@@ -34,6 +34,7 @@
 #include "options.h"
 #include "osd.h"
 #include "post.h"
+#include "config_wrapper.h"
 
 #define XINE_CONFIG_DIR  ".xine"
 #define XINE_CONFIG_FILE "config"
@@ -43,6 +44,15 @@ struct fbxine fbxine =
 	tty_fd:        -1,
 	video_port_id: "fb",
 
+        post_elements: NULL,
+        post_elements_num: 0,
+        post_enable: 1,
+          
+        deinterlace_plugin: NULL,
+        deinterlace_elements: NULL,
+        deinterlace_elements_num: 0,
+        deinterlace_enable: 0,
+
 	exit_cond: PTHREAD_COND_INITIALIZER
 };
 
@@ -93,6 +103,13 @@ static void event_listener(void *user_data, const xine_event_t *event)
 
 static int open_and_play(const char *mrl)
 {
+
+        if(!strncasecmp(mrl, "cfg:/", 5)) 
+	{
+	        config_mrl(mrl);
+		return 0;
+	}
+
 	if((!xine_open(fbxine.stream, fbxine.mrl[fbxine.current_mrl])) ||
 	   (!xine_play(fbxine.stream, 0, 0)))
 	{
@@ -114,15 +131,25 @@ static int init_video(void)
 	static struct fbxine_callback exit_callback;
 
 	fbxine_register_exit(&exit_callback, (fbxine_callback_t)exit_video);
-
-	fbxine.video_port =
-		xine_open_video_driver(fbxine.xine, fbxine.video_port_id,
+	
+	if (!strcmp(fbxine.video_port_id, "dxr3"))
+	    fbxine.video_port =
+	        xine_open_video_driver(fbxine.xine, fbxine.video_port_id,
+				       XINE_VISUAL_TYPE_X11, NULL);
+	else if (!strcmp(fbxine.video_port_id, "none"))
+	    fbxine.video_port =
+	        xine_open_video_driver(fbxine.xine, fbxine.video_port_id,
+				       XINE_VISUAL_TYPE_NONE, NULL);
+	else
+	    fbxine.video_port =
+	        xine_open_video_driver(fbxine.xine, fbxine.video_port_id,
 				       XINE_VISUAL_TYPE_FB, NULL);
 	if(!fbxine.video_port)
 	{
-		fprintf(stderr, "Video port failed.\n");
-		return 0;
+	    fprintf(stderr, "Video port failed.\n");
+	    return 0;
 	}
+	
 
 	return 1;
 }
@@ -257,6 +284,11 @@ static int fbxine_init(int argc, char **argv)
 	  pplugin_rewire_posts();
 	}
 
+	post_deinterlace_init(NULL);
+
+	if (fbxine.deinterlace_enable)
+	  post_deinterlace();
+
 	osd_init();
 	fbxine.osd.enabled = 1;
 	return 1;
