@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: actions.c,v 1.208 2003/07/25 22:52:57 f1rmb Exp $
+ * $Id: actions.c,v 1.234 2004/04/14 22:45:54 f1rmb Exp $
  *
  * implementation of all the various actions for the gui (play, stop, open, pause...)
  *
@@ -46,6 +46,75 @@ extern _panel_t        *panel;
 
 static pthread_mutex_t new_pos_mutex =  PTHREAD_MUTEX_INITIALIZER;
 
+void reparent_all_windows(void) {
+  int                    i;
+  typedef struct {
+    int                 (*visible)(void);
+    void                (*reparent)(void);
+  } reparent_t;
+  reparent_t _reparent[] = {
+    { panel_is_visible,         panel_reparent },
+    { mrl_browser_is_visible,   mrl_browser_reparent },
+    { playlist_is_visible,      playlist_reparent },
+    { control_is_visible,       control_reparent },
+    { setup_is_visible,         setup_reparent },
+    { viewlog_is_visible,       viewlog_reparent },
+    { kbedit_is_visible,        kbedit_reparent },
+    { event_sender_is_visible,  event_sender_reparent },
+    { stream_infos_is_visible,  stream_infos_reparent },
+    { tvset_is_visible,         tvset_reparent },
+    { pplugin_is_visible,       pplugin_reparent },
+    { help_is_visible,          help_reparent },
+    { NULL,                     NULL}
+  };
+  
+  for(i = 0; _reparent[i].visible; i++) {
+    if(_reparent[i].visible())
+      _reparent[i].reparent();
+  }
+
+}
+
+void raise_window(Window window, int visible, int running) {
+  if(window) {
+    if(visible && running) {
+      reparent_window(window);
+      layer_above_video(window);
+    }
+  }
+}
+
+void toggle_window(Window window, xitk_widget_list_t *widget_list, int *visible, int running) {
+  if(window && (*visible) && running) {
+    XLockDisplay(gGui->display);
+    if(gGui->use_root_window) {
+      if(xitk_is_window_visible(gGui->display, window))
+        XIconifyWindow(gGui->display, window, gGui->screen);
+      else
+	XMapWindow(gGui->display, window);
+    }
+    else {
+      *visible = 0;
+      xitk_hide_widgets(widget_list);
+      XUnmapWindow(gGui->display, window);
+    }
+    XUnlockDisplay(gGui->display);
+  }
+  else {
+    if(running) {
+      *visible = 1;
+      xitk_show_widgets(widget_list);
+      XLockDisplay(gGui->display);
+      XRaiseWindow(gGui->display, window);
+      XMapWindow(gGui->display, window);
+      if(!gGui->use_root_window)
+        XSetTransientForHint (gGui->display, window, gGui->video_window);
+      XUnlockDisplay(gGui->display);
+      layer_above_video(window);
+    }
+  }
+}
+
 int gui_xine_get_pos_length(xine_stream_t *stream, int *pos, int *time, int *length) {
   int t = 0, ret = 0;
   int lpos, ltime, llength;
@@ -79,6 +148,22 @@ int gui_xine_get_pos_length(xine_stream_t *stream, int *pos, int *time, int *len
   return ret;
 }
 
+/*
+ *
+ */
+void try_to_set_input_focus(Window window) {
+  int t = 0;
+
+  while((!xitk_is_window_visible(gGui->display, window)) && (++t < 3))
+    xine_usec_sleep(5000);
+  
+  if(xitk_is_window_visible(gGui->display, window)) {
+    XLockDisplay (gGui->display);
+    XSetInputFocus(gGui->display, window, RevertToParent, CurrentTime);
+    XUnlockDisplay (gGui->display);
+  }
+}
+
 /*
  *
  */
@@ -96,7 +181,7 @@ void gui_display_logo(void) {
     visual_anim_stop();
 
   if(gGui->display_logo)
-    (void) gui_xine_open_and_play((char *)gGui->logo_mrl, NULL, 0, 0, 0);
+    (void) gui_xine_open_and_play((char *)gGui->logo_mrl, NULL, 0, 0, 0, 0);
 
   gGui->logo_mode = 1;
   
@@ -161,6 +246,7 @@ static int _gui_xine_play(xine_stream_t *stream,
 	gGui->mmk.ident = strdup(ident);
 	gGui->playlist.mmk[gGui->playlist.cur]->ident = strdup(ident);
 	
+	video_window_set_mrl(gGui->mmk.ident);
 	playlist_mrlident_toggle();
 	panel_update_mrl_display();
 	free(ident);
@@ -176,50 +262,35 @@ static int _gui_xine_play(xine_stream_t *stream,
 	  if(!video_window_is_visible())
 	    video_window_set_visibility(1);
 	  
-	  if(gGui->auto_panel_visibility && (panel_is_visible()))
-	    panel_toggle_visibility(NULL, NULL);
-	  
-	}
-	else {
-	  
-	  if(gGui->auto_panel_visibility && (panel_is_visible()))
-	    panel_toggle_visibility(NULL, NULL);
-	  
 	}
+
+	if(gGui->auto_panel_visibility && video_window_is_visible() &&
+	   panel_is_visible() )
+	  panel_toggle_visibility(NULL, NULL);
 	
       }
       else {
 	
 	if(gGui->auto_vo_visibility) {
 	  
-	  if(gGui->auto_panel_visibility) {
-	    
-	    if(video_window_is_visible())
-	      video_window_set_visibility(0);
-	    
-	    if(!panel_is_visible())
-	      panel_toggle_visibility(NULL, NULL);
-	  }
-	  else {
-
-	    if((panel_is_visible()) && (video_window_is_visible()))
-	      video_window_set_visibility(0);
-
-	  }
-	}
-	else {
-
-	  if(gGui->auto_panel_visibility && (video_window_is_visible()) && (!panel_is_visible()))
+	  if(!panel_is_visible())
 	    panel_toggle_visibility(NULL, NULL);
-	  
-	  if(video_window_is_visible()) {
-	    if(!gGui->visual_anim.running)
-	      visual_anim_play();
-	  }
-	  else
-	    gGui->visual_anim.running = 2;
-	  
+
+	  if(video_window_is_visible())
+	    video_window_set_visibility(0);
+	    
 	}
+
+	if(gGui->auto_panel_visibility && video_window_is_visible() && 
+	  !panel_is_visible() )
+	  panel_toggle_visibility(NULL, NULL);
+	  
+	if(video_window_is_visible()) {
+	  if(!gGui->visual_anim.running)
+	    visual_anim_play();
+	}
+	else
+	  gGui->visual_anim.running = 2;
       }
       
       xine_usec_sleep(100);
@@ -345,9 +416,14 @@ int gui_xine_play(xine_stream_t *stream, int start_pos, int start_time_in_secs,
   return _gui_xine_play(stream, start_pos, start_time_in_secs, update_mmk);
 }
 
-int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos, int start_time, int offset) {
+int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos, 
+			   int start_time, int av_offset, int spu_offset) {
   char *mrl = _mrl;
   
+  if(gGui->verbosity)
+    printf("%s():\n\tmrl: '%s',\n\tsub '%s',\n\tstart_pos %d, start_time %d, av_offset %d, spu_offset %d.\n",
+	   __func__, _mrl, (_sub) ? _sub : "NONE", start_pos, start_time, av_offset, spu_offset);
+  
   if(!strncasecmp(mrl, "cfg:/", 5)) {
     config_mrl(mrl);
     gui_playlist_start_next();
@@ -393,7 +469,7 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos, int start_time
 	  fclose(fd);
 
 	  mediamark_replace_entry(&gGui->playlist.mmk[gGui->playlist.cur], 
-				  newmrl, ident, sub, start, end, 0);
+				  newmrl, ident, sub, start, end, 0, 0);
 	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
 	  mrl = gGui->mmk.mrl;
 	}
@@ -407,7 +483,7 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos, int start_time
 
   }
 
-  if(mrl_looks_playlist(mrl)) {
+  if(mrl_look_like_playlist(mrl)) {
     if(mediamark_concat_mediamarks(mrl)) {
       gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
       mrl = gGui->mmk.mrl;
@@ -416,6 +492,10 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos, int start_time
   }
   
   if(!xine_open(gGui->stream, (const char *) mrl)) {
+    
+    if(!strcmp(mrl, gGui->mmk.mrl))
+      gGui->playlist.mmk[gGui->playlist.cur]->played = 1;
+    
     gui_handle_xine_error(gGui->stream, mrl);
     return 0;
   }
@@ -429,10 +509,14 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos, int start_time
   else
     xine_close (gGui->spu_stream);
   
-  xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, offset);
+  xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, av_offset);
+  xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, spu_offset);
 
-  if(!gui_xine_play(gGui->stream, start_pos, start_time, 1))
+  if(!gui_xine_play(gGui->stream, start_pos, start_time, 1)) {
+    if(!strcmp(mrl, gGui->mmk.mrl))
+      gGui->playlist.mmk[gGui->playlist.cur]->played = 1;
     return 0;
+  }
   
   if(!strcmp(mrl, gGui->mmk.mrl))
     gGui->playlist.mmk[gGui->playlist.cur]->played = 1;
@@ -448,18 +532,22 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos, int start_time
 void gui_exit (xitk_widget_t *w, void *data) {
 
   if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
-    /* gui_stop(NULL, NULL); */
-    /* use a stripped down version of gui_stop() for faster exiting;
-     * the original version would play the logo again, which would
-     * make xine wait for one logo cycle on exit, which is intolerable
-     * with a longer, animated logo */
     gGui->ignore_next = 1;
+
+    if(gGui->visual_anim.running) {
+      xine_post_out_t * audio_source;
+
+      xine_stop(gGui->visual_anim.stream);
+
+      while(xine_get_status(gGui->visual_anim.stream) != XINE_STATUS_STOP)
+      	xine_usec_sleep(50000);
+      
+      audio_source = xine_get_audio_source(gGui->stream);
+      (void) xine_post_wire_audio_port(audio_source, gGui->ao_port);
+    }
+    
     xine_stop (gGui->stream);
     gGui->ignore_next = 0;
-    if(gGui->visual_anim.running)
-      xine_stop(gGui->visual_anim.stream);
-    while(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
-      xine_usec_sleep(50000);
   }
   
   gGui->on_quit = 1;
@@ -473,7 +561,8 @@ void gui_exit (xitk_widget_t *w, void *data) {
   if(video_window_is_visible())
     video_window_set_visibility(0);
   
-  video_window_exit ();
+  tvout_deinit(gGui->tvout);
+  video_window_exit();
 
 #ifdef HAVE_XF86VIDMODE
   /* just in case a different modeline than the original one is running,
@@ -481,7 +570,7 @@ void gui_exit (xitk_widget_t *w, void *data) {
    * the original modeline
    */
   if(gGui->XF86VidMode_fullscreen)
-    video_window_set_fullscreen_mode (0);
+    video_window_set_fullscreen_mode(WINDOWED_MODE);
   //     gui_set_fullscreen_mode(NULL,NULL);
 #endif
    
@@ -491,6 +580,7 @@ void gui_exit (xitk_widget_t *w, void *data) {
 
   osd_deinit();
 
+  config_update_num("gui.amp_level", gGui->mixer.amp_level);
   config_save();
 
   xine_close(gGui->stream);
@@ -512,9 +602,13 @@ void gui_exit (xitk_widget_t *w, void *data) {
 
   if(gGui->vo_port)
     xine_close_video_driver(gGui->xine, gGui->vo_port);
+  if(gGui->vo_none)
+    xine_close_video_driver(gGui->xine, gGui->vo_none);
 
   if(gGui->ao_port)
     xine_close_audio_driver(gGui->xine, gGui->ao_port);
+  if(gGui->ao_none)
+    xine_close_audio_driver(gGui->xine, gGui->ao_none);
 
   xine_exit(gGui->xine); 
 
@@ -557,9 +651,13 @@ void gui_play (xitk_widget_t *w, void *data) {
       return;
     }
     
-    if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, gGui->mmk.start, gGui->mmk.offset))
+    if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
+			       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset)) {
+      
+      if(mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
+	gui_exit(NULL, NULL);
       gui_display_logo();
-    
+    }
   }
   else {
     xine_set_param(gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
@@ -574,6 +672,8 @@ void gui_stop (xitk_widget_t *w, void *data) {
   xine_stop (gGui->stream);
   gGui->ignore_next = 0;
 
+  gGui->playlist.control &= !PLAYLIST_CONTROL_STOP;
+
   gGui->stream_length.pos = gGui->stream_length.time = gGui->stream_length.length = 0;
   
   mediamark_reset_played_state();
@@ -659,7 +759,8 @@ void gui_eject(xitk_widget_t *w, void *data) {
 				       gGui->playlist.mmk[i]->sub,
 				       gGui->playlist.mmk[i]->start, 
 				       gGui->playlist.mmk[i]->end,
-				       gGui->playlist.mmk[i]->offset);
+				       gGui->playlist.mmk[i]->av_offset,
+				       gGui->playlist.mmk[i]->spu_offset);
 	    new_num++;
 	  }
 	}
@@ -696,10 +797,8 @@ void gui_eject(xitk_widget_t *w, void *data) {
 
       }
 
-      if(is_playback_widgets_enabled() && (!gGui->playlist.num)) {
+      if(is_playback_widgets_enabled() && (!gGui->playlist.num))
 	enable_playback_controls(0);
-	gui_display_logo();
-      }
 
     }
     
@@ -715,20 +814,22 @@ void gui_eject(xitk_widget_t *w, void *data) {
 
 void gui_toggle_visibility(xitk_widget_t *w, void *data) {
 
-  if(panel_is_visible()) {
+  if(panel_is_visible() && (!gGui->use_root_window)) {
+
     video_window_set_visibility(!(video_window_is_visible()));
-    XLockDisplay(gGui->display);
-    XRaiseWindow(gGui->display, gGui->panel_window); 
-    XMapWindow(gGui->display, gGui->panel_window); 
-    XUnlockDisplay(gGui->display);
-    layer_above_video(gGui->panel_window);
+
+    /* We need to reparent all visible windows because of redirect tweaking */
+    reparent_all_windows();
 
     /* (re)start/stop visual animation */
     if(video_window_is_visible()) {
+      layer_above_video(gGui->panel_window);
+      
       if(gGui->visual_anim.enabled && (gGui->visual_anim.running == 2))
 	visual_anim_play();
     }
     else {
+
       if(gGui->visual_anim.running) {
 	visual_anim_stop();
 	gGui->visual_anim.running = 2;
@@ -750,8 +851,9 @@ static void set_fullscreen_mode(int fullscreen_mode) {
   int stream_infos = stream_infos_is_visible();
   int tvset        = tvset_is_visible();
   int pplugin      = pplugin_is_visible();
+  int help         = help_is_visible();
 
-  if(!(video_window_is_visible()))
+  if((!(video_window_is_visible())) || gGui->use_root_window)
     return;
 
   if(panel)
@@ -777,6 +879,8 @@ static void set_fullscreen_mode(int fullscreen_mode) {
       tvset_toggle_visibility(NULL, NULL);
     if(pplugin)
       pplugin_toggle_visibility(NULL, NULL);
+    if(help)
+      help_toggle_visibility(NULL, NULL);
   }
   
   video_window_set_fullscreen_mode(fullscreen_mode);
@@ -806,20 +910,23 @@ static void set_fullscreen_mode(int fullscreen_mode) {
       stream_infos_toggle_visibility(NULL, NULL);
     if(tvset)
       tvset_toggle_visibility(NULL, NULL);
-    if(pplugin) {
+    if(pplugin)
       pplugin_toggle_visibility(NULL, NULL);
-    }
+    if(help)
+      help_toggle_visibility(NULL, NULL);
   }
 
 }
 
 void gui_set_fullscreen_mode(xitk_widget_t *w, void *data) {
-  set_fullscreen_mode((video_window_get_fullscreen_mode() + 1));
+  set_fullscreen_mode(FULLSCR_MODE);
 }
 
-void gui_set_xinerama_fullscreen_mode(xitk_widget_t *w, void *data) {
-  set_fullscreen_mode((video_window_get_fullscreen_mode() + 4));
+#ifdef HAVE_XINERAMA
+void gui_set_xinerama_fullscreen_mode(void) {
+  set_fullscreen_mode(FULLSCR_XI_MODE);
 }
+#endif
 
 void gui_toggle_aspect(int aspect) {
   static char *ratios[XINE_VO_ASPECT_NUM_RATIOS + 1] = {
@@ -850,14 +957,9 @@ void gui_toggle_aspect(int aspect) {
 }
 
 void gui_toggle_interlaced(void) {
-  int deint;
-  
-  xine_set_param(gGui->stream, XINE_PARAM_VO_DEINTERLACE, 
-		 (1 - (xine_get_param(gGui->stream, XINE_PARAM_VO_DEINTERLACE))));
-
-  deint = xine_get_param(gGui->stream, XINE_PARAM_VO_DEINTERLACE);
-
-  osd_display_info(_("Deinterlace: %d"), deint);
+  gGui->deinterlace_enable = !gGui->deinterlace_enable;
+  osd_display_info(_("Deinterlace: %s"), (gGui->deinterlace_enable) ? _("enabled") : _("disabled"));
+  post_deinterlace();
   
   if (panel_is_visible())  {
     XLockDisplay(gGui->display);
@@ -1098,6 +1200,7 @@ void gui_seek_relative (int off_sec) {
 }
 
 void gui_dndcallback(char *filename) {
+  int more_than_one = -1;
 
   if(filename) {
     char  buffer[strlen(filename) + 10];
@@ -1120,17 +1223,37 @@ void gui_dndcallback(char *filename) {
 
       __second_stat:
 
-	if((stat(p, &pstat)) == 0)
-	  sprintf(buffer, "file:/%s", p);
+	if((stat(p, &pstat)) == 0) {
+	  if(is_a_dir(p)) {
+	    if(*(p + (strlen(p) - 1)) == '/')
+	      *(p + (strlen(p) - 1)) = '\0'; 
+	    
+	    mediamark_collect_from_directory(p);
+	    more_than_one = gGui->playlist.cur;
+	    goto __do_play;
+	  }
+	  else
+	    sprintf(buffer, "file:/%s", p);
+	}
 	else {
-	  
 	  sprintf(buffer2, "/%s", p);
-
+	  
 	  /* file don't exist, add it anyway */
 	  if((stat(buffer2, &pstat)) == -1)
 	    sprintf(buffer, "%s", filename);
-	  else
-	    sprintf(buffer, "file:/%s", buffer2);
+	  else {
+	    if(is_a_dir(buffer2)) {
+	      
+	      if(buffer2[strlen(buffer2) - 1] == '/')
+		buffer2[strlen(buffer2) - 1] = '\0'; 
+	      
+	      mediamark_collect_from_directory(buffer2);
+	      more_than_one = gGui->playlist.cur;
+	      goto __do_play;
+	    }
+	    else
+	      sprintf(buffer, "file:/%s", buffer2);
+	  }
 	  
 	}
       }
@@ -1142,16 +1265,31 @@ void gui_dndcallback(char *filename) {
     else
       sprintf(buffer, "%s", filename);
     
-    mediamark_add_entry(buffer, buffer, NULL, 0, -1, 0);
+    if(is_a_dir(buffer)) {
+      if(buffer[strlen(buffer) - 1] == '/')
+	buffer[strlen(buffer) - 1] = '\0'; 
+      
+      mediamark_collect_from_directory(buffer);
+      more_than_one = gGui->playlist.cur;
+    }
+    else
+      mediamark_add_entry(buffer, buffer, NULL, 0, -1, 0, 0);
+    
+  __do_play:
     
-    if((xine_get_status(gGui->stream) == XINE_STATUS_STOP) || gGui->logo_mode) {
-      gGui->playlist.cur = (gGui->playlist.num - 1);
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
-      if(gGui->smart_mode)
-	gui_play(NULL, NULL);
-    }   
-
     playlist_update_playlist();
+
+    if(!(gGui->playlist.control & PLAYLIST_CONTROL_IGNORE)) {
+      if((xine_get_status(gGui->stream) == XINE_STATUS_STOP) || gGui->logo_mode) {
+	if((more_than_one > -1) && ((more_than_one + 1) < gGui->playlist.num))
+	  gGui->playlist.cur = more_than_one + 1;
+	else
+	  gGui->playlist.cur = gGui->playlist.num - 1;
+	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	if(gGui->smart_mode)
+	  gui_play(NULL, NULL);
+      }
+    }
     
     if((!is_playback_widgets_enabled()) && gGui->playlist.num)
       enable_playback_controls(1);
@@ -1205,7 +1343,7 @@ void gui_direct_nextprev(xitk_widget_t *w, void *data, int value) {
 	  gGui->playlist.cur = newcur;
 	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
 	  if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				     gGui->mmk.start, gGui->mmk.offset))
+				     gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
 	    gui_display_logo();
 
 	  gGui->ignore_next = 0;
@@ -1246,7 +1384,7 @@ void gui_direct_nextprev(xitk_widget_t *w, void *data, int value) {
 	  if((gGui->playlist.cur < gGui->playlist.num)) {
 	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
 	    if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				       gGui->mmk.start, gGui->mmk.offset))
+				       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
 	      gui_display_logo();
 
 	  }
@@ -1267,7 +1405,7 @@ void gui_direct_nextprev(xitk_widget_t *w, void *data, int value) {
 	  if((gGui->playlist.cur < gGui->playlist.num)) {
 	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
 	    if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				       gGui->mmk.start, gGui->mmk.offset))
+				       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
 	      gui_display_logo();
 	  }
 	  else
@@ -1280,7 +1418,7 @@ void gui_direct_nextprev(xitk_widget_t *w, void *data, int value) {
 	  gGui->playlist.cur = newcur;
 	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
 	  if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				     gGui->mmk.start, gGui->mmk.offset))
+				     gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
 	    gui_display_logo();
 	}
 
@@ -1338,7 +1476,7 @@ void gui_mrlbrowser_show(xitk_widget_t *w, void *data) {
 }
 
 void gui_set_current_mrl(mediamark_t *mmk) {
-
+  
   if(gGui->mmk.mrl)
     free(gGui->mmk.mrl);
   if(gGui->mmk.ident)
@@ -1347,12 +1485,13 @@ void gui_set_current_mrl(mediamark_t *mmk) {
     free(gGui->mmk.sub);
   
   if(mmk) {
-    gGui->mmk.mrl    = strdup(mmk->mrl);
-    gGui->mmk.ident  = strdup(((mmk->ident) ? mmk->ident : mmk->mrl));
-    gGui->mmk.sub    = mmk->sub ? strdup(mmk->sub): NULL;
-    gGui->mmk.start  = mmk->start;
-    gGui->mmk.end    = mmk->end;
-    gGui->mmk.offset = mmk->offset;
+    gGui->mmk.mrl        = strdup(mmk->mrl);
+    gGui->mmk.ident      = strdup(((mmk->ident) ? mmk->ident : mmk->mrl));
+    gGui->mmk.sub        = mmk->sub ? strdup(mmk->sub): NULL;
+    gGui->mmk.start      = mmk->start;
+    gGui->mmk.end        = mmk->end;
+    gGui->mmk.av_offset  = mmk->av_offset;
+    gGui->mmk.spu_offset = mmk->spu_offset;
   }
   else {
     char buffer[1024];
@@ -1360,16 +1499,18 @@ void gui_set_current_mrl(mediamark_t *mmk) {
     memset(&buffer, 0, sizeof(buffer));
     sprintf(buffer, "xine-ui version %s", VERSION);
     
-    gGui->mmk.mrl    = strdup(_("There is no mrl."));
-    gGui->mmk.ident  = strdup(buffer);
-    gGui->mmk.sub    = NULL;
-    gGui->mmk.start  = 0;
-    gGui->mmk.end    = -1;
-    gGui->mmk.offset = 0;
+    gGui->mmk.mrl        = strdup(_("There is no mrl."));
+    gGui->mmk.ident      = strdup(buffer);
+    gGui->mmk.sub        = NULL;
+    gGui->mmk.start      = 0;
+    gGui->mmk.end        = -1;
+    gGui->mmk.av_offset  = 0;
+    gGui->mmk.spu_offset = 0;
   }
 
+  video_window_set_mrl(gGui->mmk.ident);
   event_sender_update_menu_buttons();
-  panel_update_mrl_display ();
+  panel_update_mrl_display();
   playlist_update_focused_entry();
 }
 
@@ -1458,6 +1599,7 @@ void gui_vpp_show(xitk_widget_t *w, void *data) {
 }
 
 void gui_vpp_enable(void) {
+
   if(pplugin_is_post_selected()) {
     gGui->post_enable = !gGui->post_enable;
     osd_display_info(_("Post plugins: %s."), (gGui->post_enable) ? _("enabled") : _("disabled"));
@@ -1474,7 +1616,7 @@ void gui_viewlog_show(xitk_widget_t *w, void *data) {
   if (viewlog_is_running() && !viewlog_is_visible())
     viewlog_toggle_visibility(NULL, NULL);
   else if(!viewlog_is_running())
-    viewlog_window();
+    viewlog_panel();
   else {
     if(gGui->use_root_window)
       viewlog_toggle_visibility(NULL, NULL);
@@ -1497,6 +1639,20 @@ void gui_kbedit_show(xitk_widget_t *w, void *data) {
   }
 }
 
+void gui_help_show(xitk_widget_t *w, void *data) {
+
+  if (help_is_running() && !help_is_visible())
+    help_toggle_visibility(NULL, NULL);
+  else if(!help_is_running())
+    help_panel();
+  else {
+    if(gGui->use_root_window)
+      help_toggle_visibility(NULL, NULL);
+    else
+      help_exit(NULL, NULL);
+  }
+}
+
 /*
  * Return 1 if layer should be set
  */
@@ -1520,7 +1676,7 @@ void layer_above_video(Window w) {
   if(!(is_layer_above()))
     return;
   
-  if (video_window_get_fullscreen_mode() && video_window_is_visible()) {
+  if ((!(video_window_get_fullscreen_mode() & WINDOWED_MODE)) && video_window_is_visible()) {
     layer = xitk_get_layer_level();
   }
   else {
@@ -1533,42 +1689,44 @@ void layer_above_video(Window w) {
   xitk_set_window_layer(w, layer);
 }
 
-void gui_increase_audio_volume(void) {
-
-  if(gGui->mixer.caps & MIXER_CAP_VOL) { 
-    if(gGui->mixer.volume_level < 100) {
-      gGui->mixer.volume_level++;
-      xine_set_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME, gGui->mixer.volume_level);
-      xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.volume_level);
-      osd_draw_bar(_("Audio Volume"), 0, 100, gGui->mixer.volume_level, OSD_BAR_STEPPER);
-    }
-  }
+void change_amp_vol(int value) {
+  gGui->mixer.amp_level = value;
+  xine_set_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL, gGui->mixer.amp_level);
+  xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.amp_level);
+  osd_draw_bar(_("Amplification Level"), 0, 200, gGui->mixer.amp_level, OSD_BAR_STEPPER);
 }
-
-void gui_decrease_audio_volume(void) {
-
-  if(gGui->mixer.caps & MIXER_CAP_VOL) { 
-    if(gGui->mixer.volume_level > 0) {
-      gGui->mixer.volume_level--;
-      xine_set_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME, gGui->mixer.volume_level);
-      xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.volume_level);
-      osd_draw_bar(_("Audio Volume"), 0, 100, gGui->mixer.volume_level, OSD_BAR_STEPPER);
-    }
-  }
-}
-
 void gui_increase_amp_level(void) {
-  if(gGui->mixer.amp < 200)
-    config_update_num("gui.amp_level", gGui->mixer.amp + 1);
+  if(gGui->mixer.amp_level < 200)
+    change_amp_vol((gGui->mixer.amp_level + 1));
 }
-
 void gui_decrease_amp_level(void) {
-  if(gGui->mixer.amp > 0)
-    config_update_num("gui.amp_level", gGui->mixer.amp - 1);
+  if(gGui->mixer.amp_level > 0)
+    change_amp_vol((gGui->mixer.amp_level - 1));
+}
+void gui_reset_amp_level(void) {
+  change_amp_vol(100);
 }
 
-void gui_reset_amp_level(void) {
-  config_update_num("gui.amp_level", 100);
+void change_audio_vol(int value) {
+  gGui->mixer.volume_level = value;
+  xine_set_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME, gGui->mixer.volume_level);
+  xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.volume_level);
+  osd_draw_bar(_("Audio Volume"), 0, 100, gGui->mixer.volume_level, OSD_BAR_STEPPER);
+}
+void gui_increase_audio_volume(void) {
+  if((gGui->mixer.method == SOUND_CARD_MIXER) && 
+     (gGui->mixer.caps & MIXER_CAP_VOL) && (gGui->mixer.volume_level < 100))
+    change_audio_vol((gGui->mixer.volume_level + 1));
+  else if(gGui->mixer.method == SOFTWARE_MIXER)
+    gui_increase_amp_level();
+  
+}
+void gui_decrease_audio_volume(void) {
+  if((gGui->mixer.method == SOUND_CARD_MIXER) &&
+     (gGui->mixer.caps & MIXER_CAP_VOL) && (gGui->mixer.volume_level > 0))
+    change_audio_vol((gGui->mixer.volume_level - 1));
+  else if(gGui->mixer.method == SOFTWARE_MIXER)
+    gui_decrease_amp_level();
 }
 
 void gui_change_zoom(int zoom_dx, int zoom_dy) {
@@ -1649,51 +1807,67 @@ void gui_add_mediamark(void) {
       secs /= 1000;
       
       mediamark_add_entry(gGui->mmk.mrl, gGui->mmk.ident, 
-			  gGui->mmk.sub, secs, -1, gGui->mmk.offset);
+			  gGui->mmk.sub, secs, -1, gGui->mmk.av_offset, gGui->mmk.spu_offset);
       playlist_update_playlist();
     }
   }
 }
 
 static void fileselector_cancel_callback(filebrowser_t *fb) {
-  sprintf(gGui->curdir, "%s", (filebrowser_get_current_dir(fb)));
-  config_update_string("input.file_origin_path", gGui->curdir);
+  char *cur_dir = filebrowser_get_current_dir(fb);
+
+  if(cur_dir && strlen(cur_dir)) {
+    sprintf(gGui->curdir, "%s", cur_dir);
+    config_update_string("input.file_origin_path", gGui->curdir);
+  }
 }
+
 static void fileselector_callback(filebrowser_t *fb) {
   char *file;
+  char *cur_dir = filebrowser_get_current_dir(fb);
+  
+  if(cur_dir && strlen(cur_dir)) {
+    sprintf(gGui->curdir, "%s", cur_dir);
+    config_update_string("input.file_origin_path", gGui->curdir);
+  }
+  
+  if(((file = filebrowser_get_full_filename(fb)) != NULL) && strlen(file)) {
+    int first  = gGui->playlist.num;
 
-  sprintf(gGui->curdir, "%s", (filebrowser_get_current_dir(fb)));
-  config_update_string("input.file_origin_path", gGui->curdir);
-
-  if((file = filebrowser_get_full_filename(fb)) != NULL) {
-    int first = gGui->playlist.num;
-
-    if(file)
+    if(file)  {
+      gGui->playlist.control |= PLAYLIST_CONTROL_IGNORE;
       gui_dndcallback(file);
-    free(file);
-
-    if(gGui->smart_mode) {
-      if(xine_get_status(gGui->stream) == XINE_STATUS_PLAY)
-	gui_stop(NULL, NULL);
+      gGui->playlist.control &= !PLAYLIST_CONTROL_IGNORE;
+      free(file);
+    }
 
+    if((first != gGui->playlist.num) && gGui->smart_mode) {
+      if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
+	gGui->ignore_next = 1;
+	xine_stop(gGui->stream);
+	gGui->ignore_next = 0;
+      }
+      
       gGui->playlist.cur = first;
       gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
       gui_play(NULL, NULL);
-    }   
+    }
   }
-  
 }
+
 static void fileselector_all_callback(filebrowser_t *fb) {
   char **files;
-  char  *path;
-
-  sprintf(gGui->curdir, "%s", (filebrowser_get_current_dir(fb)));
-  config_update_string("input.file_origin_path", gGui->curdir);
-
+  char  *path = filebrowser_get_current_dir(fb);
+  
+  if(path && strlen(path)) {
+    sprintf(gGui->curdir, "%s", path);
+    config_update_string("input.file_origin_path", gGui->curdir);
+  }
+  
   if((files = filebrowser_get_all_files(fb)) != NULL) {
     int i = 0;
 
-    if((path = filebrowser_get_current_dir(fb)) != NULL) {
+    if(path && strlen(path)) {
       char pathname[XITK_PATH_MAX + XITK_NAME_MAX + 1];
       char fullfilename[XITK_PATH_MAX + XITK_NAME_MAX + 1];
       int  first = gGui->playlist.num;
@@ -1703,21 +1877,25 @@ static void fileselector_all_callback(filebrowser_t *fb) {
       else
 	sprintf(pathname, "%s", path);
       
+      gGui->playlist.control |= PLAYLIST_CONTROL_IGNORE;
       while(files[i]) {
 	sprintf(fullfilename, "%s%s", pathname, files[i++]);
 	gui_dndcallback(fullfilename);
       }
+      gGui->playlist.control &= !PLAYLIST_CONTROL_IGNORE;
       
       free(path);
 
-      if(gGui->smart_mode) {
-	if(xine_get_status(gGui->stream) == XINE_STATUS_PLAY)
-	  gui_stop(NULL, NULL);
-
+      if((first != gGui->playlist.num) && gGui->smart_mode) {
+	if(xine_get_status(gGui->stream) == XINE_STATUS_PLAY) {
+	  gGui->ignore_next = 1;
+	  xine_stop(gGui->stream);
+	  gGui->ignore_next = 0;
+	}
 	gGui->playlist.cur = first;
 	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
 	gui_play(NULL, NULL);
-      }   
+      }
     }
 
     i = 0;
@@ -1727,12 +1905,13 @@ static void fileselector_all_callback(filebrowser_t *fb) {
     free(files);
   }
 }
+
 void gui_file_selector(void) {
   filebrowser_callback_button_t  cbb[3];
   
   cbb[0].label = _("Select");
   cbb[0].callback = fileselector_callback;
-  cbb[0].need_a_file = 1;
+  cbb[0].need_a_file = 0;
   cbb[1].label = _("Select all");
   cbb[1].callback = fileselector_all_callback;
   cbb[1].need_a_file = 0;
@@ -1744,15 +1923,13 @@ void gui_file_selector(void) {
 static void subselector_callback(filebrowser_t *fb) {
   char *file;
 
-  sprintf(gGui->curdir, "%s", (filebrowser_get_current_dir(fb)));
-
   if((file = filebrowser_get_full_filename(fb)) != NULL) {
     if(file) {
       mediamark_t *mmk = mediamark_clone_mmk((mediamark_t *) mediamark_get_current_mmk());
       
       if(mmk) {
-	mediamark_replace_entry(&gGui->playlist.mmk[gGui->playlist.cur],
-				mmk->mrl, mmk->ident, file, mmk->start, mmk->end, mmk->offset);
+	mediamark_replace_entry(&gGui->playlist.mmk[gGui->playlist.cur], mmk->mrl, mmk->ident, 
+				file, mmk->start, mmk->end, mmk->av_offset, mmk->spu_offset);
 	mediamark_free_mmk(&mmk);
 
 	mmk = (mediamark_t *) mediamark_get_current_mmk();
@@ -1778,25 +1955,44 @@ static void subselector_callback(filebrowser_t *fb) {
     free(file);
   }
 }
+
 void gui_select_sub(void) {
   
   if(gGui->playlist.num) {
-    filebrowser_callback_button_t  cbb[2];
+    filebrowser_callback_button_t  cbb;
     const mediamark_t *mmk;
     
     mmk = mediamark_get_current_mmk();
 
     if(mmk) {
+      char *path, *open_path;
       
-      cbb[0].label = _("Select");
-      cbb[0].callback = subselector_callback;
-      cbb[0].need_a_file = 1;
-      cbb[1].callback = fileselector_cancel_callback;
-      cbb[1].need_a_file = 0;
-      (void *) create_filebrowser(_("Pick a subtitle file"), mmk->sub, &cbb[0], NULL, &cbb[1]);
+      cbb.label = _("Select");
+      cbb.callback = subselector_callback;
+      cbb.need_a_file = 1;
+      
+      path = mmk->sub ? mmk->sub : mmk->mrl;
+      
+      if(mrl_look_like_file(path)) {
+	char *p;
+	
+	xine_strdupa(open_path, path);
+	
+	if(!strncasecmp(path, "file:", 5))
+	  path += 5;
+
+	p = strrchr(open_path, '/');
+	if (p && strlen(p))
+	  *p = '\0';
+      }
+      else
+	open_path = gGui->curdir;
+      
+      (void *) create_filebrowser(_("Pick a subtitle file"), open_path, &cbb, NULL, NULL);
     }
   }
 }
+
 /*
  *
  */
