@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: config_wrapper.c,v 1.14 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: config_wrapper.c,v 1.18 2004/04/13 19:55:47 f1rmb Exp $
  *
  */
 
@@ -25,9 +25,21 @@
 #include "config.h"
 #endif
 
+#ifdef HAVE_ALLOCA_H   
+#include <alloca.h>
+#endif
+
 #include "common.h"
 
-extern gGui_t                 *gGui;
+extern gGui_t  *gGui;
+
+/* 
+ * experience level above this level 
+ * can't be changed by cfg:/ mrl type.
+ * This reflect xine-lib policy about security issue
+ */
+#define XINE_CONFIG_SECURITY   30
+
 
 static void config_update(xine_cfg_entry_t *entry, int type, int min, int max, int value, char *string) {
 
@@ -128,9 +140,18 @@ void config_reset(void) {
  * Handle 'cfg:/' mrl style
  */
 void config_mrl(char *mrl) {
+  xine_cfg_entry_t entry;
   char *key;
   char *config;
   char *_mrl;
+  
+  if (!xine_config_lookup_entry(gGui->xine, "misc.implicit_config", &entry) ||
+      entry.type != XINE_CONFIG_TYPE_BOOL || !entry.num_value) {
+    xine_info(_("You tried to change the configuration with a cfg: MRL.\n"
+		"This is not allowed unless you enable the 'misc.implicit_config' setting "
+		"after reading and understanding its help text."));
+    return;
+  }
 
   xine_strdupa(_mrl, mrl);
   config = strchr(_mrl, '/');
@@ -145,10 +166,15 @@ void config_mrl(char *mrl) {
       *str_value++ = '\0';
 
     if(str_value && strlen(str_value)) {
-      xine_cfg_entry_t entry;
       
       if(xine_config_lookup_entry(gGui->xine, key, &entry)) {
 
+	if(entry.exp_level >= XINE_CONFIG_SECURITY) {
+	  xine_info(_("For security reason, you're not allowed to change "
+		      "the configuration entry named '%s'."), entry.key);
+	  return;
+	}
+
 	switch(entry.type) {
 
 	case XINE_CONFIG_TYPE_STRING:
