@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: control.c,v 1.90 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: control.c,v 1.100 2004/04/13 19:55:47 f1rmb Exp $
  *
  *
  */
@@ -250,9 +250,17 @@ static void active_sliders_video_settings(void) {
     xitk_enable_widget(control->contr);
 }
 
-void control_config_register(void) {
+static void probe_active_controls(void) {
+  hue_ena    = test_vo_property(XINE_PARAM_VO_HUE);
+  bright_ena = test_vo_property(XINE_PARAM_VO_BRIGHTNESS);
+  sat_ena    = test_vo_property(XINE_PARAM_VO_SATURATION);
+  contr_ena  = test_vo_property(XINE_PARAM_VO_CONTRAST);
+}
 
-  hue_ena = test_vo_property(XINE_PARAM_VO_HUE);
+void control_config_register(void) {
+  
+  probe_active_controls();
+  
   if(hue_ena)
     set_current_param(XINE_PARAM_VO_HUE,
 		    (gGui->video_settings.hue = 
@@ -265,7 +273,6 @@ void control_config_register(void) {
 						hue_changes_cb, 
 						CONFIG_NO_DATA)));
   
-  bright_ena = test_vo_property(XINE_PARAM_VO_BRIGHTNESS);
   if(bright_ena)
     set_current_param(XINE_PARAM_VO_BRIGHTNESS, 
 		    (gGui->video_settings.brightness = 
@@ -278,7 +285,6 @@ void control_config_register(void) {
 						brightness_changes_cb, 
 						CONFIG_NO_DATA)));
 
-  sat_ena = test_vo_property(XINE_PARAM_VO_SATURATION);
   if(sat_ena)
     set_current_param(XINE_PARAM_VO_SATURATION,
 		    (gGui->video_settings.saturation = 
@@ -291,7 +297,6 @@ void control_config_register(void) {
 						saturation_changes_cb, 
 						CONFIG_NO_DATA)));
 
-  contr_ena = test_vo_property(XINE_PARAM_VO_CONTRAST);
   if(contr_ena)
     set_current_param(XINE_PARAM_VO_CONTRAST,
 		    (gGui->video_settings.contrast = 
@@ -383,57 +388,17 @@ int control_is_visible(void) {
  * Raise control->window
  */
 void control_raise_window(void) {
-  if(control != NULL) {
-    if(control->window) {
-      if(control->visible && control->running) {
-	XLockDisplay(gGui->display);
-	XUnmapWindow(gGui->display, control->window);
-	XRaiseWindow(gGui->display, control->window);
-	XMapWindow(gGui->display, control->window);
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, control->window, gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(control->window);
-      }
-    }
-  }
+  if(control != NULL)
+    raise_window(control->window, control->visible, control->running);
 }
 
 /*
  * Hide/show the control panel
  */
 void control_toggle_visibility (xitk_widget_t *w, void *data) {
-  
-  if(control != NULL) {
-    if (control->visible && control->running) {
-      XLockDisplay(gGui->display);
-      if(gGui->use_root_window) {
-	if(xitk_is_window_visible(gGui->display, control->window))
-	  XIconifyWindow(gGui->display, control->window, gGui->screen);
-	else
-	  XMapWindow(gGui->display, control->window);
-      }
-      else {
-	control->visible = 0;
-	xitk_hide_widgets(control->widget_list);
-	XUnmapWindow (gGui->display, control->window);
-      }
-      XUnlockDisplay(gGui->display);
-    }
-    else {
-      if(control->running) {
-	control->visible = 1;
-	xitk_show_widgets(control->widget_list);
-	XLockDisplay(gGui->display);
-	XRaiseWindow(gGui->display, control->window); 
-	XMapWindow(gGui->display, control->window); 
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, control->window, gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(control->window);
-      }
-    }
-  }
+  if(control != NULL)
+    toggle_window(control->window, control->widget_list,
+		  &control->visible, control->running);
 }
 
 /*
@@ -444,10 +409,14 @@ void control_handle_event(XEvent *event, void *data) {
   switch(event->type) {
     
   case KeyPress:
-    gui_handle_event(event, data);
-    break;      
+    if(xitk_get_key_pressed(event) == XK_Escape)
+      control_exit(NULL, NULL);
+    else
+      gui_handle_event(event, data);
+    break;
+    
   }
-
+  
 }
 
 /*
@@ -501,6 +470,9 @@ void control_change_skins(void) {
     
     XUnlockDisplay(gGui->display);
 
+    if(control_is_visible())
+      control_raise_window();
+
     xitk_skin_unlock(gGui->skin_config);
     
     xitk_change_skins_widget_list(control->widget_list, gGui->skin_config);
@@ -526,6 +498,11 @@ void control_deinit(void) {
   }
 }
 
+void control_reparent(void) {
+  if(control)
+    reparent_window(control->window);
+}
+
 /*
  * Create control panel window
  */
@@ -608,11 +585,16 @@ void control_panel(void) {
 				   CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 				   &attr);
   
-  XSetStandardProperties(gGui->display, control->window, title, title,
- 			 None, NULL, 0, &hint);
+  XmbSetWMProperties(gGui->display, control->window, title, title, NULL, 0, 
+                     &hint, NULL, NULL);
   
   XSelectInput(gGui->display, control->window, INPUT_MOTION | KeymapStateMask);
   XUnlockDisplay(gGui->display);
+
+  if(!video_window_is_visible())
+    xitk_set_wm_window_type(control->window, WINDOW_TYPE_NORMAL);
+  else
+    xitk_unset_wm_window_type(control->window, WINDOW_TYPE_NORMAL);
   
   if(is_layer_above())
     xitk_set_layer_above(control->window);
@@ -636,13 +618,16 @@ void control_panel(void) {
     xclasshint->res_name = title;
     xclasshint->res_class = "xine";
     XSetClassHint(gGui->display, control->window, xclasshint);
+    XFree(xclasshint);
+    
   }
 
   wm_hint = XAllocWMHints();
   if (wm_hint != NULL) {
     wm_hint->input         = True;
     wm_hint->initial_state = NormalState;
-    wm_hint->flags         = InputHint | StateHint;
+    wm_hint->icon_pixmap   = gGui->icon;
+    wm_hint->flags         = InputHint | StateHint | IconPixmapHint;
     XSetWMHints(gGui->display, control->window, wm_hint);
     XFree(wm_hint);
   }
@@ -759,6 +744,7 @@ void control_panel(void) {
 			    xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
     xitk_disable_widget(control->contr);
 
+    probe_active_controls();
     active_sliders_video_settings();
   }
 
@@ -796,7 +782,7 @@ void control_panel(void) {
 			    xitk_browser_create(control->widget_list, gGui->skin_config, &br)));
 
   xitk_browser_update_list(control->skinlist, 
-			   control->skins, control->skins_num, 0);
+			   control->skins, NULL, control->skins_num, 0);
 
 
   lb.skin_element_name = "CtlDismiss";
@@ -822,12 +808,6 @@ void control_panel(void) {
   control->visible = 1;
   control->running = 1;
   control_raise_window();
-
-  while(!xitk_is_window_visible(gGui->display, control->window))
-    xine_usec_sleep(5000);
-
-  XLockDisplay (gGui->display);
-  XSetInputFocus(gGui->display, control->window, RevertToParent, CurrentTime);
-  XUnlockDisplay (gGui->display);
-
+  
+  try_to_set_input_focus(control->window);
 }
