@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: download.c,v 1.9 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: download.c,v 1.12 2004/04/13 19:55:47 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -39,15 +39,23 @@
 extern gGui_t   *gGui;
 
 #ifdef HAVE_CURL
+
+#ifndef TRUE
+#define TRUE 1
+#endif
+#ifndef FALSE
+#define FALSE 0
+#endif
+
 static int progress_callback(void *userdata, 
 			     double dltotal, double dlnow, double ultotal, double ulnow) {
   download_t  *download = (download_t *) userdata;
   char         buffer[1024];
   int          percent = (dltotal > 0.0) ? (int) (dlnow * 100.0 / dltotal) : 0;
-
+  
   osd_draw_bar(_("Download in progress"), 0, 100, percent, OSD_BAR_POS);
   memset(&buffer, 0, sizeof(buffer));
-  sprintf(buffer, _("Download progress: %d%%."), percent);
+  snprintf(buffer, sizeof(buffer), _("Download progress: %d%%."), percent);
   gGui->mrl_overrided = 3;
   panel_set_title(buffer);
   
@@ -80,7 +88,7 @@ int network_download(const char *url, download_t *download) {
 #ifdef HAVE_CURL
   CURL        *curl;
   CURLcode     res;
-  
+
   pthread_mutex_lock(&gGui->download_mutex);
 
   curl_global_init(CURL_GLOBAL_DEFAULT);
