@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: event.c,v 1.221 2003/07/22 21:33:08 f1rmb Exp $
+ * $Id: event.c,v 1.249 2004/04/14 22:38:29 f1rmb Exp $
  *
  * gui initialization and top-level event handling stuff
  *
@@ -50,6 +50,7 @@
  * global variables
  */
 extern gGui_t          *gGui;
+extern _panel_t        *panel;
 
 static pid_t            xine_pid;
 
@@ -94,6 +95,21 @@ static char *visual_anim_style[] = {
   NULL
 };
 
+static char *mixer_control_method[] = {
+  "Sound card",
+  "Software",
+  NULL
+};
+
+static char *shortcut_style[] = {
+  "Windows style",
+  "Emacs style",
+  NULL
+};
+
+void dummy_config_cb(void *data, xine_cfg_entry_t *cfg) {
+  /* It exist to avoid "restart" window message in setup window */
+}
 static void auto_vo_visibility_cb(void *data, xine_cfg_entry_t *cfg) {
   gGui->auto_vo_visibility = cfg->num_value;
 }
@@ -151,6 +167,9 @@ static void skin_server_url_cb(void *data, xine_cfg_entry_t *cfg) {
 static void osd_enabled_cb(void *data, xine_cfg_entry_t *cfg) {
   gGui->osd.enabled = cfg->num_value;
 }
+static void osd_use_unscaled_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->osd.use_unscaled = cfg->num_value;
+}
 static void osd_timeout_cb(void *data, xine_cfg_entry_t *cfg) {
   gGui->osd.timeout = cfg->num_value;
 }
@@ -166,10 +185,28 @@ static void play_anyway_cb(void *data, xine_cfg_entry_t *cfg) {
 static void exp_level_cb(void *data, xine_cfg_entry_t *cfg) {
   gGui->experience_level = (cfg->num_value * 10);
 }
-static void amp_level_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui->mixer.amp = cfg->num_value;
-  xine_set_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL, gGui->mixer.amp);
-  osd_draw_bar(_("Amplification Level"), 0, 200, gGui->mixer.amp, OSD_BAR_STEPPER);
+
+static void audio_mixer_method_cb(void *data, xine_cfg_entry_t *cfg) {
+  int max = 100, vol = 0;
+  
+  gGui->mixer.method = cfg->num_value;
+
+  switch(gGui->mixer.method) {
+  case SOUND_CARD_MIXER:
+    max = 100;
+    vol = gGui->mixer.volume_level;
+    break;
+  case SOFTWARE_MIXER:
+    max = 200;
+    vol = gGui->mixer.amp_level;
+    break;
+  }
+
+  xitk_slider_set_max(panel->mixer.slider, max);
+  xitk_slider_set_pos(panel->mixer.slider, vol);
+}
+static void shortcut_style_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->shortcut_style = cfg->num_value;
 }
 
 int wm_not_ewmh_only(void) {
@@ -244,6 +281,29 @@ static void gui_signal_handler (int sig, void *data) {
 
 }
 
+
+/*
+ * convert pts value to string
+ */
+char *pts2str(int64_t pts) {
+  static char buf[40];
+  int64_t min;
+  double sec;
+  int ds;
+  int sign;
+
+  if ((sign = pts < 0) != 0) pts = -pts;
+  min = pts / (90000 * 60);
+  sec = (double)pts / 90000 - 60 * min;
+  ds = sec / 10;
+  sec -= 10 * ds;
+
+  snprintf(buf, sizeof(buf), "%s%02" PRIi64 ":%d%.2f (%" PRIi64 " pts)", sign ? "-" : "", min, ds, sec, pts);
+
+  return buf;
+}
+
+
 /*
  *
  */
@@ -263,7 +323,7 @@ void gui_execute_action_id(action_id_t action) {
 	gGui->numeric.arg += (action - ACTID_EVENT_NUMBER_0);
       }
       else
-	printf("WARNING: Input number canceled, avoid overflow\n");
+	fprintf(stderr, "WARNING: Input number canceled, avoid overflow\n");
 
     } 
     else if(action == ACTID_EVENT_NUMBER_10_ADD) {
@@ -276,7 +336,7 @@ void gui_execute_action_id(action_id_t action) {
       if((gGui->numeric.arg + 10) <= INT_MAX)
 	gGui->numeric.arg += 10;
       else
-	printf("WARNING: Input number canceled, avoid overflow\n");
+	fprintf(stderr, "WARNING: Input number canceled, avoid overflow\n");
 
     }
     else {
@@ -285,10 +345,10 @@ void gui_execute_action_id(action_id_t action) {
     }
 
     /* events for advanced input plugins. */
-    xine_event.type = action & ~ACTID_IS_INPUT_EVENT;
+    xine_event.type        = action & ~ACTID_IS_INPUT_EVENT;
     xine_event.data_length = 0;
-    xine_event.data = NULL;
-    xine_event.stream = gGui->stream;
+    xine_event.data        = NULL;
+    xine_event.stream      = gGui->stream;
     gettimeofday(&xine_event.tv, NULL);
     
     xine_event_send(gGui->stream, &xine_event);
@@ -374,8 +434,8 @@ void gui_execute_action_id(action_id_t action) {
     break;
 
 #ifdef HAVE_XINERAMA
-  case ACTID_TOGGLE_XINERAMA_FULLSCR:
-    gui_set_xinerama_fullscreen_mode(NULL, NULL);
+  case ACTID_TOGGLE_XINERAMA_FULLSCREEN:
+    gui_set_xinerama_fullscreen_mode();
     break;
 #endif
 
@@ -529,25 +589,25 @@ void gui_execute_action_id(action_id_t action) {
     
   case ACTID_AV_SYNC_m3600:
     {
-      int offset = (xine_get_param(gGui->stream, XINE_PARAM_AV_OFFSET) - 3600);
+      int av_offset = (xine_get_param(gGui->stream, XINE_PARAM_AV_OFFSET) - 3600);
 
-      gGui->mmk.offset = offset;
+      gGui->mmk.av_offset = av_offset;
       if (gGui->playlist.cur < gGui->playlist.num)
-        gGui->playlist.mmk[gGui->playlist.cur]->offset = offset;
-      xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, offset);
-      osd_display_info(_("A/V offset: %d"), offset);
+        gGui->playlist.mmk[gGui->playlist.cur]->av_offset = av_offset;
+      xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, av_offset);
+      osd_display_info(_("A/V offset: %s"), pts2str(av_offset));
     }
     break;
     
   case ACTID_AV_SYNC_p3600:
     {
-      int offset = (xine_get_param(gGui->stream, XINE_PARAM_AV_OFFSET) + 3600);
+      int av_offset = (xine_get_param(gGui->stream, XINE_PARAM_AV_OFFSET) + 3600);
       
-      gGui->mmk.offset = offset;
+      gGui->mmk.av_offset = av_offset;
       if (gGui->playlist.cur < gGui->playlist.num)
-        gGui->playlist.mmk[gGui->playlist.cur]->offset = offset;
-      xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, offset);
-      osd_display_info(_("A/V offset: %d"), offset);
+        gGui->playlist.mmk[gGui->playlist.cur]->av_offset = av_offset;
+      xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, av_offset);
+      osd_display_info(_("A/V offset: %s"), pts2str(av_offset));
     }
     break;
 
@@ -558,22 +618,29 @@ void gui_execute_action_id(action_id_t action) {
 
   case ACTID_SV_SYNC_p:
     {
-      int offset = xine_get_param(gGui->stream, XINE_PARAM_SPU_OFFSET) + 3600;
+      int spu_offset = xine_get_param(gGui->stream, XINE_PARAM_SPU_OFFSET) + 3600;
       
-      xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, offset);
-      osd_display_info(_("SPU Offset: %d"), offset);
+      xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, spu_offset);
+      
+      osd_display_info(_("SPU Offset: %s"), pts2str(spu_offset));
     }
     break;
 
   case ACTID_SV_SYNC_m:
     {
-      int offset = xine_get_param(gGui->stream, XINE_PARAM_SPU_OFFSET) - 3600;
+      int spu_offset = xine_get_param(gGui->stream, XINE_PARAM_SPU_OFFSET) - 3600;
       
-      xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, offset);
-      osd_display_info(_("SPU Offset: %d"), offset);
+      xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, spu_offset);
+      
+      osd_display_info(_("SPU Offset: %s"), pts2str(spu_offset));
     }
     break;
     
+  case ACTID_SV_SYNC_RESET:
+    xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, 0);
+    osd_display_info(_("SPU Offset: reset."));
+    break;
+
   case ACTID_SPEED_FAST:
     gui_change_speed_playback(NULL, (void*)GUI_PREV);
     break;
@@ -700,7 +767,6 @@ void gui_execute_action_id(action_id_t action) {
     if(gGui->playlist.loop == PLAYLIST_LOOP_MODES_NUM)
       gGui->playlist.loop = PLAYLIST_LOOP_NO_LOOP;
     
-    /* Temporary informations */    
     switch(gGui->playlist.loop) {
     case PLAYLIST_LOOP_NO_LOOP:
       osd_display_info(_("Playlist: no loop."));
@@ -804,6 +870,21 @@ void gui_execute_action_id(action_id_t action) {
     gui_vpp_enable();
     break;
 
+  case ACTID_HELP_SHOW:
+    gui_help_show(NULL, NULL);
+    break;
+
+  case ACTID_PLAYLIST_STOP:
+    if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
+      if(gGui->playlist.control & PLAYLIST_CONTROL_STOP)
+	gGui->playlist.control &= !PLAYLIST_CONTROL_STOP;
+      else
+	gGui->playlist.control |= PLAYLIST_CONTROL_STOP;
+      osd_display_info(_("Playlist: %s"), 
+		       (gGui->playlist.control & PLAYLIST_CONTROL_STOP) ? _("Stop") : _("Continue"));
+    }
+    break;
+    
   default:
     break;
   }
@@ -873,6 +954,12 @@ void gui_playlist_start_next(void) {
 
   panel_reset_slider ();
   
+  if(gGui->playlist.control & PLAYLIST_CONTROL_STOP) {
+    gGui->playlist.control &= !PLAYLIST_CONTROL_STOP;
+    gui_display_logo();
+    return;
+  }
+  
   switch(gGui->playlist.loop) {
     
   case PLAYLIST_LOOP_NO_LOOP:
@@ -882,8 +969,14 @@ void gui_playlist_start_next(void) {
     if(gGui->playlist.cur < gGui->playlist.num) {
       gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
       if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				 gGui->mmk.start, gGui->mmk.offset))
+				 gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset)) {
+
+	if((gGui->playlist.loop == PLAYLIST_LOOP_NO_LOOP) && 
+	   mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
+	  gui_exit(NULL, NULL);
+
 	gui_display_logo();
+      }
     }
     else {
       
@@ -898,9 +991,12 @@ void gui_playlist_start_next(void) {
       else if(gGui->playlist.loop == PLAYLIST_LOOP_LOOP) {
 	gGui->playlist.cur = 0;
 	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
-	if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				   gGui->mmk.start, gGui->mmk.offset))
+	if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0,
+				   gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset)) {
+	  if(mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
+	    gui_exit(NULL, NULL);
 	  gui_display_logo();
+	}
       }
 
     }
@@ -909,7 +1005,7 @@ void gui_playlist_start_next(void) {
   case PLAYLIST_LOOP_REPEAT:
     gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
     if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-			       gGui->mmk.start, gGui->mmk.offset))
+			       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
       gui_display_logo();
     break;
 
@@ -922,15 +1018,21 @@ void gui_playlist_start_next(void) {
       
       gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
       if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				 gGui->mmk.start, gGui->mmk.offset))
-	gui_display_logo();
+				 gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset)) {
+	if(!mediamark_all_played())
+	  goto __shuffle_restart;
+	else
+	  gui_display_logo();
+      }
     }
     else {
       mediamark_reset_played_state();
 
       if(gGui->playlist.loop == PLAYLIST_LOOP_SHUF_PLUS)
 	goto __shuffle_restart;
-
+      else if(gGui->actions_on_start[0] == ACTID_QUIT)
+      	gui_exit(NULL, NULL);
+      
       gui_display_logo();
     }
     break;
@@ -1046,14 +1148,29 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
    * init playlist
    */
   for (i = 0; i < nfiles; i++) {
-    char *sub;
-    
-    if((sub = strstr(filenames[i], "::")) != NULL) {
-      *sub = 0;
-      sub += 2;
+    char *sub = NULL;
+    char *file = atoa(filenames[i]);
+
+    /* grab recursively all files from dir */
+    if(is_a_dir(file)) {
+      
+      if(file[strlen(file) - 1] == '/')
+	file[strlen(file) - 1] = '\0'; 
+      
+      mediamark_collect_from_directory(file);
+    }
+    else {
+      if((sub = (char *)get_last_double_semicolon(file)) != NULL) {
+	if(is_ipv6_last_double_semicolon(file))
+	  sub = NULL;
+	else {
+	  *sub = 0;
+	  sub += 2;
+	}
+      }
+      
+      mediamark_add_entry((const char *)file, (const char *)file, sub, 0, -1, 0, 0);
     }
-    
-    mediamark_add_entry((const char *)filenames[i], (const char *)filenames[i], sub, 0, -1, 0);
   }
   
   if((gGui->playlist.loop == PLAYLIST_LOOP_SHUFFLE) || 
@@ -1061,10 +1178,8 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
     gGui->playlist.cur = mediamark_get_shuffle_next();
   
   gGui->is_display_mrl = 0;
-  gGui->mrl_overrided = 0;
-  gGui->new_pos = -1;
-
-  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+  gGui->mrl_overrided  = 0;
+  gGui->new_pos        = -1;
 
   /*
    * X / imlib stuff
@@ -1089,7 +1204,9 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
 				 CONFIG_NO_CB,
 				 CONFIG_NO_DATA)) {
 
+    XLockDisplay(gGui->display);
     XSynchronize (gGui->display, True);
+    XUnlockDisplay(gGui->display);
     fprintf (stderr, _("Warning! Synchronized X activated - this is way slow...\n"));
   }
 
@@ -1206,6 +1323,15 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
 			      osd_enabled_cb,
 			      CONFIG_NO_DATA);
 
+  gGui->osd.use_unscaled = 
+    xine_config_register_bool(gGui->xine, "gui.osd_use_unscaled", 
+			      1,
+			      _("Use unscaled OSD"),
+			      _("Use unscaled (full screen resolution) OSD if possible"), 
+			      CONFIG_LEVEL_ADV,
+			      osd_use_unscaled_cb,
+			      CONFIG_NO_DATA);
+
   gGui->osd.timeout = 
     xine_config_register_num(gGui->xine, "gui.osd_timeout", 
 			      3,
@@ -1244,13 +1370,13 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
 			       exp_level_cb, 
 			       CONFIG_NO_DATA)) * 10;
 
-  gGui->mixer.amp = xine_config_register_range(gGui->xine, "gui.amp_level", 
-					       100, 0, 200,
-					       _("Amplification level"),
-					       NULL,
-					       CONFIG_LEVEL_ADV,
-					       amp_level_cb, 
-					       CONFIG_NO_DATA);
+  gGui->mixer.amp_level = xine_config_register_range(gGui->xine, "gui.amp_level", 
+						     100, 0, 200,
+						     _("Amplification level"),
+						     NULL,
+						     CONFIG_LEVEL_DEB,
+						     dummy_config_cb, 
+						     CONFIG_NO_DATA);
 
   gGui->splash = 
     gGui->splash ? (xine_config_register_bool(gGui->xine, "gui.splash", 
@@ -1258,9 +1384,27 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
 					      _("Display splash screen"),
 					      _("If enabled, xine will display its splash screen"), 
 					      CONFIG_LEVEL_BEG,
-					      CONFIG_NO_CB,
+					      dummy_config_cb,
 					      CONFIG_NO_DATA)) : 0;
   
+  gGui->mixer.method = 
+    xine_config_register_enum(gGui->xine, "gui.audio_mixer_method", 
+			      SOUND_CARD_MIXER, mixer_control_method,
+			      _("Audio mixer control method"),
+			      _("Which method used to control audio volume."), 
+			      CONFIG_LEVEL_ADV,
+			      audio_mixer_method_cb,
+			      CONFIG_NO_DATA);
+
+  gGui->shortcut_style = 
+    xine_config_register_enum(gGui->xine, "gui.shortcut_style", 
+			      0, shortcut_style,
+			      _("Menu shortcut style"),
+			      _("Shortcut representation in menu, 'Ctrl,Alt' or 'C,M'."), 
+			      CONFIG_LEVEL_ADV,
+			      shortcut_style_cb,
+			      CONFIG_NO_DATA);
+
   gGui->numeric.set = 0;
   gGui->numeric.arg = 0;
   
@@ -1269,27 +1413,12 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
   gGui->screen = DefaultScreen(gGui->display);
 
   /* Some infos */
-  printf(_("XServer Vendor: %s. Release: %d,\n"), 
-	 XServerVendor(gGui->display), XVendorRelease(gGui->display));
-  printf(_("        Protocol Version: %d, Revision: %d,\n"), 
-	 XProtocolVersion(gGui->display), XProtocolRevision(gGui->display));
-  printf(_("        Available Screen(s): %d, using %d\n")
-	 , XScreenCount(gGui->display), gGui->screen);
-  printf(_("        Depth: %d.\n"),
-	 XDisplayPlanes(gGui->display, gGui->screen));
-#ifdef HAVE_SHM
-  if(XShmQueryExtension(gGui->display)) {
-    int major, minor, ignore;
-    Bool pixmaps;
-    
-    if(XQueryExtension(gGui->display, "MIT-SHM", &ignore, &ignore, &ignore)) {
-      if(XShmQueryVersion(gGui->display, &major, &minor, &pixmaps ) == True) {
-	printf(_("        XShmQueryVersion: %d.%d.\n"), major, minor);
-      }
-    }
+  if(gGui->verbosity) {
+    dump_host_info();
+    dump_cpu_infos();
+    dump_xfree_info(gGui->display, gGui->screen, (gGui->verbosity >= XINE_VERBOSITY_DEBUG) ? 1 : 0);
   }
-#endif
-  
+
   gui_find_visual(&gGui->visual, &gGui->depth);
 
   gui_init_imlib (gGui->visual);
@@ -1301,8 +1430,10 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
    */
   xine_pid = getppid();
   
-  xitk_init(gGui->display);
-
+  xitk_init(gGui->display, (gGui->verbosity) ? 1 : 0);
+  
+  preinit_skins_support();
+  
   if(gGui->splash)
     splash_create();
 
@@ -1319,6 +1450,8 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
    * when done before the video_window_init(). */
   gGui->kbindings = kbindings_init_kbinding();
 
+  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+
   panel_init ();
 }
 
@@ -1372,12 +1505,17 @@ void gui_init_imlib (Visual *vis) {
 /*
  *
  */
+typedef struct {
+  int      start;
+  char   **session_opts;
+} _startup_t;
+
 static void on_start(void *data) {
-  int start = (int) data;
+  _startup_t *startup = (_startup_t *) data;
 
   splash_destroy();
 
-  if((!start && !gGui->playlist.num) || (!start && gGui->playlist.num)) {
+  if((!startup->start && !gGui->playlist.num) || (!startup->start && gGui->playlist.num)) {
 
     gui_display_logo();
     
@@ -1391,10 +1529,10 @@ static void on_start(void *data) {
    * The lib seems unable to display OSD until the stream got 
    * any video data.
    */
-  if(start || (!start && !gGui->display_logo)) {
+  if(startup->start || (!startup->start && !gGui->display_logo)) {
     (void) xine_open(gGui->stream, (const char *) XINE_BLACK_STREAM);
     
-    if(!start && !gGui->display_logo)
+    if(!startup->start && !gGui->display_logo)
       xine_play(gGui->stream, 0, 0);
     
     gGui->ignore_next = 1;
@@ -1402,13 +1540,23 @@ static void on_start(void *data) {
     gGui->ignore_next = 0;
   }
   
-  if(start)
+  if(startup->session_opts) {
+    int i = 0;
+    int dummy_session;
+    
+    while(startup->session_opts[i])
+      session_handle_subopt(startup->session_opts[i++], &dummy_session);
+    
+  }
+  
+  if(startup->start)
     gui_execute_action_id(ACTID_PLAY);
   
 }
 
-void gui_run(void) {
-  int       i, auto_start = 0;
+void gui_run(char **session_opts) {
+  _startup_t  startup;
+  int         i, auto_start = 0;
   
   video_window_change_skins();
   panel_add_autoplay_buttons();
@@ -1427,7 +1575,7 @@ void gui_run(void) {
 
     if(autoscan_plugins) {
 
-      for(i=0; autoscan_plugins[i] != NULL; ++i) {
+      for(i = 0; autoscan_plugins[i] != NULL; ++i) {
 
 	if(!strcasecmp(autoscan_plugins[i], gGui->autoscan_plugin)) {
 	  int    num_mrls, j;
@@ -1438,7 +1586,7 @@ void gui_run(void) {
 	  if(autoplay_mrls) {
 	    for (j = 0; j < num_mrls; j++)
 	      mediamark_add_entry((const char *)autoplay_mrls[j],
-				  (const char *)autoplay_mrls[j], NULL, 0, -1, 0);
+				  (const char *)autoplay_mrls[j], NULL, 0, -1, 0, 0);
 	   
 	    gGui->playlist.cur = 0;
 	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
@@ -1474,12 +1622,19 @@ void gui_run(void) {
 						 gui_dndcallback, 
 						 NULL, NULL);
   
+#ifdef HAVE_READLINE
   start_remote_server();
+#endif
   init_session();
 
-  /* Need for tvmode */
-  video_window_set_fullscreen_mode((video_window_get_fullscreen_mode()));
-
+  if(gGui->tvout) {
+    int w, h;
+    
+    video_window_get_visible_size(&w, &h);
+    tvout_set_fullscreen_mode(gGui->tvout, 
+			      ((video_window_get_fullscreen_mode() & WINDOWED_MODE) ? 0 : 1), w, h);
+  }
+  
   if(gGui->actions_on_start[0] != ACTID_NOKEY) {
 
     /* Popup setup window if there is no config file */
@@ -1507,9 +1662,11 @@ void gui_run(void) {
     for (i = actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_FULLSCREEN); i > 0; i--)
       gui_execute_action_id(ACTID_TOGGLE_FULLSCREEN);
 
+#ifdef HAVE_XINERAMA
     /* The user wants to see in xinerama fullscreen mode  */
-    for (i = actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_XINERAMA_FULLSCR); i > 0; i--)
-      gui_execute_action_id(ACTID_TOGGLE_XINERAMA_FULLSCR);
+    for (i = actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_XINERAMA_FULLSCREEN); i > 0; i--)
+      gui_execute_action_id(ACTID_TOGGLE_XINERAMA_FULLSCREEN);
+#endif
 
     /* User load a playlist on startup */
     if(actions_on_start(gGui->actions_on_start, ACTID_PLAYLIST)) {
@@ -1536,7 +1693,10 @@ void gui_run(void) {
     }
   }
 
-  xitk_run(on_start, (void *)auto_start);
+  startup.start        = auto_start;
+  startup.session_opts = session_opts;
+
+  xitk_run(on_start, (void *)&startup);
 
   /* save playlist */
   if(gGui->playlist.mmk && gGui->playlist.num) {
