@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: event.h,v 1.64 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: event.h,v 1.67 2004/04/13 19:55:47 f1rmb Exp $
  *
  */
 
@@ -32,11 +32,12 @@
 
 #include "common.h"
 
+void dummy_config_cb(void *data, xine_cfg_entry_t *cfg);
 int actions_on_start(action_id_t actions[], action_id_t a);
 void gui_deinit(void);
 void gui_init(int nfiles, char *filenames[], window_attributes_t *window_attribute);
 void gui_init_imlib(Visual *vis);
-void gui_run (void);
+void gui_run(char **session_opts);
 void gui_playlist_start_next(void);
 void gui_dndcallback(char *filename);
 void gui_execute_action_id(action_id_t);
