@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: file_browser.c,v 1.36 2003/03/13 12:50:08 f1rmb Exp $
+ * $Id: file_browser.c,v 1.49 2004/04/13 19:55:47 f1rmb Exp $
  *
  */
 
@@ -81,23 +81,48 @@ typedef struct {
 } filebrowser_filter_t;
 
 static filebrowser_filter_t __fb_filters[] = {
-  { NULL                  , "*"                        },
-  { NULL                  , "pls,m3u,sfv,tox,asx,smil" }, /* playlists */
-  { "*.vob"               , "vob"                      }, /* mpeg block */
-  { "*.mpv"               , "mpv"                      }, /* elementary */
-  { "*.mpg, *.mpeg, *.mpe", "mpg,mpeg,mpe"             }, /* mpeg */
-  { "*.avi"               , "avi"                      }, /* avi */
-  { "*.mp3"               , "mp3"                      }, /* mp3 */
-  { "*.asf, *.wma, *.wmv" , "asf,wma,wmv"              }, /* asf */
-  { "*.cpk, *.cak, *.film", "cpk,cak,film"             }, /* film */
-  { "*.ogg"               , "ogg"                      }, /* ogg */
-  { "*.vdr"               , "vdr"                      }, /* pes */
-  { "*.mov, *.mp4, *.qt"  , "mov,mp4,qt"               }, /* QT */
-  { "*.rm"                , "rm"                       }, /* rm */
-  { "*.roq"               , "roq"                      }, /* roq */
-  { "*.m2t, *.ts, *.trp"  , "m2t,ts,trp"               }, /* ts */
-  { "*.dv"                , "dv"                       }, /* dv */
-  { NULL                  , NULL                       }
+  { NULL, "*"                                                                      },
+ /* subtitles */
+  { NULL, ".sub .srt .asc .smi .ssa "                                              },
+  /* playlists */
+  { NULL, ".pls .m3u .sfv .tox .asx .smil .smi .xml "                              },
+  { "*.4xm", ".4xm "                                                               },
+  { "*.ac3", ".ac3 "                                                               },
+  { "*.aif, *.aiff,", ".aif .aiff "                                                },
+  {"*.asf, *.wmv, *.wma, *.wvx, *.wax", ".asf .wmv .wma .wvx .wax "                },
+  { "*.aud", ".aud "                                                               },
+  { "*.avi", ".avi "                                                               },
+  { "*.cin,", ".cin "                                                              },
+  { "*.cpk, *.cak, *.film,", ".cpk .cak .film "                                    },
+  { "*.dv, *.dif", ".dv .dif "                                                     },
+  { "*.fli, *.flc,", ".fli .flc "                                                  },
+  { "*.mp3, *.mp2, *.mpa, *.mpega", ".mp3 .mp2 .mpa .mpega "                       },
+  { "*.mjpg", ".mjpg "                                                             },
+  { "*.mov, *.qt, *.mp4,", ".mov .qt .mp4 "                                        },
+  { "*.mpg, *.mpeg", ".mpg .mpeg "                                                 },
+  { "*.mpv", ".mpv "                                                               },
+  { "*.mve, *.mv8", ".mve .mv8 "                                                   },
+  { "*.nsf", ".nsf "                                                               },
+  { "*.nsv", ".nsv "                                                               },
+  { "*.ogg, *.ogm, *.spx,", ".ogg .ogm .spx "                                      },
+  { "*.pes", ".pes "                                                               },
+  { "*.png, *.mng,", ".png .mng "                                                  },
+  { "*.pva", ".pva "                                                               },
+  { "*.ra", ".ra "                                                                 },
+  { "*.rm, *.ram", ".rm .ram "                                                     },
+  { "*.roq", ".roq "                                                               },
+  { "*.str, *.iki, *.ik2, *.dps, *.dat, *.xa1, *.xa2, *.xas, *.xap, *.xa,",
+    ".str .iki .ik2 .dps .dat .xa1 .xa2 .xas .xap .xa "                            },
+  { "*.snd, *.au", ".snd .au "                                                     },
+  { "*.ts, *.m2t, *.trp,", ".ts .m2t .trp "                                        },
+  { "*.vqa,", ".vqa "                                                              },
+  { "*.vob", ".vob "                                                               },
+  { "*.voc", ".voc "                                                               },
+  { "*.vox", ".vox "                                                               },
+  { "*.wav", ".wav "                                                               },
+  { "*.wve", ".wve "                                                               },
+  { "*.y4m", ".y4m "                                                               },
+  { NULL, NULL                                                                     }
 };
 
 typedef struct {
@@ -232,7 +257,7 @@ static void fne_destroy(filename_editor_t *fne) {
     xitk_destroy_widgets(fne->widget_list);
     xitk_window_destroy_window(gGui->imlib_data, fne->xwin);
 
-    fne->xwin = None;
+    fne->xwin = NULL;
     xitk_list_free((XITK_WIDGET_LIST_LIST(fne->widget_list)));
     
     XLockDisplay(gGui->display);
@@ -259,6 +284,15 @@ static void fne_cancel_cb(xitk_widget_t *w, void *data) {
   fb_reactivate(fne->fb);
   fne_destroy(fne);
 }
+static void fne_handle_event(XEvent *event, void *data) {
+  switch(event->type) {
+    
+  case KeyPress:
+    if(xitk_get_key_pressed(event) == XK_Escape)
+      fne_cancel_cb(NULL, data);
+    break;
+  }
+}
 static void fb_create_input_window(char *title, char *text,
 				   xitk_string_callback_t cb, filebrowser_t *fb) {
   filename_editor_t          *fne;
@@ -281,6 +315,10 @@ static void fb_create_input_window(char *title, char *text,
 
   fne->xwin = xitk_window_create_dialog_window(gGui->imlib_data, title, x, y, width, height);
 
+  xitk_set_wm_window_type((xitk_window_get_window(fne->xwin)), WINDOW_TYPE_NORMAL);
+  change_class_name((xitk_window_get_window(fne->xwin)));
+  change_icon((xitk_window_get_window(fne->xwin)));
+
   XLockDisplay(gGui->display);
   gc = XCreateGC(gGui->display, (xitk_window_get_window(fne->xwin)), None, None);
   XUnlockDisplay(gGui->display);
@@ -308,6 +346,7 @@ static void fb_create_input_window(char *title, char *text,
 	    (fne->input = 
 	     xitk_noskin_inputtext_create(fne->widget_list, &inp,
 					  x, y, w, 20, "Black", "Black", fontname)));
+  xitk_enable_and_show_widget(fne->input);
 
   y = height - 23 - 15;
   x = width - (100 * 2) - (15 * 2);
@@ -325,6 +364,7 @@ static void fb_create_input_window(char *title, char *text,
 	    xitk_noskin_labelbutton_create(fne->widget_list, 
 					   &lb, x, y, w, 23,
 					   "Black", "Black", "White", btnfontname)));
+  xitk_enable_and_show_widget(fne->button_apply);
 
   x += 100 + 15;
 
@@ -340,18 +380,19 @@ static void fb_create_input_window(char *title, char *text,
 	    xitk_noskin_labelbutton_create(fne->widget_list, 
 					   &lb, x, y, w, 23,
 					   "Black", "Black", "White", btnfontname)));
-  
+  xitk_enable_and_show_widget(fne->button_cancel);
+
   {
     char buffer[256];
     sprintf(buffer, "filenameed%d", (unsigned int) time(NULL));
     
     fne->widget_key = xitk_register_event_handler(buffer, 
 						  (xitk_window_get_window(fne->xwin)),
-						  NULL,
+						  fne_handle_event,
 						  NULL,
 						  NULL,
 						  fne->widget_list,
-						  NULL);
+						  (void *)fne);
   }
   
   XLockDisplay(gGui->display);
@@ -363,12 +404,7 @@ static void fb_create_input_window(char *title, char *text,
   XUnlockDisplay(gGui->display);
   layer_above_video(xitk_window_get_window(fne->xwin));
   
-  while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(fne->xwin)))
-    xine_usec_sleep(5000);
-  
-  XLockDisplay(gGui->display);
-  XSetInputFocus(gGui->display, xitk_window_get_window(fne->xwin), RevertToParent, CurrentTime);
-  XUnlockDisplay(gGui->display);
+  try_to_set_input_focus(xitk_window_get_window(fne->xwin));
 }
 /*
  * ************************** END OF filename editor **************************
@@ -382,37 +418,24 @@ static void fb_create_input_window(char *title, char *text,
  * Return 1 if file match with current filter, otherwise 0.
  */
 static int is_file_match_to_filter(filebrowser_t *fb, char *file) {
-  char  *filter_ends, *m, *ending;
-  
+
   if(!fb->filter_selected)
     return 1;
-  
-  xine_strdupa(filter_ends, __fb_filters[fb->filter_selected].ending);
-  if((ending = strrchr(file, '.')) != NULL) {
-    
-    while((m = xine_strsep(&filter_ends, ",")) != NULL) {
+  else {
+    char *ending;
+
+    if((ending = strrchr(file, '.'))) {
+      char ext[strlen(ending) + 2];
       
-      if(!strcasecmp((ending + 1), m))
+      sprintf(ext, "%s ", ending);
+      if(strstr(__fb_filters[fb->filter_selected].ending, ext))
 	return 1;
       
     }
   }
-  
   return 0;
 }
 
-/*
- * Return 1 is filepathname is a directory, otherwise 0
- */
-static int is_a_dir(char *filepathname) {
-  struct stat  pstat;
-  
-  if((stat(filepathname, &pstat)) < 0)
-    return 0;
-
-  return (S_ISDIR(pstat.st_mode));
-}
-
 static void fb_update_origin(filebrowser_t *fb) {
   char   buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
   
@@ -436,14 +459,27 @@ static void fb_extract_path_and_file(filebrowser_t *fb, char *filepathname) {
   if(filepathname) {
     char *filename = NULL;
     char *_filepathname;
+    char *tilde;
     
-    xine_strdupa(_filepathname, filepathname);
-    
+    if((tilde = strchr(filepathname, '~')) && (*(tilde - 1) != '\\')) {
+      const char *homedir = xine_get_homedir();
+      int         len = strlen(homedir) + (tilde - filepathname);
+      
+      tilde++;
+      
+      if(*tilde == '/')
+	len += strlen(tilde);
+      
+      _filepathname = (char *) alloca(len + 1);
+      sprintf(_filepathname, "%s%s", homedir, (tilde) ? tilde : "");
+    }
+    else 
+      xine_strdupa(_filepathname, filepathname);
+
     if(!is_a_dir((char *)_filepathname))
       filename = strrchr(_filepathname, '/');
     
     if(filename && (strlen(filename) > 1)) {
-      
       *filename++ = '\0';
       sprintf(fb->filename, "%s", filename);
       
@@ -566,7 +602,7 @@ static void sort_files(filebrowser_t *fb) {
     fb->files[i] = NULL;
   }
   
-  xitk_browser_update_list(fb->files_browser, (const char *const *)fb->files, fb->files_num, 0);
+  xitk_browser_update_list(fb->files_browser, (const char *const *)fb->files, NULL, fb->files_num, 0);
 }
 
 static void sort_directories(filebrowser_t *fb) {
@@ -601,7 +637,7 @@ static void sort_directories(filebrowser_t *fb) {
   }
 
   xitk_browser_update_list(fb->directories_browser, 
-			   (const char *const *)fb->directories, fb->directories_num, 0);
+			   (const char *const *)fb->directories, NULL, fb->directories_num, 0);
 }
 
 static void fb_getdir(filebrowser_t *fb) {
@@ -861,7 +897,7 @@ static void fb_exit(xitk_widget_t *w, void *data) {
     xitk_destroy_widgets(fb->widget_list);
     xitk_window_destroy_window(gGui->imlib_data, fb->xwin);
 
-    fb->xwin = None;
+    fb->xwin = NULL;
     xitk_list_free((XITK_WIDGET_LIST_LIST(fb->widget_list)));
     
     XLockDisplay(gGui->display);
@@ -1057,7 +1093,9 @@ static void fb_handle_events(XEvent *event, void *data) {
   switch(event->type) {
 
   case KeyPress:
-    {
+    if(xitk_get_key_pressed(event) == XK_Escape)
+      _fb_exit(NULL, data);
+    else {
       int sel = xitk_browser_get_current_selected(fb->files_browser);
       
       if(sel >= 0)
@@ -1068,21 +1106,8 @@ static void fb_handle_events(XEvent *event, void *data) {
 }
 
 void filebrowser_raise_window(filebrowser_t *fb) {
-  if(fb != NULL) {
-    if(fb->xwin) {
-      if(fb->visible && fb->running) {
-	  XLockDisplay(gGui->display);
-	  XUnmapWindow(gGui->display, xitk_window_get_window(fb->xwin));
-	  XRaiseWindow(gGui->display, xitk_window_get_window(fb->xwin));
-	  XMapWindow(gGui->display, xitk_window_get_window(fb->xwin));
-	  if(!gGui->use_root_window)
-	    XSetTransientForHint (gGui->display, 
-				  xitk_window_get_window(fb->xwin), gGui->video_window);
-	  XUnlockDisplay(gGui->display);
-	  layer_above_video(xitk_window_get_window(fb->xwin));
-      }
-    }
-  }
+  if(fb != NULL)
+    raise_window(xitk_window_get_window(fb->xwin), fb->visible, fb->running);
 }
 
 void filebrowser_end(filebrowser_t *fb) {
@@ -1185,6 +1210,10 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 					      (window_title) ? window_title : _("File Browser"), 
 					      x, y, WINDOW_WIDTH, WINDOW_HEIGHT);
   
+  xitk_set_wm_window_type((xitk_window_get_window(fb->xwin)), WINDOW_TYPE_NORMAL);
+  change_class_name((xitk_window_get_window(fb->xwin)));
+  change_icon((xitk_window_get_window(fb->xwin)));
+
   fb->directories                = NULL;
   fb->directories_num            = 0;
   fb->files                      = NULL;
@@ -1204,7 +1233,8 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
   fb->directories = (char **) xine_xmalloc(sizeof(char *) * 2);
   
   __fb_filters[0].name = _("All files");
-  __fb_filters[1].name = _("All known playlists");
+  __fb_filters[1].name = _("All known subtitles");
+  __fb_filters[2].name = _("All known playlists");
 
   fb->file_filters = (char **) xine_xmalloc(sizeof(filebrowser_filter_t) * ((sizeof(__fb_filters) / sizeof(__fb_filters[0])) + 1));
   
@@ -1257,6 +1287,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 	    (fb->origin = 
 	     xitk_noskin_inputtext_create(fb->widget_list, &inp,
 					  x, y, w, 20, "Black", "Black", fontname)));
+  xitk_enable_and_show_widget(fb->origin);
 
   fb_update_origin(fb);
 
@@ -1279,7 +1310,8 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 	    xitk_noskin_browser_create(fb->widget_list, &br,
 				       (XITK_WIDGET_LIST_GC(fb->widget_list)),
 				       x, y, w, 20, 12, fontname)));
-
+  xitk_enable_and_show_widget(fb->directories_browser);
+  
   draw_rectangular_inner_box(gGui->imlib_data, bg, x - 1, y - 1,
 			     xitk_get_widget_width(fb->directories_browser) + 2,
 			     xitk_get_widget_height(fb->directories_browser) + 2);
@@ -1292,6 +1324,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(fb->widget_list)), 
 	   (fb->directories_sort =
 	    xitk_noskin_button_create(fb->widget_list, &b, x - 1, y, w + 12 + 2, 15)));
+  xitk_enable_and_show_widget(fb->directories_sort);
 
   x = WINDOW_WIDTH - (w + 15 + 12);
   y += 16;
@@ -1312,7 +1345,8 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 	    xitk_noskin_browser_create(fb->widget_list, &br,
 				       (XITK_WIDGET_LIST_GC(fb->widget_list)),
 				       x, y, w, 20, 12, fontname)));
-  
+  xitk_enable_and_show_widget(fb->files_browser);
+
   draw_rectangular_inner_box(gGui->imlib_data, bg, x - 1, y - 1,
 			     xitk_get_widget_width(fb->files_browser) + 2,
 			     xitk_get_widget_height(fb->files_browser) + 2);
@@ -1325,6 +1359,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(fb->widget_list)), 
 	   (fb->files_sort =
 	    xitk_noskin_button_create(fb->widget_list, &b, x - 1, y, w + 12 + 2, 15)));
+  xitk_enable_and_show_widget(fb->files_sort);
 
 
   {
@@ -1449,6 +1484,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
   w = xitk_get_widget_width(fb->files_browser);
 
   cmb.skin_element_name = NULL;
+  cmb.layer_above       = (is_layer_above());
   cmb.parent_wlist      = fb->widget_list;
   cmb.entries           = fb->file_filters;
   cmb.parent_wkey       = &fb->widget_key;
@@ -1458,6 +1494,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 	   (fb->filters = 
 	    xitk_noskin_combo_create(fb->widget_list, &cmb, x, y, w, NULL, NULL)));
   xitk_combo_set_select(fb->filters, fb->filter_selected);
+  xitk_enable_and_show_widget(fb->filters);
 
   x = 15;
 
@@ -1467,8 +1504,8 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
   xitk_list_append_content ((XITK_WIDGET_LIST_LIST(fb->widget_list)),
 	    (fb->show_hidden = 
 	     xitk_noskin_checkbox_create(fb->widget_list, &cb, x, y+5, 10, 10)));
-
   xitk_checkbox_set_state(fb->show_hidden, fb->show_hidden_files);
+  xitk_enable_and_show_widget(fb->show_hidden);
 
   lbl.window            = xitk_window_get_window(fb->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(fb->widget_list));
@@ -1479,6 +1516,8 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(fb->widget_list)), 
 	   (widget = 
 	    xitk_noskin_label_create(fb->widget_list, &lbl, x + 15, y, w - 15, 20, fontname)));
+  xitk_enable_and_show_widget(widget);
+  
 
   y += 30;
   w = (WINDOW_WIDTH - (4 * 15)) / 3;
@@ -1495,6 +1534,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 	    xitk_noskin_labelbutton_create(fb->widget_list, 
 					   &lb, x, y, w, 23,
 					   "Black", "Black", "White", btnfontname)));
+  xitk_enable_and_show_widget(fb->rename);
 
   x += w + 15;
 
@@ -1510,7 +1550,8 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 	    xitk_noskin_labelbutton_create(fb->widget_list, 
 					   &lb, x, y, w, 23,
 					   "Black", "Black", "White", btnfontname)));
-
+  xitk_enable_and_show_widget(fb->delete);
+  
   x += w + 15;
 
   lb.button_type       = CLICK_BUTTON;
@@ -1525,7 +1566,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 	    xitk_noskin_labelbutton_create(fb->widget_list, 
 					   &lb, x, y, w, 23,
 					   "Black", "Black", "White", btnfontname)));
-
+  xitk_enable_and_show_widget(fb->create);
 
   fb->cb_buttons[0] = fb->cb_buttons[1] = NULL;
 
@@ -1545,7 +1586,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 	      xitk_noskin_labelbutton_create(fb->widget_list, 
 					     &lb, x, y, w, 23,
 					     "Black", "Black", "White", btnfontname)));
-
+    xitk_enable_and_show_widget(fb->cb_buttons[0]);
 
     if(fb->cbb[1].label) {
       x += w + 15;
@@ -1562,6 +1603,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 		xitk_noskin_labelbutton_create(fb->widget_list, 
 					       &lb, x, y, w, 23,
 					       "Black", "Black", "White", btnfontname)));
+      xitk_enable_and_show_widget(fb->cb_buttons[1]);
     }
   }
 
@@ -1579,7 +1621,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 	   (fb->close = xitk_noskin_labelbutton_create(fb->widget_list, 
 						       &lb, x, y, w, 23,
 						       "Black", "Black", "White", btnfontname)));
-  
+  xitk_enable_and_show_widget(fb->close);
 
   xitk_window_change_background(gGui->imlib_data, fb->xwin, bg->pixmap, width, height);
   xitk_image_destroy_xitk_pixmap(bg);
@@ -1606,12 +1648,8 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
     
   fb_getdir(fb);
 
-  while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(fb->xwin)))
-    xine_usec_sleep(5000);
-
-  XLockDisplay(gGui->display);
-  XSetInputFocus(gGui->display, xitk_window_get_window(fb->xwin), RevertToParent, CurrentTime);
-  XUnlockDisplay(gGui->display);
-
+  layer_above_video(xitk_window_get_window(fb->xwin));
+  try_to_set_input_focus(xitk_window_get_window(fb->xwin));
+  
   return fb;
 }
