@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: kbindings.h,v 1.44 2003/07/17 08:35:40 f1rmb Exp $
+ * $Id: kbindings.h,v 1.52 2004/04/15 15:32:40 f1rmb Exp $
  *
  */
 #ifndef KBINDINGS_H
@@ -54,7 +54,9 @@ typedef enum {
   ACTID_PAUSE,
   ACTID_TOGGLE_VISIBLITY,
   ACTID_TOGGLE_FULLSCREEN,
-  ACTID_TOGGLE_XINERAMA_FULLSCR,
+#ifdef HAVE_XINERAMA
+  ACTID_TOGGLE_XINERAMA_FULLSCREEN,
+#endif
   ACTID_TOGGLE_ASPECT_RATIO,
   ACTID_STREAM_INFOS,
   ACTID_TOGGLE_INTERLEAVE,
@@ -126,6 +128,7 @@ typedef enum {
   ACTID_SUBSELECT,
   ACTID_SV_SYNC_p,
   ACTID_SV_SYNC_m,
+  ACTID_SV_SYNC_RESET,
   ACTID_HUECONTROLp,
   ACTID_HUECONTROLm,
   ACTID_SATURATIONCONTROLp,
@@ -136,6 +139,8 @@ typedef enum {
   ACTID_CONTRASTCONTROLm,
   ACTID_VPP,
   ACTID_VPP_ENABLE,
+  ACTID_HELP_SHOW,
+  ACTID_PLAYLIST_STOP,
 
   /*
    * The below events map one-to-one with definitions in xine-lib's events.h 
@@ -174,7 +179,7 @@ typedef enum {
   
 kbinding_t *kbindings_init_kbinding(void);
 void kbindings_save_kbinding(kbinding_t *);
-void kbindings_reset_kbinding(kbinding_t **);
+void kbindings_reset_kbinding(kbinding_t *);
 void kbindings_free_kbinding(kbinding_t **);
 void kbindings_display_current_bindings(kbinding_t *);
 void kbindings_display_default_lirc_bindings(void);
@@ -182,6 +187,7 @@ void kbindings_display_default_bindings(void);
 kbinding_entry_t *kbindings_lookup_action(kbinding_t *, const char *);
 void kbindings_handle_kbinding(kbinding_t *, XEvent *);
 action_id_t kbindings_get_action_id(kbinding_entry_t *);
+char *kbindings_get_shortcut(kbinding_t *, char *);
 
 void kbedit_window(void);
 void kbedit_exit(xitk_widget_t *, void *);
@@ -189,5 +195,6 @@ int kbedit_is_visible(void);
 int kbedit_is_running(void);
 void kbedit_toggle_visibility(xitk_widget_t *, void *);
 void kbedit_raise_window(void);
+void kbedit_reparent(void);
 
 #endif
