@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: lirc.c,v 1.24 2003/07/22 21:33:08 f1rmb Exp $
+ * $Id: lirc.c,v 1.27 2004/04/13 19:55:47 f1rmb Exp $
  *
  * lirc-specific stuff
  *
@@ -71,7 +71,7 @@ static void lirc_get_playlist(char *from) {
 	
 	for (j = 0; j < num_mrls; j++)
 	  mediamark_add_entry((const char *)autoplay_mrls[j], 
-			      (const char *)autoplay_mrls[j], NULL, 0, -1, 0);
+			      (const char *)autoplay_mrls[j], NULL, 0, -1, 0, 0);
 	
 	gGui->playlist.cur = 0;
 	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
@@ -98,7 +98,7 @@ static void *xine_lirc_loop(void *dummy) {
 
     select(lirc.fd + 1, &set, NULL, NULL, &tv);
     
-    while(lirc_nextcode(&code) == 0) {
+    while((ret = lirc_nextcode(&code)) == 0) {
 
       if(code == NULL) 
 	break;
@@ -134,6 +134,8 @@ static void *xine_lirc_loop(void *dummy) {
 	break;
 
     }
+    if(ret == -1) 
+      break;
   }
   
   pthread_exit(NULL);
