@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mediamark.h,v 1.11 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: mediamark.h,v 1.16 2004/04/13 19:55:48 f1rmb Exp $
  *
  */
 
@@ -34,16 +34,17 @@ typedef struct {
   char                     *sub;
   int                       start;  /*  0..x (secs)                     */
   int                       end;    /* -1 == <till the end> else (secs) */
-  int                       offset;
+  int                       av_offset;
+  int                       spu_offset;
   int                       played; /* used with shuffle loop mode */
 } mediamark_t;
 
 int mediamark_free_mmk(mediamark_t **mmk);
-int mediamark_store_mmk(mediamark_t **mmk, const char *mrl, const char *ident, const char *sub, int start, int end, int offset);
+int mediamark_store_mmk(mediamark_t **mmk, const char *mrl, const char *ident, const char *sub, int start, int end, int av_offset, int spu_offset);
 mediamark_t *mediamark_clone_mmk(mediamark_t *mmk);
-void mediamark_add_entry(const char *mrl, const char *ident, const char *sub, int start, int end, int offset);
+void mediamark_add_entry(const char *mrl, const char *ident, const char *sub, int start, int end, int av_offset, int spu_offset);
 void mediamark_free_mediamarks(void);
-void mediamark_replace_entry(mediamark_t **mmk, const char *mrl, const char *ident, const char *sub, int start, int end, int offset);
+void mediamark_replace_entry(mediamark_t **mmk, const char *mrl, const char *ident, const char *sub, int start, int end, int av_offset, int spu_offset);
 void mediamark_free_entry(int offset);
 void mediamark_reset_played_state(void);
 int mediamark_all_played(void);
@@ -59,7 +60,9 @@ int mediamark_concat_mediamarks(const char *filename);
 void mediamark_load_mediamarks(const char *filename);
 void mediamark_save_mediamarks(const char *filename);
 
-int mrl_looks_playlist(char *mrl);
+int mrl_look_like_playlist(char *mrl);
+int mrl_look_like_file(char *mrl);
+void mediamark_collect_from_directory(char *filepathname);
 
 void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data);
 int mmk_editor_is_visible(void);
