@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: osd.c,v 1.20 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: osd.c,v 1.28 2004/04/13 19:55:48 f1rmb Exp $
  *
  */
 
@@ -114,6 +114,23 @@ static uint8_t trans[OVL_PALETTE_SIZE];
 #define BAR_WIDTH 336
 #define BAR_HEIGHT 25
 
+#define MINIMUM_WIN_WIDTH  300
+#define FONT_SIZE          20
+#define UNSCALED_FONT_SIZE 24
+
+static void  _xine_osd_show(xine_osd_t *osd, int64_t vpts) {
+  if( gGui->osd.use_unscaled && gGui->osd.unscaled_available )
+    xine_osd_show_unscaled(osd, vpts);
+  else
+    xine_osd_show(osd, vpts);
+}
+
+static void _osd_get_output_size(int *w, int *h) {
+  if( gGui->osd.use_unscaled && gGui->osd.unscaled_available )
+    video_window_get_output_size(w, h);
+  else
+    video_window_get_frame_size(w, h);
+}
 
 static char *_osd_get_speed_sym(int speed) {
   int i;
@@ -137,7 +154,7 @@ static char *_osd_get_status_sym(int status) {
 }
 
 void osd_init(void) {
-  int fonth = 20;
+  int fonth = FONT_SIZE;
 
   gGui->osd.sinfo = xine_osd_new(gGui->stream, 0, 0, 900, (fonth * 6) + (5 * 3));
   xine_osd_set_font(gGui->osd.sinfo, "sans", fonth);
@@ -155,8 +172,7 @@ void osd_init(void) {
   xine_osd_set_text_palette(gGui->osd.bar[1], 
 			    XINE_TEXTPALETTE_WHITE_BLACK_TRANSPARENT, XINE_OSD_TEXT1);
   
-  gGui->osd.status = xine_osd_new(gGui->stream, 0, 0, 300, fonth + (fonth >> 1));
-  xine_osd_set_font(gGui->osd.status, "cetus", fonth);
+  gGui->osd.status = xine_osd_new(gGui->stream, 0, 0, 300, 2 * fonth);
   xine_osd_set_text_palette(gGui->osd.status, 
 			    XINE_TEXTPALETTE_WHITE_BLACK_TRANSPARENT, XINE_OSD_TEXT1);
 
@@ -165,6 +181,8 @@ void osd_init(void) {
   xine_osd_set_text_palette(gGui->osd.info, 
 			    XINE_TEXTPALETTE_WHITE_BLACK_TRANSPARENT, XINE_OSD_TEXT1);
 
+  gGui->osd.unscaled_available =
+    (xine_osd_get_capabilities(gGui->osd.status) & XINE_OSD_CAP_UNSCALED );
 }
 
 void osd_deinit(void) {
@@ -233,7 +251,8 @@ void osd_update(void) {
 void osd_stream_infos(void) {
 
   if(gGui->osd.enabled) {
-    uint32_t    vwidth, vheight, asrate;
+    int         vwidth, vheight, asrate;
+    int         wwidth, wheight;
     const char *vcodec, *acodec;
     char        buffer[256], *p;
     int         x, y;
@@ -276,12 +295,14 @@ void osd_stream_infos(void) {
       }
     }
 
+    _osd_get_output_size(&wwidth, &wheight);
+
     y = x = 0;
 
     sprintf(buffer, "%s", (gGui->is_display_mrl) ? gGui->mmk.mrl : gGui->mmk.ident);
     xine_osd_get_text_size(gGui->osd.sinfo, buffer, &osdw, &h);
     p = buffer;
-    while(osdw > (vwidth - 40)) {
+    while(osdw > (wwidth - 40)) {
       *(p++) = '\0';
       *(p)   = '.';
       *(p+1) = '.';
@@ -364,10 +385,10 @@ void osd_stream_infos(void) {
       osd_stream_position(pos);
     }
     
-    x = (vwidth - osdw) - 40;
+    x = (wwidth - osdw) - 40;
     xine_osd_set_position(gGui->osd.sinfo, (x >= 0) ? x : 0, 15);
 
-    xine_osd_show(gGui->osd.sinfo, 0);
+    _xine_osd_show(gGui->osd.sinfo, 0);
     gGui->osd.sinfo_visible = gGui->osd.timeout;
   }
 }
@@ -375,7 +396,7 @@ void osd_stream_infos(void) {
 void osd_draw_bar(char *title, int min, int max, int val, int type) {
 
   if(gGui->osd.enabled) {
-    uint32_t vwidth, vheight;
+    int      wwidth, wheight;
     int      bar_color[40];
     int      i, x;
     float    _val = (int) val;
@@ -394,6 +415,8 @@ void osd_draw_bar(char *title, int min, int max, int val, int type) {
     
     pos = (int) (_val + -_min) / ((_max + -_min) / 40);
     
+    _osd_get_output_size(&wwidth, &wheight);
+
     xine_osd_clear(gGui->osd.bar[0]);
     xine_osd_clear(gGui->osd.bar[1]);
     
@@ -465,34 +488,20 @@ void osd_draw_bar(char *title, int min, int max, int val, int type) {
       xine_osd_draw_text(gGui->osd.bar[1], (BAR_WIDTH - tw) >> 1, 0, title, XINE_OSD_TEXT1);
     }
     
-    vwidth  = xine_get_stream_info(gGui->stream, XINE_STREAM_INFO_VIDEO_WIDTH);
-    vheight = xine_get_stream_info(gGui->stream, XINE_STREAM_INFO_VIDEO_HEIGHT);
+    x = (wwidth - BAR_WIDTH) >> 1;
+    xine_osd_set_position(gGui->osd.bar[0], (x >= 0) ? x : 0, (wheight - BAR_HEIGHT) - 40);
+    xine_osd_set_position(gGui->osd.bar[1], (x >= 0) ? x : 0, (wheight - (BAR_HEIGHT * 2)) - 40);
     
-    /* We're in visual animation mode */
-    if((vwidth == 0) && (vheight == 0)) {
-      if(gGui->visual_anim.running) {
-	if(gGui->visual_anim.enabled == 1) {
-	  video_window_get_frame_size(&vwidth, &vheight);
-	}
-	else if(gGui->visual_anim.enabled == 2) {
-	  vwidth  = xine_get_stream_info(gGui->visual_anim.stream, XINE_STREAM_INFO_VIDEO_WIDTH);
-	  vheight = xine_get_stream_info(gGui->visual_anim.stream, XINE_STREAM_INFO_VIDEO_HEIGHT);
-	}
-      }
-      else
-	video_window_get_frame_size(&vwidth, &vheight);
-
+    /* don't even bother drawing osd over those small streams.
+     * it would look pretty bad.
+     */
+    if( wwidth > MINIMUM_WIN_WIDTH ) {
+      _xine_osd_show(gGui->osd.bar[0], 0);
+      if(title)
+        _xine_osd_show(gGui->osd.bar[1], 0);
+   
+      gGui->osd.bar_visible = gGui->osd.timeout;
     }
-
-    x = (vwidth - BAR_WIDTH) >> 1;
-    xine_osd_set_position(gGui->osd.bar[0], (x >= 0) ? x : 0, (vheight - BAR_HEIGHT) - 40);
-    xine_osd_set_position(gGui->osd.bar[1], (x >= 0) ? x : 0, (vheight - (BAR_HEIGHT * 2)) - 40);
-    
-    xine_osd_show(gGui->osd.bar[0], 0);
-    if(title)
-      xine_osd_show(gGui->osd.bar[1], 0);
-    
-    gGui->osd.bar_visible = gGui->osd.timeout;
   }
 }
 
@@ -532,7 +541,7 @@ void osd_display_info(char *info, ...) {
 
     xine_osd_draw_text(gGui->osd.info, 0, 0, buf, XINE_OSD_TEXT1);
     xine_osd_set_position(gGui->osd.info, 20, 10 + 30);
-    xine_osd_show(gGui->osd.info, 0);
+    _xine_osd_show(gGui->osd.info, 0);
     gGui->osd.info_visible = gGui->osd.timeout;
     SAFE_FREE(buf);
   }
@@ -543,7 +552,8 @@ void osd_update_status(void) {
   if(gGui->osd.enabled) {
     int  status;
     char buffer[256];
-    
+    int wwidth, wheight;
+
     status = xine_get_status(gGui->stream);
     
     xine_osd_clear(gGui->osd.status);
@@ -589,17 +599,35 @@ void osd_update_status(void) {
       /* noop */
       break;
     }
-    
+
+    if( gGui->osd.use_unscaled && gGui->osd.unscaled_available )
+      xine_osd_set_font(gGui->osd.status, "cetus", UNSCALED_FONT_SIZE);
+    else
+      xine_osd_set_font(gGui->osd.status, "cetus", FONT_SIZE);
+
+    /* set latin1 encoding (NULL) for status text with special characters,
+     * then switch back to locale encoding ("") 
+     */
+    xine_osd_set_encoding(gGui->osd.status, NULL);
     xine_osd_draw_text(gGui->osd.status, 0, 0, buffer, XINE_OSD_TEXT1);
+    xine_osd_set_encoding(gGui->osd.status, "");
     xine_osd_set_position(gGui->osd.status, 20, 10);
-    xine_osd_show(gGui->osd.status, 0);
-    gGui->osd.status_visible = gGui->osd.timeout;
+
+    _osd_get_output_size(&wwidth, &wheight);
+
+    /* don't even bother drawing osd over those small streams.
+     * it would look pretty bad.
+     */
+    if( wwidth > MINIMUM_WIN_WIDTH ) {
+      _xine_osd_show(gGui->osd.status, 0);
+      gGui->osd.status_visible = gGui->osd.timeout;
+    }
   }
 }
 
 void osd_display_spu_lang(void) {
-  char   buffer[32];
-  char   lang_buffer[10];
+  char   buffer[XINE_LANG_MAX+128];
+  char   lang_buffer[XINE_LANG_MAX];
   char  *lang = NULL;
   int    channel;
   
@@ -627,8 +655,8 @@ void osd_display_spu_lang(void) {
 }
 
 void osd_display_audio_lang(void) {
-  char   buffer[32];
-  char   lang_buffer[10];
+  char   buffer[XINE_LANG_MAX+128];
+  char   lang_buffer[XINE_LANG_MAX];
   char  *lang = NULL;
   int    channel;
 
