@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: panel.c,v 1.181 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: panel.c,v 1.204 2004/04/13 19:55:48 f1rmb Exp $
  *
  * xine panel related stuff
  *
@@ -180,7 +180,10 @@ void panel_change_skins(void) {
   Imlib_apply_image(gGui->imlib_data, new_img, gGui->panel_window);
   
   XUnlockDisplay(gGui->display);
-  
+
+  if(panel_is_visible())
+    raise_window(gGui->panel_window, 1, 1);
+
   xitk_skin_unlock(gGui->skin_config);
   
   xitk_change_skins_widget_list(panel->widget_list, gGui->skin_config);
@@ -189,16 +192,17 @@ void panel_change_skins(void) {
    * Update position of dynamic buttons.
    */
   {
-    int i = 0, x, y, dir;
+    int i = 0, x, y, dir, max;
     
-    x = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayGUI");
-    y = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayGUI");
+    x   = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayGUI");
+    y   = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayGUI");
     dir = xitk_skin_get_direction(gGui->skin_config, "AutoPlayGUI");
+    max = xitk_skin_get_max_buttons(gGui->skin_config, "AutoPlayGUI");
 
     switch(dir) {
     case DIRECTION_UP:
     case DIRECTION_DOWN:
-      while(panel->autoplay_plugins[i] != NULL) {
+      while((max && ((i < max) && (panel->autoplay_plugins[i] != NULL))) || (!max && panel->autoplay_plugins[i] != NULL)) {
 	
 	(void) xitk_set_widget_pos(panel->autoplay_plugins[i], x, y);
 	
@@ -212,7 +216,7 @@ void panel_change_skins(void) {
       break;
     case DIRECTION_LEFT:
     case DIRECTION_RIGHT:
-      while(panel->autoplay_plugins[i] != NULL) {
+      while((max && ((i < max) && (panel->autoplay_plugins[i] != NULL))) || (!max && panel->autoplay_plugins[i] != NULL)) {
 	
 	(void) xitk_set_widget_pos(panel->autoplay_plugins[i], x, y);
 	
@@ -225,8 +229,15 @@ void panel_change_skins(void) {
       }
       break;
     }
-  }
 
+    if(max) {
+      while(panel->autoplay_plugins[i] != NULL) {
+	xitk_disable_and_hide_widget(panel->autoplay_plugins[i]);
+	i++;
+      }
+    }
+  }
+  
   enable_playback_controls(panel->playback_widgets.enabled);
   xitk_paint_widget_list(panel->widget_list);
 
@@ -370,13 +381,22 @@ static void *slider_loop(void *dummy) {
 	    }
 	    
 	    if(!(i % 20)) {
+	      int level = 0;
+	      
 	      panel_update_channel_display();
 	      
-	      if(gGui->mixer.caps & MIXER_CAP_VOL) {
-		gGui->mixer.volume_level = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
-		xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.volume_level);
-		panel_check_mute();
+	      if((gGui->mixer.method == SOUND_CARD_MIXER) &&
+		 (gGui->mixer.caps & MIXER_CAP_VOL)) {
+		level = gGui->mixer.volume_level = 
+		  xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
 	      }
+	      else if(gGui->mixer.method == SOFTWARE_MIXER) {
+		level = gGui->mixer.amp_level = 
+		  xine_get_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL);
+	      }
+	      
+	      xitk_slider_set_pos(panel->mixer.slider, level);
+	      panel_check_mute();
 	      
 	      i = 0;
 	    }
@@ -387,7 +407,7 @@ static void *slider_loop(void *dummy) {
 	    stream_infos_update_infos();
 	}
       }
-
+      
       if(gGui->got_reference_stream) {
 	gGui->got_reference_stream--;
 	gui_play(NULL, NULL);
@@ -508,6 +528,13 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
       pplugin_toggle_visibility(NULL, NULL);
   }
 
+  if(!panel->visible && help_is_visible()) {
+  }
+  else {
+    if(!gGui->use_root_window || (gGui->use_root_window && visible && help_is_visible()))
+      help_toggle_visibility(NULL, NULL);
+  }
+
   if (panel->visible) {
     
     XLockDisplay(gGui->display);
@@ -526,17 +553,19 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
 	xitk_hide_widgets(panel->widget_list);
       }
     }
+    else
+      XIconifyWindow(gGui->display, gGui->panel_window, (XDefaultScreen(gGui->display)));
 
     if(gGui->cursor_grabbed)
        XGrabPointer(gGui->display, gGui->video_window, 1, None, GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
     
     /* Give focus to video output window */
-    XSetInputFocus(gGui->display, gGui->video_window, RevertToParent, CurrentTime);
+    try_to_set_input_focus(gGui->video_window);
     XUnlockDisplay(gGui->display);
      
   }
   else {
-    
+
     panel->visible = 1;
     xitk_show_widgets(panel->widget_list);
     
@@ -556,27 +585,46 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
       XUnlockDisplay (gGui->display);
     }
     
-    /* TODO: Currently this is a quick hack
-     * We should rather test whether screen size has changed
-     * and move the panel on screen if it doesn't fit any longer */
-    if (video_window_get_fullscreen_mode() > 1 
+#if defined(HAVE_XINERAMA) || defined(HAVE_XF86VIDMODE)
+    if(
+#ifdef HAVE_XINERAMA
+       (((video_window_get_fullscreen_mode()) & (WINDOWED_MODE | FULLSCR_MODE)) &&
+	(!((video_window_get_fullscreen_mode()) & FULLSCR_XI_MODE)))
 #ifdef HAVE_XF86VIDMODE
-        || gGui->XF86VidMode_fullscreen
+       ||
 #endif
-	) {
-    /*
-     * necessary to place the panel in a visible area (otherwise it might
-     * appear off the video window while switched to a differenti
-     * modeline or tv mode)
-     */
-      XLockDisplay (gGui->display);
-      XMoveWindow(gGui->display, gGui->panel_window, 40, 40);
-      XUnlockDisplay (gGui->display);
+#endif
+#ifdef HAVE_XF86VIDMODE
+       (gGui->XF86VidMode_fullscreen)
+#endif
+       ) {
+      int x, y, w, h, desktopw, desktoph;
+      
+      xitk_get_window_position(gGui->display, gGui->panel_window, &x, &y, &w, &h);
+      
+      XLockDisplay(gGui->display);
+      desktopw = DisplayWidth(gGui->display, gGui->screen);
+      desktoph = DisplayHeight(gGui->display, gGui->screen);
+      XUnlockDisplay(gGui->display);
+      
+      if(((x + w) <= 0) || ((y + h) <= 0) || (x >= desktopw) || (y >= desktoph)) {
+	int newx, newy;
+	
+	newx = (desktopw - w) >> 1;
+	newy = (desktoph - h) >> 1;
+	
+	XLockDisplay(gGui->display);
+	XMoveWindow(gGui->display, gGui->panel_window, newx, newy);
+	XUnlockDisplay(gGui->display);
+	
+	panel_store_new_position(newx, newy, w, h);
+      }
     }
-
+#endif
+    
     if(gGui->logo_mode == 0) {
       int pos;
-
+      
       if(xitk_is_widget_enabled(panel->playback_widgets.slider_play)) {
 	if(gui_xine_get_pos_length(gGui->stream, &pos, NULL, NULL))
 	  xitk_slider_set_pos(panel->playback_widgets.slider_play, pos);
@@ -638,7 +686,7 @@ void panel_reset_slider (void) {
  */
 void panel_update_channel_display (void) {
   int   channel;
-  char  buffer[10];
+  char  buffer[XINE_LANG_MAX];
   char *lang = NULL;
 
   memset(&buffer, 0, sizeof(buffer));
@@ -719,7 +767,12 @@ void panel_update_mrl_display (void) {
  */
 void panel_toggle_audio_mute(xitk_widget_t *w, void *data, int state) {
 
-  if(gGui->mixer.caps & MIXER_CAP_MUTE) {
+  if(gGui->mixer.method == SOFTWARE_MIXER) {
+    gGui->mixer.mute = state;
+    xine_set_param(gGui->stream, XINE_PARAM_AUDIO_AMP_MUTE, gGui->mixer.mute);
+    osd_display_info(_("Amp: %s"), gGui->mixer.mute ? _("Muted") : _("Unmuted"));
+  }
+  else if(gGui->mixer.caps & MIXER_CAP_MUTE) {
     gGui->mixer.mute = state;
     xine_set_param(gGui->stream, XINE_PARAM_AUDIO_MUTE, gGui->mixer.mute);
     osd_display_info(_("Audio: %s"), gGui->mixer.mute ? _("Muted") : _("Unmuted"));
@@ -759,13 +812,16 @@ static void panel_slider_cb(xitk_widget_t *w, void *data, int pos) {
 	  xitk_slider_set_pos(panel->playback_widgets.slider_play, pos);
 	
 	panel_update_runtime_display();
-        }
       }
+    }
   }
   else if(w == panel->mixer.slider) {
-    gGui->mixer.volume_level = pos;
-    xine_set_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME, gGui->mixer.volume_level);
-    osd_draw_bar(_("Audio Volume"), 0, 100, gGui->mixer.volume_level, OSD_BAR_STEPPER);
+
+    if(gGui->mixer.method == SOUND_CARD_MIXER)
+      change_audio_vol(pos);
+    else if(gGui->mixer.method == SOFTWARE_MIXER)
+      change_amp_vol(pos);
+    
   }
   
   panel_check_pause();
@@ -777,7 +833,15 @@ static void panel_slider_cb(xitk_widget_t *w, void *data, int pos) {
 void panel_handle_event(XEvent *event, void *data) {
 
   switch(event->type) {
+  case ButtonPress:
+    {
+      XButtonEvent *bevent = (XButtonEvent *) event;
 
+      if(bevent->button == Button3)
+	video_window_menu(panel->widget_list);
+    }
+    break;
+    
   case KeyPress:
   case ButtonRelease:
     gui_handle_event(event, data);
@@ -800,17 +864,18 @@ void panel_handle_event(XEvent *event, void *data) {
  */
 void panel_add_autoplay_buttons(void) {
   int                        x, y, dir;
-  int                        i = 0;
+  int                        i = 0, max;
   xitk_labelbutton_widget_t  lb;
   const char *const         *autoplay_plugins = xine_get_autoplay_input_plugin_ids(gGui->xine);
   const char                *autoplay_label;
   
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
 
-  x = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayGUI");
-  y = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayGUI");
+  x   = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayGUI");
+  y   = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayGUI");
   dir = xitk_skin_get_direction(gGui->skin_config, "AutoPlayGUI");
-  
+  max = xitk_skin_get_max_buttons(gGui->skin_config, "AutoPlayGUI");
+
   if(autoplay_plugins) {
     autoplay_label = *autoplay_plugins++;
     while(autoplay_label) {
@@ -849,7 +914,10 @@ void panel_add_autoplay_buttons(void) {
 	  x -= (xitk_get_widget_width(panel->autoplay_plugins[i]) + 1);
 	break;
       }
-
+      
+      if(max && (i >= max))
+	xitk_disable_and_hide_widget(panel->autoplay_plugins[i]);
+      
       i++;
       
       autoplay_label = *autoplay_plugins++;
@@ -874,11 +942,19 @@ void panel_add_mixer_control(void) {
     if((xine_get_param(gGui->stream, XINE_PARAM_AUDIO_MUTE)) != -1)
       gGui->mixer.caps |= MIXER_CAP_MUTE;
   }
+  
+  if(((gGui->mixer.method == SOUND_CARD_MIXER) && gGui->mixer.caps & MIXER_CAP_VOL) ||
+     (gGui->mixer.method == SOFTWARE_MIXER)) {
+    int level = 0;
 
-  if(gGui->mixer.caps & MIXER_CAP_VOL) { 
     xitk_enable_widget(panel->mixer.slider);
-    gGui->mixer.volume_level = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
-    xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.volume_level);
+    
+    if(gGui->mixer.method == SOUND_CARD_MIXER)
+      level = gGui->mixer.volume_level = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
+    else if(gGui->mixer.method == SOFTWARE_MIXER)
+      level = gGui->mixer.amp_level = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL);
+    
+    xitk_slider_set_pos(panel->mixer.slider, level);
   }
 
   if(gGui->mixer.caps & MIXER_CAP_MUTE) {
@@ -906,6 +982,12 @@ void panel_deinit(void) {
   xitk_unregister_event_handler(&panel->widget_key);
 }
 
+void panel_reparent(void) {
+  if(panel) {
+    reparent_window(gGui->panel_window);
+  }
+}
+
 /*
  * Create the panel window, and fit all widgets in.
  */
@@ -1003,13 +1085,17 @@ void panel_init (void) {
 				      CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 				      &attr);
 
-  XSetStandardProperties(gGui->display, gGui->panel_window, title, title,
-			 None, NULL, 0, &hint);
+  XmbSetWMProperties(gGui->display, gGui->panel_window, title, title, NULL, 0,
+                     &hint, NULL, NULL);
 
   XSelectInput(gGui->display, gGui->panel_window, INPUT_MOTION | KeymapStateMask);
-
   XUnlockDisplay(gGui->display);
-
+  
+  if(!video_window_is_visible())
+    xitk_set_wm_window_type(gGui->panel_window, WINDOW_TYPE_NORMAL);
+  else
+    xitk_unset_wm_window_type(gGui->panel_window, WINDOW_TYPE_NORMAL);
+  
   if(is_layer_above())
     xitk_set_layer_above(gGui->panel_window);
   
@@ -1037,6 +1123,7 @@ void panel_init (void) {
     xclasshint->res_name = title;
     xclasshint->res_class = "xine";
     XSetClassHint(gGui->display, gGui->panel_window, xclasshint);
+    XFree(xclasshint);
   }
 
   wm_hint = XAllocWMHints();
@@ -1246,7 +1333,7 @@ void panel_init (void) {
   /* Mixer volume slider */
   sl.skin_element_name = "SliderVol";
   sl.min               = 0;
-  sl.max               = 100;
+  sl.max               = (gGui->mixer.method == SOUND_CARD_MIXER) ? 100 : 200;
   sl.step              = 1;
   sl.callback          = NULL;
   sl.userdata          = NULL;
@@ -1376,14 +1463,14 @@ void panel_init (void) {
   
   {
     pthread_attr_t       pth_attrs;
-#if ! defined (__OpenBSD__)
+#if !defined (__OpenBSD__)
     struct sched_param   pth_params;
 #endif
     
     pthread_attr_init(&pth_attrs);
 
     /* this won't work on linux, freebsd 5.0 */
-#if ! defined (__OpenBSD__)
+#if !defined (__OpenBSD__)
     pthread_attr_getschedparam(&pth_attrs, &pth_params);
     pth_params.sched_priority = sched_get_priority_min(SCHED_OTHER);
     pthread_attr_setschedparam(&pth_attrs, &pth_params);
@@ -1392,14 +1479,8 @@ void panel_init (void) {
     pthread_create(&panel->slider_thread, &pth_attrs, slider_loop, NULL);
   }
 
-  if(panel->visible) {
-    while(!xitk_is_window_visible(gGui->display, gGui->panel_window))
-      xine_usec_sleep(5000);
-  
-    XLockDisplay (gGui->display);
-    XSetInputFocus(gGui->display, gGui->panel_window, RevertToParent, CurrentTime);
-    XUnlockDisplay (gGui->display);
-  }
+  if(panel->visible)
+    try_to_set_input_focus(gGui->panel_window);
 }
 
 void panel_set_title(char *title) {
