@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: playlist.c,v 1.126 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: playlist.c,v 1.147 2004/04/14 22:38:30 f1rmb Exp $
  *
  * Playlist Editor
  *
@@ -70,9 +70,6 @@ typedef struct {
 
 static _playlist_t   *playlist;
 
-#define MOVEUP 1
-#define MOVEDN 2
-
 void playlist_handle_event(XEvent *event, void *data);
 
 static void _playlist_enability(int enable) {
@@ -111,11 +108,11 @@ static void _playlist_update_browser_list(int start) {
 
   if(gGui->is_display_mrl)
     xitk_browser_update_list(playlist->playlist, 
-			     (const char **)playlist->playlist_mrls, 
+			     (const char **)playlist->playlist_mrls, NULL, 
 			     playlist->playlist_len, start);
   else
     xitk_browser_update_list(playlist->playlist, 
-			     (const char **)playlist->playlist_idents, 
+			     (const char **)playlist->playlist_idents, NULL,
 			     playlist->playlist_len, start);
 
   if((sel >= 0) && ((start == -1) || (old_start == start)))
@@ -186,7 +183,7 @@ static void _playlist_xine_play(void) {
     if(!is_playback_widgets_enabled())
       enable_playback_controls(1);
 
-    if(mrl_looks_playlist(mmk->mrl)) {
+    if(mrl_look_like_playlist(mmk->mrl)) {
       if(mediamark_concat_mediamarks(mmk->mrl)) {
 	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
 	mmk = mediamark_get_current_mmk();
@@ -221,7 +218,7 @@ static void _playlist_xine_play(void) {
 /*
  * Start playing an MRL
  */
-static void _playlist_play(xitk_widget_t *w, void *data) {
+void playlist_play_current(xitk_widget_t *w, void *data) {
   int j;
   
   j = xitk_browser_get_current_selected(playlist->playlist);
@@ -251,7 +248,7 @@ static void _playlist_play_on_dbl_click(xitk_widget_t *w, void *data, int select
 /*
  * Delete selected MRL
  */
-static void _playlist_delete(xitk_widget_t *w, void *data) {
+void playlist_delete_current(xitk_widget_t *w, void *data) {
   int i, j;
   
   mmk_editor_end();
@@ -307,7 +304,10 @@ void playlist_delete_all(xitk_widget_t *w, void *data) {
   
   if(playlist && playlist->winput)
     xitk_inputtext_change_text(playlist->winput, NULL);
-  
+
+  if(playlist)
+    xitk_browser_release_all_buttons(playlist->playlist);
+
   gui_set_current_mrl(NULL);
   enable_playback_controls(0);
 }
@@ -315,7 +315,7 @@ void playlist_delete_all(xitk_widget_t *w, void *data) {
 /*
  * Move entry up/down in playlist
  */
-static void _playlist_move_updown(xitk_widget_t *w, void *data) {
+void playlist_move_current_updown(xitk_widget_t *w, void *data) {
   int j;
 
   mmk_editor_end();
@@ -325,16 +325,22 @@ static void _playlist_move_updown(xitk_widget_t *w, void *data) {
     int          start = xitk_browser_get_current_start(playlist->playlist);
     int          max_vis_len = xitk_browser_get_num_entries(playlist->playlist);
 
-    if(((int)data) == MOVEUP && (j > 0)) {
+    if(((int)data) == DIRECTION_UP && (j > 0)) {
       mmk = gGui->playlist.mmk[j - 1];
+      
+      if(j == gGui->playlist.cur)
+	gGui->playlist.cur--;
 
       gGui->playlist.mmk[j - 1] = gGui->playlist.mmk[j];
       gGui->playlist.mmk[j] = mmk;
       j--;
     }
-    else if(((int)data) == MOVEDN && (j < (gGui->playlist.num - 1))) {
+    else if(((int)data) == DIRECTION_DOWN && (j < (gGui->playlist.num - 1))) {
       mmk = gGui->playlist.mmk[j + 1];
       
+      if(j == gGui->playlist.cur)
+	gGui->playlist.cur++;
+
       gGui->playlist.mmk[j + 1] = gGui->playlist.mmk[j];
       gGui->playlist.mmk[j] = mmk;
       j++;
@@ -350,6 +356,8 @@ static void _playlist_move_updown(xitk_widget_t *w, void *data) {
       _playlist_update_browser_list(-1);
 
     xitk_browser_set_select(playlist->playlist, j);
+
+    
   }
 }
 
@@ -437,17 +445,27 @@ static void _playlist_add_input(xitk_widget_t *w, void *data, char *filename) {
  * Handle X events here.
  */
 static void _playlist_handle_event(XEvent *event, void *data) {
-  XKeyEvent      mykeyevent;
-  KeySym         mykey;
-  char           kbuf[256];
-  int            len;
 
   switch(event->type) {
 
   case ButtonPress: {
     XButtonEvent *bevent = (XButtonEvent *) event;
 
-    if((bevent->button == Button4) ||(bevent->button == Button5))
+    if(bevent->button == Button3) {
+      xitk_widget_t *w = xitk_get_focused_widget(playlist->widget_list);
+      
+      if(w && ((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)) {
+	int wx, wy;
+
+	xitk_get_window_position(gGui->display, playlist->window, &wx, &wy, NULL, NULL);
+
+	playlist_menu(playlist->widget_list, 
+		      bevent->x + wx, bevent->y + wy, 
+		      (xitk_browser_get_current_selected(playlist->playlist) >= 0));
+      }
+
+    }
+    else if((bevent->button == Button4) || (bevent->button == Button5))
       mmk_editor_end();
 
   }
@@ -463,52 +481,54 @@ static void _playlist_handle_event(XEvent *event, void *data) {
     break;
 
   case KeyPress:
-    
-    if(!playlist)
-      return;
-    
-    mykeyevent = event->xkey;
-    
-    XLockDisplay(gGui->display);
-    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-    XUnlockDisplay(gGui->display);
-    
-    switch (mykey) {
+    {
+      KeySym mkey;
       
-    case XK_Down:
-    case XK_Next: {
-      xitk_widget_t *w;
-      
-      mmk_editor_end();
-      
-      w = xitk_get_focused_widget(playlist->widget_list);
-      if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
-	if(mykey == XK_Down)
-	  xitk_browser_step_up(playlist->playlist, NULL);
-	else
-	  xitk_browser_page_up(playlist->playlist, NULL);
-      }
-    }
-      break;
-      
-    case XK_Up:
-    case XK_Prior: {
-      xitk_widget_t *w;
+      if(!playlist)
+	return;
 
-      mmk_editor_end();
-      w = xitk_get_focused_widget(playlist->widget_list);
-      if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
-	if(mykey == XK_Up)
-	  xitk_browser_step_down(playlist->playlist, NULL);
-	else
-	  xitk_browser_page_down(playlist->playlist, NULL);
+      mkey = xitk_get_key_pressed(event);
+
+      switch (mkey) {
+	
+      case XK_Down:
+      case XK_Next: {
+	xitk_widget_t *w;
+	
+	mmk_editor_end();
+	w = xitk_get_focused_widget(playlist->widget_list);
+	if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
+	  if(mkey == XK_Down)
+	    xitk_browser_step_up(playlist->playlist, NULL);
+	  else
+	    xitk_browser_page_up(playlist->playlist, NULL);
+	}
+      }
+	break;
+	
+      case XK_Up:
+      case XK_Prior: {
+	xitk_widget_t *w;
+	
+	mmk_editor_end();
+	w = xitk_get_focused_widget(playlist->widget_list);
+	if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
+	  if(mkey == XK_Up)
+	    xitk_browser_step_down(playlist->playlist, NULL);
+	  else
+	    xitk_browser_page_down(playlist->playlist, NULL);
+	}
+      }
+	break;
+
+      case XK_Escape:
+	playlist_exit(NULL, NULL);
+	break;
+	
+      default:
+	gui_handle_event(event, data);
+	break;
       }
-    }
-      break;
-      
-    default:
-      gui_handle_event(event, data);
-      break;
     }
     break;
 
@@ -530,72 +550,67 @@ void playlist_mmk_editor(void) {
   if(playlist) {
     int sel = xitk_browser_get_current_selected(playlist->playlist);
 
-    if(sel >= 0) {
+    if(sel >= 0)
       mmk_edit_mediamark(&gGui->playlist.mmk[sel], _playlist_apply_cb, NULL);
-    }
   
   }
 }
 
+static void _scan_for_playlist_infos(xine_stream_t *stream, int n) {
+  
+  if(xine_open(stream, gGui->playlist.mmk[n]->mrl)) {
+    char  *ident;
+    
+    if((ident = stream_infos_get_ident_from_stream(stream)) != NULL) {
+      
+      if(gGui->playlist.mmk[n]->ident)
+	free(gGui->playlist.mmk[n]->ident);
+      
+      gGui->playlist.mmk[n]->ident = strdup(ident);
+      
+      if(n == gGui->playlist.cur) {
+	
+	if(gGui->mmk.ident)
+	  free(gGui->mmk.ident);
+	
+	gGui->mmk.ident = strdup(ident);
+	
+	panel_update_mrl_display();
+      }
+      
+      free(ident);
+    }
+    xine_close(stream);
+  }
+}
+
+void playlist_scan_for_infos_selected(void) {
+
+  if(gGui->playlist.num) {
+    int                 selected = xitk_browser_get_current_selected(playlist->playlist);
+    xine_stream_t      *stream;
+    
+    stream = xine_stream_new(gGui->xine, gGui->ao_none, gGui->vo_none);
+    _scan_for_playlist_infos(stream, selected);
+    
+    xine_dispose(stream);
+    
+    playlist_mrlident_toggle();
+  }
+}
+
 void playlist_scan_for_infos(void) {
   
   if(gGui->playlist.num) {
-    int            i;
-    char          *ident;
-    int            old_pos = 0, rerun = 0;
-    xine_stream_t *stream = gGui->playlist.scan_stream;
-    int            mute = gGui->mixer.mute;
+    int                 i;
+    xine_stream_t      *stream;
     
-    /* 
-     * Because we share audio driver (and since we can't stat infos of
-     * audio only streams without audio driver), pausing the gGui->stream
-     * avoid some sync/crash problems.
-     */
-    if(((xine_get_status(gGui->stream)) != XINE_STATUS_STOP) && (gGui->logo_mode == 0)) {
-      if(!gui_xine_get_pos_length(gGui->stream, &old_pos, NULL, NULL))
-	old_pos = 0;
-      gui_stop(NULL, NULL);
-      rerun = 1;
-    }
-    if((!mute) && gGui->mixer.caps & MIXER_CAP_MUTE) {
-      mute = 1;
-      xine_set_param(gGui->stream, XINE_PARAM_AUDIO_MUTE, 1);
-    }
-    else
-      mute = 0;
+    stream = xine_stream_new(gGui->xine, gGui->ao_none, gGui->vo_none);
     
-    for(i = 0; i < gGui->playlist.num; i++) {
-      
-      if(xine_open(stream, gGui->playlist.mmk[i]->mrl)) {
-
-	if((ident = stream_infos_get_ident_from_stream(stream)) != NULL) {
-	  
-	  if(gGui->playlist.mmk[i]->ident)
-	    free(gGui->playlist.mmk[i]->ident);
-	  
-	  gGui->playlist.mmk[i]->ident = strdup(ident);
-	  
-	  if(i == gGui->playlist.cur) {
-	    
-	    if(gGui->mmk.ident)
-	      free(gGui->mmk.ident);
-	    
-	    gGui->mmk.ident = strdup(ident);
-	    
-	    panel_update_mrl_display();
-	  }
-	  
-	  free(ident);
-	}
-      }
-    }
+    for(i = 0; i < gGui->playlist.num; i++)
+      _scan_for_playlist_infos(stream, i);
     
-    /* Restoring previous play status */
-    if(mute)
-      xine_set_param(gGui->stream, XINE_PARAM_AUDIO_MUTE, !mute);
-    
-    if(rerun)
-      gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, old_pos, 0, gGui->mmk.offset);
+    xine_dispose(stream);
     
     playlist_mrlident_toggle();
   }
@@ -725,11 +740,15 @@ void playlist_scan_input(xitk_widget_t *w, void *ip) {
       
       if(!strcasecmp(autoplay_plugins[i], xitk_labelbutton_get_label(w))) {
 	int                num_mrls = 0;
-	char             **autoplay_mrls = 
-	  xine_get_autoplay_mrls (gGui->xine, autoplay_plugins[i], &num_mrls);
+	char             **autoplay_mrls = xine_get_autoplay_mrls(gGui->xine, autoplay_plugins[i], &num_mrls);
+	xine_stream_t      *stream;
 	
 	if(autoplay_mrls) {
-	  int j;
+	  int                j;
+	  int                cdda_mode = 0;
+
+	  if(!strcasecmp(autoplay_plugins[i], "cd"))
+	    cdda_mode = 1;
 	  
 	  /* Flush playlist in newbie mode */
 	  if(gGui->smart_mode) {
@@ -739,12 +758,28 @@ void playlist_scan_input(xitk_widget_t *w, void *ip) {
 	      xitk_inputtext_change_text(playlist->winput, NULL);
 	    gGui->playlist.cur = 0;
 	  }
-
+	  
 	  if(!gGui->playlist.num)
 	    gGui->playlist.cur = 0;
 	  
-	  for (j = 0; j < num_mrls; j++)
-	    mediamark_add_entry(autoplay_mrls[j], autoplay_mrls[j], NULL, 0, -1, 0);
+
+	  stream = xine_stream_new(gGui->xine, gGui->ao_none, gGui->vo_none);
+	  
+	  for (j = 0; j < num_mrls; j++) {
+	    char *ident = NULL;
+	    
+	    if(cdda_mode && xine_open(stream, autoplay_mrls[j])) {
+	      ident = stream_infos_get_ident_from_stream(stream);
+	      xine_close(stream);
+	    }
+	    
+	    mediamark_add_entry(autoplay_mrls[j], ident ? ident : autoplay_mrls[j], NULL, 0, -1, 0, 0);
+	    
+	    if(ident)
+	      free(ident);
+	  }
+
+	  xine_dispose(stream);
 	  
 	  if(gGui->playlist.cur == 0)
 	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
@@ -758,7 +793,6 @@ void playlist_scan_input(xitk_widget_t *w, void *ip) {
 	      gui_stop(NULL, NULL);
 	    gui_play(NULL, NULL);
 	  }
-	  
 	}
       }
       
@@ -781,18 +815,7 @@ void playlist_scan_input(xitk_widget_t *w, void *ip) {
 void playlist_raise_window(void) {
   
   if(playlist != NULL) {
-    if(playlist->window) {
-      if(playlist->visible && playlist->running) {
-	XLockDisplay(gGui->display);
-	XUnmapWindow(gGui->display, playlist->window); 
-	XRaiseWindow(gGui->display, playlist->window); 
-	XMapWindow(gGui->display, playlist->window); 
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, playlist->window, gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(playlist->window);
-      }
-    }
+    raise_window(playlist->window, playlist->visible, playlist->running);
     mmk_editor_raise_window();
   }
 }
@@ -803,34 +826,8 @@ void playlist_raise_window(void) {
 void playlist_toggle_visibility (xitk_widget_t *w, void *data) {
 
   if(playlist != NULL) {
-    if (playlist->visible && playlist->running) {
-      XLockDisplay(gGui->display);
-      if(gGui->use_root_window) {
-	if(xitk_is_window_visible(gGui->display, playlist->window))
-	  XIconifyWindow(gGui->display, playlist->window, gGui->screen);
-	else
-	  XMapWindow(gGui->display, playlist->window);
-      }
-      else {
-	playlist->visible = 0;
-	xitk_hide_widgets(playlist->widget_list);
-	XUnmapWindow (gGui->display, playlist->window);
-      }
-      XUnlockDisplay(gGui->display);
-    } 
-    else {
-      if(playlist->running) {
-	playlist->visible = 1;
-	xitk_show_widgets(playlist->widget_list);
-	XLockDisplay(gGui->display);
-	XRaiseWindow(gGui->display, playlist->window); 
-	XMapWindow(gGui->display, playlist->window); 
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, playlist->window, gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(playlist->window);
-      }
-    }
+    toggle_window(playlist->window, playlist->widget_list, 
+		  &playlist->visible, playlist->running);
     mmk_editor_toggle_visibility();
   }
 }
@@ -858,12 +855,11 @@ void playlist_update_focused_entry(void) {
 	    
 	    if(gGui->playlist.num <= max_displayed)
 	      _playlist_update_browser_list(-1);
-	    else if(gGui->playlist.cur <= (gGui->playlist.num - max_displayed)) {
+	    else if(gGui->playlist.cur <= (gGui->playlist.num - max_displayed))
 	      _playlist_update_browser_list(gGui->playlist.cur);
-	    }
 	    else
 	      _playlist_update_browser_list(gGui->playlist.num - max_displayed);
-
+	    
 	    if(selected >= 0) {
 	      xitk_browser_set_select(playlist->playlist, selected);
  	      xitk_inputtext_change_text(playlist->winput,
@@ -930,25 +926,28 @@ void playlist_change_skins(void) {
     
     Imlib_destroy_image(gGui->imlib_data, old_img);
     Imlib_apply_image(gGui->imlib_data, new_img, playlist->window);
-    
     XUnlockDisplay(gGui->display);
-    
+
+    if(playlist_is_visible())
+      playlist_raise_window();
+
     xitk_skin_unlock(gGui->skin_config);
     
     xitk_change_skins_widget_list(playlist->widget_list, gGui->skin_config);
     
     {
       int x, y, dir;
-      int i = 0;
+      int i = 0, max;
       
-      x = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayBG");
-      y = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayBG");
+      x   = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayBG");
+      y   = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayBG");
       dir = xitk_skin_get_direction(gGui->skin_config, "AutoPlayBG");
-      
+      max = xitk_skin_get_max_buttons(gGui->skin_config, "AutoPlayBG");
+
       switch(dir) {
       case DIRECTION_UP:
       case DIRECTION_DOWN:
-	while(playlist->autoplay_plugins[i] != NULL) {
+	while((max && ((i < max) && (playlist->autoplay_plugins[i] != NULL))) || (!max && playlist->autoplay_plugins[i] != NULL)) {
 	  
 	  (void) xitk_set_widget_pos(playlist->autoplay_plugins[i], x, y);
 	  
@@ -962,7 +961,7 @@ void playlist_change_skins(void) {
 	break;
       case DIRECTION_LEFT:
       case DIRECTION_RIGHT:
-	while(playlist->autoplay_plugins[i] != NULL) {
+	while((max && ((i < max) && (playlist->autoplay_plugins[i] != NULL))) || (!max && playlist->autoplay_plugins[i] != NULL)) {
 	  
 	  (void) xitk_set_widget_pos(playlist->autoplay_plugins[i], x, y);
 	  
@@ -975,6 +974,13 @@ void playlist_change_skins(void) {
 	}
 	break;
       }
+
+      if(max) {
+	while(playlist->autoplay_plugins[i] != NULL) {
+	  xitk_disable_and_hide_widget(playlist->autoplay_plugins[i]);
+	  i++;
+	}
+      }
     }
     
     xitk_paint_widget_list(playlist->widget_list);
@@ -989,6 +995,11 @@ void playlist_deinit(void) {
   }
 }
 
+void playlist_reparent(void) {
+  if(playlist)
+    reparent_window(playlist->window);
+}
+
 /*
  * Create playlist editor window
  */
@@ -1070,11 +1081,17 @@ void playlist_editor(void) {
 				    CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 				    &attr);
   
-  XSetStandardProperties(gGui->display, playlist->window, title, title,
-			 None, NULL, 0, &hint);
+  XmbSetWMProperties(gGui->display, playlist->window, title, title, NULL, 0,
+                     &hint, NULL, NULL);
   
   XSelectInput(gGui->display, playlist->window, INPUT_MOTION | KeymapStateMask);
+  XUnlockDisplay (gGui->display);
   
+  if(!video_window_is_visible())
+    xitk_set_wm_window_type(playlist->window, WINDOW_TYPE_NORMAL);
+  else
+    xitk_unset_wm_window_type(playlist->window, WINDOW_TYPE_NORMAL);
+
   if(is_layer_above())
     xitk_set_layer_above(playlist->window);
 
@@ -1082,6 +1099,7 @@ void playlist_editor(void) {
    * wm, no border please
    */
 
+  XLockDisplay (gGui->display);
   prop = XInternAtom(gGui->display, "_MOTIF_WM_HINTS", True);
   mwmhints.flags = MWM_HINTS_DECORATIONS;
   mwmhints.decorations = 0;
@@ -1096,13 +1114,15 @@ void playlist_editor(void) {
     xclasshint->res_name = title;
     xclasshint->res_class = "xine";
     XSetClassHint(gGui->display, playlist->window, xclasshint);
+    XFree(xclasshint);
   }
 
   wm_hint = XAllocWMHints();
   if (wm_hint != NULL) {
     wm_hint->input = True;
     wm_hint->initial_state = NormalState;
-    wm_hint->flags = InputHint | StateHint;
+    wm_hint->icon_pixmap   = gGui->icon;
+    wm_hint->flags = InputHint | StateHint | IconPixmapHint;
     XSetWMHints(gGui->display, playlist->window, wm_hint);
     XFree(wm_hint);
   }
@@ -1116,7 +1136,7 @@ void playlist_editor(void) {
   /*
    * Widget-list
    */
-  playlist->widget_list                = xitk_widget_list_new();
+  playlist->widget_list = xitk_widget_list_new();
   xitk_widget_list_set(playlist->widget_list, WIDGET_LIST_LIST, (xitk_list_new()));
   xitk_widget_list_set(playlist->widget_list, WIDGET_LIST_WINDOW, (void *) playlist->window);
   xitk_widget_list_set(playlist->widget_list, WIDGET_LIST_GC, gc);
@@ -1125,29 +1145,28 @@ void playlist_editor(void) {
   lbl.gc              = (XITK_WIDGET_LIST_GC(playlist->widget_list));
 
   b.skin_element_name = "PlMoveUp";
-  b.callback          = _playlist_move_updown;
-  b.userdata          = (void *)MOVEUP;
+  b.callback          = playlist_move_current_updown;
+  b.userdata          = (void *)DIRECTION_UP;
   xitk_list_append_content ((XITK_WIDGET_LIST_LIST(playlist->widget_list)), 
     (playlist->move_up = xitk_button_create (playlist->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(playlist->move_up, _("Move up selected MRL"));
 
-
   b.skin_element_name = "PlMoveDn";
-  b.callback          = _playlist_move_updown;
-  b.userdata          = (void *)MOVEDN;
+  b.callback          = playlist_move_current_updown;
+  b.userdata          = (void *)DIRECTION_DOWN;
   xitk_list_append_content ((XITK_WIDGET_LIST_LIST(playlist->widget_list)), 
     (playlist->move_down = xitk_button_create (playlist->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(playlist->move_down, _("Move down selected MRL"));
 
   b.skin_element_name = "PlPlay";
-  b.callback          = _playlist_play;
+  b.callback          = playlist_play_current;
   b.userdata          = NULL;
   xitk_list_append_content ((XITK_WIDGET_LIST_LIST(playlist->widget_list)), 
     (playlist->play = xitk_button_create (playlist->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(playlist->play, _("Start playback of selected MRL"));
   
   b.skin_element_name = "PlDelete";
-  b.callback          = _playlist_delete;
+  b.callback          = playlist_delete_current;
   b.userdata          = NULL;
   xitk_list_append_content ((XITK_WIDGET_LIST_LIST(playlist->widget_list)), 
     (playlist->delete = xitk_button_create (playlist->widget_list, gGui->skin_config, &b)));
@@ -1170,7 +1189,6 @@ void playlist_editor(void) {
     (playlist->add = xitk_labelbutton_create (playlist->widget_list, gGui->skin_config, &lb)));
   xitk_set_widget_tips(playlist->add, _("Add one or more entries in playlist"));
     
-
   lb.skin_element_name = "PlLoad";
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Load");
@@ -1233,13 +1251,14 @@ void playlist_editor(void) {
 
   {
     int                x, y, dir;
-    int                i = 0;
+    int                i = 0, max;
     const char *const *autoplay_plugins = xine_get_autoplay_input_plugin_ids(gGui->xine);
     
-    x = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayBG");
-    y = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayBG");
+    x   = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayBG");
+    y   = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayBG");
     dir = xitk_skin_get_direction(gGui->skin_config, "AutoPlayBG");
-      
+    max = xitk_skin_get_max_buttons(gGui->skin_config, "AutoPlayBG");
+
     while(autoplay_plugins[i] != NULL) {
 
       lb.skin_element_name = "AutoPlayBG";
@@ -1273,6 +1292,9 @@ void playlist_editor(void) {
 	break;
       }
 
+      if(max && (i >= max))
+	xitk_disable_and_hide_widget(playlist->autoplay_plugins[i]);
+      
       i++;
 
     }
@@ -1299,12 +1321,7 @@ void playlist_editor(void) {
 
   playlist_raise_window();
 
-  while(!xitk_is_window_visible(gGui->display, playlist->window))
-    xine_usec_sleep(5000);
-
-  XLockDisplay(gGui->display);
-  XSetInputFocus(gGui->display, playlist->window, RevertToParent, CurrentTime);
-  XUnlockDisplay(gGui->display);
+  try_to_set_input_focus(playlist->window);
 
   xitk_set_focus_to_widget(playlist->winput);
 }
