@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: post.c,v 1.24 2003/07/30 20:07:40 f1rmb Exp $
+ * $Id: post.c,v 1.50 2004/04/13 19:55:48 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -25,6 +25,10 @@
 #endif
 
 #include <stdio.h>
+
+#ifdef HAVE_ALLOCA_H   
+#include <alloca.h>
+#endif
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -33,7 +37,9 @@
 
 extern gGui_t    *gGui;
 
-#undef TRACE_WIRE
+#undef TRACE_REWIRE
+
+#define DEFAULT_DEINTERLACER "tvtime:method=LinearBlend,cheap_mode=1,pulldown=0,use_progressive_frame_flag=1"
 
 #define WINDOW_WIDTH        500
 #define WINDOW_HEIGHT       500
@@ -41,8 +47,14 @@ extern gGui_t    *gGui;
 #define FRAME_WIDTH         (WINDOW_WIDTH - 40)
 #define FRAME_HEIGHT        80
 
+#define HELP_WINDOW_WIDTH   400
+#define HELP_WINDOW_HEIGHT  300
+
 #define MAX_DISPLAY_FILTERS 5
 
+#define MAX_DISP_ENTRIES    11
+#define BROWSER_LINE_WIDTH  55
+
 #define DISABLE_ME(widget)                                                                      \
     do {                                                                                        \
       if(widget)                                                                                \
@@ -55,6 +67,7 @@ extern gGui_t    *gGui;
         xitk_enable_and_show_widget(widget);                                                    \
     } while(0)
  
+static char                  *br_fontname      = "-misc-fixed-medium-r-normal-*-10-*-*-*-*-*-*-*";
 static char                  *btnfontname      = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
 static char                  *fontname         = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
 static char                  *boldfontname     = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
@@ -78,6 +91,8 @@ typedef struct {
   xitk_widget_t               *up;
   xitk_widget_t               *down;
 
+  xitk_widget_t               *help;
+
   int                          readonly;
 
   char                       **properties_names;
@@ -105,32 +120,57 @@ typedef struct {
   int                         visible;
   xitk_register_key_t         widget_key;
 
+  /* help window stuff */
+  xitk_window_t              *helpwin;
+  xitk_widget_list_t         *help_widget_list;
+  char                      **help_text;
+  xitk_register_key_t         help_widget_key;
+  int                         help_running;
+  xitk_widget_t              *help_browser;
 } _pplugin_t;
 
 static _pplugin_t    *pplugin = NULL;
 static char         **post_audio_plugins;
 
-static pthread_t      rewire_thread;
+/* Some functions prototype */
+static void _pplugin_unwire(void);
+static post_element_t **pplugin_parse_and_load(const char *, int *);
+static void _pplugin_rewire(void);
+void _pplugin_rewire_from_post_elements(post_element_t **, int);
+static post_element_t **_pplugin_join_deinterlace_and_post_elements(int *);
+static void _pplugin_save_chain(void);
+static void _pplugin_close_help(xitk_widget_t *, void *);
 
-static void *post_rewire_thread(void *data) {
-  void (*rewire)(void) = data;
+static void post_deinterlace_plugin_cb(void *data, xine_cfg_entry_t *cfg) {
+  post_element_t **posts = NULL;
+  int              num, i;
   
-  pthread_detach(pthread_self());
-  rewire();
-  pthread_exit(NULL);
-  return NULL;
+  gGui->deinterlace_plugin = cfg->str_value;
+  
+  if(gGui->deinterlace_enable)
+    _pplugin_unwire();
+  
+  for(i = 0; i < gGui->deinterlace_elements_num; i++) {
+    xine_post_dispose(gGui->xine, gGui->deinterlace_elements[i]->post);
+    free(gGui->deinterlace_elements[i]->name);
+    free(gGui->deinterlace_elements[i]);
+  }
+  
+  SAFE_FREE(gGui->deinterlace_elements);
+  gGui->deinterlace_elements_num = 0;
+  
+  if((posts = pplugin_parse_and_load(gGui->deinterlace_plugin, &num))) {
+    gGui->deinterlace_elements     = posts;
+    gGui->deinterlace_elements_num = num;
+  }
+
+  if(gGui->deinterlace_enable)
+    _pplugin_rewire();
 }
 
 static void post_audio_plugin_cb(void *data, xine_cfg_entry_t *cfg) {
-  int err;
-  
-  gGui->visual_anim.post_plugin_num = cfg->num_value;
-  
-  if ((err = pthread_create(&rewire_thread,
-			    NULL, post_rewire_thread, (void *)post_rewire_visual_anim)) != 0) {
-    printf(_("%s(): can't create new thread (%s)\n"), __XINE_FUNCTION__, strerror(err));
-    abort();
-  }
+  gGui->visual_anim.post_plugin_num = cfg->num_value;  
+  post_rewire_visual_anim();
 }
 
 const char **post_get_audio_plugins_names(void) {
@@ -226,16 +266,29 @@ int post_rewire_audio_post_to_stream(xine_stream_t *stream) {
 
 static void _pplugin_unwire(void) {
   xine_post_out_t  *vo_source;
-
+  
   vo_source = xine_get_video_source(gGui->stream);
+
   (void) xine_post_wire_video_port(vo_source, gGui->vo_port);
-  /* Waiting a small chunk of time helps to avoid crashing */
-  xine_usec_sleep(500000);
 }
 
 static void _pplugin_rewire(void) {
 
-  if(pplugin && gGui->post_enable) {
+  static post_element_t **post_elements;
+  int post_elements_num;
+
+  _pplugin_save_chain();
+
+  post_elements = _pplugin_join_deinterlace_and_post_elements(&post_elements_num);
+
+  if( post_elements ) {
+    _pplugin_rewire_from_post_elements(post_elements, post_elements_num);
+
+    free(post_elements);
+  }
+
+#if 0
+  if(pplugin && (gGui->post_enable && !gGui->deinterlace_enable)) {
     xine_post_out_t  *vo_source;
     int               post_num = pplugin->object_num;
     int               i = 0;
@@ -244,6 +297,10 @@ static void _pplugin_rewire(void) {
       post_num--;
     
     if(post_num) {
+      int paused = 0;
+      
+      if((paused = (xine_get_param(gGui->stream, XINE_PARAM_SPEED) == XINE_SPEED_PAUSE)))
+	xine_set_param(gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_SLOW_4);
       
       for(i = (post_num - 1); i >= 0; i--) {
 	const char *const *outs = xine_post_list_outputs(pplugin->post_objects[i]->post);
@@ -274,8 +331,13 @@ static void _pplugin_rewire(void) {
 
       vo_source = xine_get_video_source(gGui->stream);
       xine_post_wire_video_port(vo_source, pplugin->post_objects[0]->post->video_input[0]);
+      
+      if(paused)
+	xine_set_param(gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
+
     }
   }
+#endif
 }
 
 static int _pplugin_get_object_offset(post_object_t *pobj) {
@@ -320,6 +382,7 @@ static void _pplugin_hide_obj(post_object_t *pobj) {
     DISABLE_ME(pobj->comment);
     DISABLE_ME(pobj->up);
     DISABLE_ME(pobj->down);
+    DISABLE_ME(pobj->help);
   }
 }
 
@@ -336,6 +399,9 @@ static void _pplugin_show_obj(post_object_t *pobj) {
     if(pobj->properties)
       xitk_set_widget_pos(pobj->properties, pobj->x + 200, pobj->y + 2);
     
+    if(pobj->help)
+      xitk_set_widget_pos(pobj->help, pobj->x + 390, pobj->y);
+
     if(pobj->comment)
       xitk_set_widget_pos(pobj->comment, pobj->x + 30, (pobj->y + 27));
     
@@ -354,6 +420,9 @@ static void _pplugin_show_obj(post_object_t *pobj) {
     ENABLE_ME(pobj->value);
     ENABLE_ME(pobj->comment);
 
+    if(pobj->help)
+      ENABLE_ME(pobj->help);
+    
     if((!_pplugin_is_first_filter(pobj)) && (xitk_combo_get_current_selected(pobj->plugins)))
       ENABLE_ME(pobj->up);
     
@@ -499,7 +568,7 @@ static void _pplugin_set_param_char(xitk_widget_t *w, void *data, char *text) {
     xitk_inputtext_change_text(pobj->value, (char *)(pobj->param_data + pobj->param->offset));
   }
   else
-    xine_error(_("parameter type POST_PARAM_TYPE_STRING) not supported yet.\n"));
+    xine_error(_("parameter type POST_PARAM_TYPE_STRING not supported yet.\n"));
 
 }
 
@@ -562,7 +631,7 @@ static void _pplugin_add_parameter_widget(post_object_t *pobj) {
 	
 	if(pobj->param->enum_values) {
 	  xitk_combo_widget_t         cmb;
-	  
+
 	  XITK_WIDGET_INIT(&cmb, gGui->imlib_data);
 	  cmb.skin_element_name = NULL;
 	  cmb.layer_above       = (is_layer_above());
@@ -749,6 +818,14 @@ static void _pplugin_destroy_only_obj(post_object_t *pobj) {
       _pplugin_destroy_widget(pobj->value);
       pobj->value = NULL;
     }
+
+    if(pobj->api && pobj->help) {
+      if(pplugin->help_running)
+	_pplugin_close_help(NULL,NULL);
+      _pplugin_destroy_widget(pobj->help);
+      pobj->help = NULL;
+    }
+    
   }
 }
 
@@ -832,11 +909,10 @@ static int __pplugin_retrieve_parameters(post_object_t *pobj) {
     
     parm = api_descr->parameter;
     pobj->param_data = malloc(api_descr->struct_size);
+    post_api->get_parameters(pobj->post, pobj->param_data);
     
     while(parm->type != POST_PARAM_TYPE_LAST) {
       
-      post_api->get_parameters(pobj->post, pobj->param_data);
-      
       if(!pnum)
 	pobj->properties_names = (char **) xine_xmalloc(sizeof(char *) * 2);
       else
@@ -859,8 +935,188 @@ static int __pplugin_retrieve_parameters(post_object_t *pobj) {
   return 0;
 }
 
+static void _pplugin_close_help(xitk_widget_t *w, void *data) {
+
+  pplugin->help_running = 0;
+  
+  xitk_unregister_event_handler(&pplugin->help_widget_key);
+  
+  xitk_destroy_widgets(pplugin->help_widget_list);
+  xitk_window_destroy_window(gGui->imlib_data, pplugin->helpwin);
+  
+  pplugin->helpwin = NULL;
+  xitk_list_free((XITK_WIDGET_LIST_LIST(pplugin->help_widget_list)));
+    
+  XLockDisplay(gGui->display);
+  XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(pplugin->help_widget_list)));
+  XUnlockDisplay(gGui->display);
+    
+  free(pplugin->help_widget_list);
+}
+
+static int __line_wrap(char *s, int pos, int line_size)
+{
+  int word_size = 0;
+
+  while( *s && *s != ' ' && *s != '\n' ) {
+    s++;
+    word_size++;
+  }
+
+  if( word_size >= line_size )
+    return pos > line_size; 
+
+  return word_size + pos > line_size;
+}
+
+static void _pplugin_show_help(xitk_widget_t *w, void *data) {
+  post_object_t *pobj = (post_object_t *) data;
+  GC                          gc;
+  xitk_pixmap_t              *bg = NULL;
+  int                         x, y, width, height;
+  xitk_labelbutton_widget_t   lb;
+  xitk_browser_widget_t       br;
+
+  if(!pobj->api) {
+    if(pplugin->help_running)
+      _pplugin_close_help(NULL,NULL);
+    return;
+  }
+  
+  /* create help window if needed */
+
+  if( !pplugin->help_running ) {
+    x = y = 100;
+    pplugin->helpwin = xitk_window_create_dialog_window(gGui->imlib_data, 
+  						 _("Plugin Help"), x, y,
+  						 HELP_WINDOW_WIDTH, HELP_WINDOW_HEIGHT);
+
+    set_window_states_start((xitk_window_get_window(pplugin->helpwin)));
+
+    XLockDisplay (gGui->display);
+    gc = XCreateGC(gGui->display, 
+  		 (xitk_window_get_window(pplugin->helpwin)), None, None);
+    XUnlockDisplay (gGui->display);
+    
+    pplugin->help_widget_list = xitk_widget_list_new();
+    xitk_widget_list_set(pplugin->help_widget_list, WIDGET_LIST_LIST, (xitk_list_new()));
+    xitk_widget_list_set(pplugin->help_widget_list, 
+  		       WIDGET_LIST_WINDOW, (void *) (xitk_window_get_window(pplugin->helpwin)));
+    xitk_widget_list_set(pplugin->help_widget_list, WIDGET_LIST_GC, gc);
+    
+    xitk_window_get_window_size(pplugin->helpwin, &width, &height);
+    bg = xitk_image_create_xitk_pixmap(gGui->imlib_data, width, height);
+  
+    XLockDisplay (gGui->display);
+    XCopyArea(gGui->display, (xitk_window_get_background(pplugin->helpwin)), bg->pixmap,
+  	    bg->gc, 0, 0, width, height, 0, 0);
+    XUnlockDisplay (gGui->display);
+  
+    XITK_WIDGET_INIT(&lb, gGui->imlib_data);
+    lb.button_type       = CLICK_BUTTON;
+    lb.label             = _("Ok");
+    lb.align             = ALIGN_CENTER;
+    lb.callback          = _pplugin_close_help;
+    lb.state_callback    = NULL;
+    lb.userdata          = NULL;
+    lb.skin_element_name = NULL;
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(pplugin->help_widget_list)), 
+     (w = xitk_noskin_labelbutton_create(pplugin->help_widget_list, 
+  				       &lb, 150, 270, 100, 23,
+  				       "Black", "Black", "White", btnfontname)));
+    xitk_enable_and_show_widget(w);
+  
+  
+    XITK_WIDGET_INIT(&br, gGui->imlib_data);
+    br.arrow_up.skin_element_name    = NULL;
+    br.slider.skin_element_name      = NULL;
+    br.arrow_dn.skin_element_name    = NULL;
+    br.browser.skin_element_name     = NULL;
+    br.browser.max_displayed_entries = MAX_DISP_ENTRIES;
+    br.browser.num_entries           = 0;
+    br.browser.entries               = NULL;
+    br.callback                      = NULL;
+    br.dbl_click_callback            = NULL;
+    br.parent_wlist                  = pplugin->help_widget_list;
+    br.userdata                      = NULL;
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(pplugin->help_widget_list)), 
+     (pplugin->help_browser = 
+      xitk_noskin_browser_create(pplugin->help_widget_list, &br,
+  			       (XITK_WIDGET_LIST_GC(pplugin->help_widget_list)), 15, 25, 
+  			       HELP_WINDOW_WIDTH - (30 + 16), 20,
+  			       16, br_fontname)));
+  }
+
+  /* load text to the browser widget */
+  {
+    char  *p, **hbuf = NULL;
+    int    lines = 0, i;
+    
+    p = pobj->api->get_help();
+    
+    do {
+      if(!lines)
+	hbuf = (char **) xine_xmalloc(sizeof(char *) * 2);
+      else
+	hbuf  = (char **) realloc(hbuf, sizeof(char *) * (lines + 2));
+
+      hbuf[lines]    = malloc(BROWSER_LINE_WIDTH+1);
+      hbuf[lines+1]  = NULL;
+
+      for(i = 0; !__line_wrap(p,i,BROWSER_LINE_WIDTH) && *p && *p != '\n'; i++)
+        hbuf[lines][i] = *p++;
+
+      hbuf[lines][i] = '\0';
+      if( *p && (*p == '\n' || *p == ' ') )
+        p++;
+
+      lines++;
+    } while( *p );
+    
+    if(lines) {
+      char **ohbuf = pplugin->help_text;
+      int    i = 0;
+
+      pplugin->help_text = hbuf;
+      xitk_browser_update_list(pplugin->help_browser, (const char **)pplugin->help_text, NULL,
+			       lines, 0);
+      
+      if(ohbuf) {
+	while(ohbuf[i++])
+	  free(ohbuf[i]);
+	free(ohbuf);
+      }
+    }
+  }
+
+  if( !pplugin->help_running ) {
+
+    xitk_enable_and_show_widget(pplugin->help_browser);
+    xitk_browser_set_alignment(pplugin->help_browser, ALIGN_LEFT);
+  
+  
+    xitk_window_change_background(gGui->imlib_data, pplugin->helpwin, bg->pixmap, width, height);
+    xitk_image_destroy_xitk_pixmap(bg);
+  
+    pplugin->help_widget_key = xitk_register_event_handler("pplugin_help", 
+  						    (xitk_window_get_window(pplugin->helpwin)),
+  						    NULL,
+  						    NULL,
+  						    NULL,
+  						    pplugin->help_widget_list,
+  						    NULL);
+  
+    pplugin->help_running = 1;
+  }
+
+  raise_window(xitk_window_get_window(pplugin->helpwin), 1, pplugin->help_running);
+  
+  try_to_set_input_focus(xitk_window_get_window(pplugin->helpwin));
+}
+
 static void _pplugin_retrieve_parameters(post_object_t *pobj) {
   xitk_combo_widget_t         cmb;
+  xitk_labelbutton_widget_t   lb;
   
   if(__pplugin_retrieve_parameters(pobj)) {
     
@@ -880,6 +1136,24 @@ static void _pplugin_retrieve_parameters(post_object_t *pobj) {
     
     xitk_combo_set_select(pobj->properties, 0);
     xitk_combo_callback_exec(pobj->properties);
+
+    if(pobj->api && pobj->api->get_help) {
+
+      XITK_WIDGET_INIT(&lb, gGui->imlib_data);
+      lb.button_type       = CLICK_BUTTON;
+      lb.label             = _("Help");
+      lb.align             = ALIGN_CENTER;
+      lb.callback          = _pplugin_show_help; 
+      lb.state_callback    = NULL;
+      lb.userdata          = pobj;
+      lb.skin_element_name = NULL;
+      xitk_list_append_content((XITK_WIDGET_LIST_LIST(pplugin->widget_list)), 
+			       (pobj->help = xitk_noskin_labelbutton_create(pplugin->widget_list, 
+									    &lb, pobj->x + 390, pobj->y, 50, 20,
+									    "Black", "Black", "White", btnfontname)));
+      xitk_show_widget(pobj->help);
+      xitk_enable_widget(pobj->help);
+    }
   }
   else {
     xitk_label_widget_t   lb;
@@ -943,6 +1217,9 @@ static void _pplugin_select_filter(xitk_widget_t *w, void *data, int select) {
       xitk_enable_widget(pplugin->new_filter);
 
   }
+  
+  if(pplugin->help_running)
+    _pplugin_show_help(NULL, pobj);
 
   _pplugin_rewire();
   _pplugin_paint_widgets();
@@ -1010,8 +1287,8 @@ static void _pplugin_create_filter_object(void) {
   pplugin->object_num++;
 
   image = xitk_image_create_image(gGui->imlib_data, FRAME_WIDTH, FRAME_HEIGHT);
+
   XLockDisplay(gGui->display);
-  
   XSetForeground(gGui->display, (XITK_WIDGET_LIST_GC(pplugin->widget_list)),
 		 xitk_get_pixel_color_gray(gGui->imlib_data));
   XFillRectangle(gGui->display, image->image->pixmap,
@@ -1039,7 +1316,6 @@ static void _pplugin_create_filter_object(void) {
   draw_inner_frame(gGui->imlib_data, image->image, NULL, boldfontname,
 		    5, FRAME_HEIGHT - 16 - 5, 16, 16);
 
-
   XITK_WIDGET_INIT(&im, gGui->imlib_data);
   im.skin_element_name = NULL;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(pplugin->widget_list)),
@@ -1125,6 +1401,34 @@ static void _pplugin_rebuild_filters(void) {
   _pplugin_paint_widgets();
 }
 
+static post_element_t **_pplugin_join_deinterlace_and_post_elements(int *post_elements_num) {
+  post_element_t **post_elements;
+  int i, j;
+
+  *post_elements_num = 0;
+  if( gGui->post_enable )
+    *post_elements_num += gGui->post_elements_num;
+  if( gGui->deinterlace_enable )
+    *post_elements_num += gGui->deinterlace_elements_num;
+
+  if( *post_elements_num == 0 )
+    return NULL;
+
+  post_elements = (post_element_t **) 
+    xine_xmalloc(sizeof(post_element_t *) * (*post_elements_num));
+
+  for( i = 0; gGui->deinterlace_enable && i < gGui->deinterlace_elements_num; i++ ) {
+    post_elements[i] = gGui->deinterlace_elements[i];
+  }
+
+  for( j = 0; gGui->post_enable && j < gGui->post_elements_num; j++ ) {
+    post_elements[i+j] = gGui->post_elements[j];
+  }
+  
+  return post_elements;
+}
+
+
 static void _pplugin_save_chain(void) {
   
   if(pplugin) {
@@ -1196,6 +1500,16 @@ static void pplugin_exit(xitk_widget_t *w, void *data) {
     window_info_t wi;
     int           i;
     
+    if( pplugin->help_running )
+     _pplugin_close_help(NULL,NULL);
+
+    if(pplugin->help_text) {
+      i = 0;
+      while(pplugin->help_text[i++])
+	free(pplugin->help_text[i]);
+      free(pplugin->help_text);
+    }
+    
     _pplugin_save_chain();
 
     pplugin->running = 0;
@@ -1232,7 +1546,7 @@ static void pplugin_exit(xitk_widget_t *w, void *data) {
     xitk_destroy_widgets(pplugin->widget_list);
     xitk_window_destroy_window(gGui->imlib_data, pplugin->xwin);
 
-    pplugin->xwin = None;
+    pplugin->xwin = NULL;
     xitk_list_free((XITK_WIDGET_LIST_LIST(pplugin->widget_list)));
     
     XLockDisplay(gGui->display);
@@ -1261,24 +1575,15 @@ static void _pplugin_handle_event(XEvent *event, void *data) {
     }
     break;
 
-  case KeyPress: {
-    XKeyEvent      mykeyevent;
-    KeySym         mykey;
-    char           kbuf[256];
-    int            len;
-    int            modifier;
-    
+  case KeyPress:
     if(xitk_is_widget_enabled(pplugin->slider)) {
-      
-      mykeyevent = event->xkey;
+      KeySym         mkey;
+      int            modifier;
       
       xitk_get_key_modifier(event, &modifier);
-      
-      XLockDisplay(gGui->display);
-      len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-      XUnlockDisplay(gGui->display);
-      
-      switch(mykey) {
+      mkey = xitk_get_key_pressed(event);
+
+      switch(mkey) {
 	
       case XK_Up:
 	if((modifier & 0xFFFFFFEF) == MODIFIER_NOMOD) {
@@ -1293,29 +1598,34 @@ static void _pplugin_handle_event(XEvent *event, void *data) {
 	  xitk_slider_callback_exec(pplugin->slider);
 	}
 	break;
-
-
-      case XK_Next: {
-	int pos, max = xitk_slider_get_max(pplugin->slider);
 	
-	pos = max - (pplugin->first_displayed + MAX_DISPLAY_FILTERS);
-	xitk_slider_set_pos(pplugin->slider, (pos >= 0) ? pos : 0);
-	xitk_slider_callback_exec(pplugin->slider);
-      }
-	break;
-
-      case XK_Prior: {
-	int pos, max = xitk_slider_get_max(pplugin->slider);
 	
-	pos = max - (pplugin->first_displayed - MAX_DISPLAY_FILTERS);
-	xitk_slider_set_pos(pplugin->slider, (pos <= max) ? pos : max);
-	xitk_slider_callback_exec(pplugin->slider);
-      }
+      case XK_Next:
+	{
+	  int pos, max = xitk_slider_get_max(pplugin->slider);
+	  
+	  pos = max - (pplugin->first_displayed + MAX_DISPLAY_FILTERS);
+	  xitk_slider_set_pos(pplugin->slider, (pos >= 0) ? pos : 0);
+	  xitk_slider_callback_exec(pplugin->slider);
+	}
 	break;
 	
+      case XK_Prior:
+	{
+	  int pos, max = xitk_slider_get_max(pplugin->slider);
+	  
+	  pos = max - (pplugin->first_displayed - MAX_DISPLAY_FILTERS);
+	  xitk_slider_set_pos(pplugin->slider, (pos <= max) ? pos : max);
+	  xitk_slider_callback_exec(pplugin->slider);
+	}
+	break;
+	
+      case XK_Escape:
+	pplugin_exit(NULL, NULL);
+	break;
+
       }
     }
-  }
     break;
 
   }
@@ -1348,36 +1658,42 @@ void pplugin_rewire_from_posts_window(void) {
   }
 }
 
-void pplugin_rewire_posts(void) {
-
-  _pplugin_unwire();
-
-  if(gGui->post_elements_num) {
-
-    if(gGui->post_enable) {
-      xine_post_out_t   *vo_source;
-      int                i = 0;
-      
-      for(i = (gGui->post_elements_num - 1); i >= 0; i--) {
-	const char *const *outs = xine_post_list_outputs(gGui->post_elements[i]->post);
-	const xine_post_out_t *vo_out = xine_post_output(gGui->post_elements[i]->post, (char *) *outs);
-	if(i == (gGui->post_elements_num - 1)) {
-	  xine_post_wire_video_port((xine_post_out_t *) vo_out, gGui->vo_port);
-	}
-	else {
-	  const xine_post_in_t *vo_in = xine_post_input(gGui->post_elements[i + 1]->post, "video");
-	  int                   err;
-	  
-	  err = xine_post_wire((xine_post_out_t *) vo_out, (xine_post_in_t *) vo_in);	
-	}
+void _pplugin_rewire_from_post_elements(post_element_t **post_elements, int post_elements_num) {
+  
+  if(post_elements_num) {
+    xine_post_out_t   *vo_source;
+    int                i = 0;
+    
+    for(i = (post_elements_num - 1); i >= 0; i--) {
+      const char *const *outs = xine_post_list_outputs(post_elements[i]->post);
+      const xine_post_out_t *vo_out = xine_post_output(post_elements[i]->post, (char *) *outs);
+      if(i == (post_elements_num - 1)) {
+	xine_post_wire_video_port((xine_post_out_t *) vo_out, gGui->vo_port);
+      }
+      else {
+	const xine_post_in_t *vo_in = xine_post_input(post_elements[i + 1]->post, "video");
+	int                   err;
+	
+	err = xine_post_wire((xine_post_out_t *) vo_out, (xine_post_in_t *) vo_in);	
       }
-      
-      vo_source = xine_get_video_source(gGui->stream);
-      xine_post_wire_video_port(vo_source, gGui->post_elements[0]->post->video_input[0]);
     }
+    
+    vo_source = xine_get_video_source(gGui->stream);
+    xine_post_wire_video_port(vo_source, post_elements[0]->post->video_input[0]);
   }
 }
 
+void pplugin_rewire_posts(void) {
+  
+  _pplugin_unwire();
+  _pplugin_rewire();
+
+#if 0
+  if(gGui->post_enable && !gGui->deinterlace_enable)
+    _pplugin_rewire_from_post_elements(gGui->post_elements, gGui->post_elements_num);
+#endif
+}
+
 void pplugin_end(void) {
   pplugin_exit(NULL, NULL);
 }
@@ -1402,56 +1718,16 @@ int pplugin_is_running(void) {
   return 0;
 }
 
-void pplugin_toggle_visibility(xitk_widget_t *w, void *data) {
-  if(pplugin != NULL) {
-    if (pplugin->visible && pplugin->running) {
-      XLockDisplay(gGui->display);
-      if(gGui->use_root_window) {
-	if(xitk_is_window_visible(gGui->display, xitk_window_get_window(pplugin->xwin)))
-	  XIconifyWindow(gGui->display, xitk_window_get_window(pplugin->xwin), gGui->screen);
-	else
-	  XMapWindow(gGui->display, xitk_window_get_window(pplugin->xwin));
-      }
-      else {
-	pplugin->visible = 0;
-	xitk_hide_widgets(pplugin->widget_list);
-	XUnmapWindow (gGui->display, xitk_window_get_window(pplugin->xwin));
-      }
-      XUnlockDisplay(gGui->display);
-    } 
-    else {
-      if(pplugin->running) {
-	pplugin->visible = 1;
-	xitk_show_widgets(pplugin->widget_list);
-	XLockDisplay(gGui->display);
-	XRaiseWindow(gGui->display, xitk_window_get_window(pplugin->xwin)); 
-	XMapWindow(gGui->display, xitk_window_get_window(pplugin->xwin)); 
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, 
-				xitk_window_get_window(pplugin->xwin), gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(xitk_window_get_window(pplugin->xwin));
-      }
-    }
-  }
+void pplugin_raise_window(void) {
+  if(pplugin != NULL)
+    raise_window(xitk_window_get_window(pplugin->xwin), pplugin->visible, pplugin->running);
 }
 
-void pplugin_raise_window(void) {
-  if(pplugin != NULL) {
-    if(pplugin->xwin) {
-      if(pplugin->visible && pplugin->running) {
-	  XLockDisplay(gGui->display);
-	  XUnmapWindow(gGui->display, xitk_window_get_window(pplugin->xwin));
-	  XRaiseWindow(gGui->display, xitk_window_get_window(pplugin->xwin));
-	  XMapWindow(gGui->display, xitk_window_get_window(pplugin->xwin));
-	  if(!gGui->use_root_window)
-	    XSetTransientForHint (gGui->display, 
-				  xitk_window_get_window(pplugin->xwin), gGui->video_window);
-	  XUnlockDisplay(gGui->display);
-	  layer_above_video(xitk_window_get_window(pplugin->xwin));
-      }
-    }
-  }
+
+void pplugin_toggle_visibility(xitk_widget_t *w, void *data) {
+  if(pplugin != NULL)
+    toggle_window(xitk_window_get_window(pplugin->xwin), pplugin->widget_list,
+		  &pplugin->visible, pplugin->running);
 }
 
 void pplugin_update_enable_button(void) {
@@ -1459,6 +1735,11 @@ void pplugin_update_enable_button(void) {
     xitk_labelbutton_set_state(pplugin->enable, gGui->post_enable);
 }
 
+void pplugin_reparent(void) {
+  if(pplugin)
+    reparent_window((xitk_window_get_window(pplugin->xwin)));
+}
+
 void pplugin_panel(void) {
   GC                          gc;
   xitk_labelbutton_widget_t   lb;
@@ -1471,6 +1752,7 @@ void pplugin_panel(void) {
 
   pplugin = (_pplugin_t *) xine_xmalloc(sizeof(_pplugin_t));
   pplugin->first_displayed = 0;
+  pplugin->help_text       = NULL;
   
   x = xine_config_register_num (gGui->xine, "gui.pplugin_x", 
 				100,
@@ -1491,6 +1773,8 @@ void pplugin_panel(void) {
 						   _("Chain Reaction"), x, y,
 						   WINDOW_WIDTH, WINDOW_HEIGHT);
   
+  set_window_states_start((xitk_window_get_window(pplugin->xwin)));
+
   XLockDisplay (gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(pplugin->xwin)), None, None);
@@ -1622,139 +1906,252 @@ void pplugin_panel(void) {
 
   pplugin_raise_window();
   
-  while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(pplugin->xwin)))
-    xine_usec_sleep(5000);
-
-  XLockDisplay (gGui->display);
-  XSetInputFocus(gGui->display, xitk_window_get_window(pplugin->xwin), RevertToParent, CurrentTime);
-  XUnlockDisplay (gGui->display);
+  try_to_set_input_focus(xitk_window_get_window(pplugin->xwin));
 }
 
-void pplugin_parse_and_store_post(const char *post) {
+/* pchain: "<post1>:option1=value1,option2=value2..;<post2>:...." */
+static post_element_t **pplugin_parse_and_load(const char *pchain, int *post_elements_num) {
+  post_element_t **post_elements = NULL;
+  char            *post_chain;
 
-  /* -post <name>:option1=value1,option2=value2... */
-  if(post && strlen(post)) {
-    char          *p = (char *) post;
-    char          *plugin, *args = NULL;
-    xine_post_t   *post;
+  *post_elements_num = 0;
+  
+  if(pchain && strlen(pchain)) {
+    char *p;
     
-    while(*p == ' ')
-      p++;
+    xine_strdupa(post_chain, pchain);
     
-    xine_strdupa(plugin, p);
-    
-    if((p = strchr(plugin, ':')))
-      *p++ = '\0';
-    
-    if(p && (strlen(p) > 1))
-      args = p;
-
-    post = xine_post_init(gGui->xine, plugin, 0, &gGui->ao_port, &gGui->vo_port);
-
-    if(post) {
-      post_object_t  pobj;
-
-      if(!gGui->post_elements_num)
-	gGui->post_elements = (post_element_t **) xine_xmalloc(sizeof(post_element_t *) * 2);
-      else
-	gGui->post_elements = (post_element_t **) 
-	  realloc(gGui->post_elements, sizeof(post_element_t *) * (gGui->post_elements_num + 2));
+    while((p = xine_strsep(&post_chain, ";"))) {
       
-      gGui->post_elements[gGui->post_elements_num] = (post_element_t *) 
-	xine_xmalloc(sizeof(post_element_t));
-      gGui->post_elements[gGui->post_elements_num]->post = post;
-      gGui->post_elements[gGui->post_elements_num]->name = strdup(plugin);
-      gGui->post_elements_num++;
-      gGui->post_elements[gGui->post_elements_num] = NULL;
-      
-      memset(&pobj, 0, sizeof(post_object_t));
-      pobj.post = post;
-
-      if(__pplugin_retrieve_parameters(&pobj)) {
-	int   i;
-
-	if(pobj.properties_names && args) {
-	  char *param;
+      if(p && strlen(p)) {
+	char          *plugin, *args = NULL;
+	xine_post_t   *post;
+	
+	while(*p == ' ')
+	  p++;
+	
+	plugin = strdup(p);
+	
+	if((p = strchr(plugin, ':')))
+	  *p++ = '\0';
+	
+	if(p && (strlen(p) > 1))
+	  args = p;
+	
+	post = xine_post_init(gGui->xine, plugin, 0, &gGui->ao_port, &gGui->vo_port);
+	
+	if(post) {
+	  post_object_t  pobj;
+	 
+	  if(!(*post_elements_num))
+	    post_elements = (post_element_t **) xine_xmalloc(sizeof(post_element_t *) * 2);
+	  else
+	    post_elements = (post_element_t **) 
+	      realloc(post_elements, sizeof(post_element_t *) * ((*post_elements_num) + 2));
 	  
-	  while((param = xine_strsep(&args, ",")) != NULL) {
+	  post_elements[(*post_elements_num)] = (post_element_t *) 
+	    xine_xmalloc(sizeof(post_element_t));
+	  post_elements[(*post_elements_num)]->post = post;
+	  post_elements[(*post_elements_num)]->name = strdup(plugin);
+	  (*post_elements_num)++;
+	  post_elements[(*post_elements_num)] = NULL;
+	  
+	  memset(&pobj, 0, sizeof(post_object_t));
+	  pobj.post = post;
+	  
+	  if(__pplugin_retrieve_parameters(&pobj)) {
+	    int   i;
 	    
-	    p = param;
-	    
-	    while((*p != '\0') && (*p != '='))
-	      p++;
-	    
-	    if(p && strlen(p)) {
-	      int param_num = 0;
+	    if(pobj.properties_names && args) {
+	      char *param;
 	      
-	      *p++ = '\0';
-	      
-	      while(pobj.properties_names[param_num]
-		    && strcmp(pobj.properties_names[param_num], param))
-		param_num++;
-	      
-	      if(pobj.properties_names[param_num]) {
+	      while((param = xine_strsep(&args, ",")) != NULL) {
 		
-		pobj.param    = pobj.descr->parameter;
-		pobj.param    += param_num;
-		pobj.readonly = pobj.param->readonly;
+		p = param;
+		
+		while((*p != '\0') && (*p != '='))
+		  p++;
+		
+		if(p && strlen(p)) {
+		  int param_num = 0;
+		  
+		  *p++ = '\0';
+		  
+		  while(pobj.properties_names[param_num]
+			&& strcasecmp(pobj.properties_names[param_num], param))
+		    param_num++;
+		  
+		  if(pobj.properties_names[param_num]) {
+		    
+		    pobj.param    = pobj.descr->parameter;
+		    pobj.param    += param_num;
+		    pobj.readonly = pobj.param->readonly;
+		    
+		    switch(pobj.param->type) {
+		    case POST_PARAM_TYPE_INT:
+		      if(!pobj.readonly) {
+			if(pobj.param->enum_values) {
+			  char **values = pobj.param->enum_values;
+			  int    i = 0;
+	  
+			  while(values[i]) {
+			    if(!strcasecmp(values[i], p)) {
+			      *(int *)(pobj.param_data + pobj.param->offset) = i;
+			      break;
+			    }
+			    i++;
+			  }
 
-		switch(pobj.param->type) {
-		case POST_PARAM_TYPE_INT:
-		  if(!pobj.readonly) {
-		    *(int *)(pobj.param_data + pobj.param->offset) = (int) strtol(p, &p, 10);
-		    _pplugin_update_parameter(&pobj);
-		  }
-		  break;
-
-		case POST_PARAM_TYPE_DOUBLE:
-		  if(!pobj.readonly) {
-		    *(double *)(pobj.param_data + pobj.param->offset) = strtod(p, &p);
-		    _pplugin_update_parameter(&pobj);
-		  }
-		  break;
-
-		case POST_PARAM_TYPE_CHAR:
-		case POST_PARAM_TYPE_STRING:
-		  if(!pobj.readonly) {
-		    if(pobj.param->type == POST_PARAM_TYPE_CHAR) {
-		      int maxlen = pobj.param->size / sizeof(char);
+			  if( !values[i] ) 
+			    *(int *)(pobj.param_data + pobj.param->offset) = (int) strtol(p, &p, 10);
+			} else {
+			  *(int *)(pobj.param_data + pobj.param->offset) = (int) strtol(p, &p, 10);
+			}
+			_pplugin_update_parameter(&pobj);
+		      }
+		      break;
 		      
-		      snprintf((char *)(pobj.param_data + pobj.param->offset), maxlen, "%s", p);
-		      _pplugin_update_parameter(&pobj);
+		    case POST_PARAM_TYPE_DOUBLE:
+		      if(!pobj.readonly) {
+			*(double *)(pobj.param_data + pobj.param->offset) = strtod(p, &p);
+			_pplugin_update_parameter(&pobj);
+		      }
+		      break;
+		      
+		    case POST_PARAM_TYPE_CHAR:
+		    case POST_PARAM_TYPE_STRING:
+		      if(!pobj.readonly) {
+			if(pobj.param->type == POST_PARAM_TYPE_CHAR) {
+			  int maxlen = pobj.param->size / sizeof(char);
+			  
+			  snprintf((char *)(pobj.param_data + pobj.param->offset), maxlen, "%s", p);
+			  _pplugin_update_parameter(&pobj);
+			}
+			else
+			  fprintf(stderr, "parameter type POST_PARAM_TYPE_STRING not supported yet.\n");
+		      }
+		      break;
+		      
+		    case POST_PARAM_TYPE_STRINGLIST: /* unsupported */
+		      if(!pobj.readonly)
+			fprintf(stderr, "parameter type POST_PARAM_TYPE_STRINGLIST not supported yet.\n");
+		      break;
+		      
+		    case POST_PARAM_TYPE_BOOL:
+		      if(!pobj.readonly) {
+			*(int *)(pobj.param_data + pobj.param->offset) = ((int) strtol(p, &p, 10)) ? 1 : 0;
+			_pplugin_update_parameter(&pobj);
+		      }
+		      break;
 		    }
-		    else
-		     fprintf(stderr, "parameter type POST_PARAM_TYPE_STRING) not supported yet.\n");
 		  }
-		  break;
-
-		case POST_PARAM_TYPE_STRINGLIST: // unsupported */
-		  if(!pobj.readonly)
-		    fprintf(stderr, "parameter type POST_PARAM_TYPE_STRINGLIST not supported yet.\n");
-		  break;
-
-		case POST_PARAM_TYPE_BOOL:
-		  if(!pobj.readonly) {
-		    *(int *)(pobj.param_data + pobj.param->offset) = ((int) strtol(p, &p, 10)) ? 1 : 0;
-		    _pplugin_update_parameter(&pobj);
-		  }
-		  break;
 		}
+	      } 
+	      
+	      i = 0;
+	      
+	      while(pobj.properties_names[i]) {
+		free(pobj.properties_names[i]);
+		i++;
 	      }
+	      
+	      free(pobj.properties_names);
 	    }
-	  } 
-
-	  i = 0;
-	  
-	  while(pobj.properties_names[i]) {
-	    free(pobj.properties_names[i]);
-	    i++;
+	    
+	    free(pobj.param_data);
 	  }
-	  
-	  free(pobj.properties_names);
 	}
 	
-	free(pobj.param_data);
+	free(plugin);
       }
     }
   }
+
+  return post_elements;
+}
+
+void pplugin_parse_and_store_post(const char *post_chain) {
+  post_element_t **posts = NULL;
+  int              num;
+
+  if((posts = pplugin_parse_and_load(post_chain, &num))) {
+    if(gGui->post_elements_num) {
+      int i;
+      int ptot = gGui->post_elements_num + num;
+      
+      gGui->post_elements = (post_element_t **) realloc(gGui->post_elements, 
+							sizeof(post_element_t *) * (ptot + 1));
+      for(i = gGui->post_elements_num; i <  ptot; i++)
+	gGui->post_elements[i] = posts[i - gGui->post_elements_num];
+
+      gGui->post_elements[i] = NULL;
+      gGui->post_elements_num += num;
+
+    }
+    else {
+      gGui->post_elements     = posts;
+      gGui->post_elements_num = num;;
+    }
+  }
+  
+}
+
+static char *_pplugin_get_default_deinterlacer(void) {
+  return DEFAULT_DEINTERLACER;
+}
+
+void post_deinterlace_init(const char *deinterlace_post) {
+  post_element_t **posts = NULL;
+  int              num;
+  char            *deinterlace_default;
+
+  deinterlace_default = _pplugin_get_default_deinterlacer();
+  
+  gGui->deinterlace_plugin = 
+    (char *) xine_config_register_string (gGui->xine, "gui.deinterlace_plugin", 
+					  deinterlace_default,
+					  _("Deinterlace plugin."),
+					  _("Plugin (with optional parameters) to use "
+					    "when deinterlace is used (plugin separator is ';')."),
+					  CONFIG_LEVEL_ADV,
+					  post_deinterlace_plugin_cb,
+					  CONFIG_NO_DATA);
+  if((posts = pplugin_parse_and_load((deinterlace_post && strlen(deinterlace_post)) ? 
+				     deinterlace_post : gGui->deinterlace_plugin, &num))) {
+    gGui->deinterlace_elements     = posts;
+    gGui->deinterlace_elements_num = num;
+  }
+  
+}
+
+void post_deinterlace(void) {
+
+  if( !gGui->deinterlace_elements_num ) {
+    /* fallback to the old method */
+    xine_set_param(gGui->stream, XINE_PARAM_VO_DEINTERLACE,
+                   gGui->deinterlace_enable);
+  }
+  else {
+    _pplugin_unwire();
+    _pplugin_rewire();
+  }
+
+#if 0
+  if(gGui->deinterlace_enable) {
+    if(gGui->post_enable)
+      _pplugin_unwire();
+    
+    _pplugin_rewire_from_post_elements(gGui->deinterlace_elements, gGui->deinterlace_elements_num);
+  }
+  else {
+    _pplugin_unwire();
+    
+    if(gGui->post_enable) {
+      if(pplugin_is_visible() && pplugin)
+	_pplugin_rewire();
+      else
+	_pplugin_rewire_from_post_elements(gGui->post_elements, gGui->post_elements_num);
+    }
+  }
+#endif
 }
