@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: session.h,v 1.4 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: session.h,v 1.8 2004/04/13 19:55:49 f1rmb Exp $
  *
  */
 #ifndef CTRLSOCK_H
@@ -41,11 +41,16 @@ typedef enum {
   CMD_SPU_PREV,
   CMD_PLAYLIST_FLUSH,
   CMD_PLAYLIST_ADD,
-  CMD_PLAYLIST_NEXT,
+  CMD_PLAYLIST_FIRST,
   CMD_PLAYLIST_PREV,
+  CMD_PLAYLIST_NEXT,
+  CMD_PLAYLIST_LAST,
   CMD_PLAYLIST_LOAD,
+  CMD_PLAYLIST_STOP,
+  CMD_PLAYLIST_CONTINUE,
   CMD_VOLUME,
   CMD_AMP,
+  CMD_LOOP,
   CMD_GET_VERSION,
   CMD_PING
 } ctrl_commands_t;
