@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: setup.c,v 1.81 2003/07/25 22:52:57 f1rmb Exp $
+ * $Id: setup.c,v 1.94 2004/04/13 19:55:49 f1rmb Exp $
  *
  *
  */
@@ -26,13 +26,10 @@
 #endif
 
 #include <stdio.h>
-#include <errno.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 #include <X11/cursorfont.h>
-#include <pthread.h>
-#include <assert.h>
 
 #include "common.h"
 
@@ -43,13 +40,12 @@ static char                     *boldfontname = "-*-helvetica-bold-r-*-*-10-*-*-
 static char                     *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
 
 #define WINDOW_WIDTH             500
-#define WINDOW_HEIGHT            480
+#define WINDOW_HEIGHT            500
 
 #define FRAME_WIDTH              350
 #define FRAME_HEIGHT             40
 
 #define MAX_DISPLAY_WIDGETS      8
-#define BROWSER_MAX_DISP_ENTRIES 17
 
 #define NORMAL_CURS              0
 #define WAIT_CURS                1
@@ -70,7 +66,6 @@ static char                     *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-
     xitk_font_unload_font(fs);                                                                  \
                                                                                                 \
     XLockDisplay(gGui->display);                                                                \
-                                                                                                \
     XSetForeground(gGui->display, (XITK_WIDGET_LIST_GC(setup->widget_list)),                    \
 		   xitk_get_pixel_color_gray(gGui->imlib_data));                                \
     XFillRectangle(gGui->display, image->image->pixmap,                                         \
@@ -94,13 +89,17 @@ static char                     *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-
   }
 
 #define ADD_LABEL(widget, cb, data) {                                                           \
-    int           wx, wy, wh, fh;                                                               \
-    xitk_font_t  *fs;                                                                           \
+    int            wx, wy, wh, fh;                                                              \
+    xitk_font_t   *fs;                                                                          \
     xitk_widget_t *lbl;                                                                         \
+    char           _labelkey[strlen(labelkey) + 5];                                             \
+                                                                                                \
+    sprintf(_labelkey, "%s%s", labelkey,                                                        \
+      (!entry->callback_data && !entry->callback) ? " (*)" : "");                               \
                                                                                                 \
     fs = xitk_font_load_font(gGui->display, fontname);                                          \
     xitk_font_set_font(fs, (XITK_WIDGET_LIST_GC(setup->widget_list)));                          \
-    fh = xitk_font_get_string_height(fs, labelkey);                                             \
+    fh = xitk_font_get_string_height(fs, _labelkey);                                            \
     xitk_font_unload_font(fs);                                                                  \
                                                                                                 \
     xitk_get_widget_pos(widget, &wx, &wy);                                                      \
@@ -108,7 +107,8 @@ static char                     *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-
     wh = xitk_get_widget_height(widget);                                                        \
                                                                                                 \
     lbl = setup_add_label (wx + 20, (wy + (wh >> 1)) - (fh>>1),                                 \
-                           (FRAME_WIDTH - wx), labelkey, cb, data);                             \
+                           (FRAME_WIDTH - (xitk_get_widget_width(widget) + 35)),                \
+                           _labelkey, cb, data);                                                \
     wt->label = lbl;                                                                            \
   }
 
@@ -170,13 +170,6 @@ typedef struct {
   int                   num_wg;
   int                   first_displayed;
 
-  const char          **config_content;
-  int                   config_lines;
-  const char          **readme_content;
-  int                   readme_lines;
-  const char          **faq_content;
-  int                   faq_lines;
-
   Cursor                cursor[2];
 
   xitk_register_key_t   kreg;
@@ -199,7 +192,6 @@ void setup_exit(xitk_widget_t *w, void *data) {
 
   if(setup) {
     window_info_t wi;
-    int           i;
     
     setup->running = 0;
     setup->visible = 0;
@@ -215,7 +207,7 @@ void setup_exit(xitk_widget_t *w, void *data) {
     xitk_destroy_widgets(setup->widget_list);
     xitk_window_destroy_window(gGui->imlib_data, setup->xwin);
     
-    setup->xwin = None;
+    setup->xwin = NULL;
     xitk_list_free((XITK_WIDGET_LIST_LIST(setup->widget_list)));
     xitk_list_free(setup->widgets);
     
@@ -227,27 +219,6 @@ void setup_exit(xitk_widget_t *w, void *data) {
     
     free(setup->widget_list);
     
-    if(setup->config_content) {
-      for(i = 0; i < setup->config_lines; i++) {
-	free((char *)setup->config_content[i]);
-      }
-      free(setup->config_content);
-    }
-    
-    if(setup->faq_content) {
-      for(i = 0; i < setup->faq_lines; i++) {
-	free((char *)setup->faq_content[i]);
-      }
-      free(setup->faq_content);
-    }
-    
-    if(setup->readme_content) {
-      for(i = 0; i < setup->readme_lines; i++) {
-	free((char **)setup->readme_content[i]);
-      }
-      free(setup->readme_content);
-    }
-    
     free(setup);
     setup = NULL;
   }
@@ -283,63 +254,21 @@ int setup_is_visible(void) {
  * Raise setup->xwin
  */
 void setup_raise_window(void) {
-  
-  if(setup != NULL) {
-    if(setup->xwin) {
-      if(setup->visible && setup->running) {
-	XLockDisplay(gGui->display);
-	XUnmapWindow(gGui->display, xitk_window_get_window(setup->xwin));
-	XRaiseWindow(gGui->display, xitk_window_get_window(setup->xwin));
-	XMapWindow(gGui->display, xitk_window_get_window(setup->xwin));
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, 
-				xitk_window_get_window(setup->xwin), gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(xitk_window_get_window(setup->xwin));
-      }
-    }
-  }
+  if(setup != NULL)
+    raise_window(xitk_window_get_window(setup->xwin), setup->visible, setup->running);
 }
 
 /*
  * Hide/show the setup panel
  */
 void setup_toggle_visibility (xitk_widget_t *w, void *data) {
-  
-  if(setup != NULL) {
-    if (setup->visible && setup->running) {
-      XLockDisplay(gGui->display);
-      if(gGui->use_root_window) {
-	if(xitk_is_window_visible(gGui->display, xitk_window_get_window(setup->xwin)))
-	  XIconifyWindow(gGui->display, xitk_window_get_window(setup->xwin), gGui->screen);
-	else
-	  XMapWindow(gGui->display, xitk_window_get_window(setup->xwin));
-      }
-      else {
-	setup->visible = 0;
-	xitk_hide_widgets(setup->widget_list);
-	XUnmapWindow (gGui->display, xitk_window_get_window(setup->xwin));
-      }
-      XUnlockDisplay(gGui->display);
-    } 
-    else {
-      if(setup->running) {
-	setup->visible = 1;
-	xitk_show_widgets(setup->widget_list);
-	XLockDisplay(gGui->display);
-	XRaiseWindow(gGui->display, xitk_window_get_window(setup->xwin)); 
-	XMapWindow(gGui->display, xitk_window_get_window(setup->xwin)); 
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, 
-				xitk_window_get_window(setup->xwin), gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(xitk_window_get_window(setup->xwin));
-      }
-    }
-  }
+  if(setup != NULL)
+    toggle_window(xitk_window_get_window(setup->xwin), setup->widget_list,
+		  &setup->visible, setup->running);
 }
 
 static void setup_apply(xitk_widget_t *w, void *data) {
+  int need_restart = 0;
 
   if(setup->num_wg > 0) {
     int i;
@@ -355,6 +284,11 @@ static void setup_apply(xitk_widget_t *w, void *data) {
 	  int            numval = 0;
 	  char          *strval = NULL;
 	  
+	  if(!need_restart) {
+	    if(setup->wg[i]->changed && (!setup->wg[i]->cfg->callback_data && !setup->wg[i]->cfg->callback))
+	      need_restart = 1;
+	  }
+
 	  setup->wg[i]->changed = 0;
 	  
 	  switch(type & WIDGET_TYPE_MASK) {
@@ -404,6 +338,23 @@ static void setup_apply(xitk_widget_t *w, void *data) {
 
     if(w != setup->ok)
       setup_change_section(setup->tabs, NULL, xitk_tabs_get_current_selected(setup->tabs));
+
+    if(need_restart) {
+      xitk_window_t *xw;
+      
+      xw = xitk_window_dialog_ok(gGui->imlib_data, _("Important Notice"),
+				 NULL, NULL,
+				 ALIGN_CENTER,
+				 _("You changed some configuration value which require"
+				   " to restart xine to take effect."));
+      if(!gGui->use_root_window) {
+	XLockDisplay(gGui->display);
+	XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
+	XUnlockDisplay(gGui->display);
+      }
+      layer_above_video(xitk_window_get_window(xw));
+    }
+
   }
 }
 
@@ -429,7 +380,7 @@ static void setup_clear_tab(void) {
   xitk_image_t *im;
 
   im = xitk_image_create_image(gGui->imlib_data, (WINDOW_WIDTH - 40), 
-  			       (WINDOW_HEIGHT - (51 + 57) + 1 + 4));
+  			       (WINDOW_HEIGHT - (51 + 57 + 20) + 1 + 4));
 
   draw_outter(gGui->imlib_data, im->image, im->width, im->height);
 
@@ -511,24 +462,15 @@ static void setup_handle_event(XEvent *event, void *data) {
     }
     break;
 
-  case KeyPress: {
-    XKeyEvent      mykeyevent;
-    KeySym         mykey;
-    char           kbuf[256];
-    int            len;
-    int            modifier;
-    
+  case KeyPress:
     if(xitk_is_widget_enabled(setup->slider_wg)) {
-      
-      mykeyevent = event->xkey;
+      KeySym         mkey;
+      int            modifier;
       
       xitk_get_key_modifier(event, &modifier);
+      mkey = xitk_get_key_pressed(event);
       
-      XLockDisplay(gGui->display);
-      len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-      XUnlockDisplay(gGui->display);
-      
-      switch(mykey) {
+      switch(mkey) {
 	
       case XK_Up:
 	if((modifier & 0xFFFFFFEF) == MODIFIER_NOMOD) {
@@ -543,29 +485,32 @@ static void setup_handle_event(XEvent *event, void *data) {
 	  xitk_slider_callback_exec(setup->slider_wg);
 	}
 	break;
-
-
-      case XK_Next: {
-	int pos, max = xitk_slider_get_max(setup->slider_wg);
 	
-	pos = max - (setup->first_displayed + MAX_DISPLAY_WIDGETS);
-	xitk_slider_set_pos(setup->slider_wg, (pos >= 0) ? pos : 0);
-	xitk_slider_callback_exec(setup->slider_wg);
-      }
-	break;
-
-      case XK_Prior: {
-	int pos, max = xitk_slider_get_max(setup->slider_wg);
-	
-	pos = max - (setup->first_displayed - MAX_DISPLAY_WIDGETS);
-	xitk_slider_set_pos(setup->slider_wg, (pos <= max) ? pos : max);
-	xitk_slider_callback_exec(setup->slider_wg);
-      }
+      case XK_Next:
+	{
+	  int pos, max = xitk_slider_get_max(setup->slider_wg);
+	  
+	  pos = max - (setup->first_displayed + MAX_DISPLAY_WIDGETS);
+	  xitk_slider_set_pos(setup->slider_wg, (pos >= 0) ? pos : 0);
+	  xitk_slider_callback_exec(setup->slider_wg);
+	}
 	break;
 	
+      case XK_Prior:
+	{
+	  int pos, max = xitk_slider_get_max(setup->slider_wg);
+	  
+	  pos = max - (setup->first_displayed - MAX_DISPLAY_WIDGETS);
+	  xitk_slider_set_pos(setup->slider_wg, (pos <= max) ? pos : max);
+	  xitk_slider_callback_exec(setup->slider_wg);
+	}
+	break;
+	
+      case XK_Escape:
+	setup_exit(NULL, NULL);
+	break;
       }
     }
-  }
     break;
     
   }
@@ -653,11 +598,11 @@ static widget_triplet_t *setup_add_nothing_available(const char *title, int x, i
  *
  */
 static widget_triplet_t *setup_add_slider (const char *title, const char *labelkey, 
-					   int x, int y, xine_cfg_entry_t *entry ) {
+					   int x, int y, xine_cfg_entry_t *entry) {
   xitk_slider_widget_t     sl;
   xitk_widget_t           *slider;
-  static widget_triplet_t *wt; 
-  
+  static widget_triplet_t *wt;
+
   wt = (widget_triplet_t *) xine_xmalloc(sizeof(widget_triplet_t));
 
   XITK_WIDGET_INIT(&sl, gGui->imlib_data);
@@ -696,7 +641,7 @@ static widget_triplet_t *setup_add_inputnum(const char *title, const char *label
   xitk_intbox_widget_t      ib;
   xitk_widget_t            *intbox, *wi, *wbu, *wbd;
   static widget_triplet_t  *wt;
-
+  
   wt = (widget_triplet_t *) xine_xmalloc(sizeof(widget_triplet_t));
 
   XITK_WIDGET_INIT(&ib, gGui->imlib_data);
@@ -776,7 +721,7 @@ static widget_triplet_t *setup_add_checkbox (const char *title, const char *labe
   xitk_checkbox_widget_t    cb;
   xitk_widget_t            *checkbox;
   static widget_triplet_t  *wt;
-
+  
   wt = (widget_triplet_t *) xine_xmalloc(sizeof(widget_triplet_t));
 
   XITK_WIDGET_INIT(&cb, gGui->imlib_data);
@@ -790,7 +735,7 @@ static widget_triplet_t *setup_add_checkbox (const char *title, const char *labe
   xitk_list_append_content ((XITK_WIDGET_LIST_LIST(setup->widget_list)),
 			   (checkbox = 
 			    xitk_noskin_checkbox_create(setup->widget_list, &cb,
-							x, y, 10, 10)));
+							x, y, 12, 12)));
   xitk_checkbox_set_state (checkbox, entry->num_value);  
   ADD_LABEL(checkbox, label_cb, (void *) checkbox);
 
@@ -842,49 +787,6 @@ static widget_triplet_t *setup_add_combo (const char *title, const char *labelke
   return wt;
 }
 
-/*
- * Add a browser (needed for help files display).
- */
-static widget_triplet_t *setup_list_browser(int x, int y, const char **content, int len) {
-  xitk_browser_widget_t     br;
-  xitk_widget_t            *browser;
-  static widget_triplet_t  *wt;
-
-  wt = (widget_triplet_t *) xine_xmalloc(sizeof(widget_triplet_t));
-
-  XITK_WIDGET_INIT(&br, gGui->imlib_data);
-
-  br.arrow_up.skin_element_name    = NULL;
-  br.slider.skin_element_name      = NULL;
-  br.arrow_dn.skin_element_name    = NULL;
-  br.browser.skin_element_name     = NULL;
-  br.browser.max_displayed_entries = BROWSER_MAX_DISP_ENTRIES;
-  br.browser.num_entries           = len;
-  br.browser.entries               = content;
-  br.callback                      = NULL;
-  br.dbl_click_callback            = NULL;
-  br.parent_wlist                  = setup->widget_list;
-  br.userdata                      = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(setup->widget_list)), 
-			   (browser = 
-			    xitk_noskin_browser_create(setup->widget_list, &br,
-						       (XITK_WIDGET_LIST_GC(setup->widget_list)), 
-						       x, y,
-						       (WINDOW_WIDTH - 50) - 16,
-						       20, 16, fontname)));
-  
-  xitk_browser_set_alignment(browser, ALIGN_LEFT);
-  xitk_browser_update_list(browser, content, len, 0);
-
-  add_widget_to_list(browser);
-
-  wt->widget = browser;
-  wt->frame  = NULL;
-  wt->label  = NULL;
-
-  return wt;
-}
-
 /*
  *
  */
@@ -901,119 +803,99 @@ static void setup_section_widgets(int s) {
   xitk_disable_widget(setup->slider_wg);
   xitk_hide_widget(setup->slider_wg);
   
-  /* Selected tab is one of help sections */
-  if(s >= (setup->num_sections - 3)) {
-    if(s == (setup->num_sections - 3)) {
-      setup->wg[setup->num_wg] = setup_list_browser (25, 70, 
-						     setup->config_content, setup->config_lines);
-    }
-    else if(s == (setup->num_sections - 2)) {
-      setup->wg[setup->num_wg] = setup_list_browser (25, 70, 
-						     setup->readme_content, setup->readme_lines);
-    }
-    else if(s == (setup->num_sections - 1)) {
-      setup->wg[setup->num_wg] = setup_list_browser (25, 70, 
-						     setup->faq_content, setup->faq_lines);
-    }
+  section = setup->sections[s];
+  len     = strlen (section);
+  entry   = (xine_cfg_entry_t *)xine_xmalloc(sizeof(xine_cfg_entry_t));
+  cfg_err_result   = xine_config_get_first_entry(gGui->xine, entry);
     
-    DISABLE_ME(setup->wg[setup->num_wg]);
-    setup->num_wg++;
-  }
-  else {
-    
-    section = setup->sections[s];
-    len     = strlen (section);
-    entry   = (xine_cfg_entry_t *)xine_xmalloc(sizeof(xine_cfg_entry_t));
-    cfg_err_result   = xine_config_get_first_entry(gGui->xine, entry);
-    
-    while (cfg_err_result) {
+  while (cfg_err_result) {
       
-      if((entry->exp_level <= gGui->experience_level) &&
-	 ((!strncmp(entry->key, section, len)) && entry->description)) {
+    if((entry->exp_level <= gGui->experience_level) &&
+       ((!strncmp(entry->key, section, len)) && entry->description)) {
 	
-	labelkey = &entry->key[len+1];
+      labelkey = &entry->key[len+1];
 	
-	switch (entry->type) {
+      switch (entry->type) {
 	  
-	case XINE_CONFIG_TYPE_RANGE: /* slider */
-	  setup->wg[setup->num_wg] = setup_add_slider (entry->description, labelkey, x, y, entry);
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  DISABLE_ME(setup->wg[setup->num_wg]);
-	  setup->num_wg++;
-	  break;
+      case XINE_CONFIG_TYPE_RANGE: /* slider */
+	setup->wg[setup->num_wg] = setup_add_slider (entry->description, labelkey, x, y, entry);
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	DISABLE_ME(setup->wg[setup->num_wg]);
+	setup->num_wg++;
+	break;
 	  
-	case XINE_CONFIG_TYPE_STRING:
-	  setup->wg[setup->num_wg] = setup_add_inputtext (entry->description, labelkey, x, y, entry);
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  DISABLE_ME(setup->wg[setup->num_wg]);
-	  setup->num_wg++;
-	  break;
+      case XINE_CONFIG_TYPE_STRING:
+	setup->wg[setup->num_wg] = setup_add_inputtext (entry->description, labelkey, x, y, entry);
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	DISABLE_ME(setup->wg[setup->num_wg]);
+	setup->num_wg++;
+	break;
 	  
-	case XINE_CONFIG_TYPE_ENUM:
-	  setup->wg[setup->num_wg] = setup_add_combo (entry->description, labelkey, x, y, entry);
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  DISABLE_ME(setup->wg[setup->num_wg]);
-	  setup->num_wg++;
-	  break;
+      case XINE_CONFIG_TYPE_ENUM:
+	setup->wg[setup->num_wg] = setup_add_combo (entry->description, labelkey, x, y, entry);
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	DISABLE_ME(setup->wg[setup->num_wg]);
+	setup->num_wg++;
+	break;
 	  
-	case XINE_CONFIG_TYPE_NUM:
-	  setup->wg[setup->num_wg] = setup_add_inputnum (entry->description, labelkey, x, y, entry);
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  DISABLE_ME(setup->wg[setup->num_wg]);
-	  setup->num_wg++;
-	  break;
+      case XINE_CONFIG_TYPE_NUM:
+	setup->wg[setup->num_wg] = setup_add_inputnum (entry->description, labelkey, x, y, entry);
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	DISABLE_ME(setup->wg[setup->num_wg]);
+	setup->num_wg++;
+	break;
 	  
-	case XINE_CONFIG_TYPE_BOOL:
-	  setup->wg[setup->num_wg] = setup_add_checkbox (entry->description, labelkey, x, y, entry);
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  DISABLE_ME(setup->wg[setup->num_wg]);
-	  setup->num_wg++;
-	  break;
+      case XINE_CONFIG_TYPE_BOOL:
+	setup->wg[setup->num_wg] = setup_add_checkbox (entry->description, labelkey, x, y, entry);
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	DISABLE_ME(setup->wg[setup->num_wg]);
+	setup->num_wg++;
+	break;
 	  
-	}
-	
-      } else
-        free(entry);
+      }
       
-      entry = (xine_cfg_entry_t *)xine_xmalloc(sizeof(xine_cfg_entry_t));
-      cfg_err_result = xine_config_get_next_entry(gGui->xine, entry);
-    }
-    free(entry);
+    } else
+      free(entry);
+      
+    entry = (xine_cfg_entry_t *)xine_xmalloc(sizeof(xine_cfg_entry_t));
+    cfg_err_result = xine_config_get_next_entry(gGui->xine, entry);
+  }
+  free(entry);
 
     
-    if(setup->num_wg == 0) {
-      setup->wg[setup->num_wg++] = 
-	setup_add_nothing_available(_("There is no configuration option available in "
-				      "this user experience level."), x, y);
-    }
-
-
-    if(setup->num_wg > MAX_DISPLAY_WIDGETS) {
-      slidmax = setup->num_wg - MAX_DISPLAY_WIDGETS;
-      xitk_show_widget(setup->slider_wg);
-      xitk_enable_widget(setup->slider_wg);
-    }
-    else
-      slidmax = 1;
-
-    xitk_slider_set_max(setup->slider_wg, slidmax);
-    xitk_slider_set_pos(setup->slider_wg, slidmax);
+  if(setup->num_wg == 0) {
+    setup->wg[setup->num_wg++] = 
+      setup_add_nothing_available(_("There is no configuration option available in "
+				    "this user experience level."), x, y);
   }
+
+
+  if(setup->num_wg > MAX_DISPLAY_WIDGETS) {
+    slidmax = setup->num_wg - MAX_DISPLAY_WIDGETS;
+    xitk_show_widget(setup->slider_wg);
+    xitk_enable_widget(setup->slider_wg);
+  }
+  else
+    slidmax = 1;
+
+  xitk_slider_set_max(setup->slider_wg, slidmax);
+  xitk_slider_set_pos(setup->slider_wg, slidmax);
+
 }
 
 /*
@@ -1105,13 +987,6 @@ static void setup_sections (void) {
     cfg_err_result = xine_config_get_next_entry(gGui->xine, &entry);
   }
 
-  setup->sections[setup->num_sections] = strdup(_("Help"));
-  setup->num_sections++;
-  setup->sections[setup->num_sections] = strdup(_("README"));
-  setup->num_sections++;
-  setup->sections[setup->num_sections] = strdup(_("FAQ"));
-  setup->num_sections++;
-
   XITK_WIDGET_INIT(&tab, gGui->imlib_data);
 
   tab.skin_element_name = NULL;
@@ -1134,7 +1009,7 @@ static void setup_sections (void) {
   XUnlockDisplay(gGui->display);
   
   draw_rectangular_outter_box(gGui->imlib_data, bg, 20, 51, 
-			      (WINDOW_WIDTH - 40) - 1, (WINDOW_HEIGHT - (51 + 57)) + 4);
+			      (WINDOW_WIDTH - 40) - 1, (WINDOW_HEIGHT - (51 + 57 + 20)) + 4);
   xitk_window_change_background(gGui->imlib_data, setup->xwin, bg->pixmap,
 				WINDOW_WIDTH, WINDOW_HEIGHT);
   
@@ -1168,66 +1043,9 @@ static void setup_nextprev_wg(xitk_widget_t *w, void *data, int pos) {
   }
 }
 
-/*
- * Read adn store some files.
- */
-static const char **_setup_read_given(char *given, int *ndest) {
-  char            buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-  char            buffer[256], *ln;
-  const langs_t  *l;
-  FILE           *fd;
-  int             first_try = 1;
-  char          **dest = NULL;
-
-  if((given == NULL) || (ndest == NULL))
-    return NULL;
-  
-  *ndest = 0;
-  
-  memset(&buf, 0, sizeof(buf));
-  
-  l = get_lang();
-
- __redo:
-
-  sprintf(buf, "%s/%s%s", XINE_DOCDIR, given, l->ext);
-
-  if((fd = fopen(buf, "r")) != NULL) {
-    
-    while((ln = fgets(buffer, 255, fd)) != NULL) {
-      dest = (char **) realloc(dest, sizeof(char *) * (*ndest + 2));
-      
-      /* label widget hate empty labels */
-      dest[*ndest] = strdup(strlen(ln) ? ln : " ");
-      
-      /* Remove newline */
-      if(dest[*ndest][strlen(dest[*ndest]) - 1] == '\n')
-	dest[*ndest][strlen(dest[*ndest]) - 1] = '\0';
-      
-      (*ndest)++;
-    }
-    
-    dest[*ndest] = NULL;
-    
-    fclose(fd);
-  }
-
-  if(first_try && (dest == NULL)) {
-    l = get_default_lang();
-    first_try--;
-    goto __redo;
-  }
-
-  return (const char **)dest;  
-}
-static void setup_read_config(void) {
-  setup->config_content = _setup_read_given("README.config", &setup->config_lines);
-}
-static void setup_read_readme(void) {
-  setup->readme_content = _setup_read_given("README", &setup->readme_lines);
-}
-static void setup_read_faq(void) {
-  setup->faq_content =  _setup_read_given("FAQ", &setup->faq_lines);
+void setup_reparent(void) {
+  if(setup)
+    reparent_window((xitk_window_get_window(setup->xwin)));
 }
 
 /*
@@ -1237,6 +1055,7 @@ void setup_panel(void) {
   GC                         gc;
   xitk_labelbutton_widget_t  lb;
   xitk_slider_widget_t       sl;
+  xitk_label_widget_t        lbl;
   int                        x, y;
   xitk_widget_t             *w;
 
@@ -1261,18 +1080,14 @@ void setup_panel(void) {
   setup->xwin = xitk_window_create_dialog_window(gGui->imlib_data,
 						 _("xine setup"), 
 						 x, y, WINDOW_WIDTH, WINDOW_HEIGHT);
-
-  /* Read some help files */
-  setup_read_config();
-  setup_read_readme();
-  setup_read_faq();
   
+  set_window_states_start((xitk_window_get_window(setup->xwin)));
+
   XLockDisplay (gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(setup->xwin)), None, None);
   setup->cursor[NORMAL_CURS] = XCreateFontCursor(gGui->display, XC_left_ptr);
   setup->cursor[WAIT_CURS] = XCreateFontCursor(gGui->display, XC_watch);
-  XUnlockDisplay(gGui->display);
   XUnlockDisplay (gGui->display);
 
   setup->widget_list                = xitk_widget_list_new();
@@ -1294,11 +1109,37 @@ void setup_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(setup->widget_list)),
    (setup->slider_wg = xitk_noskin_slider_create(setup->widget_list, &sl,
 						 (WINDOW_WIDTH - 41), 70, 
-						 16, (WINDOW_HEIGHT - 140), XITK_VSLIDER)));
+						 16, (WINDOW_HEIGHT - 160), XITK_VSLIDER)));
 
   setup_sections();
   setup_paint_widgets();
 
+  {
+    char         *label = _("(*)  you need to restart xine for this setting to take effect");
+    xitk_font_t  *fs;
+    int           len;
+    
+    fs = xitk_font_load_font(gGui->display, fontname);
+    xitk_font_set_font(fs, (XITK_WIDGET_LIST_GC(setup->widget_list)));
+    len = xitk_font_get_string_length(fs, (const char *) label);
+    xitk_font_unload_font(fs);
+    
+    XITK_WIDGET_INIT(&lbl, gGui->imlib_data);
+    
+    lbl.window              = xitk_window_get_window(setup->xwin);
+    lbl.gc                  = (XITK_WIDGET_LIST_GC(setup->widget_list));
+    lbl.skin_element_name   = NULL;
+    lbl.label               = label;
+    lbl.callback            = NULL;
+    lbl.userdata            = NULL;
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(setup->widget_list)),
+			     (w = xitk_noskin_label_create(setup->widget_list, &lbl,
+							   (WINDOW_WIDTH - len) >> 1,
+							   (WINDOW_HEIGHT - (57 + 20)) + 1 + 4,
+							   len + 3, 15, fontname)));
+    xitk_enable_and_show_widget(w);
+  }
+
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
 
   x = (WINDOW_WIDTH - (3 * 100))>>2;
@@ -1354,10 +1195,5 @@ void setup_panel(void) {
   setup->running = 1;
   setup_raise_window();
 
-  while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(setup->xwin)))
-    xine_usec_sleep(5000);
-
-  XLockDisplay (gGui->display);
-  XSetInputFocus(gGui->display, xitk_window_get_window(setup->xwin), RevertToParent, CurrentTime);
-  XUnlockDisplay (gGui->display);
+  try_to_set_input_focus(xitk_window_get_window(setup->xwin));
 }
