@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skins.c,v 1.54 2003/07/17 20:46:26 f1rmb Exp $
+ * $Id: skins.c,v 1.67 2004/04/13 19:55:49 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -26,8 +26,9 @@
 
 #include <stdio.h>
 #include <dirent.h>
-#include <sys/stat.h>       
+#include <sys/stat.h>
 #include <errno.h>
+#include <string.h>
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -81,7 +82,7 @@ static skin_downloader_t  *skdloader = NULL;
 #undef SKIN_DEBUG
 
 #define DEFAULT_SKIN        "xinetic"
-#define SKIN_IFACE_VERSION  4
+#define SKIN_IFACE_VERSION  5
 
 extern gGui_t             *gGui;
 
@@ -101,7 +102,6 @@ static void get_available_skins_from(char *path) {
     return;
 
   if((pdir = opendir(path)) != NULL) {
-    struct stat   pstat, sstat;
     char          fullfilename[XITK_PATH_MAX + XITK_NAME_MAX + 1];
     char          skcfgname[XITK_PATH_MAX + XITK_NAME_MAX + 1];
     
@@ -109,9 +109,7 @@ static void get_available_skins_from(char *path) {
       memset(&fullfilename, 0, sizeof(fullfilename));
       sprintf(fullfilename, "%s/%s", path, pdirent->d_name);
       
-      stat(fullfilename, &pstat);
-      
-      if((S_ISDIR(pstat.st_mode))
+      if((is_a_dir(fullfilename))
 	 && (!(strlen(pdirent->d_name) == 1 && pdirent->d_name[0] == '.' )
 	     && !(strlen(pdirent->d_name) == 2 
 		  && (pdirent->d_name[0] == '.' && pdirent->d_name[1] == '.')))) {
@@ -121,7 +119,7 @@ static void get_available_skins_from(char *path) {
 	 */
 	memset(&skcfgname, 0, sizeof(skcfgname));
 	sprintf(skcfgname, "%s/%s", fullfilename, "skinconfig");
-	if(((stat(skcfgname, &sstat)) > -1) && (S_ISREG(sstat.st_mode))) {
+	if(is_a_file(skcfgname)) {
 	  
 	  skins_avail = (skins_locations_t **) realloc(skins_avail, 
 						       (skins_avail_num + 2) * sizeof(skins_locations_t*));
@@ -171,7 +169,7 @@ static void looking_for_available_skins(void) {
  */
 char *skin_get_skindir(void) {
   static char          tmp[2048];
-  xine_cfg_entry_t  entry;
+  xine_cfg_entry_t     entry;
   char                *skin;
   
   memset(&tmp, 0, 2048);
@@ -261,6 +259,7 @@ static int change_skin(skins_locations_t *sk) {
   int                  twice = 0, twice_load = 0;
   int                  ret = 0;
   char                *skin_anim;
+  xitk_skin_config_t  *nskin_config, *oskin_config;
 
   if(!sk)
     return -1;
@@ -269,15 +268,13 @@ static int change_skin(skins_locations_t *sk) {
 
   old_skin = DEFAULT_SKIN;
   
-  xitk_skin_unload_config(gGui->skin_config);
-
-  gGui->skin_config = xitk_skin_init_config();
+  nskin_config = xitk_skin_init_config(gGui->imlib_data);
 
  __reload_skin:
   memset(&buf, 0, sizeof(buf));
   sprintf(buf, "%s/%s", sks->pathname, sks->skin);
 
-  if(!xitk_skin_load_config(gGui->skin_config, buf, "skinconfig")) {
+  if(!xitk_skin_load_config(nskin_config, buf, "skinconfig")) {
     skins_locations_t   *osks;
     
     ret = get_skin_offset(old_skin);
@@ -305,9 +302,9 @@ static int change_skin(skins_locations_t *sk) {
   twice_load = 0;
 
   /* Check skin version */
-  if(xitk_skin_check_version(gGui->skin_config, SKIN_IFACE_VERSION) < 1) {
-    xitk_skin_unload_config(gGui->skin_config);
-    gGui->skin_config = xitk_skin_init_config();
+  if(xitk_skin_check_version(nskin_config, SKIN_IFACE_VERSION) < 1) {
+    xitk_skin_unload_config(nskin_config);
+    nskin_config = xitk_skin_init_config(gGui->imlib_data);
     xine_error(_("Failed to load %s, wrong version. Load fallback skin '%s'.\n"), 
 	       buf, DEFAULT_SKIN);
     ret = get_skin_offset(DEFAULT_SKIN);
@@ -320,10 +317,13 @@ static int change_skin(skins_locations_t *sk) {
     free(gGui->visual_anim.mrls[gGui->visual_anim.num_mrls]);
     gGui->visual_anim.mrls[gGui->visual_anim.num_mrls--] = NULL;
   }
-  if((skin_anim = xitk_skin_get_animation(gGui->skin_config)) != NULL) {
+  if((skin_anim = xitk_skin_get_animation(nskin_config)) != NULL) {
     gGui->visual_anim.mrls[gGui->visual_anim.num_mrls++] = strdup(skin_anim);
   }
   
+  oskin_config = gGui->skin_config;
+  gGui->skin_config = nskin_config;
+
   { /* Now, change skins for each window */
     typedef struct {
       void (*change_skins)(void);
@@ -345,6 +345,8 @@ static int change_skin(skins_locations_t *sk) {
     }
   }
 
+  xitk_skin_unload_config(oskin_config);
+
   return ret;
 }
 
@@ -382,19 +384,38 @@ void select_new_skin(int selected) {
   config_update_num("gui.skin", selected);
 }
 
+/*
+ * Return path of current skin
+ */
+const char *skin_get_current_skin_dir(void) {
+  xine_cfg_entry_t     entry;
+  int                  skin_num = 0;
+  static char          skin_dir[XINE_PATH_MAX + XINE_NAME_MAX];
+  
+  if((skins_avail == NULL) || (skins_avail_num == 0)) {
+    fprintf(stderr, _("No available skin found. Say goodbye.\n"));
+    exit(-1);
+  }
+  
+  memset(&entry, 0, sizeof(xine_cfg_entry_t)); 
+  (void) xine_config_lookup_entry(gGui->xine, "gui.skin", &entry);
+  skin_num = entry.num_value;
+  
+  sprintf(skin_dir, "%s/%s", skins_avail[skin_num]->pathname, skins_avail[skin_num]->skin);
+
+  return (const char *) &skin_dir;
+}
+
 /*
  * Initialize skin support.
  */
-void init_skins_support(void) {
-  skins_locations_t   *sk;
-  char                 buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-  int                  twice = 0, twice_load = 0;
-  int                  skin_num, i;
-  char                *skin_anim;
-    
-  change_config_entry = 0;
+void preinit_skins_support(void) {
+  int  skin_num;
+  int  i;
 
-  gGui->skin_config = xitk_skin_init_config();
+  change_config_entry = 0;
+  
+  gGui->skin_config = xitk_skin_init_config(gGui->imlib_data);
   
   looking_for_available_skins();
   
@@ -418,7 +439,24 @@ void init_skins_support(void) {
 					CONFIG_LEVEL_BEG,
 					skin_change_cb, 
 					CONFIG_NO_DATA);
+}
 
+void init_skins_support(void) {
+  skins_locations_t   *sk;
+  char                 buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+  int                  twice = 0, twice_load = 0;
+  int                  skin_num;
+  char                *skin_anim;
+  xine_cfg_entry_t     entry;
+
+  memset(&entry, 0, sizeof(xine_cfg_entry_t)); 
+  if(xine_config_lookup_entry(gGui->xine, "gui.skin", &entry))
+    skin_num = entry.num_value;
+  else {
+    fprintf(stderr, _("Ooch, gui.skin config entry isn't registered. Say goodbye.\n"));
+    exit(-1);
+  }
+  
   sk = (skin_num < skins_avail_num) ? skins_avail[skin_num] : NULL;
   
   if(!sk) {
@@ -455,7 +493,7 @@ void init_skins_support(void) {
   /* Check skin version */
   if(xitk_skin_check_version(gGui->skin_config, SKIN_IFACE_VERSION) < 1) {
     xitk_skin_unload_config(gGui->skin_config);
-    gGui->skin_config = xitk_skin_init_config();
+    gGui->skin_config = xitk_skin_init_config(gGui->imlib_data);
     fprintf(stderr, _("Failed to load %s skin, wrong version. Load fallback skin '%s'.\n"), 
 	    buf, (skins_avail[(get_skin_offset(DEFAULT_SKIN))]->skin));
     config_update_num("gui.skin", (get_skin_offset(DEFAULT_SKIN)));
@@ -650,7 +688,7 @@ static void download_skin_cancel(xitk_widget_t *w, void *data) {
   xitk_destroy_widgets(skdloader->widget_list);
   xitk_window_destroy_window(gGui->imlib_data, skdloader->xwin);
 
-  skdloader->xwin = None;
+  skdloader->xwin = NULL;
 
   xitk_list_free((XITK_WIDGET_LIST_LIST(skdloader->widget_list)));
   
@@ -694,38 +732,36 @@ static void download_update_blank_preview(void) {
 
 }
 
-static void download_update_preview(void) {
-
+static void redraw_preview(void) {
+  int  x, y;
+  
   download_update_blank_preview();
-
-  if(skdloader->preview_image->image && skdloader->preview_image->image->pixmap) {
-    GC   lgc;
-    int  x, y;
-    
+  
+  y = 30 + ((skdloader->preview_image->image->height - skdloader->preview_height) >> 1);
+  x = 10 + ((skdloader->preview_image->image->width - skdloader->preview_width) >> 1);
+  
+  if(skdloader->preview_image->mask && skdloader->preview_image->mask->pixmap) {
     XLockDisplay (gGui->display);
-    lgc = XCreateGC(gGui->display, (XITK_WIDGET_LIST_WINDOW(skdloader->widget_list)), None, None);
-    XCopyGC(gGui->display, 
-	    (XITK_WIDGET_LIST_GC(skdloader->widget_list)), (1 << GCLastBit) - 1, lgc);
-    XUnlockDisplay (gGui->display);
-    
-    y = 30 + ((skdloader->preview_image->image->height - skdloader->preview_height) >> 1);
-    x = 10 + ((skdloader->preview_image->image->width - skdloader->preview_width) >> 1);
-    
-    if(skdloader->preview_image->mask && skdloader->preview_image->mask->pixmap) {
-      XLockDisplay (gGui->display);
-      XSetClipOrigin(gGui->display, lgc, x, y);
-      XSetClipMask(gGui->display, lgc, skdloader->preview_image->mask->pixmap);
-      XUnlockDisplay (gGui->display);
-    }
-    
-    XLockDisplay (gGui->display);
-    XCopyArea (gGui->display, skdloader->preview_image->image->pixmap, 
-	       (XITK_WIDGET_LIST_WINDOW(skdloader->widget_list)), lgc, 0, 0,
-	       skdloader->preview_width, skdloader->preview_height, x, y);
-    XSync(gGui->display, False);
-    XFreeGC(gGui->display, lgc);
+    XSetClipOrigin(gGui->display, (XITK_WIDGET_LIST_GC(skdloader->widget_list)), x, y);
+    XSetClipMask(gGui->display, (XITK_WIDGET_LIST_GC(skdloader->widget_list)), skdloader->preview_image->mask->pixmap);
     XUnlockDisplay (gGui->display);
   }
+  
+  XLockDisplay (gGui->display);
+  XCopyArea (gGui->display, skdloader->preview_image->image->pixmap, 
+	     (XITK_WIDGET_LIST_WINDOW(skdloader->widget_list)), (XITK_WIDGET_LIST_GC(skdloader->widget_list)), 0, 0,
+	     skdloader->preview_width, skdloader->preview_height, x, y);
+  XSync(gGui->display, False);
+  XUnlockDisplay (gGui->display);
+}
+
+static void download_update_preview(void) {
+
+  if(skdloader->preview_image->image && skdloader->preview_image->image->pixmap)
+    redraw_preview();
+  else
+    download_update_blank_preview();
+
 }
 
 static void download_skin_preview(xitk_widget_t *w, void *data, int selected) {
@@ -810,7 +846,7 @@ static void download_skin_preview(xitk_widget_t *w, void *data, int selected) {
 
 	download_update_preview();
       }
-      
+
       unlink(tmpfile);
     }
   }
@@ -872,22 +908,33 @@ static void download_skin_select(xitk_widget_t *w, void *data) {
 	sprintf(tmpskin, "/tmp/%d%s", (unsigned int)time(NULL), filename);
 	
 	if((fd = fopen(tmpskin, "w+b")) != NULL) {
-	  char   buffer[2048];
-	  int    i, skin_found = -1;
-	  
+	  char      buffer[2048];
+	  char      fskin_path[XITK_PATH_MAX];
+	  int       i, skin_found = -1;
+
 	  fwrite(download.buf, download.size, 1, fd);
 	  fflush(fd);
 	  fclose(fd);
 
 	  memset(&buffer, 0, sizeof(buffer));
 	  
-	  sprintf(buffer, "`which tar`||tar -C %s -xzf %s", skindir, tmpskin);
+	  sprintf(buffer, "which tar > /dev/null 2>&1 && tar -C %s -xzf %s", skindir, tmpskin);
 	  xine_system(0, buffer);
 	  unlink(tmpskin);
-	  
+
 	  memset(&buffer, 0, sizeof(buffer));
 	  snprintf(buffer, ((strlen(filename) + 1) - 7), "%s", filename);
 
+	  memset(&fskin_path, 0, sizeof(fskin_path));
+	  sprintf(fskin_path, "%s/%s/%s", skindir, buffer, "doinst.sh");
+	  if(is_a_file(fskin_path)) {
+	    char doinst[2048];
+
+	    memset(&doinst, 0, sizeof(doinst));
+	    sprintf(doinst, "cd %s/%s && ./doinst.sh", skindir, buffer);
+	    xine_system(0, doinst);
+	  }
+	  
 	  for(i = 0; i < skins_avail_num; i++) {
 	    if((!strcmp(skins_avail[i]->pathname, skindir)) 
 	       && (!strcmp(skins_avail[i]->skin, buffer))) {
@@ -952,52 +999,50 @@ static void download_skin_select(xitk_widget_t *w, void *data) {
 static void download_skin_handle_event(XEvent *event, void *data) {
   
   switch(event->type) {
+
+  case Expose:
+    redraw_preview();
+    break;
     
   case KeyPress: {
-    XKeyEvent      mykeyevent;
-    KeySym         mykey;
-    char           kbuf[256];
-    int            len;
+    KeySym         mkey;
     int            modifier;
     
-    mykeyevent = event->xkey;
-    
     xitk_get_key_modifier(event, &modifier);
+    mkey = xitk_get_key_pressed(event);
     
-    XLockDisplay(gGui->display);
-    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-    XUnlockDisplay(gGui->display);
-    
-    switch(mykey) {
+    switch(mkey) {
       
     case XK_Down:
-    case XK_Next: {
-      xitk_widget_t *w;
-      
-      w = xitk_get_focused_widget(skdloader->widget_list);
-      if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
-	if(mykey == XK_Down)
-	  xitk_browser_step_up(skdloader->browser, NULL);
-	else
-	  xitk_browser_page_up(skdloader->browser, NULL);
+    case XK_Next:
+      {
+	xitk_widget_t *w;
+	
+	w = xitk_get_focused_widget(skdloader->widget_list);
+	if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
+	  if(mkey == XK_Down)
+	    xitk_browser_step_up(skdloader->browser, NULL);
+	  else
+	    xitk_browser_page_up(skdloader->browser, NULL);
+	}
       }
-    }
       break;
       
     case XK_Up:
-    case XK_Prior: {
-      xitk_widget_t *w;
-      
-      w = xitk_get_focused_widget(skdloader->widget_list);
-      if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
-	if(mykey == XK_Up)
-	  xitk_browser_step_down(skdloader->browser, NULL);
-	else
-	  xitk_browser_page_down(skdloader->browser, NULL);
+    case XK_Prior:
+      {
+	xitk_widget_t *w;
+	
+	w = xitk_get_focused_widget(skdloader->widget_list);
+	if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
+	  if(mkey == XK_Up)
+	    xitk_browser_step_down(skdloader->browser, NULL);
+	  else
+	    xitk_browser_page_down(skdloader->browser, NULL);
+	}
       }
-    }
       break;
-     
+      
     case XK_Escape:
       download_skin_cancel(NULL, NULL);
       break;
@@ -1013,6 +1058,7 @@ void download_skin(char *url) {
   xitk_window_t        *xwin;
   xitk_pixmap_t        *bg;
   int                   x, y, w, h, width, height;
+  xitk_widget_t        *widget;
 
   w = 300;
   h = 50;
@@ -1037,7 +1083,7 @@ void download_skin(char *url) {
   layer_above_video(xitk_window_get_window(xwin));
   
   if((slxs = skins_get_slx_entries(url)) != NULL) {
-    char                      *fontname    = "*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
+    char                      *fontname    = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
     char                      *btnfontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
     int                        i;
     xitk_browser_widget_t      br;
@@ -1074,11 +1120,13 @@ void download_skin(char *url) {
 						       _("Choose a skin to download..."),
 						       x, y, WINDOW_WIDTH, WINDOW_HEIGHT);
     
+    set_window_states_start((xitk_window_get_window(skdloader->xwin)));
+
     XLockDisplay(gGui->display);
     skdloader->cursor[NORMAL_CURS] = XCreateFontCursor(gGui->display, XC_left_ptr);
     skdloader->cursor[WAIT_CURS] = XCreateFontCursor(gGui->display, XC_watch);
     gc = XCreateGC(gGui->display, 
-		   (xitk_window_get_window(skdloader->xwin)), None, None);
+    		   (xitk_window_get_window(skdloader->xwin)), None, None);
     XUnlockDisplay(gGui->display);
 
     xitk_window_get_window_size(skdloader->xwin, &width, &height);
@@ -1134,8 +1182,9 @@ void download_skin(char *url) {
 							 WINDOW_WIDTH - (20 + 10) , 20, 12, fontname)));
     
     xitk_browser_update_list(skdloader->browser, 
-    			     (const char *const *)skdloader->entries, skdloader->num, 0);
+    			     (const char *const *)skdloader->entries, NULL, skdloader->num, 0);
     
+    xitk_enable_and_show_widget(skdloader->browser);
     
     draw_rectangular_inner_box(gGui->imlib_data, bg, 8, y - 2, (WINDOW_WIDTH - 20) + 5, (5 * 20) + 10 + 5);
     
@@ -1153,9 +1202,11 @@ void download_skin(char *url) {
     lb.userdata          = NULL;
     lb.skin_element_name = NULL;
     xitk_list_append_content((XITK_WIDGET_LIST_LIST(skdloader->widget_list)), 
-			     xitk_noskin_labelbutton_create(skdloader->widget_list, 
-							    &lb, x, y, 100, 23,
-							    "Black", "Black", "White", btnfontname));
+	     (widget = xitk_noskin_labelbutton_create(skdloader->widget_list, 
+						      &lb, x, y, 100, 23,
+						      "Black", "Black", "White", btnfontname)));
+    xitk_enable_and_show_widget(widget);
+
     x = WINDOW_WIDTH - 115;
     
     lb.button_type       = CLICK_BUTTON;
@@ -1166,10 +1217,11 @@ void download_skin(char *url) {
     lb.userdata          = NULL;
     lb.skin_element_name = NULL;
     xitk_list_append_content((XITK_WIDGET_LIST_LIST(skdloader->widget_list)), 
-			     xitk_noskin_labelbutton_create(skdloader->widget_list, 
-							    &lb, x, y, 100, 23,
-							    "Black", "Black", "White", btnfontname));
-
+	     (widget = xitk_noskin_labelbutton_create(skdloader->widget_list, 
+						      &lb, x, y, 100, 23,
+						      "Black", "Black", "White", btnfontname)));
+    xitk_enable_and_show_widget(widget);
+    
     skdloader->widget_key = xitk_register_event_handler("skdloader", 
 							(xitk_window_get_window(skdloader->xwin)),
 							download_skin_handle_event,
@@ -1187,13 +1239,7 @@ void download_skin(char *url) {
     XUnlockDisplay(gGui->display);
     layer_above_video(xitk_window_get_window(skdloader->xwin));
 
-    while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(skdloader->xwin)))
-      xine_usec_sleep(5000);
-    
-    XLockDisplay (gGui->display);
-    XSetInputFocus(gGui->display, xitk_window_get_window(skdloader->xwin), RevertToParent, CurrentTime);
-    XUnlockDisplay (gGui->display);
-
+    try_to_set_input_focus(xitk_window_get_window(skdloader->xwin));
     download_update_blank_preview();
   }
   else
