@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skins.h,v 1.8 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: skins.h,v 1.10 2004/04/13 19:55:49 f1rmb Exp $
  *
  */
 
@@ -32,8 +32,10 @@ typedef struct {
 
 skins_locations_t **get_available_skins(void);
 int get_available_skins_num(void);
+void preinit_skins_support(void);
 void init_skins_support(void);
 void select_new_skin(int selected);
 void download_skin(char *url);
+const char *skin_get_current_skin_dir(void);
 
 #endif
