@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: stream_infos.c,v 1.31 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: stream_infos.c,v 1.36 2004/04/13 19:55:49 f1rmb Exp $
  *
  *
  */
@@ -269,7 +269,7 @@ static void stream_infos_exit(xitk_widget_t *w, void *data) {
     xitk_destroy_widgets(sinfos->widget_list);
     xitk_window_destroy_window(gGui->imlib_data, sinfos->xwin);
 
-    sinfos->xwin = None;
+    sinfos->xwin = NULL;
     xitk_list_free((XITK_WIDGET_LIST_LIST(sinfos->widget_list)));
     
     XLockDisplay(gGui->display);
@@ -283,6 +283,16 @@ static void stream_infos_exit(xitk_widget_t *w, void *data) {
   }
 }
 
+static void handle_event(XEvent *event, void *data) {
+  switch(event->type) {
+    
+  case KeyPress:
+    if(xitk_get_key_pressed(event) == XK_Escape)
+      stream_infos_exit(NULL, NULL);
+    break;
+  }
+}
+
 int stream_infos_is_visible(void) {
   
   if(sinfos != NULL) {
@@ -303,40 +313,6 @@ int stream_infos_is_running(void) {
   return 0;
 }
 
-void stream_infos_toggle_visibility(xitk_widget_t *w, void *data) {
-  if(sinfos != NULL) {
-    if (sinfos->visible && sinfos->running) {
-      XLockDisplay(gGui->display);
-      if(gGui->use_root_window) {
-	if(xitk_is_window_visible(gGui->display, xitk_window_get_window(sinfos->xwin)))
-	  XIconifyWindow(gGui->display, xitk_window_get_window(sinfos->xwin), gGui->screen);
-	else
-	  XMapWindow(gGui->display, xitk_window_get_window(sinfos->xwin));
-      }
-      else {
-	sinfos->visible = 0;
-	xitk_hide_widgets(sinfos->widget_list);
-	XUnmapWindow (gGui->display, xitk_window_get_window(sinfos->xwin));
-      }
-      XUnlockDisplay(gGui->display);
-    } 
-    else {
-      if(sinfos->running) {
-	sinfos->visible = 1;
-	xitk_show_widgets(sinfos->widget_list);
-	XLockDisplay(gGui->display);
-	XRaiseWindow(gGui->display, xitk_window_get_window(sinfos->xwin)); 
-	XMapWindow(gGui->display, xitk_window_get_window(sinfos->xwin)); 
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, 
-				xitk_window_get_window(sinfos->xwin), gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(xitk_window_get_window(sinfos->xwin));
-      }
-    }
-  }
-}
-
 void stream_infos_toggle_auto_update(void) {
   if(sinfos != NULL) {
 
@@ -354,21 +330,14 @@ void stream_infos_toggle_auto_update(void) {
 }
 
 void stream_infos_raise_window(void) {
-  if(sinfos != NULL) {
-    if(sinfos->xwin) {
-      if(sinfos->visible && sinfos->running) {
-	  XLockDisplay(gGui->display);
-	  XUnmapWindow(gGui->display, xitk_window_get_window(sinfos->xwin));
-	  XRaiseWindow(gGui->display, xitk_window_get_window(sinfos->xwin));
-	  XMapWindow(gGui->display, xitk_window_get_window(sinfos->xwin));
-	  if(!gGui->use_root_window)
-	    XSetTransientForHint (gGui->display, 
-				  xitk_window_get_window(sinfos->xwin), gGui->video_window);
-	  XUnlockDisplay(gGui->display);
-	  layer_above_video(xitk_window_get_window(sinfos->xwin));
-      }
-    }
-  }
+  if(sinfos != NULL)
+    raise_window(xitk_window_get_window(sinfos->xwin), sinfos->visible, sinfos->running);
+}
+
+void stream_infos_toggle_visibility(xitk_widget_t *w, void *data) {
+  if(sinfos != NULL)
+    toggle_window(xitk_window_get_window(sinfos->xwin), sinfos->widget_list, 
+		  &sinfos->visible, sinfos->running);
 }
 
 void stream_infos_end(void) {
@@ -459,6 +428,11 @@ void stream_infos_update_infos(void) {
   }
 }
 
+void stream_infos_reparent(void) {
+  if(sinfos)
+    reparent_window((xitk_window_get_window(sinfos->xwin)));
+}
+
 void stream_infos_panel(void) {
   GC                          gc;
   xitk_labelbutton_widget_t   lb;
@@ -489,6 +463,8 @@ void stream_infos_panel(void) {
   sinfos->xwin = xitk_window_create_dialog_window(gGui->imlib_data, _("Stream Information"), x, y,
 						  WINDOW_WIDTH, WINDOW_HEIGHT);
   
+  set_window_states_start((xitk_window_get_window(sinfos->xwin)));
+
   XLockDisplay (gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(sinfos->xwin)), None, None);
@@ -1072,7 +1048,7 @@ void stream_infos_panel(void) {
 
   sinfos->widget_key = xitk_register_event_handler("sinfos", 
 						   (xitk_window_get_window(sinfos->xwin)),
-						   NULL,
+						   handle_event,
 						   NULL,
 						   NULL,
 						   sinfos->widget_list,
@@ -1084,11 +1060,5 @@ void stream_infos_panel(void) {
   sinfos->running = 1;
   stream_infos_raise_window();
 
-
-  while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(sinfos->xwin)))
-    xine_usec_sleep(5000);
-
-  XLockDisplay (gGui->display);
-  XSetInputFocus(gGui->display, xitk_window_get_window(sinfos->xwin), RevertToParent, CurrentTime);
-  XUnlockDisplay (gGui->display);
+  try_to_set_input_focus(xitk_window_get_window(sinfos->xwin));
 }
