@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: tvset.c,v 1.4 2003/07/25 22:52:57 f1rmb Exp $
+ * $Id: tvset.c,v 1.9 2004/04/13 19:55:49 f1rmb Exp $
  *
  *
  */
@@ -141,7 +141,6 @@ static struct _std_list std_list[] = {
 };
 
 
-
 static void tvset_update(xitk_widget_t *w, void *data) {
   xine_event_t          xine_event;
   xine_set_v4l2_data_t *ev_data;
@@ -184,7 +183,7 @@ static void tvset_exit(xitk_widget_t *w, void *data) {
     xitk_destroy_widgets(tvset->widget_list);
     xitk_window_destroy_window(gGui->imlib_data, tvset->xwin);
 
-    tvset->xwin = None;
+    tvset->xwin = NULL;
     xitk_list_free((XITK_WIDGET_LIST_LIST(tvset->widget_list)));
     
     XLockDisplay(gGui->display);
@@ -212,6 +211,17 @@ static void tvset_exit(xitk_widget_t *w, void *data) {
   }
 }
 
+static void tvset_handle_event(XEvent *event, void *data) {
+  
+  switch(event->type) {
+    
+  case KeyPress:
+    if(xitk_get_key_pressed(event) == XK_Escape)
+      tvset_exit(NULL, NULL);
+    break;
+  }  
+}
+
 int tvset_is_visible(void) {
   
   if(tvset != NULL) {
@@ -232,57 +242,15 @@ int tvset_is_running(void) {
   return 0;
 }
 
-void tvset_toggle_visibility(xitk_widget_t *w, void *data) {
-  if(tvset != NULL) {
-    if (tvset->visible && tvset->running) {
-      XLockDisplay(gGui->display);
-      if(gGui->use_root_window) {
-	if(xitk_is_window_visible(gGui->display, xitk_window_get_window(tvset->xwin)))
-	  XIconifyWindow(gGui->display, xitk_window_get_window(tvset->xwin), gGui->screen);
-	else
-	  XMapWindow(gGui->display, xitk_window_get_window(tvset->xwin));
-      }
-      else {
-	tvset->visible = 0;
-	xitk_hide_widgets(tvset->widget_list);
-	XUnmapWindow (gGui->display, xitk_window_get_window(tvset->xwin));
-      }
-      XUnlockDisplay(gGui->display);
-    } 
-    else {
-      if(tvset->running) {
-	tvset->visible = 1;
-	xitk_show_widgets(tvset->widget_list);
-	XLockDisplay(gGui->display);
-	XRaiseWindow(gGui->display, xitk_window_get_window(tvset->xwin)); 
-	XMapWindow(gGui->display, xitk_window_get_window(tvset->xwin)); 
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, 
-				xitk_window_get_window(tvset->xwin), gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(xitk_window_get_window(tvset->xwin));
-      }
-    }
-  }
+void tvset_raise_window(void) {
+  if(tvset != NULL)
+    raise_window(xitk_window_get_window(tvset->xwin), tvset->visible, tvset->running);
 }
 
-
-void tvset_raise_window(void) {
-  if(tvset != NULL) {
-    if(tvset->xwin) {
-      if(tvset->visible && tvset->running) {
-	  XLockDisplay(gGui->display);
-	  XUnmapWindow(gGui->display, xitk_window_get_window(tvset->xwin));
-	  XRaiseWindow(gGui->display, xitk_window_get_window(tvset->xwin));
-	  XMapWindow(gGui->display, xitk_window_get_window(tvset->xwin));
-	  if(!gGui->use_root_window)
-	    XSetTransientForHint (gGui->display, 
-				  xitk_window_get_window(tvset->xwin), gGui->video_window);
-	  XUnlockDisplay(gGui->display);
-	  layer_above_video(xitk_window_get_window(tvset->xwin));
-      }
-    }
-  }
+void tvset_toggle_visibility(xitk_widget_t *w, void *data) {
+  if(tvset != NULL)
+    toggle_window(xitk_window_get_window(tvset->xwin), tvset->widget_list,
+		  &tvset->visible, tvset->running);
 }
 
 void tvset_end(void) {
@@ -322,6 +290,11 @@ static void system_combo_select(xitk_widget_t *w, void *data, int select) {
 }
 
 
+void tvset_reparent(void) {
+  if(tvset)
+    reparent_window((xitk_window_get_window(tvset->xwin)));
+}
+
 void tvset_panel(void) {
   GC                          gc;
   xitk_labelbutton_widget_t   lb;
@@ -357,6 +330,8 @@ void tvset_panel(void) {
 						 _("TV/Analog Video parameters"), x, y,
 						 WINDOW_WIDTH, WINDOW_HEIGHT);
   
+  set_window_states_start((xitk_window_get_window(tvset->xwin)));
+
   XLockDisplay (gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(tvset->xwin)), None, None);
@@ -514,7 +489,6 @@ void tvset_panel(void) {
   w = 130;
 
 
-
   x = 5;
   y += 45;
   draw_outter_frame(gGui->imlib_data, bg, _("MPEG2"), btnfontname, 
@@ -562,22 +536,16 @@ void tvset_panel(void) {
   xitk_image_destroy_xitk_pixmap(bg);
 
   tvset->widget_key = xitk_register_event_handler("tvset", 
-						   (xitk_window_get_window(tvset->xwin)),
-						   NULL,
-						   NULL,
-						   NULL,
-						   tvset->widget_list,
-						   NULL);
+						  (xitk_window_get_window(tvset->xwin)),
+						  tvset_handle_event,
+						  NULL,
+						  NULL,
+						  tvset->widget_list,
+						  NULL);
   
   tvset->visible = 1;
   tvset->running = 1;
   tvset_raise_window();
 
-
-  while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(tvset->xwin)))
-    xine_usec_sleep(5000);
-
-  XLockDisplay (gGui->display);
-  XSetInputFocus(gGui->display, xitk_window_get_window(tvset->xwin), RevertToParent, CurrentTime);
-  XUnlockDisplay (gGui->display);
+  try_to_set_input_focus(xitk_window_get_window(tvset->xwin));
 }
