@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: viewlog.c,v 1.39 2003/07/14 22:44:10 f1rmb Exp $
+ * $Id: viewlog.c,v 1.46 2004/04/13 19:55:49 f1rmb Exp $
  *
  *
  */
@@ -93,7 +93,7 @@ void viewlog_exit(xitk_widget_t *w, void *data) {
     xitk_destroy_widgets(viewlog->widget_list);
     xitk_window_destroy_window(gGui->imlib_data, viewlog->xwin);
     
-    viewlog->xwin = None;
+    viewlog->xwin = NULL;
     xitk_list_free(XITK_WIDGET_LIST_LIST(viewlog->widget_list));
     
     XLockDisplay(gGui->display);
@@ -137,70 +137,16 @@ int viewlog_is_visible(void) {
  * Raise viewlog->xwin
  */
 void viewlog_raise_window(void) {
-  
-  if(viewlog != NULL) {
-    if(viewlog->xwin) {
-      if(viewlog->visible && viewlog->running) {
-	XLockDisplay(gGui->display);
-	XUnmapWindow(gGui->display, xitk_window_get_window(viewlog->xwin));
-	XRaiseWindow(gGui->display, xitk_window_get_window(viewlog->xwin));
-	XMapWindow(gGui->display, xitk_window_get_window(viewlog->xwin));
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, 
-				xitk_window_get_window(viewlog->xwin), gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(xitk_window_get_window(viewlog->xwin));
-      }
-    }
-  }
+  if(viewlog != NULL)
+    raise_window(xitk_window_get_window(viewlog->xwin), viewlog->visible, viewlog->running);
 }
 /*
  * Hide/show the viewlog window.
  */
 void viewlog_toggle_visibility (xitk_widget_t *w, void *data) {
-  
-  if(viewlog != NULL) {
-    if (viewlog->visible && viewlog->running) {
-      XLockDisplay(gGui->display);
-      if(gGui->use_root_window) {
-	if(xitk_is_window_visible(gGui->display, xitk_window_get_window(viewlog->xwin)))
-	  XIconifyWindow(gGui->display, xitk_window_get_window(viewlog->xwin), gGui->screen);
-	else
-	  XMapWindow(gGui->display, xitk_window_get_window(viewlog->xwin));
-      }
-      else {
-	viewlog->visible = 0;
-	xitk_hide_widgets(viewlog->widget_list);
-	XUnmapWindow (gGui->display, xitk_window_get_window(viewlog->xwin));
-      }
-      XUnlockDisplay(gGui->display);
-    } 
-    else {
-      if(viewlog->running) {
-	viewlog->visible = 1;
-	xitk_show_widgets(viewlog->widget_list);
-	XLockDisplay(gGui->display);
-	XRaiseWindow(gGui->display, xitk_window_get_window(viewlog->xwin)); 
-	XMapWindow(gGui->display, xitk_window_get_window(viewlog->xwin)); 
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, 
-				xitk_window_get_window(viewlog->xwin), gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(xitk_window_get_window(viewlog->xwin));
-      }
-    }
-  }
-}
-
-/*
- * Handle X events here.
- */
-static void viewlog_handle_event(XEvent *event, void *data) {
-
-  /*
-  switch(event->type) {
-  }
-  */
+  if(viewlog != NULL)
+    toggle_window(xitk_window_get_window(viewlog->xwin), viewlog->widget_list, 
+		  &viewlog->visible, viewlog->running);
 }
 
 /*
@@ -257,8 +203,6 @@ static void viewlog_change_section(xitk_widget_t *wx, void *data, int section) {
 
       memset(&buf, 0, sizeof(buf));
       
-      //      printf("handling line: %d '%s'\n", i, log[i]);
-      
       p = &log[i][0];
       
       if(strlen(log[i]) > 0) {
@@ -280,13 +224,11 @@ static void viewlog_change_section(xitk_widget_t *wx, void *data, int section) {
 	      viewlog->log = (const char **) realloc(viewlog->log, sizeof(char **) * ((j + 1) + 1));
 	      viewlog->log[j++] = strdup(buf);
 	      viewlog->real_num_entries++;
-	      // printf("added line '%s'\n", viewlog->log[j-1]);
 	    }
 	    memset(&buf, 0, sizeof(buf));
 	    break;
 	    
 	  default:
-	    // printf("- %c", *p);
 	    sprintf(buf, "%s%c", buf, *p);
 	    break;
 	  }
@@ -309,14 +251,9 @@ static void viewlog_change_section(xitk_widget_t *wx, void *data, int section) {
     }
     
     /* I like null terminated arrays ;-) */
-    //    printf("viewlog->log[%d] = NULL\n", j);
     viewlog->log[j]      = NULL;
     viewlog->log_entries = j;
     
-    /*
-    for(i = 0; i < j; i++)
-      printf("line %d '%s'\n", i, viewlog->log[i]);
-    */
   }
   
 #if DEBUG_VIEWLOG
@@ -326,8 +263,21 @@ static void viewlog_change_section(xitk_widget_t *wx, void *data, int section) {
 			  _("There is no log entry for logging section '%s'.\n"), 
 			  xitk_tabs_get_current_tab_selected(viewlog->tabs));
 #endif
+
+  if(gGui->verbosity) {
+    const char   *const *log_sections = xine_get_log_names(gGui->xine);
+
+    printf("\nLOG SECTION [%s]\n", log_sections[section]);
+    i = 0;
+    while(viewlog->log[i]) {
+      if((strlen(viewlog->log[i]) > 1) || (viewlog->log[i][0] != ' '))
+	printf(" '%s'\n", viewlog->log[i]);
+      i++;
+    }
+    printf("-----------------------------------\n\n");
+  }
   
-  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, viewlog->real_num_entries, 0);
+  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, NULL, viewlog->real_num_entries, 0);
   
   viewlog_clear_tab();
   viewlog_paint_widgets();
@@ -396,11 +346,26 @@ static void viewlog_end(xitk_widget_t *w, void *data) {
   viewlog_exit(NULL, NULL);
 }
 
+static void viewlog_handle_event(XEvent *event, void *data) {
+  switch(event->type) {
+    
+  case KeyPress:
+    if(xitk_get_key_pressed(event) == XK_Escape)
+      viewlog_exit(NULL, NULL);
+    break;
+    
+  }
+}
+
+void viewlog_reparent(void) {
+  if(viewlog)
+    reparent_window((xitk_window_get_window(viewlog->xwin)));
+}
 
 /*
  * Create viewlog window
  */
-void viewlog_window(void) {
+void viewlog_panel(void) {
   GC                         gc;
   xitk_labelbutton_widget_t  lb;
   xitk_browser_widget_t      br;
@@ -430,6 +395,8 @@ void viewlog_window(void) {
 						   _("xine log viewer"), 
 						   x, y, WINDOW_WIDTH, WINDOW_HEIGHT);
   
+  set_window_states_start((xitk_window_get_window(viewlog->xwin)));
+
   XLockDisplay (gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(viewlog->xwin)), None, None);
@@ -465,7 +432,7 @@ void viewlog_window(void) {
   xitk_enable_and_show_widget(viewlog->browser_widget);
 
   xitk_browser_set_alignment(viewlog->browser_widget, ALIGN_LEFT);
-  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, viewlog->real_num_entries, 0);
+  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, NULL, viewlog->real_num_entries, 0);
   
   viewlog_paint_widgets();
   
@@ -514,11 +481,5 @@ void viewlog_window(void) {
   viewlog->running = 1;
   viewlog_raise_window();
   
-
-  while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(viewlog->xwin)))
-    xine_usec_sleep(5000);
-
-  XLockDisplay (gGui->display);
-  XSetInputFocus(gGui->display, xitk_window_get_window(viewlog->xwin), RevertToParent, CurrentTime);
-  XUnlockDisplay (gGui->display);
+  try_to_set_input_focus(xitk_window_get_window(viewlog->xwin));
 }
