@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: viewlog.h,v 1.8 2003/07/14 22:44:10 f1rmb Exp $
+ * $Id: viewlog.h,v 1.11 2004/04/13 19:55:49 f1rmb Exp $
  *
  */
 
@@ -30,11 +30,12 @@
 
 #include "xitk.h"
 
-void viewlog_window(void);
+void viewlog_panel(void);
 void viewlog_exit(xitk_widget_t *, void *);
 int viewlog_is_visible(void);
 int viewlog_is_running(void);
 void viewlog_toggle_visibility(xitk_widget_t *, void *);
 void viewlog_raise_window(void);
+void viewlog_reparent(void);
 
 #endif
