@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: _config.h,v 1.9 2003/07/14 22:44:10 f1rmb Exp $
+ * $Id: _config.h,v 1.13 2004/04/13 19:55:49 f1rmb Exp $
  *
  *
  */
@@ -36,6 +36,7 @@ typedef struct {
   struct {
     char           *fallback;
     char           *system;
+    int             xmb;
   } fonts;
 
   struct {
@@ -56,17 +57,25 @@ typedef struct {
   struct {
     int             shm;
     int             oldbarstyle;
+    int             checkstyle;
   } features;
 
+  struct {
+    int             shortcuts;
+  } menus;
+
 } xitk_config_t;
 
 
 xitk_config_t *xitk_config_init(void);
 void xitk_config_deinit(xitk_config_t *xtcf);
 int xitk_config_get_barstyle_feature(xitk_config_t *xtcf);
+int xitk_config_get_checkstyle_feature(xitk_config_t *xtcf);
 int xitk_config_get_shm_feature(xitk_config_t *xtcf);
 char *xitk_config_get_system_font(xitk_config_t *xtcf);
 char *xitk_config_get_default_font(xitk_config_t *xtcf);
+int xitk_config_get_xmb_enability(xitk_config_t *xtcf);
+void xitk_config_set_xmb_enability(xitk_config_t *xtcf, int value);
 int xitk_config_get_black_color(xitk_config_t *xtcf);
 int xitk_config_get_white_color(xitk_config_t *xtcf);
 int xitk_config_get_background_color(xitk_config_t *xtcf);
@@ -76,5 +85,6 @@ unsigned long xitk_config_get_timer_label_animation(xitk_config_t *xtcf);
 long xitk_config_get_timer_dbl_click(xitk_config_t *xtcf);
 unsigned long xitk_config_get_warning_foreground(xitk_config_t *xtcf);
 unsigned long xitk_config_get_warning_background(xitk_config_t *xtcf);
+int xitk_config_get_menu_shortcuts_enability(xitk_config_t *xtcf);
 
 #endif
