@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: browser.c,v 1.55 2003/07/14 22:44:10 f1rmb Exp $
+ * $Id: browser.c,v 1.62 2004/04/13 19:55:49 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -30,12 +30,6 @@
 
 #include "_xitk.h"
 
-typedef struct {
-  xitk_widget_t    *itemlist;
-  int               sel;
-} btnlist_t;
-
-
 #define WBUP    0  /*  Position of button up in item_tree  */
 #define WSLID   1  /*  Position of slider in item_tree  */
 #define WBDN    2  /*  Position of button down in item_tree  */
@@ -149,8 +143,13 @@ static void notify_destroy(xitk_widget_t *w) {
 
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
            (w->type & WIDGET_GROUP_WIDGET))) {
+    int  i;
+
     private_data = (browser_private_data_t *) w->private_data;
     
+    for(i = WBSTART; i < private_data->max_length + WBSTART; i++)
+      XITK_FREE(private_data->bt[i]);
+    
     XITK_FREE(private_data->skin_element_name);
     XITK_FREE(private_data);
   }
@@ -318,8 +317,14 @@ void xitk_browser_rebuild_browser(xitk_widget_t *w, int start) {
 
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
+    char         *max_shortcut = NULL;
+    int           item_width;
+    char         *label_font;
+    xitk_font_t  *fs = NULL;
     
     private_data = (browser_private_data_t *) w->private_data;
+    item_width   = xitk_get_widget_width(private_data->item_tree[WBSTART]);
+    label_font   = xitk_labelbutton_get_fontname(private_data->item_tree[WBSTART]);
     
     xitk_browser_release_all_buttons(w);
 
@@ -344,27 +349,55 @@ void xitk_browser_rebuild_browser(xitk_widget_t *w, int start) {
 
     private_data->labels_offset = 0;
 
-    for(i = 0; i < private_data->max_length; i++) {
-      xitk_labelbutton_set_label_offset(private_data->item_tree[i + WBSTART], 0);
+    if(label_font) {
+      fs = xitk_font_load_font(private_data->imlibdata->x.disp, label_font);
+      xitk_font_set_font(fs, private_data->parent_wlist->gc);
     }
+
+    if(private_data->shortcuts_content) {
+      int maxlen = 0;
+      
+      for(i = 0; i < private_data->max_length; i++) {
+	char *nshortcut;
+	
+	if((private_data->shortcuts_content[private_data->current_start + i]) && 
+	   (nshortcut = private_data->shortcuts_content[private_data->current_start + i])) {
+	  int nlen = strlen(nshortcut);
+	  
+	  if(nlen > maxlen) {
+	    maxlen = nlen;
+	    max_shortcut = nshortcut;
+	  }
+	}
+      }
+    }
+    
+    for(i = 0; i < private_data->max_length; i++)
+      xitk_labelbutton_set_label_offset(private_data->item_tree[i + WBSTART], 0);
     
     for(i = 0; i < private_data->max_length; i++) {
       if (((private_data->current_start + i) < private_data->list_length) 
 	  && (private_data->content[private_data->current_start + i] != NULL)) {
-	int item_width = xitk_get_widget_width(private_data->item_tree[WBSTART]);
-	char *label_font = xitk_labelbutton_get_fontname(private_data->item_tree[WBSTART]);
 	
-	xitk_labelbutton_change_label(private_data->item_tree[i + WBSTART], 
+	xitk_labelbutton_change_label(private_data->item_tree[i + WBSTART],
 				      private_data->content[private_data->current_start + i]);
+
+	if(private_data->shortcuts_content) {
+	  if(private_data->shortcuts_content[private_data->current_start + i])
+	    xitk_labelbutton_change_shortcut_label(private_data->item_tree[i + WBSTART],
+						   private_data->shortcuts_content[private_data->current_start + i], 
+						   item_width - (xitk_font_get_string_length(fs, private_data->shortcuts_content[private_data->current_start + i]) + 10),
+						   NULL);
+	  else
+	    xitk_labelbutton_change_shortcut_label(private_data->item_tree[i + WBSTART], "", -1, NULL);
+	}
 	
-	if(label_font) {
-	  xitk_font_t *fs = NULL;
-	  int          label_width;
+	if(fs) {
+	  int  label_width;
 	  
-	  fs = xitk_font_load_font(private_data->imlibdata->x.disp, label_font);
-	  xitk_font_set_font(fs, private_data->parent_wlist->gc);
-	  label_width = xitk_font_get_string_length(fs, private_data->content[private_data->current_start + i]);
-	  xitk_font_unload_font(fs);
+	  label_width = (private_data->content[private_data->current_start + i] && 
+			 strlen(private_data->content[private_data->current_start + i])) ? 
+	    xitk_font_get_string_length(fs, private_data->content[private_data->current_start + i]) : 0;
 	  
 	  if(label_width > (item_width - 2)) {
 	    int diff_width = (label_width - (item_width - 2));
@@ -375,14 +408,19 @@ void xitk_browser_rebuild_browser(xitk_widget_t *w, int start) {
 	      private_data->labels_offset = diff_width;
 	    
 	  }
-	  XITK_FREE(label_font);
 	}
       }
       else {
 	xitk_labelbutton_change_label(private_data->item_tree[i+WBSTART], "");
+	xitk_labelbutton_change_shortcut_label(private_data->item_tree[i + WBSTART], "", -1, NULL);
       }
     }
 
+    XITK_FREE(label_font);
+
+    if(fs)
+      xitk_font_unload_font(fs);
+
     if(private_data->need_h_slider) {
       int align = xitk_labelbutton_get_alignment(private_data->item_tree[WBSTART]);
 
@@ -434,7 +472,7 @@ void xitk_browser_rebuild_browser(xitk_widget_t *w, int start) {
 /**
  * Update the list, and rebuild button list
  */
-void xitk_browser_update_list(xitk_widget_t *w, const char *const *list, int len, int start) {
+void xitk_browser_update_list(xitk_widget_t *w, const char *const *list, const char *const *shortcut, int len, int start) {
   browser_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
@@ -442,6 +480,7 @@ void xitk_browser_update_list(xitk_widget_t *w, const char *const *list, int len
     
     private_data = (browser_private_data_t *) w->private_data;
     private_data->content = (char **)list;
+    private_data->shortcuts_content = (char **)shortcut;
     private_data->list_length = len;
     xitk_browser_rebuild_browser(w, start);
   }
@@ -491,11 +530,15 @@ static void browser_hslidmove(xitk_widget_t *w, void *data, int pos) {
 	  
 	  xitk_labelbutton_change_label(private_data->item_tree[i + WBSTART], 
 					private_data->content[private_data->current_start + i]);
+	  if(private_data->shortcuts_content)
+	    xitk_labelbutton_change_shortcut_label(private_data->item_tree[i + WBSTART], 
+						   private_data->shortcuts_content[private_data->current_start + i], -1, NULL);
 	  
 	}
-	else
+	else {
 	  xitk_labelbutton_change_label(private_data->item_tree[i+WBSTART], "");
-	
+	  xitk_labelbutton_change_shortcut_label(private_data->item_tree[i + WBSTART], "", -1, NULL);
+	}
       }
     }
     else
@@ -715,6 +758,8 @@ static void browser_select(xitk_widget_t *w, void *data, int state) {
 	  
 	  /* Ok, double click occur, call cb */
 	  if(click_diff < private_data->dbl_click_time) {
+	    private_data->click_time.tv_sec -= (xitk_get_timer_dbl_click() / 1000.0);
+	    private_data->click_time.tv_usec -= (xitk_get_timer_dbl_click() * 1000.0);
 	    if(private_data->dbl_click_callback)
 	      cb2 = 1;
 	  }
@@ -757,6 +802,8 @@ static void browser_select(xitk_widget_t *w, void *data, int state) {
 	
 	/* Ok, double click occur, call cb */
 	if(click_diff < private_data->dbl_click_time) {
+	  private_data->click_time.tv_sec -= (xitk_get_timer_dbl_click() / 1000.0);
+	  private_data->click_time.tv_usec -= (xitk_get_timer_dbl_click() * 1000.0);
 	  if(private_data->dbl_click_callback)
 	    private_data->dbl_click_callback(((btnlist_t*)data)->itemlist, 
 					     private_data->userdata, cur_clicked);
@@ -925,10 +972,12 @@ static xitk_widget_t *_xitk_browser_create(xitk_widget_list_t *wl,
   private_data->skin_element_name    = (skin_element_name == NULL) ? NULL : strdup(br->browser.skin_element_name);
   private_data->jumped               = -1;
   private_data->content              = (char **)br->browser.entries;
+  private_data->shortcuts_content    = NULL;
   private_data->list_length          = br->browser.num_entries;
   private_data->max_length           = br->browser.max_displayed_entries;
 
   private_data->dbl_click_time       = xitk_get_timer_dbl_click();
+  gettimeofday(&private_data->click_time, 0);
 
   if(br->dbl_click_callback)
     private_data->dbl_click_callback = br->dbl_click_callback;
@@ -996,22 +1045,21 @@ xitk_widget_t *xitk_browser_create(xitk_widget_list_t *wl,
   {
 
     int x, y, i;
-    btnlist_t *bt;
     
     x = xitk_skin_get_coord_x(skonfig, br->browser.skin_element_name);
     y = xitk_skin_get_coord_y(skonfig, br->browser.skin_element_name);
     
     for(i = WBSTART; i < (EXTRA_BTNS + WBSTART); i++) {
       
-      bt = (btnlist_t *) xitk_xmalloc(sizeof(btnlist_t));
-      bt->itemlist = mywidget;
-      bt->sel = i;
+      private_data->bt[i]           = (btnlist_t *) xitk_xmalloc(sizeof(btnlist_t));
+      private_data->bt[i]->itemlist = mywidget;
+      private_data->bt[i]->sel      = i;
       
       lb.button_type       = RADIO_BUTTON;
       lb.label             = "";
       lb.callback          = NULL;
       lb.state_callback    = browser_select;
-      lb.userdata          = (void *)(bt);
+      lb.userdata          = (void *)(private_data->bt[i]);
       lb.skin_element_name = br->browser.skin_element_name;
       xitk_list_append_content(br->parent_wlist->l, 
 			       (private_data->item_tree[i] = 
@@ -1121,21 +1169,20 @@ xitk_widget_t *xitk_noskin_browser_create(xitk_widget_list_t *wl,
 
   {
     int            ix = x, iy = y, i;
-    btnlist_t     *bt;
     xitk_image_t  *wimage;
 
     for(i = WBSTART; i < (br->browser.max_displayed_entries+WBSTART); i++) {
 
-      bt = (btnlist_t *) xitk_xmalloc(sizeof(btnlist_t));
-      bt->itemlist = mywidget;
-      bt->sel = i;
-      
+      private_data->bt[i]           = (btnlist_t *) xitk_xmalloc(sizeof(btnlist_t));
+      private_data->bt[i]->itemlist = mywidget;
+      private_data->bt[i]->sel      = i;
+
       lb.button_type       = RADIO_BUTTON;
       lb.label             = "";
       lb.align             = ALIGN_LEFT;
       lb.callback          = NULL;
       lb.state_callback    = browser_select;
-      lb.userdata          = (void *)(bt);
+      lb.userdata          = (void *)(private_data->bt[i]);
       lb.skin_element_name = NULL;
       xitk_list_append_content(br->parent_wlist->l, 
 			       (private_data->item_tree[i] = 
