@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: combo.c,v 1.34 2003/07/14 22:44:10 f1rmb Exp $
+ * $Id: combo.c,v 1.39 2004/04/13 19:55:49 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -36,12 +36,10 @@ static void _combo_rollunroll(xitk_widget_t *w, void *data, int state);
  *
  */
 static void enability(xitk_widget_t *w) {
-  combo_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
 
     if(w->enable == WIDGET_ENABLE) {
       xitk_enable_widget(private_data->label_widget);
@@ -63,12 +61,10 @@ static void enability(xitk_widget_t *w) {
 }
 
 static void notify_destroy(xitk_widget_t *w) {
-  combo_private_data_t *private_data;
-  
+
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
     
     if(private_data->visible)
       _combo_rollunroll(private_data->button_widget, (void *)w, 0);
@@ -83,10 +79,10 @@ static void notify_destroy(xitk_widget_t *w) {
     XFreeGC(private_data->imlibdata->x.disp, private_data->widget_list->gc);
     XUNLOCK(private_data->imlibdata->x.disp);
 
-    XITK_FREE(private_data->widget_list);
+    free(private_data->widget_list);
 
     XITK_FREE(private_data->skin_element_name);
-    XITK_FREE(private_data);
+    free(private_data);
   }
 }
 
@@ -94,12 +90,10 @@ static void notify_destroy(xitk_widget_t *w) {
  *
  */
 static void paint(xitk_widget_t *w) {
-  combo_private_data_t *private_data;
 
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
 
     if(private_data->visible == 1 && (w->visible < 1)) {
       xitk_checkbox_set_state(private_data->button_widget, 0);
@@ -160,7 +154,7 @@ static void combo_select(xitk_widget_t *w, void *data, int selected) {
     
     xitk_browser_release_all_buttons(private_data->browser_widget);
     xitk_browser_update_list(private_data->browser_widget, 
-			     (const char* const*)private_data->entries, 
+			     (const char* const*)private_data->entries, NULL,
 			     private_data->num_entries, 0);
     
     xitk_checkbox_set_state(private_data->button_widget, 0);
@@ -199,11 +193,10 @@ static void _combo_handle_event(XEvent *event, void *data) {
  *
  */
 static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
-  combo_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
 
     if(private_data->skin_element_name) {
       int x, y;
@@ -378,12 +371,10 @@ void xitk_combo_rollunroll(xitk_widget_t *w) {
    *
    */
 void xitk_combo_set_select(xitk_widget_t *w, int select) {
-  combo_private_data_t *private_data;
 
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    
-    private_data = (combo_private_data_t *)w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *)w->private_data;
     
     if(private_data->entries && private_data->entries[select]) {
       private_data->selected = select;
@@ -397,15 +388,13 @@ void xitk_combo_set_select(xitk_widget_t *w, int select) {
  *
  */
 void xitk_combo_update_pos(xitk_widget_t *w) {
-  combo_private_data_t  *private_data;
-  int                    xx = 0, yy = 0;
-  window_info_t          wi;
-  XSizeHints             hint;
 
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    
-    private_data = (combo_private_data_t *)w->private_data;
+    combo_private_data_t  *private_data = (combo_private_data_t *)w->private_data;
+    int                    xx = 0, yy = 0;
+    window_info_t          wi;
+    XSizeHints             hint;
     
     if(private_data->visible) {
       if((xitk_get_window_info(*(private_data->parent_wkey), &wi))) {
@@ -456,11 +445,11 @@ void xitk_combo_update_pos(xitk_widget_t *w) {
  *
  */
 int xitk_combo_get_current_selected(xitk_widget_t *w) {
-  combo_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
+
     return private_data->selected;
   }
 
@@ -471,11 +460,10 @@ int xitk_combo_get_current_selected(xitk_widget_t *w) {
  * 
  */
 char *xitk_combo_get_current_entry_selected(xitk_widget_t *w) {
-  combo_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
 
     if(private_data->entries && private_data->selected >= 0)
       return(private_data->entries[private_data->selected]);
@@ -488,22 +476,36 @@ char *xitk_combo_get_current_entry_selected(xitk_widget_t *w) {
  *
  */
 void xitk_combo_update_list(xitk_widget_t *w, char **list, int len) {
-  combo_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
- 
-    private_data              = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
+
     private_data->entries     = list;
     private_data->num_entries = len;
     private_data->selected    = -1;
     
     xitk_browser_update_list(private_data->browser_widget, 
-			     (const char* const*)private_data->entries, 
+			     (const char* const*)private_data->entries, NULL,
 			     private_data->num_entries, 0);
   }
 }
 
+/*
+ *
+ */
+xitk_widget_t *xitk_combo_get_label_widget(xitk_widget_t *w) {
+
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
+    
+    return private_data->label_widget;
+  }
+
+  return NULL;
+}
+
 /*
  *
  */
@@ -612,7 +614,7 @@ static xitk_widget_t *_xitk_combo_create(xitk_widget_list_t *wl,
   private_data->browser_widget->type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
   
   xitk_browser_update_list(private_data->browser_widget, 
-			   (const char* const*)private_data->entries, 
+			   (const char* const*)private_data->entries, NULL,
 			   private_data->num_entries, 0);
   
   private_data->widget_key = 
@@ -687,8 +689,6 @@ xitk_widget_t *xitk_combo_create(xitk_widget_list_t *wl,
     *lw = private_data->label_widget;
   if(bw)
     *bw = private_data->button_widget;
-
-  
   
   {
     int x, y;
@@ -775,13 +775,13 @@ xitk_widget_t *xitk_noskin_combo_create(xitk_widget_list_t *wl,
     {
       xitk_image_t *wimage = xitk_get_widget_foreground_skin(private_data->label_widget);
       
-      if(wimage) {
+      if(wimage)
 	draw_rectangular_inner_box(c->imlibdata, wimage->image, 0, 0, wimage->width - 1, wimage->height - 1);
-      }
-      
+
       wimage = xitk_get_widget_foreground_skin(private_data->button_widget);
       
       if(wimage) {
+	draw_bevel_three_state(c->imlibdata, wimage);
 	draw_arrow_down(c->imlibdata, wimage);
       }
       
