@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: filebrowser.c,v 1.46 2003/07/14 22:44:10 f1rmb Exp $
+ * $Id: filebrowser.c,v 1.49 2004/04/13 19:55:49 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -740,7 +740,7 @@ static void load_files(xitk_widget_t *w, void *data) {
     private_data->dir_contents_num = 0;
   
   xitk_browser_update_list(private_data->fb_list, 
-			   (const char* const*)private_data->fc->dir_disp_contents, 
+			   (const char* const*)private_data->fc->dir_disp_contents, NULL,
 			   private_data->dir_contents_num, 0);
   
   update_current_dir(private_data);
@@ -1098,8 +1098,8 @@ xitk_widget_t *xitk_filebrowser_create(xitk_widget_list_t *wl,
 		   fb->imlibdata->x.visual,
 		   CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect, &attr);
   
-  XSetStandardProperties(fb->imlibdata->x.disp, private_data->window, title, title,
-			 None, NULL, 0, &hint);
+  XmbSetWMProperties(fb->imlibdata->x.disp, private_data->window, title, title,
+                     NULL, 0, &hint, NULL, NULL);
 
   XSelectInput(fb->imlibdata->x.disp, private_data->window, INPUT_MOTION | KeymapStateMask);
   
@@ -1269,7 +1269,7 @@ xitk_widget_t *xitk_filebrowser_create(xitk_widget_list_t *wl,
   load_files(NULL, (void *)private_data);
   
   xitk_browser_update_list(private_data->fb_list, 
-			   (const char* const*)private_data->fc->dir_disp_contents,
+			   (const char* const*)private_data->fc->dir_disp_contents, NULL,
 			   private_data->dir_contents_num, 0);
 
   XLOCK(fb->imlibdata->x.disp);
@@ -1391,8 +1391,8 @@ xitk_widget_t *xitk_noskin_filebrowser_create(xitk_widget_list_t *wl,
 		   fb->imlibdata->x.visual,
 		   CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect, &attr);
   
-  XSetStandardProperties(fb->imlibdata->x.disp, private_data->window, title, title,
-			 None, NULL, 0, &hint);
+  XmbSetWMProperties(fb->imlibdata->x.disp, private_data->window, title, title,
+                     &hint, NULL, NULL);
 
   XSelectInput(fb->imlibdata->x.disp, private_data->window, INPUT_MOTION | KeymapStateMask);
   
@@ -1562,7 +1562,7 @@ xitk_widget_t *xitk_noskin_filebrowser_create(xitk_widget_list_t *wl,
   load_files(NULL, (void *)private_data);
   
   xitk_browser_update_list(private_data->fb_list, 
-			   (const char* const*)private_data->fc->dir_disp_contents,
+			   (const char* const*)private_data->fc->dir_disp_contents, NULL,
 			   private_data->dir_contents_num, 0);
 
   XLOCK(fb->imlibdata->x.disp);
