@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: image.c,v 1.51 2003/07/18 21:34:40 f1rmb Exp $
+ * $Id: image.c,v 1.57 2004/04/13 19:55:49 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -25,7 +25,6 @@
 #endif
 
 #include <stdio.h>
-#include <assert.h>
 #include <stdarg.h>
 #include <X11/Xlib.h>
 
@@ -38,7 +37,7 @@ unsigned int xitk_get_pixel_color_from_rgb(ImlibData *im, int r, int g, int b) {
   XColor       xcolor;
   unsigned int pixcol;
 
-  assert(im);
+  ABORT_IF_NULL(im);
   
   xcolor.flags = DoRed | DoBlue | DoGreen;
   
@@ -106,7 +105,8 @@ unsigned int xitk_get_pixel_color_warning_background(ImlibData *im) {
  */
 void xitk_image_free_image(ImlibData *im, xitk_image_t **src) {
 
-  assert(im && *src);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(*src);
 
   if((*src)->mask)
     xitk_image_destroy_xitk_pixmap((*src)->mask);
@@ -124,7 +124,9 @@ void xitk_image_free_image(ImlibData *im, xitk_image_t **src) {
 Pixmap xitk_image_create_pixmap(ImlibData *im, int width, int height) {
   Pixmap p;
   
-  assert(im && width && height);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NOT_COND(width > 0);
+  ABORT_IF_NOT_COND(height > 0);
   
   XLOCK(im->x.disp);
   p = XCreatePixmap(im->x.disp, im->x.base_window, width, height, im->x.depth);
@@ -138,7 +140,7 @@ Pixmap xitk_image_create_pixmap(ImlibData *im, int width, int height) {
  */
 static void xitk_image_xitk_pixmap_destroyer(xitk_pixmap_t *xpix) {
 
-  assert(xpix);
+  ABORT_IF_NULL(xpix);
 
   XLOCK(xpix->imlibdata->x.disp);
 
@@ -176,7 +178,9 @@ xitk_pixmap_t *xitk_image_create_xitk_pixmap_with_depth(ImlibData *im,
   XShmSegmentInfo  *shminfo;
 #endif
   
-  assert(im && width && height);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NOT_COND(width > 0);
+  ABORT_IF_NOT_COND(height > 0);
   
   xpix = (xitk_pixmap_t *) xitk_xmalloc(sizeof(xitk_pixmap_t));
   xpix->imlibdata = im;
@@ -295,7 +299,7 @@ xitk_pixmap_t *xitk_image_create_xitk_mask_pixmap(ImlibData *im, int width, int
 }
 
 void xitk_image_destroy_xitk_pixmap(xitk_pixmap_t *p) {
-  assert(p);
+  ABORT_IF_NULL(p);
   p->destroy(p);
 }
 
@@ -305,7 +309,9 @@ void xitk_image_destroy_xitk_pixmap(xitk_pixmap_t *p) {
 Pixmap xitk_image_create_mask_pixmap(ImlibData *im, int width, int height) {
   Pixmap p;
   
-  assert(im && width && height);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NOT_COND(width > 0);
+  ABORT_IF_NOT_COND(height > 0);
   
   XLOCK(im->x.disp);
   p = XCreatePixmap(im->x.disp, im->x.base_window, width, height, 1);
@@ -319,7 +325,11 @@ Pixmap xitk_image_create_mask_pixmap(ImlibData *im, int width, int height) {
  */
 void xitk_image_change_image(ImlibData *im, 
 			     xitk_image_t *src, xitk_image_t *dest, int width, int height) {
-  assert(im && src && dest && width && height);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(src);
+  ABORT_IF_NULL(dest);
+  ABORT_IF_NOT_COND(width > 0);
+  ABORT_IF_NOT_COND(height > 0);
   
   if(dest->mask)
     xitk_image_destroy_xitk_pixmap(dest->mask);
@@ -358,7 +368,9 @@ void xitk_image_change_image(ImlibData *im,
 xitk_image_t *xitk_image_create_image(ImlibData *im, int width, int height) {
   xitk_image_t *i;
 
-  assert(im && width && height);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NOT_COND(width > 0);
+  ABORT_IF_NOT_COND(height > 0);
 
   i = (xitk_image_t *) xitk_xmalloc(sizeof(xitk_image_t));
   i->mask = NULL;
@@ -387,7 +399,10 @@ xitk_image_t *xitk_image_create_image_with_colors_from_string(ImlibData *im,
   int             numlines = 0;
   char            bufsubs[BUFSIZ], buf[BUFSIZ];
 
-  assert(im && fontname && str && width);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(fontname);
+  ABORT_IF_NULL(str);
+  ABORT_IF_NOT_COND(width > 0);
 
   XLOCK(im->x.disp);
   gc = XCreateGC(im->x.disp, im->x.base_window, None, None);
@@ -522,7 +537,8 @@ xitk_image_t *xitk_image_create_image_from_string(ImlibData *im,
  *
  */
 void xitk_image_add_mask(ImlibData *im, xitk_image_t *dest) {
-  assert(im && dest);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(dest);
   
   if(dest->mask)
     xitk_image_destroy_xitk_pixmap(dest->mask);
@@ -543,7 +559,8 @@ void menu_draw_arrow_branch(ImlibData *im, xitk_image_t *p) {
   short          x1, x2, x3;
   short          y1, y2, y3;
   
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
   
   w = p->width / 3;
   h = p->height;
@@ -598,7 +615,8 @@ static void _draw_arrow(ImlibData *im, xitk_image_t *p, int direction) {
   short          x1, x2, x3;
   short          y1, y2, y3;
 
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
   
   w = p->width / 3;
   h = p->height;
@@ -700,7 +718,8 @@ void draw_arrow_right(ImlibData *im, xitk_image_t *p) {
 static void _draw_rectangular_box(ImlibData *im, xitk_pixmap_t *p, 
 				  int x, int y, int excstart, int excstop,
 				  int width, int height, int relief) {
-  assert(im && p && width && height);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
 
   XLOCK(im->x.disp);
   if(relief == DRAW_OUTTER)
@@ -730,7 +749,8 @@ static void _draw_rectangular_box(ImlibData *im, xitk_pixmap_t *p,
 static void _draw_rectangular_box_light(ImlibData *im, xitk_pixmap_t *p, 
 					int x, int y, int excstart, int excstop,
 					int width, int height, int relief) {
-  assert(im && p && width && height);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
 
   XLOCK(im->x.disp);
   if(relief == DRAW_OUTTER)
@@ -761,7 +781,8 @@ static void _draw_rectangular_box_with_colors(ImlibData *im, xitk_pixmap_t *p,
 					      int x, int y, int width, int height, 
 					      unsigned int lcolor, unsigned int dcolor,
 					      int relief) {
-  assert(im && p && width && height);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
 
   XLOCK(im->x.disp);
   if(relief == DRAW_OUTTER)
@@ -804,19 +825,125 @@ void draw_rectangular_outter_box_light(ImlibData *im, xitk_pixmap_t *p,
   _draw_rectangular_box_light(im, p, x, y, 0, 0, width, height, DRAW_OUTTER);
 }
 
+static void _draw_check_round(ImlibData *im, xitk_image_t *p, int x, int y, int d, int checked) {
+  XLOCK(im->x.disp);
+  XSetForeground(im->x.disp, p->image->gc, xitk_get_pixel_color_black(im));
+  XFillArc(im->x.disp, p->image->pixmap, p->image->gc, x, y, d, d, (30 * 64), (180 * 64));
+  XUNLOCK(im->x.disp);
+  
+  XLOCK(im->x.disp);
+  XSetForeground(im->x.disp, p->image->gc, xitk_get_pixel_color_darkgray(im));
+  XFillArc(im->x.disp, p->image->pixmap, p->image->gc, x, y, d, d, (210 * 64), (180 * 64));
+  XUNLOCK(im->x.disp);
+  
+  XLOCK(im->x.disp);
+  XSetForeground(im->x.disp, p->image->gc, xitk_get_pixel_color_white(im));
+  XFillArc(im->x.disp, p->image->pixmap, p->image->gc, x + 2, y + 2, d - 4, d - 4, (0 * 64), (360 * 64));
+  XUNLOCK(im->x.disp);
+  
+  if(checked) {
+    XLOCK(im->x.disp);
+    XSetForeground(im->x.disp, p->image->gc, xitk_get_pixel_color_black(im));
+    XFillArc(im->x.disp, p->image->pixmap, p->image->gc, x + 4, y + 4, d - 8, d - 8, (0 * 64), (360 * 64));
+    XUNLOCK(im->x.disp);
+  }
+}
+static void _draw_check_check(ImlibData *im, xitk_image_t *p, int x, int y, int d, int checked) {
+  /* background */
+  XLOCK(im->x.disp);
+  XSetForeground(im->x.disp, p->image->gc, xitk_get_pixel_color_lightgray(im));
+  XFillRectangle(im->x.disp, p->image->pixmap, p->image->gc, x, y, d, d);
+  XUNLOCK(im->x.disp);
+  /* */
+  XLOCK(im->x.disp);
+  XSetForeground(im->x.disp, p->image->gc, xitk_get_pixel_color_black(im));
+  XDrawLine(im->x.disp, p->image->pixmap, p->image->gc, x, y, x + d, y);
+  XDrawLine(im->x.disp, p->image->pixmap, p->image->gc, x, y, x, y + d);
+  XUNLOCK(im->x.disp);
+  
+  XLOCK(im->x.disp);
+  XSetForeground(im->x.disp, p->image->gc, xitk_get_pixel_color_darkgray(im));
+  XDrawLine(im->x.disp, p->image->pixmap, p->image->gc, x, y + d, x + d, y + d);
+  XDrawLine(im->x.disp, p->image->pixmap, p->image->gc, x + d, y, x + d, y + d);
+  XUNLOCK(im->x.disp);
+  
+  if(checked) {
+    XLOCK(im->x.disp);
+    XSetForeground(im->x.disp, p->image->gc, xitk_get_pixel_color_black(im));
+    XDrawLine(im->x.disp, p->image->pixmap, p->image->gc, x + (d / 5), (y + ((d / 3) * 2)) - 2, x + (d / 2), y + d - 2);
+    XDrawLine(im->x.disp, p->image->pixmap, p->image->gc, x + (d / 5)+1, (y + ((d / 3) * 2)) - 2, x + (d / 2) + 1, y + d - 2);
+    XDrawLine(im->x.disp, p->image->pixmap, p->image->gc, x + (d / 2), y + d - 2, x + d - 2, y+1);
+    XDrawLine(im->x.disp, p->image->pixmap, p->image->gc, x + (d / 2) + 1, y + d - 2, x + d - 1, y+1);
+    XUNLOCK(im->x.disp);
+  }
+  
+}
+
+void draw_check_three_state_round_style(ImlibData *im, xitk_image_t *p, int x, int y, int d, int w, int checked) {
+  int i;
+
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
+
+  for(i = 0; i < 3; i++) {
+    if(i == 2) {
+      x++;
+      y++;
+    }
+
+    _draw_check_round(im, p, x, y, d, checked);
+    x += w;
+  }
+}
+
+void draw_check_three_state_check_style(ImlibData *im, xitk_image_t *p, int x, int y, int d, int w, int checked) {
+  int i;
+
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
+  
+  for(i = 0; i < 3; i++) {
+    if(i == 2) {
+      x++;
+      y++;
+    }
+
+    _draw_check_check(im, p, x, y, d, checked);
+    x += w;
+  }
+}
+
 void menu_draw_check(ImlibData *im, xitk_image_t *p, int checked) {
-  int      h, w;
-  int      relief = (checked) ? DRAW_INNER : DRAW_OUTTER;
-  int      nrelief = (checked) ? DRAW_OUTTER : DRAW_INNER;
+  int  style = xitk_get_checkstyle_feature();
   
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
   
-  w = p->width / 3;
-  h = p->height - 12;
-  
-  _draw_rectangular_box(im, p->image, 4,               6,     0, 0, 12, h, relief);
-  _draw_rectangular_box(im, p->image, w + 4,           6,     0, 0, 12, h, relief);
-  _draw_rectangular_box(im, p->image, (w * 2) + 4 + 1, 6 + 1, 0, 0, 12, h, nrelief);
+  switch(style) {
+    
+  case CHECK_STYLE_CHECK:
+    draw_check_three_state_check_style(im, p, 4, 4, p->height - 8, p->width / 3, checked);
+    break;
+    
+  case CHECK_STYLE_ROUND:
+    draw_check_three_state_round_style(im, p, 4, 4, p->height - 8, p->width / 3, checked);
+    break;
+    
+  case CHECK_STYLE_OLD:
+  default: 
+    {
+      int      relief = (checked) ? DRAW_INNER : DRAW_OUTTER;
+      int      nrelief = (checked) ? DRAW_OUTTER : DRAW_INNER;
+      int      w, h;
+      
+      w = p->width / 3;
+      h = p->height - 12;
+      _draw_rectangular_box(im, p->image, 4,               6,     0, 0, 12, h, relief);
+      _draw_rectangular_box(im, p->image, w + 4,           6,     0, 0, 12, h, relief);
+      _draw_rectangular_box(im, p->image, (w * 2) + 4 + 1, 6 + 1, 0, 0, 12, h, nrelief);
+    }
+    break;
+  }
 }
 
 /*
@@ -826,7 +953,8 @@ static void _draw_three_state(ImlibData *im, xitk_image_t *p, int style) {
   int           w;
   int           h;
 
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
 
   w = p->width / 3;
   h = p->height;
@@ -888,7 +1016,8 @@ static void _draw_two_state(ImlibData *im, xitk_image_t *p, int style) {
   int           w;
   int           h;
 
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
 
   w = p->width / 2;
   h = p->height;
@@ -925,7 +1054,8 @@ static void _draw_two_state(ImlibData *im, xitk_image_t *p, int style) {
  *
  */
 static void _draw_relief(ImlibData *im, xitk_pixmap_t *p, int w, int h, int relief, int light) {
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
   
   XLOCK(im->x.disp);
   XSetForeground(im->x.disp, p->gc, xitk_get_pixel_color_gray(im));
@@ -950,6 +1080,45 @@ static void _draw_relief(ImlibData *im, xitk_pixmap_t *p, int w, int h, int reli
 
 }
 
+void draw_checkbox_check(ImlibData *im, xitk_image_t *p) {
+  int  style = xitk_get_checkstyle_feature();
+  
+  XLOCK(im->x.disp);
+  XSetForeground(im->x.disp, p->image->gc, xitk_get_pixel_color_gray(im));
+  XFillRectangle(im->x.disp, p->image->pixmap, p->image->gc, 0, 0, p->width, p->height);
+  XUNLOCK(im->x.disp);
+  
+  switch(style) {
+    
+  case CHECK_STYLE_CHECK:
+    {
+      int w;
+      
+      w = p->width / 3;
+      _draw_check_check(im, p, 0, 0, p->height, 0);
+      _draw_check_check(im, p, w, 0, p->height, 0);
+      _draw_check_check(im, p, w * 2, 0, p->height, 1);
+    }
+    break;
+    
+  case CHECK_STYLE_ROUND:
+    {
+      int w;
+      
+      w = p->width / 3;
+      _draw_check_round(im, p, 0, 0, p->height, 0);
+      _draw_check_round(im, p, w, 0, p->height, 0);
+      _draw_check_round(im, p, w * 2, 0, p->height, 1);
+    }
+    break;
+    
+  case CHECK_STYLE_OLD:
+  default: 
+    _draw_three_state(im, p, STYLE_BEVEL);
+    break;
+  }
+}
+
 /*
  *
  */
@@ -978,7 +1147,8 @@ static void _draw_paddle_three_state(ImlibData *im, xitk_image_t *p, int directi
   int           w;
   int           h;
 
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
 
   w = p->width / 3;
   h = p->height;
@@ -1088,7 +1258,8 @@ void draw_flat(ImlibData *im, xitk_pixmap_t *p, int w, int h) {
  *
  */
 void draw_flat_with_color(ImlibData *im, xitk_pixmap_t *p, int w, int h, unsigned int color) {
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
 
   XLOCK(im->x.disp);
   XSetForeground(im->x.disp, p->gc, color);
@@ -1106,7 +1277,8 @@ static void _draw_frame(ImlibData *im, xitk_pixmap_t *p,
   int            yoff = 0, xstart = 0, xstop = 0, fheight = 0, fwidth = 0;
   char           buf[BUFSIZ];
 
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
   
   if(title) {
 
@@ -1116,8 +1288,8 @@ static void _draw_frame(ImlibData *im, xitk_pixmap_t *p,
     fwidth = xitk_font_get_string_length(fs, title);
 
     if(fwidth >= (w - 12)) {
-      int nchar = 0;
-
+      int nchar = 1;
+      
       memset(&buf, 0, sizeof(buf));
       /* Limit the title to width of frame */
       do {
@@ -1184,7 +1356,8 @@ void draw_outter_frame(ImlibData *im, xitk_pixmap_t *p, char *title, char *fontn
 void draw_tab(ImlibData *im, xitk_image_t *p) {
   int           w, h;
 
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
 
   w = p->width / 3;
   h = p->height;
@@ -1232,7 +1405,8 @@ void draw_paddle_rotate(ImlibData *im, xitk_image_t *p) {
   int           h;
   unsigned int  ccolor, fcolor, ncolor;
 
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
   
   w = p->width/3;
   h = p->height;
@@ -1277,7 +1451,8 @@ void draw_rotate_button(ImlibData *im, xitk_image_t *p) {
   int           w;
   int           h;
   
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
   
   w = p->width;
   h = p->height;
@@ -1315,7 +1490,8 @@ void draw_rotate_button(ImlibData *im, xitk_image_t *p) {
 void draw_button_plus(ImlibData *im, xitk_image_t *p) {
   int           w, h;
   
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
 
   draw_button_minus(im, p);
   
@@ -1338,7 +1514,8 @@ void draw_button_plus(ImlibData *im, xitk_image_t *p) {
 void draw_button_minus(ImlibData *im, xitk_image_t *p) {
   int           w, h;
   
-  assert(im && p);
+  ABORT_IF_NULL(im);
+  ABORT_IF_NULL(p);
   
   w = p->width / 3;
   h = p->height;
@@ -1360,7 +1537,7 @@ xitk_image_t *xitk_image_load_image(ImlibData *im, char *image) {
   ImlibImage    *img = NULL;
   xitk_image_t  *i;
 
-  assert(im);
+  ABORT_IF_NULL(im);
 
   if(image == NULL) {
     XITK_WARNING("image name is NULL\n");
@@ -1371,7 +1548,9 @@ xitk_image_t *xitk_image_load_image(ImlibData *im, char *image) {
   
   XLOCK(im->x.disp);
   if(!(img = Imlib_load_image(im, (char *)image))) {
-    XITK_DIE("%s(): couldn't find image %s\n", __FUNCTION__, image);
+    XITK_WARNING("%s(): couldn't find image %s\n", __FUNCTION__, image);
+    XUNLOCK(im->x.disp);
+    return NULL;
   }
   
   Imlib_render (im, img, img->rgb_width, img->rgb_height);
@@ -1412,7 +1591,6 @@ static void notify_destroy(xitk_widget_t *w) {
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_IMAGE)) {
     private_data = (image_private_data_t *) w->private_data;
     XITK_FREE(private_data->skin_element_name);
-    xitk_image_free_image(private_data->imlibdata, &private_data->skin);
     XITK_FREE(private_data);
   }
 }
@@ -1488,9 +1666,8 @@ static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
       
       xitk_skin_lock(skonfig);
 
-      xitk_image_free_image(private_data->imlibdata, &private_data->skin);
-      private_data->skin = xitk_image_load_image(private_data->imlibdata,
-						 xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
+      private_data->skin = xitk_skin_get_image(skonfig,
+					       xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
       
       w->x               = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
       w->y               = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
@@ -1584,9 +1761,8 @@ xitk_widget_t *xitk_image_create (xitk_widget_list_t *wl,
 			    (xitk_skin_get_coord_x(skonfig, im->skin_element_name)),
 			    (xitk_skin_get_coord_y(skonfig, im->skin_element_name)),
 			    im->skin_element_name,
-			    (xitk_image_load_image(im->imlibdata,
-						   xitk_skin_get_skin_filename(skonfig, 
-								       im->skin_element_name))));
+			    (xitk_skin_get_image(skonfig,
+						 xitk_skin_get_skin_filename(skonfig, im->skin_element_name))));
 }
 
 /*
