@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: inputtext.c,v 1.54 2003/07/18 21:34:40 f1rmb Exp $
+ * $Id: inputtext.c,v 1.63 2004/04/13 19:55:49 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -74,7 +74,6 @@ static void notify_destroy(xitk_widget_t *w) {
     XUNLOCK(private_data->imlibdata->x.disp);
 
     XITK_FREE(private_data->skin_element_name);
-    xitk_image_free_image(private_data->imlibdata, &private_data->skin);
     XITK_FREE(private_data->text);
     XITK_FREE(private_data->normal_color);
     XITK_FREE(private_data->focused_color);
@@ -161,52 +160,85 @@ int xitk_get_key_modifier(XEvent *xev, int *modifier) {
   return (*modifier != MODIFIER_NOMOD);
 }
 
+int xitk_get_keysym_and_buf(XEvent *event, KeySym *ksym, char kbuf[]) {
+  int len = 0;
+  if(event) {
+    XKeyEvent  pkeyev = event->xkey;
+    
+    XLOCK(pkeyev.display);
+    len = XLookupString(&pkeyev, kbuf, sizeof(kbuf), ksym, NULL);
+    XUNLOCK(pkeyev.display);
+  }
+  return len;
+}
+
+/*
+ * Return key pressed (XK_VoidSymbol on failure)
+ */
+KeySym xitk_get_key_pressed(XEvent *event) {
+  KeySym   pkey = XK_VoidSymbol;
+  
+  if(event) {
+    char  buf[256];
+    (void) xitk_get_keysym_and_buf(event, &pkey, buf);
+  }
+
+  return pkey;
+}
+
 /*
  * Recalculate display offsets.
  */
 static void inputtext_recalc_offsets(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
-  char         *label = private_data->text;
-  char         *p = private_data->text;
-  xitk_font_t  *fs = NULL;
-  int           width;
-  int           i = 0;
 
   if(private_data->cursor_pos >= 0) {
+    char *label;
     
-    if(private_data->fontname)
-      fs = xitk_font_load_font(private_data->imlibdata->x.disp, private_data->fontname);
-
-    if(fs == NULL) 
-      fs = xitk_font_load_font(private_data->imlibdata->x.disp, xitk_get_system_font());
-
-    if(fs == NULL)
-      XITK_DIE("%s()@%d: xitk_font_load_font() failed. Exiting\n", __FUNCTION__, __LINE__);
-
-    xitk_font_set_font(fs, w->wl->gc);
-
-    private_data->disp_offset = private_data->pos_in_pos = 0;
-    
-    while(i < private_data->cursor_pos) {
+    if((label = private_data->text)) {
+      char         *p;
+      xitk_font_t  *fs = NULL;
+      int           width;
+      int           i = 0;
       
-      p = label;
-      p += private_data->disp_offset;
+      if(private_data->fontname)
+	fs = xitk_font_load_font(private_data->imlibdata->x.disp, private_data->fontname);
       
-      width = xitk_font_get_string_length(fs, p);
-
-      if(((i == (strlen(label))) && (width > private_data->max_visible)) 
-	 || ((xitk_font_get_text_width(fs, p, 
-				       (i - private_data->disp_offset))) 
-	     >= private_data->max_visible)) {
+      if(fs == NULL) 
+	fs = xitk_font_load_font(private_data->imlibdata->x.disp, xitk_get_system_font());
+      
+      if(fs == NULL)
+	XITK_DIE("%s()@%d: xitk_font_load_font() failed. Exiting\n", __FUNCTION__, __LINE__);
+      
+      xitk_font_set_font(fs, w->wl->gc);
+      
+      private_data->disp_offset = private_data->pos_in_pos = 0;
+      
+      while(i < private_data->cursor_pos) {
 	
-	private_data->pos[++private_data->pos_in_pos] = i - 1;
-	private_data->disp_offset = private_data->pos[private_data->pos_in_pos];
+	p = label;
+	p += private_data->disp_offset;
+	
+	width = xitk_font_get_string_length(fs, p);
+	
+	if(((i == (strlen(label))) && (width > private_data->max_visible)) 
+	   || ((xitk_font_get_text_width(fs, p, 
+					 (i - private_data->disp_offset))) 
+	       >= private_data->max_visible)) {
+	  
+	  private_data->pos[++private_data->pos_in_pos] = i - 1;
+	  private_data->disp_offset = private_data->pos[private_data->pos_in_pos];
+	}
+	
+	i++;
       }
-
-      i++;
+      
+      xitk_font_unload_font(fs);
+    }
+    else {
+      private_data->pos_in_pos = 0;
+      private_data->pos[0]     = 0;
     }
-
-    xitk_font_unload_font(fs);
   }
 }
 
@@ -230,18 +262,20 @@ static void create_labelofinputtext(xitk_widget_t *w,
 
   /* Try to load font */
 
-  if(private_data->fontname)
-    fs = xitk_font_load_font(private_data->imlibdata->x.disp, private_data->fontname);
-
-  if(fs == NULL) 
-    fs = xitk_font_load_font(private_data->imlibdata->x.disp, xitk_get_system_font());
-
-  if(fs == NULL)
-    XITK_DIE("%s()@%d: xitk_font_load_font() failed. Exiting\n", __FUNCTION__, __LINE__);
-
-  xitk_font_set_font(fs, gc);
-  xitk_font_string_extent(fs, label, &lbear, &rbear, &width, &asc, &des);
-
+  if(label && strlen(label)) {
+    if(private_data->fontname)
+      fs = xitk_font_load_font(private_data->imlibdata->x.disp, private_data->fontname);
+    
+    if(fs == NULL) 
+      fs = xitk_font_load_font(private_data->imlibdata->x.disp, xitk_get_system_font());
+    
+    if(fs == NULL)
+      XITK_DIE("%s()@%d: xitk_font_load_font() failed. Exiting\n", __FUNCTION__, __LINE__);
+    
+    xitk_font_set_font(fs, gc);
+    xitk_font_string_extent(fs, label, &lbear, &rbear, &width, &asc, &des);
+  }
+  
   if((private_data->skin_element_name != NULL) ||
      ((private_data->skin_element_name == NULL) && ((fg = xitk_get_black_color()) == -1))) {
     
@@ -277,10 +311,9 @@ static void create_labelofinputtext(xitk_widget_t *w,
     }
     
     XLOCK(private_data->imlibdata->x.disp);
-    XAllocColor(private_data->imlibdata->x.disp,
-		Imlib_get_colormap(private_data->imlibdata), &xcolor);
+    XAllocColor(private_data->imlibdata->x.disp, Imlib_get_colormap(private_data->imlibdata), &xcolor);
     XUNLOCK(private_data->imlibdata->x.disp);
-
+    
     fg = xcolor.pixel;
     
   }
@@ -288,44 +321,49 @@ static void create_labelofinputtext(xitk_widget_t *w,
   XLOCK(private_data->imlibdata->x.disp);
   XSetForeground(private_data->imlibdata->x.disp, gc, fg);
   XUNLOCK(private_data->imlibdata->x.disp);
-
-  if(private_data->cursor_pos >= 0) {
-
-    if(private_data->disp_offset)
-      p += private_data->disp_offset;
-    
-    width = xitk_font_get_string_length(fs, p);
-    
-    if(private_data->disp_offset > private_data->cursor_pos && private_data->pos_in_pos) {
+  
+  if(label && strlen(label)) {
+    if(private_data->cursor_pos >= 0) {
       
-      private_data->disp_offset = private_data->pos[--private_data->pos_in_pos];
+      if(private_data->disp_offset)
+	p += private_data->disp_offset;
       
-      if(private_data->disp_offset < 0) {
-	private_data->pos[private_data->pos_in_pos] = private_data->disp_offset = 0;
+      width = xitk_font_get_string_length(fs, p);
+      
+      if(private_data->disp_offset > private_data->cursor_pos && private_data->pos_in_pos) {
+	
+	private_data->disp_offset = private_data->pos[--private_data->pos_in_pos];
+	
+	if(private_data->disp_offset < 0) {
+	  private_data->pos[private_data->pos_in_pos] = private_data->disp_offset = 0;
+	}
+	
+      }
+      else if(((private_data->cursor_pos == (strlen(label)))
+	       && (width > private_data->max_visible))
+	      || ((xitk_font_get_text_width(fs, p, 
+					    (private_data->cursor_pos - private_data->disp_offset)))
+		  >= private_data->max_visible)) {
+	
+	private_data->pos[++private_data->pos_in_pos] = private_data->cursor_pos - 1;
+	private_data->disp_offset = private_data->pos[private_data->pos_in_pos];
       }
-      
-    }
-    else if(((private_data->cursor_pos == (strlen(label)))
-	     && (width > private_data->max_visible))
-	    || ((xitk_font_get_text_width(fs, p, 
-					  (private_data->cursor_pos - private_data->disp_offset)))
-		>= private_data->max_visible)) {
-      
-      private_data->pos[++private_data->pos_in_pos] = private_data->cursor_pos - 1;
-      private_data->disp_offset = private_data->pos[private_data->pos_in_pos];
     }
+    
+    if(private_data->disp_offset)
+      plabel += private_data->disp_offset;
+
   }
-
-  if(private_data->disp_offset)
-    plabel += private_data->disp_offset;
-
+  
   /*  Put text in the right place */
   if(private_data->skin_element_name) {
-    XLOCK(private_data->imlibdata->x.disp);
-    xitk_font_draw_string(fs, pix, gc, 
-		2, ((ysize+asc+des+yoff)>>1)-des, 
-		plabel, strlen(plabel));
-    XUNLOCK(private_data->imlibdata->x.disp);
+    if(label && strlen(label)) {
+      XLOCK(private_data->imlibdata->x.disp);
+      xitk_font_draw_string(fs, pix, gc, 
+			    2, ((ysize+asc+des+yoff)>>1)-des, 
+			    plabel, strlen(plabel));
+      XUNLOCK(private_data->imlibdata->x.disp);
+    }
   }
   else {
     XWindowAttributes  attr;
@@ -342,17 +380,14 @@ static void create_labelofinputtext(xitk_widget_t *w,
     tpix = xitk_image_create_xitk_pixmap(private_data->imlibdata, xsize, ysize);
     
     XLOCK(private_data->imlibdata->x.disp);
-    XCopyArea (private_data->imlibdata->x.disp, pix, tpix->pixmap, lgc, 0, 0,
-	       xsize, ysize, 0, 0);
+    XCopyArea (private_data->imlibdata->x.disp, pix, tpix->pixmap, lgc, 0, 0, xsize, ysize, 0, 0);
     
-    xitk_font_draw_string(fs, tpix->pixmap, lgc, 
-		2, ((ysize+asc+des+yoff)>>1)-des, 
-		plabel, strlen(plabel));
+    if(label && strlen(label))
+      xitk_font_draw_string(fs, tpix->pixmap, lgc, 3 , ((ysize+asc+des+yoff)>>1)-des, plabel, strlen(plabel));
     
-    XCopyArea (private_data->imlibdata->x.disp, tpix->pixmap, pix, lgc, 0, 0,
-	       xsize - 1, ysize, 0, 0);
+    XCopyArea (private_data->imlibdata->x.disp, tpix->pixmap, pix, lgc, 0, 0, xsize - 1, ysize, 0, 0);
     XUNLOCK(private_data->imlibdata->x.disp);
-
+    
     xitk_image_destroy_xitk_pixmap(tpix);
 
     XLOCK(private_data->imlibdata->x.disp);
@@ -360,26 +395,27 @@ static void create_labelofinputtext(xitk_widget_t *w,
     XUNLOCK(private_data->imlibdata->x.disp);
   }
 
-  width = xitk_font_get_text_width(fs, plabel,
-				   (private_data->cursor_pos - private_data->disp_offset));
-  
   /* Draw cursor pointer */
   if(private_data->cursor_pos >= 0) {
     
+    width = (label && strlen(label)) ? 
+      xitk_font_get_text_width(fs, plabel, (private_data->cursor_pos - private_data->disp_offset)) : 0;
+  
     XLOCK(private_data->imlibdata->x.disp);
     XDrawLine(private_data->imlibdata->x.disp, pix, gc,
 	      width + 1, 2, width + 3, 2);
-
+    
     XDrawLine(private_data->imlibdata->x.disp, pix, gc, 
 	      width + 2, 2, width + 2, ysize - 3);
-
+    
     XDrawLine(private_data->imlibdata->x.disp, pix, gc, 
 	      width + 1, ysize - 3, width + 3, ysize - 3);
     XUNLOCK(private_data->imlibdata->x.disp);
   }
-
-  xitk_font_unload_font(fs);
-
+  
+  if(label && strlen(label))  
+    xitk_font_unload_font(fs);
+  
   if(color)
     xitk_free_color_name(color);
 }
@@ -468,11 +504,13 @@ static void paint_inputtext(xitk_widget_t *w) {
  */
 static int notify_click_inputtext(xitk_widget_t *w, int button, int bUp, int x, int y) {
   inputtext_private_data_t *private_data;
-  int pos;
+  int                       pos;
   
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
-    private_data = (inputtext_private_data_t *) w->private_data;
+    char  *p;
 
+    private_data = (inputtext_private_data_t *) w->private_data;
+    
     if(w->have_focus == FOCUS_LOST)
       w->have_focus = private_data->have_focus = FOCUS_RECEIVED;
     
@@ -482,10 +520,10 @@ static int notify_click_inputtext(xitk_widget_t *w, int button, int bUp, int x,
     
     pos = x - w->x;
 
-    {
-      char        *p = private_data->text;
+    if((p = private_data->text)) {
       xitk_font_t *fs = NULL;
       int          width = 0, i = 0;
+      int          max_len;
       
       p += private_data->disp_offset;
 
@@ -500,26 +538,32 @@ static int notify_click_inputtext(xitk_widget_t *w, int button, int bUp, int x,
       
       xitk_font_set_font(fs, w->wl->gc);
       
-      while(width < pos) {
-	
-	width = xitk_font_get_text_width(fs, p, i);
-	i++;
+      max_len = xitk_font_get_text_width(fs, p, strlen(p));
+      
+      if(pos > max_len) {
+	/* cursor behind the last character */
+	i = strlen(p) + 1;
+      } else {
+	/* cursor in the text */
+        /* FIXME: implement multibyte string support */
+        while(width < pos) {
+          width = xitk_font_get_text_width(fs, p, i + 1);
+          i++;
+	}
       }
+
       xitk_font_unload_font(fs);
-      pos = (i - 2) + private_data->disp_offset;
-    }
-    
-    if(private_data->text) {
+      pos = (i - 1) + private_data->disp_offset;
+      
       if(pos > strlen(private_data->text))
 	pos = strlen(private_data->text);
+
     }
     else
       pos = 0;
-
+    
     private_data->cursor_pos = (pos < 0) ? 0 : pos;
-
     paint_inputtext(w);
-
   }
 
   return 1;
@@ -563,9 +607,8 @@ static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
       XITK_FREE(private_data->fontname);
       private_data->fontname      = strdup(xitk_skin_get_label_fontname(skonfig, private_data->skin_element_name));
       
-      xitk_image_free_image(private_data->imlibdata, &private_data->skin);
-      private_data->skin          = xitk_image_load_image(private_data->imlibdata, 
-							  xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
+      private_data->skin          = xitk_skin_get_image(skonfig,
+							xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
       
       private_data->max_visible   = (private_data->skin->width/2);
       
@@ -597,9 +640,8 @@ static void inputtext_erase_with_delete(xitk_widget_t *w) {
   char                      *p, *pp;
   int                        offset;
   
-  if(strlen(private_data->text) > 0) {
-    if((private_data->cursor_pos >= 0)
-       && (private_data->cursor_pos < strlen(private_data->text))) {
+  if(private_data->text && (strlen(private_data->text) > 0)) {
+    if((private_data->cursor_pos >= 0) && (private_data->cursor_pos < strlen(private_data->text))) {
       
       oldtext = strdup(private_data->text);
       newtext = (char *) xitk_xmalloc(strlen(oldtext));
@@ -626,7 +668,7 @@ static void inputtext_erase_with_delete(xitk_widget_t *w) {
       *pp = 0;
       
       XITK_FREE(private_data->text);
-      private_data->text = strdup((newtext != NULL) ? newtext : "");
+      private_data->text = newtext ? strdup(newtext) : NULL;
       
       paint_inputtext(w);
       
@@ -634,10 +676,9 @@ static void inputtext_erase_with_delete(xitk_widget_t *w) {
       XITK_FREE(newtext);
     }
   }
-  else {
-    sprintf(private_data->text, "%s", "");
+  else
     paint_inputtext(w);
-  }
+
 }
 
 /*
@@ -649,8 +690,8 @@ static void inputtext_erase_with_backspace(xitk_widget_t *w) {
   char                     *p, *pp;
   int                       offset;
   
-  if(strlen(private_data->text) > 1) {
-    if(private_data->cursor_pos > 0) {
+  if(private_data->text && (strlen(private_data->text) >= 1)) {
+    if(private_data->cursor_pos >= 0) {
       
       oldtext = strdup(private_data->text);
       newtext = (char *) xitk_xmalloc(strlen(oldtext));
@@ -687,7 +728,6 @@ static void inputtext_erase_with_backspace(xitk_widget_t *w) {
     }
   }
   else {
-    sprintf(private_data->text, "%s", "");
     private_data->cursor_pos = 0;
     paint_inputtext(w);
   }
@@ -698,23 +738,18 @@ static void inputtext_erase_with_backspace(xitk_widget_t *w) {
  */
 static void inputtext_kill_line(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
-  char                      *oldtext, *newtext;
   
-  if(strlen(private_data->text) > 1) {
+  if(private_data->text && (strlen(private_data->text) > 1)) {
     if(private_data->cursor_pos >= 0) {
-
-      oldtext = strdup(private_data->text);
-      newtext = (char *) xitk_xmalloc(strlen(oldtext));
-
-      snprintf(newtext, private_data->cursor_pos + 1, "%s", oldtext);
-
+      char *newtext = NULL;
+      
+      newtext = (char *) xitk_xmalloc(private_data->cursor_pos + 1);
+      snprintf(newtext, private_data->cursor_pos + 1, "%s", private_data->text);
+      
       XITK_FREE(private_data->text);
-      private_data->text = strdup(newtext);
+      private_data->text = newtext;
       
       paint_inputtext(w);
-      
-      XITK_FREE(oldtext);
-      XITK_FREE(newtext);
     }
   }
 }
@@ -756,7 +791,7 @@ static void inputtext_exec_return(xitk_widget_t *w) {
   _cursor_focus(private_data, w->wl->win, 0);
   paint_inputtext(w);
   
-  if(strlen(private_data->text) > 0) {
+  if(private_data->text && (strlen(private_data->text) > 0)) {
     if(private_data->callback)
       private_data->callback(w, private_data->userdata, private_data->text);
   }
@@ -827,7 +862,6 @@ static void notify_keyevent_inputtext(xitk_widget_t *w, XEvent *xev) {
   inputtext_private_data_t *private_data;
   
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
-    XKeyEvent   keyevent = xev->xkey;
     KeySym      key;
     char        buf[256];
     int         len;
@@ -837,10 +871,7 @@ static void notify_keyevent_inputtext(xitk_widget_t *w, XEvent *xev) {
 
     private_data->have_focus = FOCUS_RECEIVED;
 
-    XLOCK(private_data->imlibdata->x.disp);
-    len = XLookupString(&keyevent, buf, sizeof(buf), &key, NULL);
-    XUNLOCK(private_data->imlibdata->x.disp);
-
+    len = xitk_get_keysym_and_buf(xev, &key, buf);
     buf[len] = '\0';
 
     (void) xitk_get_key_modifier(xev, &modifier);
@@ -974,25 +1005,35 @@ static void notify_keyevent_inputtext(xitk_widget_t *w, XEvent *xev) {
 	char *oldtext;
 	int pos;
 	
-	if((private_data->text != NULL) 
-	   && (strlen(private_data->text) < private_data->max_length)) {
+	if(private_data->text) {
+	  if(strlen(private_data->text) < private_data->max_length) {	   
+	    
+	    oldtext = strdup(private_data->text);
+	    
+	    if(strlen(private_data->text) <= private_data->cursor_pos)
+	      pos = strlen(private_data->text);
+	    else
+	      pos = private_data->cursor_pos;
+	    
+	    private_data->text = (char *) realloc(private_data->text, strlen(oldtext) + 3);
+
+	    sprintf(private_data->text, "%s", oldtext);
+	    sprintf(&private_data->text[pos], "%c%s", buf[0], &oldtext[pos]);
+	    private_data->text[strlen(private_data->text)] = '\0';
+	    free(oldtext);
+	    
+	    private_data->cursor_pos++;
+	    paint_inputtext(w);
+	  }
 	  
-	  oldtext = strdup(private_data->text);
+	}
+	else {
+	  private_data->text = (char *) xitk_xmalloc(2);
 	  
-	  if(strlen(private_data->text) <= private_data->cursor_pos)
-	    pos = strlen(private_data->text);
-	  else
-	    pos = private_data->cursor_pos;
-	  
-	  XITK_FREE(private_data->text);
-	  private_data->text = (char *) xitk_xmalloc(strlen(oldtext) + 3);
-	  
-	  sprintf(private_data->text, "%s", oldtext);
-	  sprintf(&private_data->text[pos], "%c%s%c", buf[0], &oldtext[pos], 0);
+	  sprintf(private_data->text, "%c%c", buf[0], 0);
 	  
 	  private_data->cursor_pos++;
 	  paint_inputtext(w);
-	  XITK_FREE(oldtext);
 	}
       }
     }
@@ -1052,7 +1093,8 @@ char *xitk_inputtext_get_text(xitk_widget_t *w) {
   
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     private_data = (inputtext_private_data_t *) w->private_data;
-    return private_data->text;
+    
+    return private_data->text ? private_data->text : "";
   }
   
   return NULL;
@@ -1066,8 +1108,11 @@ void xitk_inputtext_change_text(xitk_widget_t *w, char *text) {
   
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     private_data = (inputtext_private_data_t *) w->private_data;
+
     XITK_FREE(private_data->text);
-    private_data->text = strdup((text != NULL) ? text : "");
+    if(text)
+      private_data->text = strdup(text);
+
     private_data->disp_offset = 0;
     private_data->cursor_pos = -1;
     paint_inputtext(w);
@@ -1098,8 +1143,8 @@ static xitk_widget_t *_xitk_inputtext_create (xitk_widget_list_t *wl,
 
   private_data->iWidget           = mywidget;
 
-  private_data->text              = strdup((it->text != NULL)?it->text:"");
-
+  private_data->text = it->text ? strdup(it->text) : NULL;
+    
   private_data->fontname          = strdup(fontname);
  
   private_data->max_length        = it->max_length;
@@ -1159,8 +1204,8 @@ xitk_widget_t *xitk_inputtext_create (xitk_widget_list_t *wl,
 				 (xitk_skin_get_coord_x(skonfig, it->skin_element_name)),
 				 (xitk_skin_get_coord_y(skonfig, it->skin_element_name)),
 				 it->skin_element_name,
-				 (xitk_image_load_image(it->imlibdata, 
-							xitk_skin_get_skin_filename(skonfig, it->skin_element_name))),
+				 (xitk_skin_get_image(skonfig,
+						      xitk_skin_get_skin_filename(skonfig, it->skin_element_name))),
 				 (xitk_skin_get_label_fontname(skonfig, it->skin_element_name)),
 				 (xitk_skin_get_label_color(skonfig, it->skin_element_name)),
 				 (xitk_skin_get_label_color_focus(skonfig, it->skin_element_name)),
