@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: label.c,v 1.57 2003/07/18 21:34:40 f1rmb Exp $
+ * $Id: label.c,v 1.64 2004/04/13 19:55:50 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -30,6 +30,10 @@
 #include <pthread.h>
 #include <unistd.h>
 
+#ifdef HAVE_ALLOCA_H   
+#include <alloca.h>
+#endif
+
 #include "_xitk.h"
 
 static void _create_label_pixmap(xitk_widget_t *w) {
@@ -43,7 +47,7 @@ static void _create_label_pixmap(xitk_widget_t *w) {
   private_data->anim_offset = 0;
 
   if(private_data->animation) {
-    _label = (char *) alloca((strlen(private_data->label) * 2) + 5 + 1);
+    _label = (char *) xitk_xmalloc((strlen(private_data->label) * 2) + 5 + 1);
     
     if((strlen(private_data->label)) > private_data->length)
       sprintf(_label, "%s *** %s", private_data->label, private_data->label);
@@ -52,27 +56,26 @@ static void _create_label_pixmap(xitk_widget_t *w) {
     
   }
   else
-    xitk_strdupa(_label, private_data->label);
+    _label = strdup(private_data->label);
   
   len = strlen(_label);
   pixwidth = private_data->char_length * 
     ((private_data->length * ((len / private_data->length) + 1)) + 5);
-
   private_data->labelpix = xitk_image_create_xitk_pixmap(private_data->imlibdata,
 							 (pixwidth) ? pixwidth : 1, 
 							 private_data->char_height);
 
   x_dest = 0;
-    
-  for (i = 0; i < pixwidth; i++) {
+
+  for (i = 0; i < len; i++) {
     int c = 0;
     
     if ((i < len) && (_label[i] >= 32))
       c = _label[i] - 32;
-    
+
     if (c >= 0) {
       int px, py;
-      
+         
       px = (c % 32) * private_data->char_length;
       py = (c / 32) * private_data->char_height;
       
@@ -86,7 +89,20 @@ static void _create_label_pixmap(xitk_widget_t *w) {
     
     x_dest += private_data->char_length;
   }
+  
+  /* fill gap with spaces */
+  if(len < private_data->length) {
+    XLOCK(private_data->imlibdata->x.disp);
+    for(; i < private_data->length; i++) {
+      XCopyArea(private_data->imlibdata->x.disp, font->image->pixmap, 
+		private_data->labelpix->pixmap, private_data->labelpix->gc, 0, 0,
+		private_data->char_length, private_data->char_height, x_dest, 0);
+      x_dest += private_data->char_length;
+    }
+    XUNLOCK(private_data->imlibdata->x.disp);
+  }
 
+  free(_label);
 }
 
 /*
@@ -115,9 +131,6 @@ static void notify_destroy(xitk_widget_t *w) {
     XITK_FREE(private_data->fontname);
     XITK_FREE(private_data->skin_element_name);
 
-    if(private_data->font)
-      xitk_image_free_image(private_data->imlibdata, &private_data->font);
-
     pthread_mutex_destroy(&private_data->paint_mutex);
 
     XITK_FREE(private_data);
@@ -178,7 +191,8 @@ static void paint_label(xitk_widget_t *w) {
 
       fs = xitk_font_load_font(private_data->imlibdata->x.disp, private_data->fontname);
       xitk_font_set_font(fs, private_data->font->image->gc);
-      xitk_font_string_extent(fs, private_data->label, &lbear, &rbear, &wid, &asc, &des);
+      xitk_font_string_extent(fs, (private_data->label && strlen(private_data->label)) ? private_data->label : "Label",
+			      &lbear, &rbear, &wid, &asc, &des);
 
       bg = xitk_image_create_image(private_data->imlibdata, w->width, w->height);
 
@@ -205,13 +219,12 @@ static void paint_label(xitk_widget_t *w) {
     else {
       int width = private_data->char_length * private_data->length;
       
-
       XLOCK(private_data->imlibdata->x.disp);
       XCopyArea(private_data->imlibdata->x.disp, 
 		private_data->labelpix->pixmap, w->wl->win, font->image->gc, 
 		private_data->anim_offset, 0, width, private_data->char_height, 
 		w->x, w->y);
-
+      
       /* We can't wait here, otherwise the rolling effect is really jerky */
       if(private_data->animation && (private_data->on_change == 0))
 	XSync(private_data->imlibdata->x.disp, False);
@@ -335,9 +348,8 @@ static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
       
       xitk_skin_lock(skonfig);
 
-      xitk_image_free_image(private_data->imlibdata, &private_data->font);
-      private_data->font        = xitk_image_load_image(private_data->imlibdata, 
-							xitk_skin_get_label_skinfont_filename(skonfig, private_data->skin_element_name));
+      private_data->font        = xitk_skin_get_image(skonfig,
+						      xitk_skin_get_label_skinfont_filename(skonfig, private_data->skin_element_name));
       private_data->char_length = (private_data->font->width/32);
       private_data->char_height = (private_data->font->height/3);
       
@@ -483,8 +495,8 @@ static xitk_widget_t *_xitk_label_create(xitk_widget_list_t *wl,
     private_data->length       = width;
   }
   else {
-    private_data->font         = xitk_image_load_image(private_data->imlibdata, 
-							 xitk_skin_get_label_skinfont_filename(skonfig, private_data->skin_element_name));
+    private_data->font         = xitk_skin_get_image(skonfig, 
+						     xitk_skin_get_label_skinfont_filename(skonfig, private_data->skin_element_name));
   
     private_data->fontname     = NULL;
     private_data->char_length  = (private_data->font->width/32);
