@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: labelbutton.c,v 1.52 2003/07/18 21:34:40 f1rmb Exp $
+ * $Id: labelbutton.c,v 1.59 2004/04/13 19:55:50 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -43,8 +43,9 @@ static void notify_destroy(xitk_widget_t *w) {
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
     XITK_FREE(private_data->label);
+    XITK_FREE(private_data->shortcut_label);
+    XITK_FREE(private_data->shortcut_font);
     XITK_FREE(private_data->skin_element_name);
-    xitk_image_free_image(private_data->imlibdata, &private_data->skin);
     XITK_FREE(private_data->normcolor);
     XITK_FREE(private_data->focuscolor);
     XITK_FREE(private_data->clickcolor);
@@ -98,13 +99,13 @@ static int notify_inside(xitk_widget_t *w, int x, int y) {
 static void create_labelofbutton(xitk_widget_t *lb, 
 				 Window win, GC gc, Pixmap pix, 
 				 int xsize, int ysize, 
-				 char *label, int state) {
-  lbutton_private_data_t  *private_data = 
-    (lbutton_private_data_t *) lb->private_data;
+				 char *label, char *shortcut_label, int shortcut_pos, int state) {
+  lbutton_private_data_t  *private_data = (lbutton_private_data_t *) lb->private_data;
   xitk_font_t             *fs = NULL;
   int                      lbear, rbear, width, asc, des;
   int                      xoff = 0, yoff = 0, DefaultColor = -1;
   unsigned int             fg = 0;
+  unsigned int             origin = 0;
   XColor                   xcolor;
   xitk_color_names_t      *color = NULL;
 
@@ -121,7 +122,7 @@ static void create_labelofbutton(xitk_widget_t *lb,
     XITK_DIE("%s()@%d: xitk_font_load_font() failed. Exiting\n", __FUNCTION__, __LINE__);
   
   xitk_font_set_font(fs, gc);
-  xitk_font_string_extent(fs, label, &lbear, &rbear, &width, &asc, &des);
+  xitk_font_string_extent(fs, (label && strlen(label)) ? label : "Button", &lbear, &rbear, &width, &asc, &des);
 
   if((state == CLICK) && (private_data->label_static == 0)) {
     xoff = -4;
@@ -172,37 +173,49 @@ static void create_labelofbutton(xitk_widget_t *lb,
     
     XLOCK(private_data->imlibdata->x.disp);
     XAllocColor(private_data->imlibdata->x.disp,
-		Imlib_get_colormap(private_data->imlibdata), &xcolor);
+    Imlib_get_colormap(private_data->imlibdata), &xcolor);
     XUNLOCK(private_data->imlibdata->x.disp);
 
     fg = xcolor.pixel;
   }
   
   XLOCK(private_data->imlibdata->x.disp);
-
   XSetForeground(private_data->imlibdata->x.disp, gc, fg);
+  XUNLOCK(private_data->imlibdata->x.disp);
+
+  origin = ((ysize+asc+des+yoff)>>1)-des;
   
   /*  Put text in the right place */
   if(private_data->align == ALIGN_CENTER) {
     xitk_font_draw_string(fs, pix, gc, 
-		((xsize-(width+xoff))>>1) + private_data->label_offset, 
-		((ysize+asc+des+yoff)>>1)-des, 
-		label, strlen(label));
+			  ((xsize-(width+xoff))>>1) + private_data->label_offset, 
+			  origin, label, strlen(label));
   }
   else if(private_data->align == ALIGN_LEFT) {
     xitk_font_draw_string(fs, pix, gc, 
-		(((state != CLICK) ? 1 : 5)) + private_data->label_offset, 
-		((ysize+asc+des+yoff)>>1)-des, 
-		label, strlen(label));
+			  (((state != CLICK) ? 1 : 5)) + private_data->label_offset, 
+			  origin, label, strlen(label));
+    
+    /* shortcut is only permited if alignement is set to left */
+    if(strlen(shortcut_label) && shortcut_pos >= 0) {
+      xitk_font_t *short_font;
+      if (private_data->shortcut_font)
+	short_font = xitk_font_load_font(private_data->imlibdata->x.disp, private_data->shortcut_font);
+      else
+        short_font = fs;
+      xitk_font_draw_string(short_font, pix, gc, 
+			    (((state != CLICK) ? 1 : 5)) + shortcut_pos, 
+			    origin, shortcut_label, strlen(shortcut_label));
+      if (short_font != fs)
+	xitk_font_unload_font(short_font);
+    }
+    
   }
   else if(private_data->align == ALIGN_RIGHT) {
     xitk_font_draw_string(fs, pix, gc, 
-    		(xsize - (width + ((state != CLICK) ? 5 : 1))) + private_data->label_offset,
-    		((ysize+asc+des+yoff)>>1)-des, 
-    		label, strlen(label));
+			  (xsize - (width + ((state != CLICK) ? 5 : 1))) + private_data->label_offset, 
+			  origin, label, strlen(label));
   }
-  
-  XUNLOCK(private_data->imlibdata->x.disp);
 
   xitk_font_unload_font(fs);
 
@@ -304,7 +317,7 @@ static void paint_labelbutton (xitk_widget_t *w) {
       if(private_data->label_visible) {
 	create_labelofbutton(w, w->wl->win, w->wl->gc, btn->pixmap,
 			     button_width, skin->height, 
-			     private_data->label, state);
+			     private_data->label, private_data->shortcut_label, private_data->shortcut_pos, state);
       }
       
       XLOCK(private_data->imlibdata->x.disp);
@@ -396,6 +409,53 @@ char *xitk_labelbutton_get_label(xitk_widget_t *w) {
   return NULL;
 }
 
+/*
+ * Changing button caption
+ */
+int xitk_labelbutton_change_shortcut_label(xitk_widget_t *w, char *newlabel, int pos, char *newfont) {
+  lbutton_private_data_t *private_data;
+
+  if (w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON) && 
+	    (((w->type & WIDGET_GROUP_MASK) == WIDGET_GROUP_MENU) || ((w->type & WIDGET_GROUP_MASK) == WIDGET_GROUP_BROWSER)))) {
+    private_data = (lbutton_private_data_t *) w->private_data;
+
+    if((private_data->shortcut_label = (char *) realloc(private_data->shortcut_label, strlen(newlabel) + 1)) != NULL)
+      strcpy(private_data->shortcut_label, newlabel);
+
+    if(newfont &&
+	(private_data->shortcut_font = (char *) realloc(private_data->shortcut_font, strlen(newfont) + 1)) != NULL)
+      strcpy(private_data->shortcut_font, newfont);
+
+    if(strlen(private_data->shortcut_label)) {
+      if(pos >= 0)
+	private_data->shortcut_pos = pos;
+      else
+	private_data->shortcut_pos = -1;
+    }
+    
+    paint_labelbutton(w);
+    
+    return 1;
+  }
+
+  return 0;
+}
+
+/*
+ * Return the current button label
+ */
+char *xitk_labelbutton_get_shortcut_label(xitk_widget_t *w) {
+  lbutton_private_data_t *private_data;
+
+  if (w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON) &&
+	    (((w->type & WIDGET_GROUP_MASK) == WIDGET_GROUP_MENU) || ((w->type & WIDGET_GROUP_MASK) == WIDGET_GROUP_BROWSER)))) {
+    private_data = (lbutton_private_data_t *) w->private_data;
+    return private_data->shortcut_label;
+  }
+
+  return NULL;
+}
+
 /*
  * Handle motion on button
  */
@@ -423,9 +483,8 @@ static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
       
       xitk_skin_lock(skonfig);
       
-      xitk_image_free_image(private_data->imlibdata, &private_data->skin);
-      private_data->skin          = xitk_image_load_image(private_data->imlibdata, 
-		       xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
+      private_data->skin          = xitk_skin_get_image(skonfig, 
+							xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
       XITK_FREE(private_data->normcolor);
       private_data->normcolor     = strdup(xitk_skin_get_label_color(skonfig, 
 							     private_data->skin_element_name));
@@ -674,6 +733,9 @@ static xitk_widget_t *_xitk_labelbutton_create (xitk_widget_list_t *wl,
   private_data->state_callback    = b->state_callback;
   private_data->userdata          = b->userdata;
   private_data->label             = strdup((b->label)?b->label:"");
+  private_data->shortcut_label    = strdup("");
+  private_data->shortcut_font     = strdup(fontname);
+  private_data->shortcut_pos      = -1;
   private_data->label_visible     = label_visible;
   private_data->label_static      = label_static;
 
@@ -720,7 +782,7 @@ xitk_widget_t *xitk_labelbutton_create (xitk_widget_list_t *wl,
 		  (xitk_skin_get_coord_x(skonfig, b->skin_element_name)),
 		  (xitk_skin_get_coord_y(skonfig, b->skin_element_name)),
 		   b->skin_element_name,
-		  (xitk_image_load_image(b->imlibdata, xitk_skin_get_skin_filename(skonfig, b->skin_element_name))),
+		  (xitk_skin_get_image(skonfig, xitk_skin_get_skin_filename(skonfig, b->skin_element_name))),
 		  (xitk_skin_get_label_color(skonfig, b->skin_element_name)),
 		  (xitk_skin_get_label_color_focus(skonfig, b->skin_element_name)),
 		  (xitk_skin_get_label_color_click(skonfig, b->skin_element_name)),
