@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: menu.c,v 1.13 2003/07/18 21:34:40 f1rmb Exp $
+ * $Id: menu.c,v 1.22 2004/04/13 19:55:50 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -29,6 +29,7 @@
 #include "_xitk.h"
 
 #undef DEBUG_MENU
+#undef DUMP_MENU
 
 static void _menu_create_menu_from_branch(menu_node_t *, xitk_widget_t *, int, int);
 
@@ -89,6 +90,7 @@ static xitk_menu_entry_t *_menu_build_menu_entry(xitk_menu_entry_t *me, char *na
 
   mentry            = (xitk_menu_entry_t *) xitk_xmalloc(sizeof(xitk_menu_entry_t));
   mentry->menu      = strdup((name) ? buffer : me->menu);
+  mentry->shortcut  = (me->shortcut) ? strdup(me->shortcut) : NULL;
   mentry->type      = (me->type) ? strdup(me->type) : NULL;
   mentry->cb        = me->cb;
   mentry->user_data = me->user_data;
@@ -143,42 +145,64 @@ static int _menu_is_title(xitk_menu_entry_t *me) {
     return 1;
   return 0;
 }
+static int _menu_branch_have_check(menu_node_t *branch) {
+  while(branch) {
+    if(_menu_is_check(branch->menu_entry) || _menu_is_checked(branch->menu_entry))
+      return 1;
+    
+    branch = branch->next;
+  }
+  return 0;
+}
+static int _menu_branch_have_branch(menu_node_t *branch) {
+  while(branch) {
+    if(_menu_is_branch(branch->menu_entry))
+      return 1;
+    
+    branch = branch->next;
+  }
+  return 0;
+}
 
-#ifdef DEBUG_MENU
+#ifdef DUMP_MENU
+#ifdef	__GNUC__
+#define prints(fmt, args...) do { int j; for(j = 0; j < i; j++) printf(" "); printf(fmt, ##args); } while(0)
+#else
+#define prints(...) do { int j; for(j = 0; j < i; j++) printf(" "); printf(__VA_ARGS__); } while(0)
+#endif
+static int i = 0;
+static void _menu_dump_branch(menu_node_t *branch) {
+  
+  while(branch) {
+    prints("[%s]\n", branch->menu_entry->menu);
+
+    if(branch->branch) {
+      i += 3;
+      _menu_dump_branch(branch->branch);
+    }
+    
+    branch = branch->next;
+  }
+  i -= 3;
+}
 static void _menu_dump(menu_private_data_t *private_data) {
   menu_tree_t *mtree = private_data->mtree;
   menu_node_t *me;
   
+  printf("\n");
   me = mtree->first;
   while(me) {
     xitk_menu_entry_t *entry = me->menu_entry;
     menu_node_t *branch = me->branch;
-
+    
     if(_menu_is_separator(entry))
-      printf("===================\n");
+      prints("===================\n");
     else
-      printf("[%s]\n", entry->menu);
-
-    while(branch) {
-      xitk_menu_entry_t *sentry = branch->menu_entry;
-      menu_node_t *sbranch = branch->branch;
-
-      if(_menu_is_separator(sentry))
-	printf("  ===================\n");
-      else
-	printf("  [%s]\n", sentry->menu);
-
-      while(sbranch) {
-	xitk_menu_entry_t *ssentry = sbranch->menu_entry;
-
-	if(_menu_is_separator(ssentry))
-	  printf("    ===================\n");
-	else
-	  printf("    [%s]\n", ssentry->menu);
-
-	sbranch = sbranch->next;
-      }
-      
+      prints("[%s]\n", entry->menu);
+    
+    if(branch) {
+      i += 3;
+      _menu_dump_branch(branch);
       branch = branch->next;
     }
     
@@ -187,149 +211,6 @@ static void _menu_dump(menu_private_data_t *private_data) {
 }
 #endif
 
-static void _menu_scissor(xitk_widget_t *w,
-			  menu_private_data_t *private_data, xitk_menu_entry_t *me) {
-  char         *c;
-  int           sub, offset;
-  menu_tree_t  *mtree;
-  menu_node_t  *main_trunk = NULL, *branch = NULL, *subbranch = NULL;
-  
-  mtree        = (menu_tree_t *) xitk_xmalloc(sizeof(menu_tree_t));
-  mtree->first = NULL;
-  mtree->cur   = NULL;
-  mtree->last  = NULL;
-  
-  while(me->menu) {
-
-    sub = offset = 0;
-    c = me->menu;
-    
-    if(_menu_is_branch(me)) {
-      int     off = 0, same_branch = 0;
-      char   *cbranch = strchr(c, '/');
-
-      if(cbranch && (*(cbranch - 1) == '\\'))
-	cbranch = NULL;
-      
-      if(cbranch) {
-	cbranch++;
-	off = cbranch - c;
-
-#ifdef DEBUG_MENU
-	printf("  /[%s] << SUBBRANCH (add_in_branch, create_new_branch) \n", cbranch);
-#endif
-
-        if(branch == NULL)
-	  branch = _menu_add_to_node_branch(w, &main_trunk, me, cbranch);
-	else
-	  branch = _menu_append_to_node(w, &branch, me, cbranch);
-	
-	me++;
-	do {
-	  same_branch = 0;
-	  
-	  if(!strncmp(me->menu + off, cbranch, strlen(cbranch))) {
-
-#ifdef DEBUG_MENU
-	    printf("    |\n");
-	    if(_menu_is_separator(me)) {
-	      printf("    +- ========================== << SUBSEP (add_in_branch)\n");
-	    }
-	    else {
-	      printf("    +- [%s] << SUBENTRY (add_in_branch)\n", 
-		     me->menu + off + strlen(cbranch) + 1);
-	    }
-#endif
-
-	    if(subbranch == NULL)
-	      subbranch = _menu_add_to_node_branch(w, &branch, me, (me->menu + off + strlen(cbranch) + 1));
-	    else
-	      subbranch = _menu_append_to_node(w, &subbranch, me, (me->menu + off + strlen(cbranch) + 1));
-	    me++;
-	    same_branch = 1;
-	  }
-	  
-	} while(same_branch);
-
-#ifdef DEBUG_MENU
-	printf("(back_to_parent_branch)\n");
-#endif
-
-	subbranch = NULL;
-	
-	continue;
-      }
-      else {
-
-#ifdef DEBUG_MENU
-	printf("[%s]> << NEWBRANCH (add_in_trunk, create_new_branch)\n", me->menu);
-#endif
-
-	main_trunk = mtree->cur = _menu_append_to_node(w, &main_trunk, me, me->menu);
-	branch = subbranch = NULL;
-	me++;
-	continue;
-      }
-    }
-    
-    while( (c = strchr(c, '/')) && (c && (*(c - 1) != '\\')) ) {
-      offset = c - me->menu + 1;
-      sub++;
-      c++;
-    }
-
-    if(sub) {
-      if(sub > 1) {
-
-#ifdef DEBUG_MENU
-	printf("WHAT ?\n");
-#endif
-
-      }
-      else {
-
-#ifdef DEBUG_MENU
-	printf("  /[%s] SUB (add_in_branch)\n", me->menu + offset);
-#endif
-	if(branch == NULL)
-	  branch = _menu_add_to_node_branch(w, &main_trunk, me, (me->menu + offset));
-	else
-	  branch = _menu_append_to_node(w, &branch, me, (me->menu + offset));
-	
-      }
-    }
-    else {
-
-#ifdef DEBUG_MENU
-      if(_menu_is_separator(me)) {
-	printf("========================= << MAIN (add_in_trunk)\n");
-      }
-      else {
-	printf("(go_to_trunk)\n");
-      }
-#endif
-
-      if(main_trunk)
-	mtree->cur = main_trunk;
-      
-      branch = subbranch = NULL;
-
-#ifdef DEBUG_MENU
-      printf("[%s] << MAIN (add_in_trunk)\n", me->menu);
-#endif
-      
-      main_trunk = mtree->last = mtree->cur = _menu_append_to_node(w, &mtree->cur, me, me->menu);
-      
-      if(mtree->first == NULL)
-      	mtree->first = main_trunk;
-    }
-    
-    me++;
-  }
-  
-  private_data->mtree = mtree;  
-}
-
 static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_result_t *result) {
   int retval = 0;
 
@@ -398,17 +279,46 @@ static menu_node_t *_menu_get_wider_menuitem_node(menu_node_t *branch) {
   menu_node_t  *max = NULL;
   int           len = 0;
   
-  
   while(me) {
     if((!_menu_is_separator(me->menu_entry)) && 
-       (me->menu_entry->menu && (strlen(me->menu_entry->menu) > len))) {
+       (me->menu_entry->menu && 
+	((strlen(me->menu_entry->menu) > len)))) {
       len = strlen(me->menu_entry->menu);
       max = me;
     }
     
     me = me->next;
   }
+
+  return max;
+}
+static int _menu_branch_have_shortcut(menu_node_t *branch) {
+  menu_node_t  *me = branch;
   
+  while(me) {
+    if((!_menu_is_separator(me->menu_entry)) && me->menu_entry->shortcut)
+      return 1;
+    me = me->next;
+  }
+  
+  return 0;
+}
+static menu_node_t *_menu_get_wider_shortcut_node(menu_node_t *branch) {
+  menu_node_t  *me = branch;
+  menu_node_t  *max = NULL;
+  int           len = 0;
+  
+  while(me) {
+    if((!_menu_is_separator(me->menu_entry)) && 
+       (me->menu_entry->shortcut && 
+	((strlen(me->menu_entry->shortcut) > len)))) {
+      len = strlen(me->menu_entry->shortcut);
+      max = me;
+    }
+    
+    me = me->next;
+  }
+
   return max;
 }
 
@@ -462,6 +372,8 @@ static void _menu_destroy_ntree(menu_node_t **mn) {
   if((*mn)->menu_entry) {
     if((*mn)->menu_entry->menu)
       free((*mn)->menu_entry->menu);
+    if((*mn)->menu_entry->shortcut)
+      free((*mn)->menu_entry->shortcut);
     if((*mn)->menu_entry->type)
       free((*mn)->menu_entry->type);
     free((*mn)->menu_entry);
@@ -515,34 +427,48 @@ static void _menu_click_cb(xitk_widget_t *w, void *data) {
   menu_private_data_t  *private_data = (menu_private_data_t *) widget->private_data;
   
   if(_menu_is_branch(me->menu_entry)) {
-
+    
     if(me->branch) {
-      if(private_data->curbranch && (me->branch != private_data->curbranch)) {
-	int   wx, wy, x, y;
+      menu_node_t   *mebr = me;
+      
+      while(mebr->branch) {
 	
-	xitk_window_get_window_position(private_data->imlibdata, 
-					me->menu_window->xwin, &wx, &wy, NULL, NULL);
-	xitk_get_widget_pos(w, &x, &y);
-	
-	x += (xitk_get_widget_width(w)) + wx;
-	x -= 10;
-	y += wy;
-	
-	_menu_destroy_subs(private_data, me->menu_window);
-	_menu_create_menu_from_branch(me->branch, widget, x, y);
-      }
-      else {
-	
-	if(private_data->curbranch) {
-	  XLOCK(private_data->curbranch->menu_window->display);
-	  XRaiseWindow(private_data->curbranch->menu_window->display, 
-		       xitk_window_get_window(private_data->curbranch->menu_window->xwin));
-	  XSetInputFocus(private_data->curbranch->menu_window->display, 
-			 (xitk_window_get_window(private_data->curbranch->menu_window->xwin)),
-			 RevertToParent, CurrentTime);
-	  XUNLOCK(private_data->curbranch->menu_window->display);
+	if(private_data->curbranch && (mebr->branch != private_data->curbranch)) {
+	  int   wx = 0, wy = 0, x = 0, y = 0;
+	  
+	  xitk_window_get_window_position(private_data->imlibdata, 
+					  mebr->menu_window->xwin, &wx, &wy, NULL, NULL);
+	  xitk_get_widget_pos(mebr->button, &x, &y);
+	  
+	  x += (xitk_get_widget_width(mebr->button)) + wx;
+	  x -= 10;
+	  y += wy;
+	  
+	  
+	  if(me->branch && (me->branch == mebr->branch))
+	    _menu_destroy_subs(private_data, mebr->menu_window);
+	    
+	  _menu_create_menu_from_branch(mebr->branch, mebr->widget, x, y);
 	}
+	else {
+	  
+	  if(private_data->curbranch) {
+	    XLOCK(private_data->curbranch->menu_window->display);
+	    XRaiseWindow(private_data->curbranch->menu_window->display, 
+			 xitk_window_get_window(private_data->curbranch->menu_window->xwin));
+	    XSetInputFocus(private_data->curbranch->menu_window->display, 
+			   (xitk_window_get_window(private_data->curbranch->menu_window->xwin)),
+			   RevertToParent, CurrentTime);
+	    XUNLOCK(private_data->curbranch->menu_window->display);
+	  }
+	}
+
+	mebr = mebr->branch;
       }
+
+      private_data->curbranch = me->branch;
+
+      return;
     }
     else {
       menu_node_t *mnode = (private_data->curbranch && private_data->curbranch->prev) ? 
@@ -622,7 +548,7 @@ static void __menu_find_branch_by_name(menu_node_t *mnode, menu_node_t **fnode,
 static menu_node_t *_menu_find_branch_by_name(menu_node_t *mnode, char *name) {
   menu_node_t *m = NULL;
   
-  __menu_find_branch_by_name(mnode, &m, name);
+  if (mnode) __menu_find_branch_by_name(mnode, &m, name);
 
   return m;
 }
@@ -645,16 +571,24 @@ void xitk_menu_add_entry(xitk_widget_t *w, xitk_menu_entry_t *me) {
       
       o = c = new_entry = buffer;
 
-      branch = private_data->mtree->first;
-      while( (c = strchr(c, '/')) && (c && (*(c - 1) != '\\')) && branch) {
+      if(!(branch = private_data->mtree->first)) {
+#ifdef DEBUG_MENU
+	printf("FIRST ENTRY OF MENU\n");
+#endif
+	private_data->mtree->last = private_data->mtree->cur = private_data->mtree->first = 
+	  _menu_append_to_node(w, &(private_data->mtree->first), me, new_entry);
+	return;
+      }
+
+      while( (c = strchr(c, '/')) && (c && (*(c - 1) != '\\'))/*  && branch */) {
 	*c = '\0';
-
+	
 	in_trunk = 0;
-
+	
 #ifdef DEBUG_MENU
 	printf("LOOKING FOR '%s': ", o);
 #endif	
-
+	
 	/* Try to find branch */
 	branch = _menu_find_branch_by_name(branch, o);
 
@@ -707,15 +641,36 @@ void xitk_menu_add_entry(xitk_widget_t *w, xitk_menu_entry_t *me) {
 #ifdef DEBUG_MENU
       printf("******* BYE ************\n");
 #endif
-    }      
+#ifdef DUMP_MENU
+      _menu_dump(private_data);
+#endif
+    }
   }
 }
 
+static void _menu_scissor(xitk_widget_t *w,
+			  menu_private_data_t *private_data, xitk_menu_entry_t *me) {
+  menu_tree_t  *mtree;
+  
+  mtree        = (menu_tree_t *) xitk_xmalloc(sizeof(menu_tree_t));
+  mtree->first = NULL;
+  mtree->cur   = NULL;
+  mtree->last  = NULL;
+  
+  private_data->mtree = mtree;  
+  
+  while(me->menu) {
+    xitk_menu_add_entry(w, me);
+    me++;
+  }
+  
+}
+
 static void _menu_create_menu_from_branch(menu_node_t *branch, xitk_widget_t *w, int x, int y) {
   menu_private_data_t        *private_data;
   int                         bentries, bsep, btitle, rentries;
   menu_node_t                *maxnode, *me;
-  int                         maxlen, wwidth, wheight, swidth, sheight;
+  int                         maxlen, wwidth, wheight, swidth, sheight, shortcutlen = 0, shortcutpos = 0;
   xitk_font_t                *fs;
   static xitk_window_t       *xwin;
   menu_window_t              *menu_window;
@@ -743,10 +698,24 @@ static void _menu_create_menu_from_branch(menu_node_t *branch, xitk_widget_t *w,
 
   xitk_font_set_font(fs, private_data->widget->wl->gc);
   maxlen = xitk_font_get_string_length(fs, maxnode->menu_entry->menu);
+
+  if(xitk_get_menu_shortcuts_enability() && _menu_branch_have_shortcut(branch)) {
+    xitk_font_t *short_font;
+    short_font = xitk_font_load_font(private_data->imlibdata->x.disp, DEFAULT_FONT_12);
+    shortcutlen = xitk_font_get_string_length(short_font, (_menu_get_wider_shortcut_node(branch))->menu_entry->shortcut);
+    maxlen += shortcutlen + 15;
+    xitk_font_unload_font(short_font);
+  }
+
   xitk_font_unload_font(fs);
-  
+
   wwidth = maxlen + 40;
+
+  if(_menu_branch_have_check(branch) || _menu_branch_have_branch(branch))
+    wwidth += 20;
   wheight = (rentries * 20) + (bsep * 2) + (btitle * 2);
+
+  shortcutpos = (wwidth - shortcutlen) - 15;
   
   XLOCK(private_data->imlibdata->x.disp);
   swidth = DisplayWidth(private_data->imlibdata->x.disp, 
@@ -762,7 +731,8 @@ static void _menu_create_menu_from_branch(menu_node_t *branch, xitk_widget_t *w,
   
   xwin = xitk_window_create_simple_window(private_data->imlibdata, 
 					  x, y, wwidth + 2, wheight + 2);
-  
+  xitk_set_wm_window_type(xitk_window_get_window(xwin), WINDOW_TYPE_DOCK);
+
   if(bsep || btitle) {
     int width, height;
     
@@ -784,7 +754,8 @@ static void _menu_create_menu_from_branch(menu_node_t *branch, xitk_widget_t *w,
   while(me) {
 
     me->menu_window      = menu_window;
-
+    me->button           = NULL;
+    
     if(_menu_is_title(me->menu_entry)) {
 
       if(bg && (!got_title)) {
@@ -816,9 +787,9 @@ static void _menu_create_menu_from_branch(menu_node_t *branch, xitk_widget_t *w,
 		       1, 1, wwidth , 20);
 	XSetForeground(private_data->imlibdata->x.disp, private_data->widget->wl->gc, cfg);
 	xitk_font_draw_string(fs, bg->pixmap, 
-		    private_data->widget->wl->gc, 
-		    5, 1+ ((20+asc+des)>>1)-des, 
-		    me->menu_entry->menu, strlen(me->menu_entry->menu));
+			      private_data->widget->wl->gc, 
+			      5, 1+ ((20+asc+des)>>1)-des, 
+			      me->menu_entry->menu, strlen(me->menu_entry->menu));
 	XUNLOCK(private_data->imlibdata->x.disp);
 	
 	xitk_font_unload_font(fs);
@@ -852,7 +823,11 @@ static void _menu_create_menu_from_branch(menu_node_t *branch, xitk_widget_t *w,
 								     "Black", "Black", "White", 
 								     DEFAULT_BOLD_FONT_12)));
       btn->type |= WIDGET_GROUP | WIDGET_GROUP_MENU;
-
+      me->button = btn;
+      
+      if(xitk_get_menu_shortcuts_enability()  && me->menu_entry->shortcut)
+	xitk_labelbutton_change_shortcut_label(btn, me->menu_entry->shortcut, shortcutpos, DEFAULT_FONT_12);
+      
       wimage = xitk_get_widget_foreground_skin(btn);
       if(wimage) {
 	draw_flat_three_state(private_data->imlibdata, wimage);				
@@ -1001,26 +976,24 @@ xitk_widget_t *xitk_noskin_menu_create(xitk_widget_list_t *wl,
     abort();
   }
 
-  _menu_scissor(mywidget, private_data, m->menu_tree);
+  mywidget->private_data                 = private_data;
+  mywidget->wl                           = wl;
+  mywidget->type                         = WIDGET_GROUP | WIDGET_GROUP_WIDGET | WIDGET_GROUP_MENU | WIDGET_FOCUSABLE;
   
-#ifdef DEBUG_MENU
+  _menu_scissor(mywidget, private_data, m->menu_tree);
+
+#ifdef DEBUG_DUMP
   _menu_dump(private_data);
 #endif
 
-  mywidget->private_data                 = private_data;
-  
-  mywidget->wl                           = wl;
-  
   mywidget->enable                       = 1;
   mywidget->running                      = 1;
   mywidget->visible                      = 0;
   mywidget->have_focus                   = FOCUS_RECEIVED;
   mywidget->imlibdata                    = private_data->imlibdata;
   mywidget->x = mywidget->y = mywidget->width = mywidget->height = 0;
-  mywidget->type                         = WIDGET_GROUP | WIDGET_GROUP_WIDGET | WIDGET_GROUP_MENU | WIDGET_FOCUSABLE;
   mywidget->event                        = notify_event;
   mywidget->tips_timeout                 = 0;
   mywidget->tips_string                  = NULL;
-
   return mywidget;
 }
