@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrlbrowser.c,v 1.70 2003/07/25 22:52:57 f1rmb Exp $
+ * $Id: mrlbrowser.c,v 1.81 2004/04/13 19:55:50 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -32,7 +32,10 @@
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>                                                      
-#include <assert.h>
+
+#ifdef HAVE_ALLOCA_H   
+#include <alloca.h>
+#endif
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -194,7 +197,7 @@ static void mrlbrowser_filter_mrls(mrlbrowser_private_data_t *private_data) {
 
     private_data->mc->mrls_to_disp = 0;
 
-    xitk_strdupa(filter_ends, private_data->mrl_filters[private_data->filter_selected]->ending);
+    filter_ends = strdup(private_data->mrl_filters[private_data->filter_selected]->ending);
     xitk_strdupa(p, filter_ends);
 
     for(i = 0; i < private_data->mrls_num; i++) {
@@ -222,10 +225,15 @@ static void mrlbrowser_filter_mrls(mrlbrowser_private_data_t *private_data) {
 	      private_data->mc->mrls_to_disp++;
 	    }
 	  }
-	  xitk_strdupa(filter_ends, p);
+	  free(filter_ends);
+	  filter_ends = strdup(p);
 	}
       }    
     }
+    
+    if(filter_ends)
+      free(filter_ends);
+    
   }
   else { /* no filtering */
 
@@ -366,7 +374,7 @@ static void mrlbrowser_grab_mrls(xitk_widget_t *w, void *data) {
     update_current_origin(private_data);
     mrlbrowser_create_enlighted_entries(private_data);
     xitk_browser_update_list(private_data->mrlb_list, 
-			     (const char* const*)private_data->mc->mrls_disp, 
+			     (const char* const*)private_data->mc->mrls_disp, NULL,
 			     private_data->mc->mrls_to_disp, 0);
   }
 }
@@ -723,17 +731,19 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
     {
       int x, y;
       int i = 0;
-      int dir;
+      int dir, max;
       
-      x = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name_ip);
-      y = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name_ip);
+      x   = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name_ip);
+      y   = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name_ip);
       dir = xitk_skin_get_direction(skonfig, private_data->skin_element_name_ip);
-    
+      max = xitk_skin_get_max_buttons(skonfig, private_data->skin_element_name_ip);
+
       switch(dir) {
       case DIRECTION_UP:
       case DIRECTION_DOWN:
-	while(private_data->autodir_plugins[i] != NULL) {
-	  
+	while((max && ((i < max) && (private_data->autodir_plugins[i] != NULL))) || 
+	      (!max && private_data->autodir_plugins[i] != NULL)) {
+
 	  (void) xitk_set_widget_pos(private_data->autodir_plugins[i], x, y);
 	  
 	  if(dir == DIRECTION_DOWN)
@@ -746,7 +756,8 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
 	break;
       case DIRECTION_LEFT:
       case DIRECTION_RIGHT:
-	while(private_data->autodir_plugins[i] != NULL) {
+	while((max && ((i < max) && (private_data->autodir_plugins[i] != NULL))) || 
+	      (!max && private_data->autodir_plugins[i] != NULL)) {
 	  
 	  (void) xitk_set_widget_pos(private_data->autodir_plugins[i], x, y);
 	  
@@ -760,6 +771,12 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
 	break;
       }
 
+      if(max) {
+	while(private_data->autodir_plugins[i] != NULL) {
+	  xitk_disable_and_hide_widget(private_data->autodir_plugins[i]);
+	  i++;
+	}
+      }
     }
     
     xitk_paint_widget_list(private_data->widget_list);
@@ -829,7 +846,7 @@ static void mrlbrowser_select_mrl(mrlbrowser_private_data_t *private_data,
     update_current_origin(private_data);
     mrlbrowser_create_enlighted_entries(private_data);
     xitk_browser_update_list(private_data->mrlb_list, 
-			     (const char* const*)private_data->mc->mrls_disp, 
+			     (const char* const*)private_data->mc->mrls_disp, NULL,
 			     private_data->mc->mrls_to_disp, 0);
     
   }
@@ -903,7 +920,7 @@ static void combo_filter_select(xitk_widget_t *w, void *data, int select) {
   private_data->filter_selected = select;
   mrlbrowser_create_enlighted_entries(private_data);
   xitk_browser_update_list(private_data->mrlb_list, 
-  			  (const char* const*) private_data->mc->mrls_disp, 
+			   (const char* const*) private_data->mc->mrls_disp, NULL,
   			   private_data->mc->mrls_to_disp, 0);
 }
 
@@ -922,21 +939,13 @@ static void mrlbrowser_handle_event(XEvent *event, void *data) {
     break;
     
   case KeyPress: {
-    XKeyEvent      mykeyevent;
-    KeySym         mykey;
-    char           kbuf[256];
-    int            len;
+    KeySym         mkey;
     int            modifier;
 
     xitk_get_key_modifier(event, &modifier);
+    mkey = xitk_get_key_pressed(event);
     
-    mykeyevent = event->xkey;
-    
-    XLOCK (private_data->imlibdata->x.disp);
-    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-    XUNLOCK (private_data->imlibdata->x.disp);
-    
-    switch (mykey) {
+    switch (mkey) {
 
     case XK_d: 
     case XK_D:
@@ -961,6 +970,10 @@ static void mrlbrowser_handle_event(XEvent *event, void *data) {
     }
     break;
 
+    case XK_Escape:
+      xitk_mrlbrowser_exit(NULL, (void *)private_data->fbWidget);
+      break;
+      
     }
   }
   break;
@@ -1073,13 +1086,18 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
 					  mb->imlibdata->x.visual,
 					  CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 					  &attr);
-  
-  XSetStandardProperties(mb->imlibdata->x.disp, private_data->window, title, title,
-			 None, NULL, 0, &hint);
+
+  XmbSetWMProperties(mb->imlibdata->x.disp, private_data->window, title, title,
+                     NULL, 0, &hint, NULL, NULL);
 
   XSelectInput(mb->imlibdata->x.disp, private_data->window, INPUT_MOTION | KeymapStateMask);
   XUNLOCK(mb->imlibdata->x.disp);
-  
+
+  if(mb->set_wm_window_normal)
+    xitk_set_wm_window_type(private_data->window, WINDOW_TYPE_NORMAL);
+  else
+    xitk_unset_wm_window_type(private_data->window, WINDOW_TYPE_NORMAL);
+
   /*
    * layer above most other things, like gnome panel
    * WIN_LAYER_ABOVE_DOCK  = 10
@@ -1118,6 +1136,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
     xclasshint->res_name  = mb->resource_name ? mb->resource_name : "xitk mrl browser";
     xclasshint->res_class = mb->resource_class ? mb->resource_class : "xitk";
     XSetClassHint(mb->imlibdata->x.disp, private_data->window, xclasshint);
+    XFree(xclasshint);
   }
   XUNLOCK(mb->imlibdata->x.disp);
   
@@ -1126,7 +1145,11 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   if (wm_hint != NULL) {
     wm_hint->input         = True;
     wm_hint->initial_state = NormalState;
-    wm_hint->flags         = InputHint | StateHint;
+    if(mb->icon) {
+      wm_hint->icon_pixmap = *mb->icon;
+      wm_hint->flags       = IconPixmapHint;
+    }
+    wm_hint->flags         |= InputHint | StateHint;
     XSetWMHints(mb->imlibdata->x.disp, private_data->window, wm_hint);
     XFree(wm_hint);
   }
@@ -1219,13 +1242,14 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   {
     int x, y;
     int i = 0;
-    int dir;
+    int dir, max;
     
     private_data->skin_element_name_ip = strdup(mb->ip_name.button.skin_element_name);
-    x = xitk_skin_get_coord_x(skonfig, mb->ip_name.button.skin_element_name);
-    y = xitk_skin_get_coord_y(skonfig, mb->ip_name.button.skin_element_name);
+    x   = xitk_skin_get_coord_x(skonfig, mb->ip_name.button.skin_element_name);
+    y   = xitk_skin_get_coord_y(skonfig, mb->ip_name.button.skin_element_name);
     dir = xitk_skin_get_direction(skonfig, mb->ip_name.button.skin_element_name);
-    
+    max = xitk_skin_get_max_buttons(skonfig, mb->ip_name.button.skin_element_name);
+
     while(mb->ip_availables[i] != NULL) {
 
       lb.button_type    = CLICK_BUTTON;
@@ -1264,7 +1288,10 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
 	  x -= (xitk_get_widget_width(private_data->autodir_plugins[i]) + 1);
 	break;
       }
-
+      
+      if(max && (i >= max))
+	xitk_disable_and_hide_widget(private_data->autodir_plugins[i]);
+      
       i++;
     }
     if(i)
@@ -1303,7 +1330,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   mywidget->tips_string        = NULL;
 
   xitk_browser_update_list(private_data->mrlb_list, 
-			   (const char* const*)private_data->mc->mrls_disp, 
+			   (const char* const*)private_data->mc->mrls_disp, NULL,
 			   private_data->mrls_num, 0);
 
   XLOCK (mb->imlibdata->x.disp);
