@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skin.c,v 1.24 2003/07/14 22:44:10 f1rmb Exp $
+ * $Id: skin.c,v 1.29 2004/04/13 19:55:50 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -28,7 +28,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <assert.h>
 #include <pthread.h>
 #include <errno.h>
 
@@ -40,6 +39,55 @@
  *                                     PRIVATES
  ************************************************************************************/
 
+static pthread_mutex_t  skin_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+/*
+ *
+ */
+static void skin_load_to_cache(xitk_skin_config_t *skonfig, char *pixmap) {
+
+  if(skonfig && pixmap) {
+    xitk_image_t  *image  = xitk_image_load_image(skonfig->im, pixmap);
+    cache_entry_t *cache  = skonfig->cache;
+    cache_entry_t *ncache = NULL;
+    
+    if(image) {
+      ncache           = (cache_entry_t *) xitk_xmalloc(sizeof(cache_entry_t));
+      ncache->image    = image;
+      ncache->filename = strdup(pixmap);
+      ncache->next     = NULL;
+    }
+    else
+      return;
+    
+    if(!cache) {
+      skonfig->cache = ncache;
+    }
+    else {
+      while(cache->next)
+	cache = cache->next;
+      
+      cache->next  = ncache;
+    }
+  }
+}
+
+static void skin_free_cache(xitk_skin_config_t *skonfig) {
+  cache_entry_t *pcache, *cache  = skonfig->cache;
+  
+  while(cache) {
+    pcache = cache;
+    
+    xitk_image_free_image(skonfig->im, &(cache->image));
+    free(cache->filename);
+    
+    cache  = cache->next;
+    free(pcache);
+  }
+
+  skonfig->cache = NULL;
+}
+
 /*
  *
  */
@@ -48,7 +96,7 @@ static char *_get_expanded_command(xitk_skin_config_t *skonfig, char *cmd) {
   char *ret = NULL;
   char  buf[BUFSIZ], buf2[BUFSIZ], var[BUFSIZ];
   
-  assert(skonfig != NULL);
+  ABORT_IF_NULL(skonfig);
 
   if(cmd) {
 
@@ -150,6 +198,7 @@ static void _nullify_me(xitk_skin_element_t *s) {
   s->font            = NULL;
   s->browser_entries = -2;
   s->direction       = DIRECTION_LEFT; /* Compatibility */
+  s->max_buttons     = 0;
 }
 
 /*
@@ -171,7 +220,7 @@ static int _is_there_char(const char *str, int c) {
  */
 static int skin_begin_section(xitk_skin_config_t *skonfig) {
 
-  assert(skonfig != NULL);
+  ABORT_IF_NULL(skonfig);
 
   return _is_there_char(skonfig->ln, '{');
 }
@@ -181,7 +230,7 @@ static int skin_begin_section(xitk_skin_config_t *skonfig) {
  */
 static int skin_end_section(xitk_skin_config_t *skonfig) {
 
-  assert(skonfig != NULL);
+  ABORT_IF_NULL(skonfig);
 
   return _is_there_char(skonfig->ln, '}');
 }
@@ -207,7 +256,7 @@ static int istriplet(char *c) {
 static void skin_clean_eol(xitk_skin_config_t *skonfig) {
   char *p;
 
-  assert(skonfig != NULL);
+  ABORT_IF_NULL(skonfig);
 
   p = skonfig->ln;
 
@@ -239,7 +288,8 @@ static void skin_clean_eol(xitk_skin_config_t *skonfig) {
  */
 static void skin_get_next_line(xitk_skin_config_t *skonfig) {
 
-  assert(skonfig != NULL && skonfig->fd != NULL);
+  ABORT_IF_NULL(skonfig);
+  ABORT_IF_NULL(skonfig->fd);
   
  __get_next_line:
 
@@ -259,31 +309,6 @@ static void skin_get_next_line(xitk_skin_config_t *skonfig) {
   skin_clean_eol(skonfig);
 }
 
-/*
- * Return 0/1 from char value (valids are 1/0, true/false, 
- * yes/no, on/off. Case isn't checked.
- */
-static int skin_get_bool_value(const char *val) {
-  static struct {
-    const char *str;
-    int value;
-  } bools[] = {
-    { "1",     1 }, { "true",  1 }, { "yes",   1 }, { "on",    1 },
-    { "0",     0 }, { "false", 0 }, { "no",    0 }, { "off",   0 },
-    { NULL,    0 }
-  };
-  int i;
-  
-  assert(val != NULL);
-
-  for(i = 0; bools[i].str != NULL; i++) {
-    if(!(strcasecmp(bools[i].str, val)))
-      return bools[i].value;
-  }
-
-  return 0;
-}
-
 /*
  * Return alignement value.
  */
@@ -299,7 +324,7 @@ static int skin_get_align_value(const char *val) {
   };
   int i;
   
-  assert(val != NULL);
+  ABORT_IF_NULL(val);
   
   for(i = 0; aligns[i].str != NULL; i++) {
     if(!(strcasecmp(aligns[i].str, val)))
@@ -325,7 +350,7 @@ static int skin_get_direction(const char *val) {
   };
   int   i;
 
-  assert(val != NULL);
+  ABORT_IF_NULL(val);
   
   for(i = 0; directions[i].str != NULL; i++) {
     if(!(strcasecmp(directions[i].str, val)))
@@ -341,7 +366,7 @@ static int skin_get_direction(const char *val) {
  */
 static void skin_set_pos_to_value(char **p) {
   
-  assert(*p != NULL);
+  ABORT_IF_NULL(*p);
 
   while(*(*p) != '\0' && *(*p) != '=' && *(*p) != ':' && *(*p) != '{') ++(*p);
   while(*(*p) == '=' || *(*p) == ':' || *(*p) == ' ' || *(*p) == '\t') ++(*p);
@@ -354,7 +379,7 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
   char *p;
   int  brace_offset;
 
-  assert(skonfig != NULL);
+  ABORT_IF_NULL(skonfig);
 
   if((brace_offset = skin_begin_section(skonfig)) >= 0) {
     *(skonfig->ln + brace_offset) = '\0';
@@ -400,6 +425,7 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	  skin_set_pos_to_value(&p);
 	  skonfig->celement->pixmap_pad = (char *) xitk_xmalloc(strlen(skonfig->path) + strlen(p) + 2);
 	  sprintf(skonfig->celement->pixmap_pad, "%s/%s", skonfig->path, p);
+	  skin_load_to_cache(skonfig, skonfig->celement->pixmap_pad);
 	}
 	else if(!strncasecmp(skonfig->ln, "radius", 6)) {
 	  skin_set_pos_to_value(&p);
@@ -443,7 +469,7 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	}
 	else if(!strncasecmp(skonfig->ln, "animation", 9)) {
 	  skin_set_pos_to_value(&p);
-	  skonfig->celement->animation = skin_get_bool_value(p);
+	  skonfig->celement->animation = xitk_get_bool_value(p);
 	}
 	else if(!strncasecmp(skonfig->ln, "length", 6)) {
 	  skin_set_pos_to_value(&p);
@@ -453,10 +479,11 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	  skin_set_pos_to_value(&p);
 	  skonfig->celement->pixmap_font = (char *) xitk_xmalloc(strlen(skonfig->path) + strlen(p) + 2);
 	  sprintf(skonfig->celement->pixmap_font, "%s/%s", skonfig->path, p);
+	  skin_load_to_cache(skonfig, skonfig->celement->pixmap_font);
 	}
 	else if(!strncasecmp(skonfig->ln, "static", 6)) {
 	  skin_set_pos_to_value(&p);
-	  skonfig->celement->staticity = skin_get_bool_value(p);
+	  skonfig->celement->staticity = xitk_get_bool_value(p);
 	}
 	else if(!strncasecmp(skonfig->ln, "align", 5)) {
 	  skin_set_pos_to_value(&p);
@@ -468,7 +495,7 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	}
 	else if(!strncasecmp(skonfig->ln, "print", 5)) {
 	  skin_set_pos_to_value(&p);
-	  skonfig->celement->print = skin_get_bool_value(p);
+	  skonfig->celement->print = xitk_get_bool_value(p);
 	}
 	else if(!strncasecmp(skonfig->ln, "timer", 5)) {
 	  skin_set_pos_to_value(&p);
@@ -498,7 +525,7 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
   char section[256];
   char *p;
   
-  assert(skonfig != NULL);
+  ABORT_IF_NULL(skonfig);
 
   while(skonfig->ln != NULL) {
 
@@ -541,23 +568,27 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 	     goto  __next_subsection;
 	  }
 	  else {
-
-	    if(!strncasecmp(skonfig->ln, "direction", 9)) {
+	    if(!strncasecmp(skonfig->ln, "max_buttons", 11)) {
+	      skin_set_pos_to_value(&p);
+	      s->max_buttons = strtol(p, &p, 10);
+	    }
+	    else if(!strncasecmp(skonfig->ln, "direction", 9)) {
 	      skin_set_pos_to_value(&p);
 	      s->direction = skin_get_direction(p);
 	    }
 	    else if(!strncasecmp(skonfig->ln, "visible", 7)) {
 	      skin_set_pos_to_value(&p);
-	      s->visible = skin_get_bool_value(p);
+	      s->visible = xitk_get_bool_value(p);
 	    }
 	    else if(!strncasecmp(skonfig->ln, "pixmap", 6)) {
 	      skin_set_pos_to_value(&p);
 	      s->pixmap = (char *) xitk_xmalloc(strlen(skonfig->path) + strlen(p) + 2);
 	      sprintf(s->pixmap, "%s/%s", skonfig->path, p);
+	      skin_load_to_cache(skonfig, s->pixmap);
 	    }
 	    else if(!strncasecmp(skonfig->ln, "enable", 6)) {
 	      skin_set_pos_to_value(&p);
-	      s->enable = skin_get_bool_value(p);
+	      s->enable = xitk_get_bool_value(p);
 	    }
 	    
 	  }
@@ -629,7 +660,7 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 static void check_skonfig(xitk_skin_config_t *skonfig) {
   xitk_skin_element_t *s;
 
-  assert(skonfig != NULL);
+  ABORT_IF_NULL(skonfig);
   
   s = skonfig->first;
 
@@ -672,6 +703,7 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
       printf("  color click = '%s'\n", s->color_click);
       printf("  pixmap font = '%s'\n", s->pixmap_font);
       printf("  font        = '%s'\n", s->font);
+      printf("  max_buttons = %d\n", s->max_buttons);
       s = s->next;
     }
 
@@ -695,6 +727,7 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
       printf("  color click = '%s'\n", s->color_click);
       printf("  pixmap font = '%s'\n", s->pixmap_font);
       printf("  font        = '%s'\n", s->font);
+      printf("  max_buttons = %d\n", s->max_buttons);
       s = s->prev;
     }
   }
@@ -704,7 +737,8 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
 static xitk_skin_element_t *skin_lookup_section(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig != NULL && str != NULL);
+  ABORT_IF_NULL(skonfig);
+  ABORT_IF_NULL(str);
 
   s = skonfig->first;
   while(s) {
@@ -725,23 +759,30 @@ static xitk_skin_element_t *skin_lookup_section(xitk_skin_config_t *skonfig, con
 /*
  * Alloc a xitk_skin_config_t* memory area, nullify pointers.
  */
-xitk_skin_config_t *xitk_skin_init_config(void) {
+xitk_skin_config_t *xitk_skin_init_config(ImlibData *im) {
   xitk_skin_config_t *skonfig;
   
   if((skonfig = (xitk_skin_config_t *) xitk_xmalloc(sizeof(xitk_skin_config_t))) == NULL) {
     XITK_DIE("xitk_xmalloc() failed: %s\n", strerror(errno));
   }
-  skonfig->version = -1;
-  skonfig->first = skonfig->last = skonfig->celement = NULL;
-  skonfig->name = skonfig->author = skonfig->date = skonfig->url = 
-    skonfig->load_command = skonfig->unload_command = skonfig->logo = 
-    skonfig->animation = NULL;
+  skonfig->im       = im;
+  skonfig->cache    = NULL;
+  skonfig->version  = -1;
+  skonfig->first    = skonfig->last 
+                    = skonfig->celement 
+                    = NULL;
+  skonfig->name     = skonfig->author 
+                    = skonfig->date 
+                    = skonfig->url
+                    = skonfig->load_command 
+                    = skonfig->unload_command 
+                    = skonfig->logo 
+                    = skonfig->animation 
+                    = NULL;
   skonfig->skinfile = skonfig->path = NULL;
 
   skonfig->ln = skonfig->buf;
   
-  pthread_mutex_init(&skonfig->mutex, NULL);
-
   return skonfig;
 }
 
@@ -751,9 +792,11 @@ xitk_skin_config_t *xitk_skin_init_config(void) {
 void xitk_skin_free_config(xitk_skin_config_t *skonfig) {
   xitk_skin_element_t *s, *c;
 
-  assert(skonfig != NULL);
+  ABORT_IF_NULL(skonfig);
 
+  pthread_mutex_lock(&skin_mutex);
   if(skonfig->celement) {
+    
     s = skonfig->last;
     
     while(s) {
@@ -784,9 +827,10 @@ void xitk_skin_free_config(xitk_skin_config_t *skonfig) {
   XITK_FREE(skonfig->unload_command);
   XITK_FREE(skonfig->skinfile);
 
-  pthread_mutex_destroy(&skonfig->mutex);
+  skin_free_cache(skonfig);
 
   XITK_FREE(skonfig);
+  pthread_mutex_unlock(&skin_mutex);
 }
 
 /*
@@ -795,7 +839,11 @@ void xitk_skin_free_config(xitk_skin_config_t *skonfig) {
 int xitk_skin_load_config(xitk_skin_config_t *skonfig, char *path, char *filename) {
   char buf[2048];
 
-  assert(skonfig != NULL && path != NULL && filename != NULL);
+  ABORT_IF_NULL(skonfig);
+  ABORT_IF_NULL(path);
+  ABORT_IF_NULL(filename);
+
+  pthread_mutex_lock(&skin_mutex);
 
   skonfig->path     = strdup(path);
   skonfig->skinfile = strdup(filename);
@@ -819,12 +867,14 @@ int xitk_skin_load_config(xitk_skin_config_t *skonfig, char *path, char *filenam
   else {
     XITK_WARNING("%s(): Unable to open '%s' file.\n", __FUNCTION__, skonfig->skinfile);
     XITK_FREE(skonfig->skinfile);
+    pthread_mutex_unlock(&skin_mutex);
     return 0;
   }
 
   if(!skonfig->first) {
     XITK_WARNING("%s(): no valid skin element found in '%s/%s'.\n",
 		 __FUNCTION__, skonfig->path, skonfig->skinfile);
+    pthread_mutex_unlock(&skin_mutex);
     return 0;
   }
 
@@ -838,6 +888,8 @@ int xitk_skin_load_config(xitk_skin_config_t *skonfig, char *path, char *filenam
   if(skonfig->load_command)
     xitk_system(0, skonfig->load_command);
 
+  pthread_mutex_unlock(&skin_mutex);
+
   return 1;
 }
 
@@ -863,7 +915,7 @@ void xitk_skin_unload_config(xitk_skin_config_t *skonfig) {
  */
 int xitk_skin_check_version(xitk_skin_config_t *skonfig, int min_version) {
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if(skonfig->version == -1)
     return -1;
@@ -883,7 +935,7 @@ int xitk_skin_check_version(xitk_skin_config_t *skonfig, int min_version) {
 int xitk_skin_get_direction(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
   
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
   
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->direction;
@@ -897,7 +949,7 @@ int xitk_skin_get_direction(xitk_skin_config_t *skonfig, const char *str) {
 int xitk_skin_get_visibility(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
   
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
   
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->visible;
@@ -911,7 +963,7 @@ int xitk_skin_get_visibility(xitk_skin_config_t *skonfig, const char *str) {
 int xitk_skin_get_enability(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
   
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
   
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->enable;
@@ -925,7 +977,7 @@ int xitk_skin_get_enability(xitk_skin_config_t *skonfig, const char *str) {
 int xitk_skin_get_coord_x(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->x;
@@ -939,7 +991,7 @@ int xitk_skin_get_coord_x(xitk_skin_config_t *skonfig, const char *str) {
 int xitk_skin_get_coord_y(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->y;
@@ -953,7 +1005,7 @@ int xitk_skin_get_coord_y(xitk_skin_config_t *skonfig, const char *str) {
 char *xitk_skin_get_label_color(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->color;
@@ -967,7 +1019,7 @@ char *xitk_skin_get_label_color(xitk_skin_config_t *skonfig, const char *str) {
 char *xitk_skin_get_label_color_focus(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->color_focus;
@@ -981,7 +1033,7 @@ char *xitk_skin_get_label_color_focus(xitk_skin_config_t *skonfig, const char *s
 char *xitk_skin_get_label_color_click(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->color_click;
@@ -995,7 +1047,7 @@ char *xitk_skin_get_label_color_click(xitk_skin_config_t *skonfig, const char *s
 int xitk_skin_get_label_length(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->length;
@@ -1009,7 +1061,7 @@ int xitk_skin_get_label_length(xitk_skin_config_t *skonfig, const char *str) {
 int xitk_skin_get_label_animation(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->animation;
@@ -1023,7 +1075,7 @@ int xitk_skin_get_label_animation(xitk_skin_config_t *skonfig, const char *str)
 int xitk_skin_get_label_animation_step(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->animation_step;
@@ -1034,7 +1086,7 @@ int xitk_skin_get_label_animation_step(xitk_skin_config_t *skonfig, const char *
 unsigned long xitk_skin_get_label_animation_timer(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
   
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
   
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->animation_timer;
@@ -1048,7 +1100,7 @@ unsigned long xitk_skin_get_label_animation_timer(xitk_skin_config_t *skonfig, c
 char *xitk_skin_get_label_fontname(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->font;
@@ -1062,7 +1114,7 @@ char *xitk_skin_get_label_fontname(xitk_skin_config_t *skonfig, const char *str)
 int xitk_skin_get_label_printable(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
   
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
   
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->print;
@@ -1076,7 +1128,7 @@ int xitk_skin_get_label_printable(xitk_skin_config_t *skonfig, const char *str)
 int xitk_skin_get_label_staticity(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
   
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
   
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->staticity;
@@ -1090,7 +1142,7 @@ int xitk_skin_get_label_staticity(xitk_skin_config_t *skonfig, const char *str)
 int xitk_skin_get_label_alignment(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
   
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
   
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->align;
@@ -1104,7 +1156,7 @@ int xitk_skin_get_label_alignment(xitk_skin_config_t *skonfig, const char *str)
 char *xitk_skin_get_label_skinfont_filename(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
   
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->pixmap_font;
@@ -1118,7 +1170,7 @@ char *xitk_skin_get_label_skinfont_filename(xitk_skin_config_t *skonfig, const c
 char *xitk_skin_get_skin_filename(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->pixmap;
@@ -1132,7 +1184,7 @@ char *xitk_skin_get_skin_filename(xitk_skin_config_t *skonfig, const char *str)
 char *xitk_skin_get_slider_skin_filename(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     if(s->slider_type)
@@ -1147,7 +1199,7 @@ char *xitk_skin_get_slider_skin_filename(xitk_skin_config_t *skonfig, const char
 int xitk_skin_get_slider_type(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
 
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
 
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return((s->slider_type) ? s->slider_type : XITK_HSLIDER);
@@ -1161,7 +1213,7 @@ int xitk_skin_get_slider_type(xitk_skin_config_t *skonfig, const char *str) {
 int xitk_skin_get_slider_radius(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
   
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
   
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->radius;
@@ -1173,7 +1225,7 @@ int xitk_skin_get_slider_radius(xitk_skin_config_t *skonfig, const char *str) {
  *
  */
 char *xitk_skin_get_animation(xitk_skin_config_t *skonfig) {
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
   
   return skonfig->animation;
 }
@@ -1182,7 +1234,7 @@ char *xitk_skin_get_animation(xitk_skin_config_t *skonfig) {
  *
  */
 char *xitk_skin_get_logo(xitk_skin_config_t *skonfig) {
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
   
   return skonfig->logo;
 }
@@ -1193,28 +1245,57 @@ char *xitk_skin_get_logo(xitk_skin_config_t *skonfig) {
 int xitk_skin_get_browser_entries(xitk_skin_config_t *skonfig, const char *str) {
   xitk_skin_element_t *s;
   
-  assert(skonfig);
+  ABORT_IF_NULL(skonfig);
   
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->browser_entries;
-
+  
   return -1;
 }
 
 /*
  *
  */
-void xitk_skin_lock(xitk_skin_config_t *skonfig) {
-  assert(skonfig);
+xitk_image_t *xitk_skin_get_image(xitk_skin_config_t *skonfig, const char *str) {
+  ABORT_IF_NULL(skonfig);
 
-  pthread_mutex_lock(&skonfig->mutex);
+  if(skonfig->cache) {
+    cache_entry_t *cache = skonfig->cache;
+    
+    while(cache) {
+      if(!strcmp(str, cache->filename))
+	return cache->image;
+      
+      cache = cache->next;
+    }
+  }
+
+  return (xitk_image_load_image(skonfig->im, (char *) str));
+}
+
+int xitk_skin_get_max_buttons(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+  
+  ABORT_IF_NULL(skonfig);
+  
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->max_buttons;
+  
+  return 0;
+}
+
+/*
+ *
+ */
+void xitk_skin_lock(xitk_skin_config_t *skonfig) {
+  ABORT_IF_NULL(skonfig);
+  pthread_mutex_lock(&skin_mutex);
 }
 
 /*
  *
  */
 void xitk_skin_unlock(xitk_skin_config_t *skonfig) {
-  assert(skonfig);
-  
-  pthread_mutex_unlock(&skonfig->mutex);
+  ABORT_IF_NULL(skonfig);
+  pthread_mutex_unlock(&skin_mutex);
 }
