@@ -91,6 +91,7 @@ typedef struct {
 
   /* menu */
   xitk_widget_t        *menu;
+  int                   checked;
 
   xitk_widget_list_t   *widget_list;
   xitk_register_key_t   kreg;
@@ -138,7 +139,7 @@ static int init_test(void) {
     return 0;
   }
 
-  xitk_init(test->display);
+  xitk_init(test->display, 1);
 
   XUnlockDisplay (test->display);
 
@@ -161,9 +162,16 @@ static void deinit_test(void) {
  *
  */
 static void test_end(xitk_widget_t *w, void *data) {
-  XUnmapWindow(test->display, (xitk_window_get_window(test->xwin)));
+  xitk_unregister_event_handler(&test->kreg);
+
   xitk_destroy_widgets(test->widget_list);
+  xitk_window_destroy_window(test->imlibdata, test->xwin);
+
+  xitk_list_free((XITK_WIDGET_LIST_LIST(test->widget_list)));
+  XFreeGC(test->display, (XITK_WIDGET_LIST_GC(test->widget_list)));
+
   free(test->widget_list);
+
   xitk_stop();
 }
 
@@ -466,7 +474,7 @@ static void create_checkbox(void) {
 static void create_tabs(void) {
   xitk_pixmap_t      *bg;
   xitk_tabs_widget_t  t;
-  char               *fontname = "*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*";
+  char               *fontname = "-*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*";
   int                 x = 150, y = 200, w = 300;
   int                 width, height;
   static char        *tabs_labels[] = {
@@ -503,7 +511,7 @@ static void create_tabs(void) {
 static void create_frame(void) {
   xitk_pixmap_t  *bg;
   int             width, height;
-  char           *fontname = "*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
+  char           *fontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
   int              x = 350, y = 50, w = 200, h = 150;
   
   xitk_window_get_window_size(test->xwin, &width, &height);
@@ -523,7 +531,7 @@ static void create_frame(void) {
  */
 static void create_inputtext(void) {
   xitk_inputtext_widget_t  inp;
-  char                    *fontname = "*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
+  char                    *fontname = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
 
   XITK_WIDGET_INIT(&inp, test->imlibdata);
 
@@ -547,7 +555,7 @@ static void create_inputtext(void) {
  */
 static void create_label(void) {
   xitk_label_widget_t   lbl;
-  char                 *fontname = "*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
+  char                 *fontname = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
   int                   x = 150, y = 120, len = 100;
   xitk_font_t          *fs;
   int                   lbear, rbear, wid, asc, des;
@@ -609,7 +617,7 @@ static void create_button(void) {
     if(wimage) {
       unsigned int   col;
       xitk_font_t   *fs = NULL;
-      char          *fontname = "*-helvetica-bold-r-*-*-14-*-*-*-*-*-*-*";
+      char          *fontname = "-*-helvetica-bold-r-*-*-14-*-*-*-*-*-*-*";
       int            lbear, rbear, wid, asc, des;
       char          *label = _("Fire !!");
 
@@ -744,7 +752,7 @@ static void combo_select(xitk_widget_t *w, void *data, int select) {
  */
 static void create_combo(void) {
   xitk_combo_widget_t    cmb;
-  char                  *fontname = "*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
+  char                  *fontname = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
   int                    x = 150, y = 36, width = 100, height;
   xitk_font_t           *fs;
 
@@ -777,6 +785,7 @@ static void create_combo(void) {
 		   (test->combo = 
 		    xitk_noskin_combo_create(test->widget_list, &cmb,
 					     x, y, width, NULL, NULL)));
+  xitk_set_widget_tips_default(test->combo, "This is a combo box.");
   xitk_enable_and_show_widget(test->combo);
 }
 
@@ -785,7 +794,7 @@ static void create_combo(void) {
  */
 static void create_browser(void) {
   xitk_browser_widget_t  browser;
-  char                  *fontname = "*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
+  char                  *fontname = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
   Pixmap                 bg;
   int                    width, height;
 
@@ -837,10 +846,13 @@ static void create_browser(void) {
   xitk_enable_and_show_widget(test->browser);
   
   xitk_browser_update_list(test->browser, 
-			   (const char *const *)test->entries, test->num_entries, 0);
+			   (const char *const *)test->entries, NULL, test->num_entries, 0);
 
 }
 
+static void menu_checked(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  test->checked = !test->checked;
+}
 static void menu_test_end(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
   test_end(NULL, NULL);
 }
@@ -848,40 +860,55 @@ static void create_menu(void) {
   xitk_menu_widget_t   menu;
   int                  x = 10, y = 10;
   xitk_menu_entry_t    menu_entries[] = {
-    { "Popup menu:-------------------",  "<title>",       NULL,     NULL },
-    { "Load a file",                     NULL,            NULL,     NULL },
-    { "Reload",                          NULL,            NULL,     NULL },
-    { "Auto save",                       "<check>",       NULL,     NULL },
-    { "SEP",                             "<separator>",   NULL,     NULL },
-    { "Empty",                           "<branch>",      NULL,     NULL },
-    { "Playlist",                        "<branch>",      NULL,     NULL },
-    { "Playlist/Playlist Management",    "<title>",       NULL,     NULL },
-    { "Playlist/Load",                   NULL,            NULL,     NULL },
-    { "Playlist/Loop",                   "<branch>",      NULL,     NULL },
-    { "Playlist/Loop/no loop",           "<checked>",     NULL,     NULL },
-    { "Playlist/Loop/simple loop",       "<check>",       NULL,     NULL },
-    { "Playlist/Loop/SEP",               "<separator>",   NULL,     NULL },
-    { "Playlist/Loop/repeat entry",      "<check>",       NULL,     NULL },
-    { "Playlist/Loop/SEP",               "<separator>",   NULL,     NULL },
-    { "Playlist/Loop/shuffle",           "<check>",       NULL,     NULL },
-    { "Playlist/Loop/infinite shuffle",  "<check>",       NULL,     NULL },
-    { "Playlist/Loop/Extra",             "<branch>",      NULL,     NULL },
-    { "Playlist/Save",                   NULL,            NULL,     NULL },
-    { "Another branch",                  "<branch>",      NULL,     NULL },
-    { "Another branch/dummy",            NULL,            NULL,     NULL },
-    { "Another branch/vice\\/versa",     NULL,            NULL,     NULL },
-    { "SEP",                             "<separator>",   NULL,     NULL },
-    { "Save a file",                     NULL,            NULL,     NULL },
-    { "SEP",                             "<separator>",   NULL,     NULL },
-    { "Video",                           "<branch>",      NULL,     NULL },
-    { "Video/premier",                   "<branch>",      NULL,     NULL },
-    { "Video/premier/Le Voili",          NULL,            NULL,     NULL },
-    { "Video/second",                    NULL,            NULL,     NULL },
-    { "Video/troisieme",                 NULL,            NULL,     NULL },
-    { "Video/quatrieme",                 NULL,            NULL,     NULL },
-    { "SEP",                             "<separator>",   NULL,     NULL },
-    { "Quit",                            NULL,            menu_test_end, NULL },
-    { NULL,                              NULL,            NULL,     NULL }
+    { "Popup menu:",                     NULL,     "<title>",       NULL,     NULL },
+    { "Load a file",                     NULL,     NULL,            NULL,     NULL },
+    { "Reload",                          NULL,     NULL,            NULL,     NULL },
+    { "Auto save",                       NULL,
+      test->checked ? "<checked>" : "<check>",                      menu_checked, NULL },
+    { "SEP",                             NULL,     "<separator>",   NULL,     NULL },
+    { "Empty",                           NULL,     "<branch>",      NULL,     NULL },
+    { "Playlist",                        NULL,     "<branch>",      NULL,     NULL },
+    { "Playlist/Playlist Management",    NULL,     "<title>",       NULL,     NULL },
+    { "Playlist/Load",                   NULL,     NULL,            NULL,     NULL },
+    { "Playlist/Loop",                   NULL,     "<branch>",      NULL,     NULL },
+    { "Playlist/Loop/no loop",           NULL,     "<checked>",     NULL,     NULL },
+    { "Playlist/Loop/simple loop",       NULL,     "<check>",       NULL,     NULL },
+    { "Playlist/Loop/SEP",               NULL,     "<separator>",   NULL,     NULL },
+    { "Playlist/Loop/repeat entry",      NULL,     "<check>",       NULL,     NULL },
+    { "Playlist/Loop/SEP",               NULL,     "<separator>",   NULL,     NULL },
+    { "Playlist/Loop/shuffle",           NULL,     "<check>",       NULL,     NULL },
+    { "Playlist/Loop/infinite shuffle",  NULL,     "<check>",       NULL,     NULL },
+    { "Playlist/Loop/Extra",             NULL,     "<branch>",      NULL,     NULL },
+    { "Playlist/Save",                   NULL,     NULL,            NULL,     NULL },
+    { "Another branch",                  NULL,     "<branch>",      NULL,     NULL },
+    { "Another branch/dummy",            NULL,     NULL,            NULL,     NULL },
+    { "Another branch/vice\\/versa",     NULL,     NULL,            NULL,     NULL },
+    { "SEP",                             NULL,     "<separator>",   NULL,     NULL },
+    /* Testing deep sub-branches */
+    { "Deep",                            NULL,     "<branch>",      NULL,     NULL },
+    { "Deep/Deep 2",                     NULL,     "<branch>",      NULL,     NULL },
+    { "Deep/Deep 2/Deep 3",              NULL,     "<branch>",      NULL,     NULL },
+    { "Deep/Deep 2/Deep 3/Deep 4",       NULL,     "<branch>",      NULL,     NULL },
+    { "Deep/Deep 2/Deep 3/Deep 4/Deep 5",
+                                         NULL,     "<branch>",      NULL,     NULL },
+    { "Deep/Deep 2/Deep 3/Deep 4/Deep 5/Entry number 1",
+                                         NULL,     NULL,            NULL,     NULL },
+    { "Deep/Deep 2/Deep 3/Deep 4/Deep 5/SEP",
+                                         NULL,     "<separator>",   NULL,     NULL },
+    { "Deep/Deep 2/Deep 3/Deep 4/Deep 5/Entry number 2",
+                                         NULL,     NULL,            NULL,     NULL },
+    { "SEP",                             NULL,     "<separator>",   NULL,     NULL },
+    { "Save a file",                     NULL,     NULL,            NULL,     NULL },
+    { "SEP",                             NULL,     "<separator>",   NULL,     NULL },
+    { "Video",                           NULL,     "<branch>",      NULL,     NULL },
+    { "Video/premier",                   NULL,     "<branch>",      NULL,     NULL },
+    { "Video/premier/Le Voili",          NULL,     NULL,            NULL,     NULL },
+    { "Video/second",                    NULL,     NULL,            NULL,     NULL },
+    { "Video/troisieme",                 NULL,     NULL,            NULL,     NULL },
+    { "Video/quatrieme",                 NULL,     NULL,            NULL,     NULL },
+    { "SEP",                             NULL,     "<separator>",   NULL,     NULL },
+    { "Quit",                            "C-q",    NULL,            menu_test_end, NULL },
+    { NULL,                              NULL,     NULL,            NULL,     NULL }
   };
   
   XITK_WIDGET_INIT(&menu, test->imlibdata);
@@ -898,10 +925,8 @@ static void create_menu(void) {
   {
     xitk_menu_entry_t menu_entry;
 
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
     menu_entry.menu      = "Playlist/Loop/Extra/errrr?";
-    menu_entry.type      = NULL;
-    menu_entry.cb        = NULL;
-    menu_entry.user_data = NULL;
     xitk_menu_add_entry(test->menu, &menu_entry);
   }
   
@@ -915,7 +940,7 @@ static void create_menu(void) {
 int main(int argc, char **argv) {
   GC                          gc;
   xitk_labelbutton_widget_t   lb;
-  char                       *fontname = "*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
+  char                       *fontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
   int                         windoww = 600, windowh = 400;
   xitk_widget_t              *w;
   
@@ -995,13 +1020,10 @@ int main(int argc, char **argv) {
 
   deinit_test();
 
-  XLockDisplay (test->display);
-  XFreeGC(test->display, gc);
-  XUnlockDisplay (test->display);
-
   XCloseDisplay(test->display);
 
   free(test);
-  
+  test = NULL;
+
   return 1;
 }
