@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: widget.c,v 1.55 2003/07/18 21:34:40 f1rmb Exp $
+ * $Id: widget.c,v 1.58 2004/04/13 19:55:50 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -1103,7 +1103,7 @@ int xitk_click_notify_widget_list (xitk_widget_list_t *wl, int x, int y, int but
     }
   }
   
-  if((wl->widget_focused == NULL) && menu)
+  if(!(wl->widget_focused) && menu)
     wl->widget_focused = menu;
 
   return((bRepaint == 1));
@@ -1232,7 +1232,10 @@ void xitk_set_focus_to_next_widget(xitk_widget_list_t *wl, int backward) {
     }
     
     /* next widget will be focused */
-    if(widget == wl->widget_focused) {
+    if((widget == wl->widget_focused) ||
+       (widget && wl->widget_focused && 
+	(((wl->widget_focused->type & WIDGET_GROUP_MASK) == WIDGET_GROUP_MENU)
+	 && (wl->widget_focused->type & WIDGET_GROUP_WIDGET)))) {
       xitk_widget_t *next_widget;
       
       if ((wl->widget_focused->type & WIDGET_FOCUSABLE) &&
