@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: window.c,v 1.37 2003/07/18 21:34:40 f1rmb Exp $
+ * $Id: window.c,v 1.43 2004/04/13 19:55:50 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -248,7 +248,7 @@ xitk_window_t *xitk_window_create_window(ImlibData *im, int x, int y, int width,
 			       | CWOverrideRedirect | CWWinGravity ,
 			       &attr);
   
-  XSetStandardProperties(im->x.disp, xwin->window, title, title, None, NULL, 0, &hint);
+  XmbSetWMProperties(im->x.disp, xwin->window, title, title, NULL, 0, &hint, NULL, NULL);
 
   XSelectInput(im->x.disp, xwin->window, INPUT_MOTION | KeymapStateMask);
   
@@ -337,7 +337,7 @@ xitk_window_t *xitk_window_create_dialog_window(ImlibData *im, char *title,
 
   fs = xitk_font_load_font(im->x.disp, DEFAULT_BOLD_FONT_12);
   xitk_font_set_font(fs, bar->gc);
-  xitk_font_string_extent(fs, title, &lbear, &rbear, &wid, &asc, &des);
+  xitk_font_string_extent(fs, (title && strlen(title)) ? title : "Window", &lbear, &rbear, &wid, &asc, &des);
 
   XLOCK(im->x.disp);
   XCopyArea(im->x.disp, xwin->background->pixmap, pix_bg->pixmap, xwin->background->gc,
@@ -423,12 +423,11 @@ xitk_window_t *xitk_window_create_dialog_window(ImlibData *im, char *title,
     XSetForeground(im->x.disp, bar->gc, colorwhite);
   else
     XSetForeground(im->x.disp, bar->gc, (xitk_get_pixel_color_from_rgb(im, 85, 12, 135)));
-  
-  xitk_font_draw_string(fs, bar->pixmap, bar->gc, 
-	      (width - wid) - TITLE_BAR_HEIGHT, ((TITLE_BAR_HEIGHT+asc+des) >> 1) - des, 
-	      title, strlen(title));
   XUNLOCK(im->x.disp);
 
+  xitk_font_draw_string(fs, bar->pixmap, bar->gc, 
+			(width - wid) - TITLE_BAR_HEIGHT, ((TITLE_BAR_HEIGHT+asc+des) >> 1) - des, title, strlen(title));
+
   xitk_font_unload_font(fs);
 
   XLOCK(im->x.disp);
@@ -685,7 +684,7 @@ static void _xitk_window_destroy_window(xitk_widget_t *w, void *data) {
   }
 
   xitk_unregister_event_handler(&wd->key);
-
+  
   if(wd->widget_list) {
     xitk_destroy_widgets(wd->widget_list);
     
@@ -694,8 +693,8 @@ static void _xitk_window_destroy_window(xitk_widget_t *w, void *data) {
     XUNLOCK(wd->imlibdata->x.disp);
     
     xitk_list_free(wd->widget_list->l);
-    
-    XITK_FREE(wd->widget_list);
+
+    free(wd->widget_list);
   }
   
   XITK_FREE(wd);
