@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xitk.c,v 1.105 2003/07/18 21:34:40 f1rmb Exp $
+ * $Id: xitk.c,v 1.119 2004/04/13 19:55:50 f1rmb Exp $
  *
  */
 #define _XITK_C_
@@ -111,6 +111,7 @@ typedef struct {
 
 typedef struct {
   Display                    *display;
+  int                         verbosity;
   xitk_list_t                *list;
   xitk_list_t                *gfx;
   int                         use_xshm;
@@ -136,6 +137,15 @@ static Atom XA_WIN_LAYER = None, XA_STAYS_ON_TOP = None;
 static Atom XA_NET_WM_STATE = None, XA_NET_WM_STATE_ABOVE = None;
 static Atom XA_NET_WM_STATE_FULLSCREEN = None;
 
+static Atom XA_WM_WINDOW_TYPE = None;
+static Atom XA_WM_WINDOW_TYPE_DESKTOP = None;
+static Atom XA_WM_WINDOW_TYPE_DOCK = None;
+static Atom XA_WM_WINDOW_TYPE_TOOLBAR = None;
+static Atom XA_WM_WINDOW_TYPE_MENU = None;
+static Atom XA_WM_WINDOW_TYPE_UTILITY = None;
+static Atom XA_WM_WINDOW_TYPE_SPLASH = None;
+static Atom XA_WM_WINDOW_TYPE_DIALOG = None;
+static Atom XA_WM_WINDOW_TYPE_NORMAL = None;
 
 void widget_stop(void);
 
@@ -518,11 +528,21 @@ static uint32_t xitk_check_wm(Display *display) {
   }
 
   if(type & WM_TYPE_EWMH_COMP) {
-    XA_WIN_LAYER          = XInternAtom(display, "_NET_WM_STATE", False);
-    XA_STAYS_ON_TOP       = XInternAtom(display, "_NET_WM_STATE_STAYS_ON_TOP", False);
-    XA_NET_WM_STATE       = XInternAtom(display, "_NET_WM_STATE", False);
-    XA_NET_WM_STATE_ABOVE = XInternAtom(display, "_NET_WM_STATE_ABOVE", False);
+    XA_WIN_LAYER               = XInternAtom(display, "_NET_WM_STATE", False);
+    XA_STAYS_ON_TOP            = XInternAtom(display, "_NET_WM_STATE_STAYS_ON_TOP", False);
+    XA_NET_WM_STATE            = XInternAtom(display, "_NET_WM_STATE", False);
+    XA_NET_WM_STATE_ABOVE      = XInternAtom(display, "_NET_WM_STATE_ABOVE", False);
     XA_NET_WM_STATE_FULLSCREEN = XInternAtom(display, "_NET_WM_STATE_FULLSCREEN", False);
+
+    XA_WM_WINDOW_TYPE          = XInternAtom(display, "_NET_WM_WINDOW_TYPE", False);
+    XA_WM_WINDOW_TYPE_DESKTOP  = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DESKTOP", False);
+    XA_WM_WINDOW_TYPE_DOCK     = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DOCK", False);
+    XA_WM_WINDOW_TYPE_TOOLBAR  = XInternAtom(display, "_NET_WM_WINDOW_TYPE_TOOLBAR", False);
+    XA_WM_WINDOW_TYPE_MENU     = XInternAtom(display, "_NET_WM_WINDOW_TYPE_MENU", False);
+    XA_WM_WINDOW_TYPE_UTILITY  = XInternAtom(display, "_NET_WM_WINDOW_TYPE_UTILITY", False);
+    XA_WM_WINDOW_TYPE_SPLASH   = XInternAtom(display, "_NET_WM_WINDOW_TYPE_SPLASH", False);
+    XA_WM_WINDOW_TYPE_DIALOG   = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DIALOG", False);
+    XA_WM_WINDOW_TYPE_NORMAL   = XInternAtom(display, "_NET_WM_WINDOW_TYPE_NORMAL", False);
   }
   
   switch(type & WM_TYPE_COMP_MASK) {
@@ -552,63 +572,65 @@ static uint32_t xitk_check_wm(Display *display) {
 
   XUnlockDisplay(display);
   
-  printf("-[ WM type: ");
-  
-  if(type & WM_TYPE_GNOME_COMP)
-    printf("(GnomeCompliant) ");
-  if(type & WM_TYPE_EWMH_COMP)
-    printf("(EWMH) ");
-  
-  switch(type & WM_TYPE_COMP_MASK) {
-  case WM_TYPE_UNKNOWN:
-    printf("Unknown");
-    break;
-  case WM_TYPE_KWIN:
-    printf("KWIN");
-    break;
-  case WM_TYPE_E:
-    printf("Enlightenment");
-    break;
-  case WM_TYPE_ICE:
-    printf("Ice");
-    break;
-  case WM_TYPE_WINDOWMAKER:
-    printf("WindowMaker");
-    break;
-  case WM_TYPE_MOTIF:
-    printf("Motif(like?)");
-    break;
-  case WM_TYPE_XFCE:
-    printf("XFce");
-    break;
-  case WM_TYPE_SAWFISH:
-    printf("Sawfish");
-    break;
-  case WM_TYPE_METACITY:
-    printf("Metacity");
-    break;
-  case WM_TYPE_AFTERSTEP:
-    printf("Afterstep");
-    break;
-  case WM_TYPE_BLACKBOX:
-    printf("Blackbox");
-    break;
-  case WM_TYPE_LARSWM:
-    printf("LarsWM");
-    break;
-  case WM_TYPE_DTWM:
-    printf("dtwm");
-    break;
+  if(gXitk->verbosity) {
+    printf("[ WM type: ");
+    
+    if(type & WM_TYPE_GNOME_COMP)
+      printf("(GnomeCompliant) ");
+    if(type & WM_TYPE_EWMH_COMP)
+      printf("(EWMH) ");
+    
+    switch(type & WM_TYPE_COMP_MASK) {
+    case WM_TYPE_UNKNOWN:
+      printf("Unknown");
+      break;
+    case WM_TYPE_KWIN:
+      printf("KWIN");
+      break;
+    case WM_TYPE_E:
+      printf("Enlightenment");
+      break;
+    case WM_TYPE_ICE:
+      printf("Ice");
+      break;
+    case WM_TYPE_WINDOWMAKER:
+      printf("WindowMaker");
+      break;
+    case WM_TYPE_MOTIF:
+      printf("Motif(like?)");
+      break;
+    case WM_TYPE_XFCE:
+      printf("XFce");
+      break;
+    case WM_TYPE_SAWFISH:
+      printf("Sawfish");
+      break;
+    case WM_TYPE_METACITY:
+      printf("Metacity");
+      break;
+    case WM_TYPE_AFTERSTEP:
+      printf("Afterstep");
+      break;
+    case WM_TYPE_BLACKBOX:
+      printf("Blackbox");
+      break;
+    case WM_TYPE_LARSWM:
+      printf("LarsWM");
+      break;
+    case WM_TYPE_DTWM:
+      printf("dtwm");
+      break;
+    }
+    
+    if(wm_name)
+      printf(" {%s}", wm_name);
+    
+    printf(" ]-\n");
   }
-
-  if(wm_name) {
-    printf(" {%s}", wm_name);
+  
+  if(wm_name)
     free(wm_name);
-  }
-
-  printf(" ]-\n");
-
-
+  
   return type;
 }
 uint32_t xitk_get_wm_type(void) {
@@ -667,18 +689,18 @@ void xitk_set_layer_above(Window window) {
       XEvent xev;
       
       XLockDisplay(gXitk->display);
-      
-      xev.type                 = ClientMessage;
+      xev.xclient.type         = ClientMessage;
+      xev.xclient.display      = gXitk->display;
       xev.xclient.window       = window;
       xev.xclient.message_type = XA_NET_WM_STATE;
       xev.xclient.format       = 32;
       xev.xclient.data.l[0]    = 1;
       xev.xclient.data.l[1]    = XA_STAYS_ON_TOP;
-      xev.xclient.data.l[2]    = None;
+      xev.xclient.data.l[2]    = 0l;
+      xev.xclient.data.l[3]    = 0l;
+      xev.xclient.data.l[4]    = 0l;
       
-      XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), 
-		 False, SubstructureRedirectMask | SubstructureNotifyMask, (XEvent*) &xev);
-
+      XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), True, SubstructureRedirectMask, &xev);
       XUnlockDisplay(gXitk->display);
     }
     else {
@@ -767,29 +789,106 @@ void xitk_set_window_layer(Window window, int layer) {
   XUnlockDisplay(gXitk->display);
 }
 
-void xitk_set_ewmh_fullscreen(Window window) {
+static void _set_ewmh_state(Window window, Atom atom, int enable) {
   XEvent xev;
   
+  if((window == None) || (atom == None))
+    return;
+
+  xev.xclient.type         = ClientMessage;
+  xev.xclient.message_type = XA_NET_WM_STATE;
+  xev.xclient.display      = gXitk->display;
+  xev.xclient.window       = window;
+  xev.xclient.format       = 32;
+  xev.xclient.data.l[0]    = (enable == 1) ? 1 : 0;
+  xev.xclient.data.l[1]    = atom;
+  xev.xclient.data.l[2]    = 0l;
+  xev.xclient.data.l[3]    = 0l;
+  xev.xclient.data.l[4]    = 0l;
+
+  XLockDisplay(gXitk->display);
+  XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), True, SubstructureRedirectMask, &xev);
+  XUnlockDisplay(gXitk->display);
+}
+
+void xitk_set_ewmh_fullscreen(Window window) {
+  
   if(!(gXitk->wm_type & WM_TYPE_EWMH_COMP) || (window == None))
     return;
   
-  XLockDisplay(gXitk->display);
-  
-  xev.xclient.type         = ClientMessage;
-  xev.xclient.send_event   = True;
-  xev.xclient.display      = gXitk->display;
-  xev.xclient.window       = window;
-  xev.xclient.message_type = XA_NET_WM_STATE;
-  xev.xclient.format       = 32;
-  xev.xclient.data.l[0]    = (long) 1;
-  xev.xclient.data.l[1]    = (long) XA_NET_WM_STATE_FULLSCREEN;
-  xev.xclient.data.l[2]    = (long) None;
-  
-  XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), 
-	     False, SubstructureRedirectMask | SubstructureNotifyMask, (XEvent*) &xev);
-  
-  XUnlockDisplay(gXitk->display);
+  _set_ewmh_state(window, XA_NET_WM_STATE_FULLSCREEN, 1);
+  _set_ewmh_state(window, XA_STAYS_ON_TOP, 1);
+}
 
+void xitk_unset_ewmh_fullscreen(Window window) {
+  
+  if(!(gXitk->wm_type & WM_TYPE_EWMH_COMP) || (window == None))
+    return;
+  
+  _set_ewmh_state(window, XA_NET_WM_STATE_FULLSCREEN, 0);
+  _set_ewmh_state(window, XA_STAYS_ON_TOP, 0);
+}
+
+static void _set_wm_window_type(Window window, xitk_wm_window_type_t type, int value) {
+  if(window && (gXitk->wm_type & WM_TYPE_EWMH_COMP)) {
+    Atom *atom = NULL;
+    
+    switch(type) {
+    case WINDOW_TYPE_DESKTOP:
+      atom = &XA_WM_WINDOW_TYPE_DESKTOP;
+      break;
+    case WINDOW_TYPE_DOCK:
+      atom = &XA_WM_WINDOW_TYPE_DOCK;
+      break;
+    case WINDOW_TYPE_TOOLBAR:
+      atom = &XA_WM_WINDOW_TYPE_TOOLBAR;
+      break;
+    case WINDOW_TYPE_MENU:
+      atom = &XA_WM_WINDOW_TYPE_MENU;
+      break;
+    case WINDOW_TYPE_UTILITY:
+      atom = &XA_WM_WINDOW_TYPE_UTILITY;
+      break;
+    case WINDOW_TYPE_SPLASH:
+      atom = &XA_WM_WINDOW_TYPE_SPLASH;
+      break;
+    case WINDOW_TYPE_DIALOG:
+      atom = &XA_WM_WINDOW_TYPE_DIALOG;
+      break;
+    case WINDOW_TYPE_NORMAL:
+      atom = &XA_WM_WINDOW_TYPE_NORMAL;
+      break;
+    }
+    
+    if(atom) {
+      XEvent xev;
+
+      xev.xclient.type         = ClientMessage;
+      xev.xclient.display      = gXitk->display;
+      xev.xclient.window       = window;
+      xev.xclient.message_type = XA_WIN_LAYER;
+      xev.xclient.format       = 32;
+      xev.xclient.data.l[0]    = *atom;
+      xev.xclient.data.l[1]    = 0l;
+      xev.xclient.data.l[2]    = 0l;
+      xev.xclient.data.l[3]    = 0l;
+      xev.xclient.data.l[4]    = 0l;
+  
+      XLOCK(gXitk->display);
+      XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), False, SubstructureNotifyMask, &xev);
+
+      XRaiseWindow(gXitk->display, window);
+      XUNLOCK(gXitk->display);
+    }
+  }
+}
+
+void xitk_unset_wm_window_type(Window window, xitk_wm_window_type_t type) {
+  _set_wm_window_type(window, type, 0);
+}
+
+void xitk_set_wm_window_type(Window window, xitk_wm_window_type_t type) {
+  _set_wm_window_type(window, type, 1);
 }
 
 /*
@@ -866,12 +965,12 @@ xitk_register_key_t xitk_register_event_handler(char *name, Window window,
 						xitk_dnd_callback_t dnd_cb,
 						xitk_widget_list_t *wl, void *user_data) {
   __gfx_t   *fx;
-
+  
   //  printf("%s()\n", __FUNCTION__);
 
   fx = (__gfx_t *) xitk_xmalloc(sizeof(__gfx_t));
 
-  fx->name = (name != NULL) ? name : "NO_SET";
+  fx->name = name ? strdup(name) : strdup("NO_SET");
 
   fx->window    = window;
   fx->new_pos.x = 0;
@@ -969,7 +1068,6 @@ void xitk_unregister_event_handler(xitk_register_key_t *key) {
   
   fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
   
-  
   while(fx) {
     
     if(fx->key == *key) {
@@ -981,14 +1079,20 @@ void xitk_unregister_event_handler(xitk_register_key_t *key) {
       }
 
       fx->xevent_callback = NULL;
+      fx->newpos_callback = NULL;
+      fx->user_data       = NULL;
+
       xitk_list_delete_current(gXitk->gfx); 
+
+      free(fx->name);
       free(fx);
+      fx = NULL;
       
       MUTUNLOCK();
       return;
     }
-    fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
 
+    fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
   }
 
   MUTUNLOCK();
@@ -1005,7 +1109,7 @@ int xitk_get_window_info(xitk_register_key_t key, window_info_t *winf) {
   fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
     
   while(fx) {
-
+    
     if((fx->key == key) && (fx->window != None)) {
       Window c;
       
@@ -1497,15 +1601,9 @@ void xitk_xevent_notify(XEvent *event) {
 /*
  * Initiatization of widget internals.
  */
-void xitk_init(Display *display) {
+void xitk_init(Display *display, int verbosity) {
+  char buffer[256];
   
-  printf("-[ xiTK version %d.%d.%d "
-#ifdef WITH_XMB
-	 "[XMB]"
-#endif
-	 "]-\n",
-	 XITK_MAJOR_VERSION, XITK_MINOR_VERSION, XITK_SUB_VERSION);
-
   xitk_pid = getppid();
 
 #ifdef ENABLE_NLS
@@ -1514,6 +1612,7 @@ void xitk_init(Display *display) {
 
   gXitk = (__xitk_t *) xitk_xmalloc(sizeof(__xitk_t));
 
+  gXitk->verbosity       = verbosity;
   gXitk->list            = xitk_list_new();
   gXitk->gfx             = xitk_list_new();
   gXitk->display         = display;
@@ -1527,7 +1626,10 @@ void xitk_init(Display *display) {
   gXitk->modalw          = None;
 
   pthread_mutex_init (&gXitk->mutex, NULL);
-
+  
+  sprintf(buffer, "-[ xiTK version %d.%d.%d ", 
+	  XITK_MAJOR_VERSION, XITK_MINOR_VERSION, XITK_SUB_VERSION);
+  
   /* Check if SHM is working */
 #ifdef HAVE_SHM
   if(gXitk->use_xshm) {
@@ -1561,15 +1663,13 @@ void xitk_init(Display *display) {
 	  
 	  XShmAttach(display, &shminfo);
 	  XSync(display, False);
-	  if(xitk_x_error) {
+	  if(xitk_x_error)
 	    gXitk->use_xshm = 0;
-	    XITK_WARNING("-[ xiTK can't use XShm ]-\n");
-	  }
 	  else {
 	    XShmDetach(display, &shminfo);
-	    printf("-[ xiTK will use XShm ]-\n");
+	    sprintf(buffer, "%s%s", buffer, "[XShm]");
 	  }
-
+	  
 	  XDestroyImage(xim);
 	  shmdt(shminfo.shmaddr);
 	  shmctl(shminfo.shmid, IPC_RMID, 0);
@@ -1582,6 +1682,17 @@ void xitk_init(Display *display) {
   }
 #endif
 
+#ifdef WITH_XFT
+  sprintf(buffer, "%s%s", buffer, "[XFT]");
+#elif defined(WITH_XMB)
+  sprintf(buffer, "%s%s", buffer, "[XMB]");
+#endif
+  
+  sprintf(buffer, "%s%s", buffer, " ]-");
+
+  if(verbosity)
+    printf(buffer);
+
   gXitk->wm_type = xitk_check_wm(display);
   
   /* init font caching */
@@ -1722,14 +1833,15 @@ void xitk_run(xitk_startup_callback_t cb, void *data) {
 
   }
 
+  /* destroy font caching */
+  xitk_font_cache_done();
+  
   xitk_list_free(gXitk->list);
   xitk_list_free(gXitk->gfx);
   xitk_config_deinit(gXitk->config);
   
   XITK_FREE(gXitk);
 
-  /* destroy font caching */
-  xitk_font_cache_done();
 }
 
 /*
@@ -1745,6 +1857,12 @@ char *xitk_get_system_font(void) {
 char *xitk_get_default_font(void) {
   return xitk_config_get_default_font(gXitk->config);
 }
+int xitk_get_xmb_enability(void) {
+  return xitk_config_get_xmb_enability(gXitk->config);
+}
+void xitk_set_xmb_enability(int value) {
+  xitk_config_set_xmb_enability(gXitk->config, value);
+}
 int xitk_get_black_color(void) {
   return xitk_config_get_black_color(gXitk->config);
 }
@@ -1775,7 +1893,12 @@ long int xitk_get_timer_dbl_click(void) {
 int xitk_get_barstyle_feature(void) {
   return xitk_config_get_barstyle_feature(gXitk->config);
 }
-
+int xitk_get_checkstyle_feature(void) {
+  return xitk_config_get_checkstyle_feature(gXitk->config);
+}
+int xitk_get_menu_shortcuts_enability(void) {
+  return xitk_config_get_menu_shortcuts_enability(gXitk->config);
+}
 /*
  * copy src to dest and substitute special chars. dest should have 
  * enought space to store chars.
@@ -1816,7 +1939,8 @@ void xitk_subst_special_chars(char *src, char *dest) {
       if(*(s + 1) == '/') {
 	strcat(d, xitk_get_homedir());
 	d += (strlen(xitk_get_homedir()) - 1);
-      }
+      } else
+        *d = *s;
       break;
       
     default:
@@ -1849,7 +1973,10 @@ char *xitk_set_locale(void) {
  * Return home directory.
  */
 const char *xitk_get_homedir(void) {
-  struct passwd  pwd, *pw = NULL;
+#ifdef HAVE_GETPWUID_R
+  struct passwd  pwd;
+#endif
+  struct passwd *pw = NULL;
   static char    homedir[BUFSIZ] = {0,};
 
   if(homedir[0])
@@ -1866,8 +1993,10 @@ const char *xitk_get_homedir(void) {
       homedir[sizeof(homedir) - 1] = '\0';
     }
   } else {
-    strncpy(homedir, pw->pw_dir, sizeof(homedir));
+    char *s = strdup(pw->pw_dir);
+    strncpy(homedir, s, sizeof(homedir));
     homedir[sizeof(homedir) - 1] = '\0';
+    free(s);
   }
 
   if(!homedir[0]) {
@@ -1877,3 +2006,28 @@ const char *xitk_get_homedir(void) {
 
   return homedir;
 }
+
+/*
+ * Return 0/1 from char value (valids are 1/0, true/false, 
+ * yes/no, on/off. Case isn't checked.
+ */
+int xitk_get_bool_value(const char *val) {
+  static struct {
+    const char *str;
+    int value;
+  } bools[] = {
+    { "1",     1 }, { "true",  1 }, { "yes",   1 }, { "on",    1 },
+    { "0",     0 }, { "false", 0 }, { "no",    0 }, { "off",   0 },
+    { NULL,    0 }
+  };
+  int i;
+  
+  ABORT_IF_NULL(val);
+
+  for(i = 0; bools[i].str != NULL; i++) {
+    if(!(strcasecmp(bools[i].str, val)))
+      return bools[i].value;
+  }
+
+  return 0;
+}
