@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xitk.h,v 1.108 2003/07/18 21:34:40 f1rmb Exp $
+ * $Id: xitk.h,v 1.122 2004/04/13 19:55:50 f1rmb Exp $
  *
  * Unique public header file for xitk TK.
  *
@@ -37,14 +37,18 @@
 #include <stdarg.h>
 #include "xitk/Imlib-light/Imlib.h"
 
+#ifdef HAVE_ICONV
+#include <iconv.h>
+#endif
+
 #ifdef NEED_MRLBROWSER
 #include <xine.h>
 #endif
 
 #define XITK_MAJOR_VERSION          (0)
 #define XITK_MINOR_VERSION          (10)
-#define XITK_SUB_VERSION            (5)
-#define XITK_VERSION                "0.10.5"
+#define XITK_SUB_VERSION            (7)
+#define XITK_VERSION                "0.10.7"
 
 #define XITK_CHECK_VERSION(major, minor, sub)                          \
                                     (XITK_MAJOR_VERSION > (major) ||   \
@@ -315,6 +319,20 @@ typedef struct {
 #define WM_TYPE_LARSWM              0x0000000B
 #define WM_TYPE_DTWM                0x0000000C
 
+typedef enum {
+  WINDOW_TYPE_DESKTOP = 1,
+  WINDOW_TYPE_DOCK,
+  WINDOW_TYPE_TOOLBAR,
+  WINDOW_TYPE_MENU,
+  WINDOW_TYPE_UTILITY,
+  WINDOW_TYPE_SPLASH,
+  WINDOW_TYPE_DIALOG,
+  WINDOW_TYPE_NORMAL
+} xitk_wm_window_type_t;
+
+void xitk_set_wm_window_type(Window window, xitk_wm_window_type_t type);
+void xitk_unset_wm_window_type(Window window, xitk_wm_window_type_t type);
+
 /* 
  * See xitk_widget_list_[set/get]()
  */
@@ -500,6 +518,8 @@ typedef struct {
   char                             *skin_element_name;
   Window                            window_trans;
   int                               layer_above;
+  Pixmap                           *icon;
+  int                               set_wm_window_normal;
 
   int                               x;
   int                               y;
@@ -628,7 +648,8 @@ typedef struct {
 
 struct xitk_menu_entry_s {
   char                             *menu;
-  char                             *type; /* NULL, <separator>, <branch>, <check>, <checked> */
+  char                             *shortcut; /* displayed (can be NULL) */
+  char                             *type;     /* NULL, <separator>, <branch>, <check>, <checked> */
   xitk_menu_callback_t              cb;
   void                             *user_data;
 };
@@ -694,7 +715,7 @@ int xitk_get_window_info(xitk_register_key_t key, window_info_t *winf);
 /*
  * Initialization function, should be the first call to widget lib.
  */
-void xitk_init(Display *display);
+void xitk_init(Display *display, int verbosity);
 
 /*
  *
@@ -725,6 +746,7 @@ void xitk_set_window_layer(Window window, int layer);
  *
  */
 void xitk_set_ewmh_fullscreen(Window window);
+void xitk_unset_ewmh_fullscreen(Window window);
 
 /*
  * This function start the widget live. It's a block function,
@@ -744,6 +766,8 @@ void xitk_stop(void);
  */
 char *xitk_get_system_font(void);
 char *xitk_get_default_font(void);
+int xitk_get_xmb_enability(void);
+void xitk_set_xmb_enability(int value);
 int xitk_get_black_color(void);
 int xitk_get_white_color(void);
 int xitk_get_background_color(void);
@@ -755,6 +779,7 @@ long int xitk_get_timer_dbl_click(void);
 int xitk_get_barstyle_feature(void);
 unsigned long xitk_get_warning_foreground(void);
 unsigned long xitk_get_warning_background(void);
+int xitk_get_menu_shortcuts_enability(void);
 
 
 /*
@@ -1164,11 +1189,13 @@ xitk_widget_t *xitk_noskin_labelbutton_create (xitk_widget_list_t *wl,
  * Change label of button 'widget'.
  */
 int xitk_labelbutton_change_label(xitk_widget_t *, char *);
+int xitk_labelbutton_change_shortcut_label(xitk_widget_t *, char *, int, char *);
 
 /**
  * Return label of button 'widget'.
  */
 char *xitk_labelbutton_get_label(xitk_widget_t *);
+char *xitk_labelbutton_get_shortcut_label(xitk_widget_t *);
 
 /**
  * Get state of button 'widget'.
@@ -1337,7 +1364,7 @@ void xitk_browser_rebuild_browser(xitk_widget_t *w, int start);
 /**
  * Update the list, and rebuild button list
  */
-void xitk_browser_update_list(xitk_widget_t *w, const char *const *list, int len, int start);
+void xitk_browser_update_list(xitk_widget_t *w, const char *const *list, const char *const *shortcut, int len, int start);
 
 /**
  * slide up.
@@ -1524,6 +1551,18 @@ void xitk_mrlbrowser_set_tips_timeout(xitk_widget_t *w, int enabled, unsigned lo
 
 #endif
 
+/*
+ * return len of keystring (stored in kbuf)
+ * define ksym
+ * kbuf should be large enought (256 usually)
+ */
+int xitk_get_keysym_and_buf(XEvent *event, KeySym *ksym, char kbuf[]);
+
+/**
+ * return keypressed
+ */
+KeySym xitk_get_key_pressed(XEvent *event);
+
 /**
  * All states of modifiers (see xitk_get_key_modifier() bellow).
  */
@@ -1564,7 +1603,7 @@ void xitk_inputtext_change_text(xitk_widget_t *it, char *text);
 /*
  * Alloc a xitk_skin_config_t* memory area, nullify pointers.
  */
-xitk_skin_config_t *xitk_skin_init_config(void);
+xitk_skin_config_t *xitk_skin_init_config(ImlibData *);
 
 /*
  * Release all allocated memory of a xitk_skin_config_t* variable (element chained struct too).
@@ -1705,6 +1744,16 @@ char *xitk_skin_get_animation(xitk_skin_config_t *);
  */
 int xitk_skin_get_browser_entries(xitk_skin_config_t *, const char *);
 
+/*
+ *
+ */
+xitk_image_t *xitk_skin_get_image(xitk_skin_config_t *, const char *);
+
+/*
+ *
+ */
+int xitk_skin_get_max_buttons(xitk_skin_config_t *, const char *);
+
 /*
  *
  */
@@ -1793,6 +1842,37 @@ int xitk_font_get_descent(xitk_font_t *xtfs, const char *c);
  */
 void xitk_font_set_font(xitk_font_t *xtfs, GC gc);
 
+typedef struct {
+#ifdef HAVE_ICONV
+  iconv_t    id;
+#else
+  int        id;
+#endif
+} xitk_recode_t;
+
+/**
+ * get current locale encoding 
+ */
+char *xitk_get_system_encoding(void);
+
+/**
+ * prepare recoding
+ *   - when one of encoding is NULL, is prepared no conversion, strings will
+ *     be strdup()ed
+ *   - when encoding is "", it's got from system
+ */
+xitk_recode_t *xitk_recode_init(const char *src_encoding, const char *dst_encoding);
+
+/**
+ * destroy recoding
+ */
+void xitk_recode_done(xitk_recode_t *xr);
+
+/**
+ * recode string 'src' into 'dst' according to prepared 'xr'
+ */
+char *xitk_recode(xitk_recode_t *xr, const char *src);
+
 
 /**
  *
@@ -1848,6 +1928,11 @@ int xitk_combo_is_same_parent(xitk_widget_t *w1, xitk_widget_t *w2);
  */
 void xitk_combo_callback_exec(xitk_widget_t *w);
 
+/**
+ *
+ */
+xitk_widget_t *xitk_combo_get_label_widget(xitk_widget_t *w);
+
 /**
  *
  */
@@ -1940,6 +2025,10 @@ void draw_bevel_three_state(ImlibData *im, xitk_image_t *p);
  */
 void draw_bevel_two_state(ImlibData *im, xitk_image_t *p);
 
+void draw_three_state_round_style(ImlibData *im, xitk_image_t *p, int x, int y, int d, int w, int checked);
+void draw_three_state_check_style(ImlibData *im, xitk_image_t *p, int x, int y, int d, int w, int checked);
+
+
 void draw_paddle_three_state_vertical(ImlibData *im, xitk_image_t *p);
 void draw_paddle_three_state_horizontal(ImlibData *im, xitk_image_t *p);
 
@@ -2258,6 +2347,7 @@ xitk_widget_t *xitk_noskin_intbox_create(xitk_widget_list_t *wl,
 					 xitk_widget_t **iw, xitk_widget_t **mw, xitk_widget_t **lw);
 void xitk_intbox_set_value(xitk_widget_t *, int);
 int xitk_intbox_get_value(xitk_widget_t *);
+xitk_widget_t *xitk_intbox_get_input_widget(xitk_widget_t *w);
 
 xitk_widget_t *xitk_noskin_doublebox_create(xitk_widget_list_t *wl,
 					    xitk_doublebox_widget_t *ib,
@@ -2265,6 +2355,7 @@ xitk_widget_t *xitk_noskin_doublebox_create(xitk_widget_list_t *wl,
 					    xitk_widget_t **iw, xitk_widget_t **mw, xitk_widget_t **lw);
 void xitk_doublebox_set_value(xitk_widget_t *, double);
 double xitk_doublebox_get_value(xitk_widget_t *);
+xitk_widget_t *xitk_doublebox_get_input_widget(xitk_widget_t *w);
 
 int xitk_widget_list_set(xitk_widget_list_t *wl, int param, void *data);
 void *xitk_widget_list_get(xitk_widget_list_t *wl, int param);
